// ATSelectDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ATSelectDlg.h"
#include "CSFAbstractTree.h"
#include "SNDevice.h"
#include "SNConfig.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CATSelectDlg dialog


CATSelectDlg::CATSelectDlg(CATSelectDlg::PathType pathType,
						   bool bAllowNonUnique,
						   LPCTSTR szTitle,
						   CProntoString *pPSDev,
						   CProntoString *pPSPan,
						   CProntoString *pPSBut,
						   const CCSFAbstractTree *pAbstractTree,
						   CWnd* pParent)
	: CSuckLessDialog(CATSelectDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CATSelectDlg)
	//}}AFX_DATA_INIT

	m_pathType = pathType;
	m_bAllowNonUnique = bAllowNonUnique;

	m_csTitle = szTitle;

	m_pPSDev = pPSDev;
	m_pPSPan = pPSPan;
	m_pPSBut = pPSBut;

	m_pAbstractTree = pAbstractTree;

	setLParamToReturnToParent(FALSE);
}

void CATSelectDlg::DoDataExchange(CDataExchange* pDX)
{
	CSuckLessDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CATSelectDlg)
	DDX_Control(pDX, IDOK, m_ctrlButtonOK);
	DDX_Control(pDX, IDC_TREE, m_ctrlTree);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CATSelectDlg, CSuckLessDialog)
	//{{AFX_MSG_MAP(CATSelectDlg)
	ON_NOTIFY(TVN_SELCHANGED, IDC_TREE, OnSelchangedTree)
	ON_NOTIFY(NM_DBLCLK, IDC_TREE, OnDblclkTree)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CATSelectDlg message handlers

BOOL CATSelectDlg::OnInitDialog() 
{
	CSuckLessDialog::OnInitDialog();
	
	this->setWindowIcon(IDR_MAINFRAME);

	SetWindowText(m_csTitle);

	// -- Initial contents --

	m_ctrlButtonOK.EnableWindow(FALSE);

	m_treeImageList.Create(16, 16, ILC_MASK, 0, 4);
	m_treeImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_DEVICE));
	m_treeImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_MACROGROUP));
	m_treeImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_PANEL));
	m_treeImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_BUTTON));
	m_treeImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_HARDBUTTONS));
	m_treeImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_CROSS));
	m_treeImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_CROSS));
	m_treeImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_CROSS));
	m_treeImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_CROSS));
	m_treeImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_CROSS));
	m_ctrlTree.SetImageList(&m_treeImageList, TVSIL_NORMAL);

	if (NULL != g_pProntoFont)
	{
		m_ctrlTree.SetFont(g_pProntoFont);
	}

	if (! populateTree() )
	{
		MessageBox("Error populating tree", SUPERNUDELIST_ERROR_TITLE, MB_OK | MB_ICONEXCLAMATION);
	}

	// -- Control setup --

	addControlInfo(IDC_TREE);
	addControlInfo(IDC_BUTTON_HELP);
	addControlInfo(IDOK);
	addControlInfo(IDCANCEL);

	setControlRelationship(IDC_TREE,		IDD, CControlInfo::ACOMMON_RESIZEHEIGHT_RESIZEWIDTH);
	setControlRelationship(IDC_BUTTON_HELP,	IDD, CControlInfo::ACOMMON_BOTTOM_LEFT);
	setControlRelationship(IDOK,			IDD, CControlInfo::ACOMMON_BOTTOM_RIGHT);
	setControlRelationship(IDCANCEL,		IDD, CControlInfo::ACOMMON_BOTTOM_RIGHT);

	controlAndFrameInfoIsSetup();

	setMinXToCurrent();
	setMinYToCurrent();

	this->setRememberWindowPosition(SN_REG_PATH_WINDOWPOS, SN_REG_VALUE_WINDOWPOS_ATSELECT);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

bool CATSelectDlg::populateTree(void)
{
	bool bResult = true;

	const CCSFATDevice *pDevice;

	HTREEITEM htItemDev;
	int imgIdx;
	CString label;
	bool bSelectDev;

	for (pDevice = m_pAbstractTree->GetDevices()->GetFirst();
		 NULL != pDevice;
		 pDevice = m_pAbstractTree->GetDevices()->GetNext(pDevice))
	{
		if (!pDevice->IsUniqueName())
		{
			imgIdx = m_bAllowNonUnique ? ILIDX_DEVICE : ILIDX_DEVICE_CROSS;
			label.Format("%s  (non-unique name)", pDevice->GetName()->GetPronto(true));
		}
		else
		{
			imgIdx = ILIDX_DEVICE;
			label = pDevice->GetName()->GetPronto(true);
		}

		// Add device
		htItemDev = m_ctrlTree.InsertItem(TVIF_PARAM|TVIF_TEXT|TVIF_IMAGE|TVIF_SELECTEDIMAGE,
											label, imgIdx, imgIdx, 0, 0,
											(LPARAM)pDevice, TVI_ROOT, TVI_LAST);

		if (NULL == htItemDev)
		{
			bResult = false;
			break;
		}

		bSelectDev = (NULL != m_pPSDev) && (0 == m_pPSDev->Compare(pDevice->GetName(), false));

		if (ILIDX_DEVICE == imgIdx)
		{
			if (((PT_BUTTON == m_pathType) && (0 != pDevice->GetHardButtons()->GetNumElements())	\
				 && (!populateTreeHardButtons(pDevice->GetHardButtons(), htItemDev, bSelectDev)))	\
			||	(!populateTreePanels(pDevice->GetPanels(), htItemDev, bSelectDev))					)
			{
				bResult = false;
				break;
			}
		}
		else if (bSelectDev)
		{
			m_ctrlTree.SelectItem(htItemDev);
			m_ctrlTree.EnsureVisible(htItemDev);
		}
	}

	return(bResult);
}

bool CATSelectDlg::populateTreePanels(const CCSFATPanelList *pPanelList,
									  HTREEITEM htItemDev,
									  bool bSelectDev)
{
	bool bResult = true;

	const CCSFATPanel *pPanel;

	HTREEITEM htItemPan;
	int imgIdx;
	CString label;
	bool bSelectPan;

	for (pPanel = pPanelList->GetFirst();
		 NULL != pPanel;
		 pPanel = pPanelList->GetNext(pPanel))
	{
		if (!pPanel->IsUniqueName())
		{
			imgIdx = m_bAllowNonUnique ? ILIDX_PANEL : ILIDX_PANEL_CROSS;
			label.Format("%s  (non-unique name)", pPanel->GetName()->GetPronto(true));
		}
		else
		{
			imgIdx = ILIDX_PANEL;
			label = pPanel->GetName()->GetPronto(true);
		}

		htItemPan = m_ctrlTree.InsertItem(TVIF_PARAM|TVIF_TEXT|TVIF_IMAGE|TVIF_SELECTEDIMAGE,
											label, imgIdx, imgIdx, 0, 0,
											(LPARAM)pPanel, htItemDev, TVI_LAST);

		if (NULL == htItemPan)
		{
			bResult = false;
			break;
		}

		bSelectPan = bSelectDev && (NULL != m_pPSPan) && (0 == m_pPSPan->Compare(pPanel->GetName(), false));

		if ((PT_BUTTON == m_pathType) && (ILIDX_PANEL == imgIdx))
		{
			if (!populateTreeButtons(pPanel->GetButtons(), htItemDev, htItemPan, bSelectPan))
			{
				bResult = false;
				break;
			}
		}

		if (bSelectPan
		&&	((PT_PANEL == m_pathType) || (ILIDX_PANEL != imgIdx)
		||	 (NULL == m_pPSBut) || m_pPSBut->IsUnassigned()))
		{
			m_ctrlTree.Expand(htItemDev, TVE_EXPAND);
			m_ctrlTree.SelectItem(htItemPan);
			m_ctrlTree.EnsureVisible(htItemDev);
			m_ctrlTree.EnsureVisible(htItemPan);
		}
	}

	return(bResult);
}

bool CATSelectDlg::populateTreeHardButtons(const CCSFATButtonList *pButtonList,
										   HTREEITEM htItemDev,
										   bool bSelectDev)
{
	bool bResult = true;

	HTREEITEM htItemHardButtons;
	bool bSelectHardButtons;

	htItemHardButtons = m_ctrlTree.InsertItem(TVIF_PARAM|TVIF_TEXT|TVIF_IMAGE|TVIF_SELECTEDIMAGE,
												"Hard Buttons", ILIDX_HARDBUTTONS, ILIDX_HARDBUTTONS,
												0, 0, 0, htItemDev, TVI_LAST);

	if (NULL == htItemHardButtons)
	{
		bResult = false;
	}
	else
	{
		bSelectHardButtons = bSelectDev && ((NULL == m_pPSPan) || m_pPSPan->IsUnassigned());

		if (!populateTreeButtons(pButtonList, htItemDev, htItemHardButtons, bSelectHardButtons))
		{
			bResult = false;
		}
	}

	return(bResult);
}

bool CATSelectDlg::populateTreeButtons(const CCSFATButtonList *pButtonList,
									   HTREEITEM htItemDev,
									   HTREEITEM htItemPan,
									   bool bSelectPan)
{
	bool bResult = true;

	const CCSFATButton *pButton;

	HTREEITEM htItemBut;
	int imgIdx;
	CString label;

	for (pButton = pButtonList->GetFirst();
		 NULL != pButton;
		 pButton = pButtonList->GetNext(pButton))
	{
		if (!pButton->IsUniqueName())
		{
			imgIdx = m_bAllowNonUnique ? ILIDX_BUTTON : ILIDX_BUTTON_CROSS;
			label.Format("%s  (non-unique name)", pButton->GetName()->GetPronto(true));
		}
		else
		{
			imgIdx = ILIDX_BUTTON;
			label = pButton->GetName()->GetPronto(true);
		}

		htItemBut = m_ctrlTree.InsertItem(TVIF_PARAM|TVIF_TEXT|TVIF_IMAGE|TVIF_SELECTEDIMAGE,
											label, imgIdx, imgIdx, 0, 0,
											(LPARAM)pButton, htItemPan, TVI_LAST);

		if (NULL == htItemBut)
		{
			bResult = false;
			break;
		}

		if (bSelectPan && (NULL != m_pPSBut) && (0 == m_pPSBut->Compare(pButton->GetName(), false)))
		{
			m_ctrlTree.Expand(htItemDev, TVE_EXPAND);
			m_ctrlTree.Expand(htItemPan, TVE_EXPAND);
			m_ctrlTree.EnsureVisible(htItemDev);
			m_ctrlTree.EnsureVisible(htItemPan);
			m_ctrlTree.SelectItem(htItemBut);
			m_ctrlTree.EnsureVisible(htItemBut);
		}
	}

	return(bResult);
}

void CATSelectDlg::OnSelchangedTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

	validateSelection(pNMTreeView->itemNew.hItem);

	*pResult = 0;
}

BOOL CATSelectDlg::validateSelection(HTREEITEM htItem)
{
	BOOL bResult = FALSE;

	int nImage;
	int nSelectedImage;

	if ((NULL != htItem)											\
	&&	(NULL != m_ctrlTree.GetItemData(htItem))					\
	&&	(m_ctrlTree.GetItemImage(htItem, nImage, nSelectedImage))	\
	&&	(((PT_PANEL == m_pathType) && (ILIDX_PANEL  == nImage))	\
	||	 ((PT_BUTTON == m_pathType) && (ILIDX_BUTTON == nImage)))	)
	{
		bResult = TRUE;
	}

	m_ctrlButtonOK.EnableWindow(bResult);

	return(bResult);
}

void CATSelectDlg::OnDblclkTree(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// Make sure they double-clicked on the selected item.
	// Without this the user can select an item and double-click
	// a +/- box to expand/collapse an item and we think they
	// double-clicked something they were nowhere near pointing at.

	DWORD mPos = ::GetMessagePos();
	POINTS points = MAKEPOINTS(mPos);

	TVHITTESTINFO hitTest;
	ZeroMemory(&hitTest, sizeof(hitTest));
	hitTest.pt.x = points.x;
	hitTest.pt.y = points.y;

	m_ctrlTree.ScreenToClient(&hitTest.pt);
	
	HTREEITEM htiPoint = m_ctrlTree.HitTest(&hitTest);
	HTREEITEM htiSelected = m_ctrlTree.GetSelectedItem();

	if ((NULL != htiPoint) && (htiPoint == htiSelected) && (TVHT_ONITEM&(hitTest.flags)))
	{
		if (validateSelection(htiSelected))
		{
			OnOK();
		}
	}

	*pResult = 0;
}

void CATSelectDlg::OnOK() 
{
	HTREEITEM htiSel = m_ctrlTree.GetSelectedItem();

	if (validateSelection(htiSel))
	{
		bool bOkay = false;

		if (PT_PANEL == m_pathType)
		{
			const CCSFATPanel *pATPanel = (const CCSFATPanel *)m_ctrlTree.GetItemData(htiSel);

			if (NULL != pATPanel)
			{
				CCSFPanelPath *pPanelPath = pATPanel->CreatePath();

				if (NULL != pPanelPath)
				{
					if (NULL != m_pPSDev)
					{
						m_pPSDev->SetPronto(pPanelPath->GetDeviceName()->GetPronto(false));
					}

					if (NULL != m_pPSPan)
					{
						m_pPSPan->SetPronto(pPanelPath->GetPanelName()->GetPronto(false));
					}

					if (NULL != m_pPSBut)
					{
						m_pPSBut->Reset();
					}

					bOkay = true;

					delete pPanelPath;
				}
			}
		}
		else if (PT_BUTTON == m_pathType)
		{
			const CCSFATButton *pATButton = (const CCSFATButton *)m_ctrlTree.GetItemData(htiSel);

			if (NULL != pATButton)
			{
				CCSFButtonPath *pButtonPath = pATButton->CreatePath();

				if (NULL != pButtonPath)
				{
					if (NULL != m_pPSDev)
					{
						m_pPSDev->SetPronto(pButtonPath->GetDeviceName()->GetPronto(false));
					}

					if (NULL != m_pPSPan)
					{
						m_pPSPan->SetPronto(pButtonPath->GetPanelName()->GetPronto(false));
					}

					if (NULL != m_pPSBut)
					{
						m_pPSBut->SetPronto(pButtonPath->GetButtonName()->GetPronto(false));
					}

					bOkay = true;

					delete pButtonPath;
				}
			}
		}

		if (bOkay)
		{
			setLParamToReturnToParent(TRUE);
			CSuckLessDialog::OnOK();
		}
	}
}
