// CSFAbstractTree.cpp: implementation of the CCSFAbstractTree class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CSFSection.h"
#include "CSFAbstractTree.h"
#include "UnsignedBaseTenNumber.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------

CCSFAbstractTree::CCSFAbstractTree()
{
}

CCSFAbstractTree::~CCSFAbstractTree()
{
}

void CCSFAbstractTree::FindDevicesPronto(list<CCSFATDevice *> &deviceList, const char *device)
{
	deviceList.clear();

	if (NULL != device)
	{
		CCSFATDevice *pDev = NULL;

		for (pDev = m_devices.GetNonConst(m_devices.GetByNamePronto(device, pDev));
			 NULL != pDev;
			 pDev = m_devices.GetNonConst(m_devices.GetByNamePronto(device, pDev)))
		{
			deviceList.push_back(pDev);
		}
	}
}

void CCSFAbstractTree::FindPanelsPronto(list<const CCSFATPanel *> &panelList,
												const char *device, const char *panel) const
{
	panelList.clear();

	if ((NULL != device) && (NULL != panel))
	{
		const CCSFATDevice *pDev = NULL;

		for (pDev = GetDevices()->GetByNamePronto(device, pDev);
			 NULL != pDev;
			 pDev = GetDevices()->GetByNamePronto(device, pDev))
		{
			const CCSFATPanel *pPan = NULL;
			
			for (pPan = pDev->GetPanels()->GetByNamePronto(panel, pPan);
				 NULL != pPan;
				 pPan = pDev->GetPanels()->GetByNamePronto(panel, pPan))
			{
				panelList.push_back(pPan);
			}
		}
	}
}

void CCSFAbstractTree::FindButtonsPronto(list<const CCSFATButton *> &buttonList,
												const char *device, const char *panel,
												const char *button) const
{
	buttonList.clear();

	if ((NULL != device) && (NULL != button))
	{
		const CCSFATDevice *pDev = NULL;

		for (pDev = GetDevices()->GetByNamePronto(device, pDev);
			 NULL != pDev;
			 pDev = GetDevices()->GetByNamePronto(device, pDev))
		{
			if (NULL != panel) // soft button
			{
				const CCSFATPanel *pPan = NULL;
				
				for (pPan = pDev->GetPanels()->GetByNamePronto(panel, pPan);
					 NULL != pPan;
					 pPan = pDev->GetPanels()->GetByNamePronto(panel, pPan))
				{
					const CCSFATButton *pBut = NULL;

					for (pBut = pPan->GetButtons()->GetByNamePronto(button, pBut);
						 NULL != pBut;
						 pBut = pPan->GetButtons()->GetByNamePronto(button, pBut))
					{
						buttonList.push_back(pBut);
					}
				}
			}
			else if (NULL != button) // hard button
			{
				const CCSFATButton *pBut = NULL;

				for (pBut = pDev->GetHardButtons()->GetByNamePronto(button, pBut);
					 NULL != pBut;
					 pBut = pDev->GetHardButtons()->GetByNamePronto(button, pBut))
				{
					buttonList.push_back(pBut);
				}
			}
		}
	}
}

bool CCSFAbstractTree::Build(CTypedError *pTError, const CCSFSection *pCSFRootSection)
{
	bool bResult = true;

	CCSFATDevice *pATDev;
	char *deviceNameEscaped;

	bool bPrefixErrors = CTypedError::SafeIsNoError(pTError);

	for (CCSFSection::const_subIterator iter = pCSFRootSection->subBegin();
		 bResult && (pCSFRootSection->subEnd() != iter);
		 iter++)
	{
		if ((*iter)->ScanForDeviceEscaped(&deviceNameEscaped))
		{
			pATDev = new CCSFATDevice();
			pATDev->GetName()->SetEscaped(deviceNameEscaped);

			if ((!pATDev->Build(pTError, *iter))									\
			||  (!m_devices.AddBelowAndOwn(pTError, pATDev, m_devices.GetLast()))	)
			{
				delete pATDev;
				bResult = false;
			}

			// deviceNameEscaped allocated by ScanForDevice
			delete [] deviceNameEscaped;
		}
	}

	if ((!bResult) && bPrefixErrors)
	{
		CTypedError::SafePrefixMsgError(pTError, "Error parsing CCF file");
	}

	return(bResult);
}

const CCSFATDeviceList *CCSFAbstractTree::GetDevices(void) const
{
	return(&m_devices);
}

void CCSFAbstractTree::DeleteDeviceFromATAndCSF(CCSFATDevice *pDevice)
{
	pDevice->GetCSFSection()->GetParent()->RemoveAndDeleteSubSectionByPointer(pDevice->GetCSFSection());

	m_devices.RemoveAndDelete(pDevice);
}

//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------

CCSFATDevice::CCSFATDevice()
{
	m_pCSFSection = NULL;
}

CCSFATDevice::~CCSFATDevice()
{
}

bool CCSFATDevice::Build(CTypedError *pTError, CCSFSection *pCSFDeviceSection)
{
	bool bResult = true;

	CCSFATPanel *pATPan;
	CCSFATButton *pATBut;
	char *panelNameEscaped;
	char *hardButtonNameEscaped;

	bool bPrefixErrors = CTypedError::SafeIsNoError(pTError);

	for (CCSFSection::subIterator iter = pCSFDeviceSection->subBegin();
		 pCSFDeviceSection->subEnd() != iter;
		 iter++)
	{
		if ((*iter)->ScanForPanelEscaped(&panelNameEscaped))
		{
			pATPan = new CCSFATPanel(this);
			pATPan->GetName()->SetEscaped(panelNameEscaped);

			if ((!pATPan->Build(pTError, *iter))								\
			||  (!m_panels.AddBelowAndOwn(pTError, pATPan, m_panels.GetLast()))	)
			{
				delete pATPan;
				bResult = false;
			}

			// panelNameEscaped allocated by ScanForPanel
			delete [] panelNameEscaped;
		}
		else if ((*iter)->ScanForHardButtonEscaped(&hardButtonNameEscaped))
		{
			pATBut = new CCSFATButton(this);
			pATBut->GetName()->SetEscaped(hardButtonNameEscaped);

			if ((!pATBut->BuildHard(pTError, *iter))										\
			||	(!m_hardButtons.AddBelowAndOwn(pTError, pATBut, m_hardButtons.GetLast()))	)
			{
				delete pATBut;
				bResult = false;
			}

			// hardButtonNameEscaped allocated by ScanForHardButton
			delete [] hardButtonNameEscaped;
		}
	}

	if ((!bResult) && bPrefixErrors)
	{
		if (!this->GetName()->IsUnassigned())
		{
			CTypedError::SafePrefixMsgError(pTError, "%s \"%s\"", this->getTypeName(), this->GetName()->GetPrintable(true));
		}
		else
		{
			CTypedError::SafePrefixMsgError(pTError, "%s <unnamed>", this->getTypeName());
		}
	}

	if (bResult)
	{
		m_pCSFSection = pCSFDeviceSection;
	}

	return(bResult);
}

CCSFSection *CCSFATDevice::GetCSFSection(void) const
{
	return(m_pCSFSection);
}

/*
// Delete the object when finished with it.
CCSFDevicePath *CCSFATDevice::CreatePath(void) const
{
	CCSFDevicePath *pResPath = new CCSFDevicePath();
	pResPath->SetDeviceNamePronto(this->GetName()->GetPronto(false));

	return(pResPath);
}
*/

const char *CCSFATDevice::getTypeName(void) const
{
	return("device");
}

const CCSFATPanelList *CCSFATDevice::GetPanels(void) const
{
	return(&m_panels);
}

const CCSFATButtonList *CCSFATDevice::GetHardButtons(void) const
{
	return(&m_hardButtons);
}

CProntoString *CCSFATDevice::GetName(void)
{
	return(&m_name);
}

const CProntoString *CCSFATDevice::GetName(void) const
{
	return(&m_name);
}

bool CCSFATDevice::IsValid(CTypedError *pTError) const
{
	if (m_name.IsUnassigned())
	{
		CTypedError::SafeSetMsgError(pTError, "%s has no name", getTypeName());
		return(false);
	}

	return(true);
}

// Returns < 0 if this <  pDevice
// Returns   0 if this == pDevice
// Returns > 0 if this >  pDevice
int CCSFATDevice::CompareForSort(const CCSFATDevice *pDevice) const
{
	if (NULL == pDevice)
	{
		return(1);
	}
	else
	{
		return(m_name.Compare(pDevice->GetName(), false));
	}
}

//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------

CCSFATPanel::CCSFATPanel(const CCSFATDevice *pParentDevice)
{
	m_pParentDevice = pParentDevice;
	m_pCSFSection = NULL;
}

CCSFATPanel::~CCSFATPanel()
{

}

bool CCSFATPanel::Build(CTypedError *pTError, CCSFSection *pCSFPanelSection)
{
	bool bResult = true;

	m_csfJumpString.erase();

	CCSFATButton *pATBut;
	char *buttonNameEscaped;
	unsigned int buttonNumber;

	bool bPrefixErrors = CTypedError::SafeIsNoError(pTError);

	for (CCSFSection::subIterator iter = pCSFPanelSection->subBegin();
		 pCSFPanelSection->subEnd() != iter;
		 iter++)
	{
		if ((*iter)->ScanForSoftButtonEscaped(&buttonNameEscaped, &buttonNumber))
		{
			pATBut = new CCSFATButton(this);
			pATBut->GetName()->SetEscaped(buttonNameEscaped);

			if ((!pATBut->BuildSoft(pTError, *iter, buttonNumber))					\
			||  (!m_buttons.AddBelowAndOwn(pTError, pATBut, m_buttons.GetLast()))	)
			{
				delete pATBut;
				bResult = false;
			}

			// buttonNameEscaped allocated by ScanForButton
			delete [] buttonNameEscaped;
		}
		else if ((*iter)->ScanForFrame())
		{
			// Recursive call to go inside all frames which can contain
			// buttons and further levels of nested frames.
			if (!this->Build(pTError, *iter))
			{
				bPrefixErrors = false;
				bResult = false;
			}
		}
	}

	if ((!bResult) && bPrefixErrors)
	{
		if (! this->GetName()->IsUnassigned() )
		{
			CTypedError::SafePrefixMsgError(pTError, "%s \"%s\"", this->getTypeName(), this->GetName()->GetPrintable(true));
		}
		else
		{
			CTypedError::SafePrefixMsgError(pTError, "%s <unnamed>", this->getTypeName());
		}
	}

	if (bResult)
	{
		m_pCSFSection = pCSFPanelSection;
	}

	return(bResult);
}

CCSFSection *CCSFATPanel::GetCSFSection(void) const
{
	return(m_pCSFSection);
}

const CCSFATDevice *CCSFATPanel::GetParentDevice(void) const
{
	return(m_pParentDevice);
}

// Delete the object when finished with it.
CCSFPanelPath *CCSFATPanel::CreatePath(void) const
{
	CCSFPanelPath *pResPath = new CCSFPanelPath();
	pResPath->SetDeviceNamePronto(this->GetParentDevice()->GetName()->GetPronto(false));
	pResPath->SetPanelNamePronto(this->GetName()->GetPronto(false));

	return(pResPath);
}

const char *CCSFATPanel::getTypeName(void) const
{
	return("panel");
}

const CCSFATButtonList *CCSFATPanel::GetButtons(void) const
{
	return(&m_buttons);
}

CProntoString *CCSFATPanel::GetName(void)
{
	return(&m_name);
}

const CProntoString *CCSFATPanel::GetName(void) const
{
	return(&m_name);
}

bool CCSFATPanel::IsValid(CTypedError *pTError) const
{
	if (m_name.IsUnassigned())
	{
		CTypedError::SafeSetMsgError(pTError, "%s has no name", getTypeName());
		return(false);
	}
	else if (NULL == m_pParentDevice)
	{
		CTypedError::SafeSetMsgError(pTError, "%s \"%s\" has no parent", getTypeName(), GetName()->GetPrintable(true));
		return(false);
	}

	return(true);
}

// Returns < 0 if this <  pPanel
// Returns   0 if this == pPanel
// Returns > 0 if this >  pPanel
int CCSFATPanel::CompareForSort(const CCSFATPanel *pPanel) const
{
	if (NULL == pPanel)
	{
		return(1);
	}
	else
	{
		return(m_name.Compare(pPanel->GetName(), false));
	}
}

const char *CCSFATPanel::GetCSFJumpString(void) const
{
	if (m_csfJumpString.empty())
	{
		m_csfJumpString = "JumpDevicePanel(\"";
		m_csfJumpString += m_pParentDevice->GetName()->GetEscaped(true);
		m_csfJumpString += "\",\"";
		m_csfJumpString += GetName()->GetEscaped(true);
		m_csfJumpString += "\");";
	}

	return(m_csfJumpString.c_str());
}

//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------

// For soft buttons.
CCSFATButton::CCSFATButton(const CCSFATPanel *pParentPanel)
{
	// There are two constructors to keep up-to-date.
	m_pCSFSection = NULL;

	m_pParentPanel = pParentPanel;
	m_pParentDevice = NULL;

	m_buttonNumber = 0;

	m_bHasAction = false;
	m_bHasActionWithAlias = false;
}

// For hard buttons.
CCSFATButton::CCSFATButton(const CCSFATDevice *pParentDevice)
{
	// There are two constructors to keep up-to-date.
	m_pCSFSection = NULL;

	m_pParentPanel = NULL;
	m_pParentDevice = pParentDevice;

	m_buttonNumber = 0;

	m_bHasAction = false;
	m_bHasActionWithAlias = false;
}

CCSFATButton::~CCSFATButton()
{
}

bool CCSFATButton::BuildSoft(CTypedError *pTError, CCSFSection *pCSFButtonSection,
								unsigned int buttonNumber)
{
	bool bResult = true;

	bool bPrefixErrors = CTypedError::SafeIsNoError(pTError);

	m_csfAliasString.erase();

	m_buttonNumber = buttonNumber;

	const CCSFSection *pActionSection = pCSFButtonSection->FindSubSectionByNamePrefixEscaped("Action(");

	if (NULL != pActionSection)
	{
		m_bHasAction = true;

		if ((NULL != pActionSection->FindSubSectionByNamePrefixEscaped("AliasDeviceButton("))
		||	(NULL != pActionSection->FindSubSectionByNamePrefixEscaped("AliasDeviceKey(")))
		{
			m_bHasActionWithAlias = true;
		}
	}

	if ((!bResult) && bPrefixErrors)
	{
		if (! this->GetName()->IsUnassigned() )
		{
			CTypedError::SafePrefixMsgError(pTError, "%s \"%s\"", this->getTypeName(), this->GetName()->GetPrintable(true));
		}
		else
		{
			CTypedError::SafePrefixMsgError(pTError, "%s <unnamed>", this->getTypeName());
		}
	}

	if (bResult)
	{
		m_pCSFSection = pCSFButtonSection;
	}

	return(bResult);
}

bool CCSFATButton::BuildHard(CTypedError *pTError, CCSFSection *pCSFButtonSection)
{
	bool bResult = true;

	bool bPrefixErrors = CTypedError::SafeIsNoError(pTError);

	m_csfAliasString.erase();

	m_buttonNumber = 0;

	m_bHasAction = true;

	if ((NULL != pCSFButtonSection->FindSubSectionByNamePrefixEscaped("AliasDeviceButton("))\
	||	(NULL != pCSFButtonSection->FindSubSectionByNamePrefixEscaped("AliasDeviceKey("))	 )
	{
		m_bHasActionWithAlias = true;
	}

	if ((!bResult) && bPrefixErrors)
	{
		if (! this->GetName()->IsUnassigned() )
		{
			CTypedError::SafePrefixMsgError(pTError, "%s \"%s\"", this->getTypeName(), this->GetName()->GetPrintable(true));
		}
		else
		{
			CTypedError::SafePrefixMsgError(pTError, "%s <unnamed>", this->getTypeName());
		}
	}

	if (bResult)
	{
		m_pCSFSection = pCSFButtonSection;
	}

	return(bResult);
}

const CCSFATPanel *CCSFATButton::GetParentPanelIfSoft(void) const
{
	return(m_pParentPanel);
}

const CCSFATDevice *CCSFATButton::GetParentDeviceIfHard(void) const
{
	return(m_pParentDevice);
}

CCSFSection *CCSFATButton::GetCSFSection(void) const
{
	return(m_pCSFSection);
}

// Delete the object when finished with it.
CCSFButtonPath *CCSFATButton::CreatePath(void) const
{
	CCSFButtonPath *pResPath = new CCSFButtonPath();

	if (NULL != m_pParentPanel)
	{
		pResPath->SetDeviceNamePronto(m_pParentPanel->GetParentDevice()->GetName()->GetPronto(false));
		pResPath->SetPanelNamePronto(m_pParentPanel->GetName()->GetPronto(false));
	}
	else if (NULL != m_pParentDevice)
	{
		pResPath->SetDeviceNamePronto(m_pParentDevice->GetName()->GetPronto(false));
		pResPath->SetPanelNamePronto(NULL);
	}
	
	pResPath->SetButtonNamePronto(this->GetName()->GetPronto(false));

	return(pResPath);
}

const char *CCSFATButton::getTypeName(void) const
{
	return("button");
}

bool CCSFATButton::IsAction(void) const
{
	return(m_bHasAction);
}

bool CCSFATButton::IsActionWithAlias(void) const
{
	return(m_bHasActionWithAlias);
}

CProntoString *CCSFATButton::GetName(void)
{
	return(&m_name);
}

const CProntoString *CCSFATButton::GetName(void) const
{
	return(&m_name);
}

unsigned int CCSFATButton::GetButtonNumber(void) const
{
	return(m_buttonNumber);
}

bool CCSFATButton::IsValid(CTypedError *pTError) const
{
	if (m_name.IsUnassigned())
	{
		CTypedError::SafeSetMsgError(pTError, "%s has no name", getTypeName());
		return(false);
	}
	else if ((NULL == m_pParentPanel) && (NULL == m_pParentDevice))
	{
		CTypedError::SafeSetMsgError(pTError, "%s \"%s\" has no parent", getTypeName(), GetName()->GetPrintable(true));
		return(false);
	}
	else if ((NULL != m_pParentPanel) && (NULL != m_pParentDevice))
	{
		CTypedError::SafeSetMsgError(pTError, "%s \"%s\" built incorrectly due to internal error", getTypeName(), GetName()->GetPrintable(true));
		return(false);
	}
	else if ((0 >= m_buttonNumber) && (NULL != m_pParentPanel))
	{
		CTypedError::SafeSetMsgError(pTError, "%s \"%s\" has invalid number (%d)", getTypeName(), GetName()->GetPrintable(true), m_buttonNumber);
		return(false);
	}

	return(true);
}

// Returns < 0 if this <  pButton
// Returns   0 if this == pButton
// Returns > 0 if this >  pButton
int CCSFATButton::CompareForSort(const CCSFATButton *pButton) const
{
	if (NULL == pButton)
	{
		return(1);
	}
	else
	{
		return(m_name.Compare(pButton->GetName(), false));
	}
}

const char *CCSFATButton::GetCSFAliasString(void) const
{
	if (m_csfAliasString.empty())
	{
		if (NULL != m_pParentPanel)
		{
			CUnsignedBaseTenNumber ubtn(m_buttonNumber);

			m_csfAliasString = "AliasDeviceButton(\"";
			m_csfAliasString += m_pParentPanel->GetParentDevice()->GetName()->GetEscaped(true);
			m_csfAliasString += "\",\"";
			m_csfAliasString += m_pParentPanel->GetName()->GetEscaped(true);
			m_csfAliasString += "\",\"";
			m_csfAliasString += GetName()->GetEscaped(true);
			m_csfAliasString += "\",";
			m_csfAliasString += ubtn.GetAsciiRepresentation();
			m_csfAliasString += "); /* Reference to Button ";
			m_csfAliasString += ubtn.GetAsciiRepresentation();
			m_csfAliasString += " */";
		}
		else if (NULL != m_pParentDevice)
		{
			m_csfAliasString = "AliasDeviceKey(\"";
			m_csfAliasString += m_pParentDevice->GetName()->GetEscaped(true);
			m_csfAliasString += "\",\"";
			m_csfAliasString += GetName()->GetEscaped(true);
			m_csfAliasString += "\");";
		}
	}

	return(m_csfAliasString.c_str());
}

//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------
