// CSFAbstractTree.h: interface for the CCSFAbstractTree class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CSFABSTRACTTREE_H__36E2DE9E_F85B_4534_A6A1_DEA15F0C7BB1__INCLUDED_)
#define AFX_CSFABSTRACTTREE_H__36E2DE9E_F85B_4534_A6A1_DEA15F0C7BB1__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SNList.h"
#include "CSFPath.h"

class CCSFSection;
class CTypedError;
class CCSFATDevice;
class CCSFATPanel;

/////////////////////////////////////////////////////////////////////////

class CCSFATButton : public CSNListObject
{
	friend class CSNList<CCSFATButton>;

public:
	CCSFATButton(const CCSFATPanel *pParentPanel);		// For soft buttons.
	CCSFATButton(const CCSFATDevice *pParentDevice);	// For hard buttons.
	virtual ~CCSFATButton();

	bool IsValid(CTypedError *pTError) const;

	// Returns < 0 if this <  pButton
	// Returns   0 if this == pButton
	// Returns > 0 if this >  pButton
	int CompareForSort(const CCSFATButton *pButton) const;

	CProntoString *GetName(void);
	const CProntoString *GetName(void) const;
	bool IsAction(void) const;
	bool IsActionWithAlias(void) const;
	unsigned int GetButtonNumber(void) const;

	const CCSFATPanel *GetParentPanelIfSoft(void) const;
	const CCSFATDevice *GetParentDeviceIfHard(void) const;

	bool BuildSoft(CTypedError *pTError, CCSFSection *pCSFButtonSection, unsigned int buttonNumber);
	bool BuildHard(CTypedError *pTError, CCSFSection *pCSFButtonSection);

	// Delete the object when finished with it.
	CCSFButtonPath *CreatePath(void) const;

	const char *GetCSFAliasString(void) const;

	CCSFSection *GetCSFSection(void) const;

protected:
	const char *getTypeName(void) const;

private:
	CCSFSection *m_pCSFSection;
	CProntoString m_name;
	const CCSFATPanel *m_pParentPanel;		// NULL iff it's a hard button.
	const CCSFATDevice *m_pParentDevice;	// Non-NULL iff it's a hard button.
	unsigned int m_buttonNumber;
	bool m_bHasAction;
	bool m_bHasActionWithAlias;
	mutable string m_csfAliasString;
};

/////////////////////////////////////////////////////////////////////////

class CCSFATButtonList : public CSNList<CCSFATButton>
{
public:
	CCSFATButtonList()
	{
		SetRequireUniqueNames(false);
	}

	virtual ~CCSFATButtonList()
	{
	}
};

/////////////////////////////////////////////////////////////////////////

class CCSFATPanel : public CSNListObject
{
	friend class CSNList<CCSFATPanel>;

public:
	CCSFATPanel(const CCSFATDevice *pParentDevice);
	virtual ~CCSFATPanel();

	bool IsValid(CTypedError *pTError) const;

	// Returns < 0 if this <  pPanel
	// Returns   0 if this == pPanel
	// Returns > 0 if this >  pPanel
	int CompareForSort(const CCSFATPanel *pPanel) const;

	CProntoString *GetName(void);
	const CProntoString *GetName(void) const;
	const CCSFATButtonList *GetButtons(void) const;
	const CCSFATDevice *GetParentDevice(void) const;

	bool Build(CTypedError *pTError, CCSFSection *pCSFPanelSection);

	// Delete the object when finished with it.
	CCSFPanelPath *CreatePath(void) const;

	const char *GetCSFJumpString(void) const;

	CCSFSection *GetCSFSection(void) const;

protected:
	const char *getTypeName(void) const;

private:
	CCSFSection *m_pCSFSection;
	CProntoString m_name;
	const CCSFATDevice *m_pParentDevice;
	CCSFATButtonList m_buttons;
	mutable string m_csfJumpString;
};

/////////////////////////////////////////////////////////////////////////

class CCSFATPanelList : public CSNList<CCSFATPanel>
{
public:
	CCSFATPanelList()
	{
		SetRequireUniqueNames(false);
	}

	virtual ~CCSFATPanelList()
	{
	}
};

/////////////////////////////////////////////////////////////////////////

class CCSFATDevice : public CSNListObject
{
	friend class CSNList<CCSFATDevice>;

public:
	CCSFATDevice();
	virtual ~CCSFATDevice();

	bool IsValid(CTypedError *pTError) const;

	// Returns < 0 if this <  pDevice
	// Returns   0 if this == pDevice
	// Returns > 0 if this >  pDevice
	int CompareForSort(const CCSFATDevice *pDevice) const;

	CProntoString *GetName(void);
	const CProntoString *GetName(void) const;
	const CCSFATPanelList *GetPanels(void) const;
	const CCSFATButtonList *GetHardButtons(void) const;

	bool Build(CTypedError *pTError, CCSFSection *pCSFDeviceSection);

	// Delete the object when finished with it.
//	CCSFDevicePath *CreatePath(void) const;

	CCSFSection *GetCSFSection(void) const;

protected:
	const char *getTypeName(void) const;

private:
	CCSFSection *m_pCSFSection;
	CProntoString m_name;
	CCSFATPanelList m_panels;
	CCSFATButtonList m_hardButtons;
};

/////////////////////////////////////////////////////////////////////////

class CCSFATDeviceList : public CSNList<CCSFATDevice>
{
public:
	CCSFATDeviceList()
	{
		SetRequireUniqueNames(false);
	}
	
	virtual ~CCSFATDeviceList()
	{
	}
};

/////////////////////////////////////////////////////////////////////////

class CCSFAbstractTree  
{
public:
	CCSFAbstractTree();
	virtual ~CCSFAbstractTree();

	bool Build(CTypedError *pTError, const CCSFSection *pCSFRootSection);

	const CCSFATDeviceList *GetDevices(void) const;

	void FindDevicesPronto(list<CCSFATDevice *> &deviceList, const char *device);

	void FindPanelsPronto(list<const CCSFATPanel *> &panelList,
							const char *device, const char *panel) const;

	void FindButtonsPronto(list<const CCSFATButton *> &buttonList,
							const char *device, const char *panel, const char *button) const;

	void DeleteDeviceFromATAndCSF(CCSFATDevice *pDevice);

protected:
	CCSFATDeviceList m_devices;

};

/////////////////////////////////////////////////////////////////////////

#endif // !defined(AFX_CSFABSTRACTTREE_H__36E2DE9E_F85B_4534_A6A1_DEA15F0C7BB1__INCLUDED_)
