// CSFPath.h: interface for the CCSFPath class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CSFPATH_H__9A0792D9_7A0E_4334_9988_B5EB61AD6F9C__INCLUDED_)
#define AFX_CSFPATH_H__9A0792D9_7A0E_4334_9988_B5EB61AD6F9C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SNList.h"

/////////////////////////////////////////////////////////////////////////

class CCSFAbstractTree;
class CCSFATButton;
class CCSFATPanel;

/////////////////////////////////////////////////////////////////////////

class CCSFPanelPath
{
public:
	CCSFPanelPath();
	virtual ~CCSFPanelPath();

	void Reset(void);
	void Set(const CCSFPanelPath *pToCopy);

	bool IsValid(CTypedError *pTError) const;

	bool ResolveCSFPath(CTypedError *pTError, bool bRequireUnique, const CCSFAbstractTree *pAT,
							const char *ownerType,
							const char *ownerName,
							const char *pathName);

	// Only call after a successful ResolveCSFPath call.
	const list<const CCSFATPanel *> *GetATPanels(void) const;

	const CProntoString *GetName(void) const;

	const CProntoString *GetDeviceName(void) const;
	const CProntoString *GetPanelName(void) const;
	void SetDeviceNameEscaped(const char *name);
	void SetDeviceNamePronto(const char *name);
	void SetPanelNameEscaped(const char *name);
	void SetPanelNamePronto(const char *name);

	xmlNodePtr AddToXML(xmlNodePtr parentNode) const;
	bool BuildFromXML(CTypedError *pTError, CStringList *pSLErrorDetails, xmlNodePtr pNode);

protected:
	const char *m_separator;

	CProntoString m_deviceName;
	CProntoString m_panelName;

	mutable CProntoString m_description;

	list<const CCSFATPanel *> m_atPanels;
};

/////////////////////////////////////////////////////////////////////////

class CCSFButtonPath : public CSNListObject
{
	friend class CSNList<CCSFButtonPath>;
	friend class CSNUpdatableList<CCSFButtonPath>;
	friend class CSNSetableList<CCSFButtonPath>;

public:
	CCSFButtonPath();
	virtual ~CCSFButtonPath();

	bool IsValid(CTypedError *pTError) const;

	// Returns < 0 if this <  pButtonPath
	// Returns   0 if this == pButtonPath
	// Returns > 0 if this >  pButtonPath
	int CompareForSort(const CCSFButtonPath *pButtonPath) const;

	void Reset(void);
	void Set(const CCSFButtonPath *pToCopy);

	bool ResolveCSFPath(CTypedError *pTError, bool bRequireUnique, const CCSFAbstractTree *pAT,
							const char *ownerType,
							const char *ownerName,
							const char *pathName);

	// Only call after a successful ResolveCSFPath call.
	const list<const CCSFATButton *> *GetATButtons(void) const;

	const CProntoString *GetName(void) const;

	const CProntoString *GetDeviceName(void) const;
	const CProntoString *GetPanelName(void) const;
	const CProntoString *GetButtonName(void) const;
	void SetDeviceNameEscaped(const char *name);
	void SetDeviceNamePronto(const char *name);
	void SetPanelNameEscaped(const char *name);
	void SetPanelNamePronto(const char *name);
	void SetButtonNameEscaped(const char *name);
	void SetButtonNamePronto(const char *name);

	xmlNodePtr AddToXML(xmlNodePtr parentNode) const;
	bool BuildFromXML(CTypedError *pTError, CStringList *pSLErrorDetails, xmlNodePtr pNode);

protected:
	const char *m_separator;

	CProntoString m_deviceName;
	CProntoString m_panelName;
	CProntoString m_buttonName;

	mutable CProntoString m_description;

	list<const CCSFATButton *> m_atButtons;

	// For SNList.
	const char *getTypeName(void) const;
};

class CCSFButtonPathList : public CSNSetableList<CCSFButtonPath>
{
public:
	CCSFButtonPathList()
	{
		SetSort(true);
	}

	virtual ~CCSFButtonPathList()
	{
	}
};

/////////////////////////////////////////////////////////////////////////

#endif // !defined(AFX_CSFPATH_H__9A0792D9_7A0E_4334_9988_B5EB61AD6F9C__INCLUDED_)
