
#ifndef __CSFSECTION_H_
#define __CSFSECTION_H_

#include "SimpleTextFile.h"

/////////////////////////////////////////////////////////////////////////

class CCSFSection
{
public:
	typedef list<CCSFSection *>::iterator subIterator;
	typedef list<CCSFSection *>::const_iterator const_subIterator;

	enum FontType
	{
		FONT_0  = 0,
		FONT_8  = 8,
		FONT_10 = 10,
		FONT_18 = 18
	};

	enum Colour
	{
		COLOUR_BLACK,
		COLOUR_DARKGRAY,
		COLOUR_LIGHTGRAY,
		COLOUR_WHITE
	};

public:
	CCSFSection(CCSFSection *pSectionParent);
	virtual ~CCSFSection(void);

	CCSFSection *CCSFSection::GetParent(void);
	const CCSFSection *CCSFSection::GetParent(void) const;

	// Call this on the root before adding any buttons; it is NOT calculated automatically.
	void CalcHighestButtonNumber(void);

	const char *GetTitleEscaped(void);
	void SetTitleEscaped(const char *title);
	void SetTitleEscapedFormat(const char *format, ...);

	subIterator subBegin(void);
	subIterator subEnd(void);
	const_subIterator subBegin(void) const;
	const_subIterator subEnd(void) const;

	bool ReadNextSection(CSimpleTextFile::const_lineIterator &lineIter,
						 const CSimpleTextFile::const_lineIterator &lineIterEnd,
						 CTypedError *pTError);

	// Set *pLineCount to the number of lines written.
	bool Write(FILE *pfOutputCSF, int *pLineCount, int indentLevel) const;

	// The subsection is returned so it can have further details added when required.
	// Titles, formats, and labels can be NULL.
	CCSFSection *AddSubSection(void);
	CCSFSection *AddEscapedTitleOnlySubSection(const char *subTitle);
	CCSFSection *AddEscapedTitleOnlySubSectionFormat(const char *format, ...);
	CCSFSection *AddEscapedFrameSubSection(const char *label,
											unsigned int xpos, unsigned int ypos,
											unsigned int width, unsigned int height,
											CCSFSection::FontType fontType,
											CCSFSection::Colour labelTextColour,
											const char *iconPath);

	// See CalcHighestButtonNumber
	// label can be NULL.
	// The subsection is returned so it can have further details added when required.
	CCSFSection *AddEscapedButtonSubSection(const char *label,
											unsigned int xpos, unsigned int ypos,
											unsigned int width, unsigned int height,
											CCSFSection::FontType fontType,
											CCSFSection::Colour labelTextColour,
											const char *iconPath,
											CCSFSection **ppActionSection);

	// Returns NULL if there is no matching subsection.
	CCSFSection *FindSubSectionByNameEscaped(const char *name) const;
	CCSFSection *FindSubSectionByNamePrefixEscaped(const char *namePrefix) const;

	// Returns true if a button was parsed and the output arguments are valid.
	// If returns true and pNameBuffer not NULL on input, *pNameBuffer is the name and should be delete[]'d.
	// If returns true and pNumber not NULL on input, *pNumber is the number.
	bool ScanForSoftButtonEscaped(char **pNameBuffer, unsigned int *pNumber) const;

	// Returns true if a hard button was parsed and the output arguments are valid.
	// If returns true and pNameBuffer not NULL on input, *pNameBuffer is the name and should be delete[]'d.
	bool ScanForHardButtonEscaped(char **pNameBuffer) const;

	// Returns true if a device was parsed and the output arguments are valid.
	// If returns true and pNameBuffer not NULL on input, *pNameBuffer is the name and should be delete[]'d.
	bool ScanForDeviceEscaped(char **pNameBuffer) const;

	// Returns true if a panel was parsed and the output arguments are valid.
	// If returns true and pNameBuffer not NULL on input, *pNameBuffer is the name and should be delete[]'d.
	bool ScanForPanelEscaped(char **pNameBuffer) const;

	// Returns true if a frame was parsed.
	bool ScanForFrame(void) const;

	// Returns true if a Device Jump was parsed and the ouput arguments are valid.
	// If returns true and pDeviceNameBuffer not NULL on input, *pDeviceNameBuffer is the device name and should be delete[]'d.
	// If returns true and pPanelNameBuffer not NULL on input, *pPanelNameBuffer is the device name and should be delete[]'d.
	// pDeviceNameBuffer may be "this" (literally), although the function will attempt to resolve to a real device name.
	bool ScanForDevicePanelJumpEscaped(char **pDeviceNameBuffer, char **pPanelNameBuffer);

	void RemoveAndDeleteSubSectionByPointer(CCSFSection *pSectionToFind);

	static bool IsHardButtonName(const char *name);

protected:
	void indentLine(FILE *pfOutputCSF, int indentLevel) const;
	const char *getColourString(CCSFSection::Colour kula) const;

protected:
	CCSFSection *m_pSectionParent;
	char *m_title;
	bool m_bHasSubBody;

	unsigned int *m_pHighestButtonNumber;

	list<CCSFSection *> m_subSections;
};

/////////////////////////////////////////////////////////////////////////



#endif
