// CSVFile.cpp: implementation of the CCSVFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "CSVFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CCSVFile::CCSVFile(int numColumns)
{
	m_numColumns = numColumns;
}

CCSVFile::CCSVFile(const char *szFilename)
{
	// Read first non-empty line to get numColumns

	std::fstream fsCSVFile(szFilename, std::ios_base::in);

	int numColumns = 0;

	while (std::ios_base::goodbit == fsCSVFile.rdstate())
	{
		std::string strLine;

		std::getline(fsCSVFile, strLine);

		// Count the number of columns.
		numColumns = 0;

		const char *pszLine = strLine.c_str();
		size_t tokenBufferSize = strLine.length() + 1;
		char *tokenBuffer = new char[tokenBufferSize];

		while (Tokenize(&pszLine, ",", NULL, tokenBuffer, tokenBufferSize, false, true, true))
		{
			numColumns++;
		}

		if (1 < numColumns)
		{
			break;
		}
	}

	m_numColumns = numColumns; // Could be zero, of course.

	fsCSVFile.close();
}

CCSVFile::~CCSVFile()
{

}

int CCSVFile::GetNumColumns(void) const
{
	return(m_numColumns);
}

CCSVFile::const_lineIter CCSVFile::Begin(void) const
{
	return(m_lines.begin());
}

CCSVFile::const_lineIter CCSVFile::End(void) const
{
	return(m_lines.end());
}

CCSVFile::lineIter CCSVFile::Begin(void)
{
	return(m_lines.begin());
}

CCSVFile::lineIter CCSVFile::End(void)
{
	return(m_lines.end());
}

CCSVFile::lineIter CCSVFile::NewLine(void)
{
	vector<string> line(m_numColumns);

	m_lines.push_back(line);

	lineIter iter = m_lines.end();
	iter--;

	return(iter);
}

void CCSVFile::SetLineColumn(lineIter line, int column, const char *value)
{
	if (column < m_numColumns)
	{
		(*line)[column] = value;
	}
}

string CCSVFile::GetLineColumn(const_lineIter line, int column) const
{
	if (column < m_numColumns)
	{
		return((*line)[column]);
	}
	else
	{
		return("");
	}
}

bool CCSVFile::LoadFromFile(CTypedError *pTError, const char *szFilename)
{
	bool bResult = true;

	if (0 == m_numColumns)
	{
		CTypedError::SafeSetMsgError(pTError, "Input file appears to have no columns. Could not read from %s", szFilename);
		bResult = false;
	}
	else
	{
		std::fstream fsCSVFile(szFilename, std::ios_base::in);

		if (std::ios_base::goodbit != fsCSVFile.rdstate())
		{
			CTypedError::SafeSetMsgError(pTError, "Could not read from %s", szFilename);
			bResult = false;
		}
		else
		{
			while (std::ios_base::goodbit == fsCSVFile.rdstate())
			{
				std::string strLine;

				std::getline(fsCSVFile, strLine);

				// Skip completely empty lines.
				bool bAllSpace = true;

				for (int i = 0; i < strLine.length(); i++)
				{
					if (!isspace(strLine[i]))
					{
						bAllSpace = false;
						break;
					}
				}

				if (!bAllSpace)
				{
					// Extract line.

					int nCol = 0;

					lineIter liter = this->End();

					const char *pszLine = strLine.c_str();
					size_t tokenBufferSize = strLine.length() + 1;
					char *tokenBuffer = new char[tokenBufferSize];

					while ((nCol < m_numColumns) && Tokenize(&pszLine, ",", NULL, tokenBuffer, tokenBufferSize, false, true, true))
					{
						if (liter == this->End())
						{
							liter = this->NewLine();
						}

						SetLineColumn(liter, nCol, tokenBuffer);

						nCol++;
					}
				}
			}
		}

		fsCSVFile.close();
	}

	return(bResult);
}

bool CCSVFile::SaveToFile(CTypedError *pTError, const char *szFilename)
{
	bool bResult = true;

	FILE *pFile = fopen(szFilename, "w");

	if (NULL == pFile)
	{
		CTypedError::SafeSetMsgError(pTError, "Could not write to %s", szFilename);
		bResult = false;
	}
	else
	{
		for (lineIter iter = m_lines.begin(); bResult && (iter != m_lines.end()); iter++)
		{
			bResult = lineToFile(pTError, pFile, *iter);
		}

		fclose(pFile);
	}

	return(bResult);
}

bool CCSVFile::lineToFile(CTypedError *pTError, FILE *pFile, vector<string>&line)
{
	bool bResult = true;

	string outLine;

	for (int i = 0; i < m_numColumns; i++)
	{
		if (0 != i)
		{
			outLine += ',';
		}

		outLine += escapeColStr(line[i]);
	}

	fprintf(pFile, "%s\n", outLine.c_str());

	return(bResult);
}

string CCSVFile::escapeColStr(const string colStr)
{
	bool bDoEscape = (string::npos != colStr.find_first_of(" ,\""));

	if (!bDoEscape)
	{
		return(colStr);
	}
	else
	{
		string escapedStr = colStr;

		// Double-up all quotes.
		

		for (string::iterator iter = escapedStr.begin(); iter != escapedStr.end(); iter++)
		{
			if ((*iter) == '"')
			{
				escapedStr.insert(iter++, '"');
			}
		}

		// Surround string by quotes.
		escapedStr.insert(escapedStr.begin(), '"');
		escapedStr.insert(escapedStr.end(),   '"');

		return(escapedStr);
	}
}
