// DeviceAddDlg.cpp : implementation file
//

#include "stdafx.h"
#include "DeviceAddDlg.h"
#include "DeviceDlg.h"
#include "CSFAbstractTree.h"
#include "SNDevice.h"
#include "SNConfig.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDeviceAddDlg dialog


CDeviceAddDlg::CDeviceAddDlg(CSNDeviceList *pDeviceList, const CCSFAbstractTree *pAbstractTree,
							 const CSNDevice **ppDeviceResult, CWnd* pParent)
	: CSuckLessDialog(CDeviceAddDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDeviceAddDlg)
	//}}AFX_DATA_INIT

	m_pDeviceList = pDeviceList;
	m_pAbstractTree = pAbstractTree;
	m_ppDeviceResult = ppDeviceResult;

	if (NULL != m_ppDeviceResult)
	{
		(*m_ppDeviceResult) = NULL;
	}
}


void CDeviceAddDlg::DoDataExchange(CDataExchange* pDX)
{
	CSuckLessDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDeviceAddDlg)
	DDX_Control(pDX, IDC_LIST_DEVICES, m_ctrlListDevices);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDeviceAddDlg, CSuckLessDialog)
	//{{AFX_MSG_MAP(CDeviceAddDlg)
	ON_LBN_DBLCLK(IDC_LIST_DEVICES, OnDblclkListDevices)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDeviceAddDlg message handlers

BOOL CDeviceAddDlg::OnInitDialog() 
{
	CSuckLessDialog::OnInitDialog();

	UpdateData(TRUE);

	this->setWindowIcon(IDR_MAINFRAME);

	// -- Initial contents --

	if (NULL != g_pProntoFont)
	{
		m_ctrlListDevices.SetFont(g_pProntoFont);
	}

	const CCSFATDeviceList *pDevList = m_pAbstractTree->GetDevices();

	int blankItemIdx = m_ctrlListDevices.AddString("<None>\t-- Start from scratch");
	int itemIdx;
	CString csLabel;
	CTypedError tErrorLabel;

	for (const CCSFATDevice *pDev = pDevList->GetFirst(); pDev != NULL; pDev = pDevList->GetNext(pDev))
	{
		CTypedError::SafeReset(&tErrorLabel);

		csLabel = pDev->GetName()->GetPronto(true);

		guessDevice(pDev);

		if (m_outDevice.IsValid(&tErrorLabel))
		{
			csLabel += "\t-- Good";
		}
		else
		{
			csLabel += "\t-- ";
			csLabel += CTypedError::SafeGetFullErrorTextTempBuffer(&tErrorLabel);
		}

		itemIdx = m_ctrlListDevices.AddString(csLabel);

		if (0 <= itemIdx)
		{
			m_ctrlListDevices.SetItemDataPtr(itemIdx, (void *)pDev);
		}
	}

	if (0 <= blankItemIdx)
	{
		m_ctrlListDevices.SetItemDataPtr(blankItemIdx, NULL);
		m_ctrlListDevices.SetCurSel(blankItemIdx);
	}

	// -- Control setup --

	addControlInfo(IDC_STATIC_DEVICES);

	addControlInfo(IDC_LIST_DEVICES);
	
	addControlInfo(IDC_BUTTON_HELP);

	addControlInfo(IDOK);
	addControlInfo(IDCANCEL);

	setControlRelationship(IDC_STATIC_DEVICES,		IDD, CControlInfo::ACOMMON_TOP_LEFT);

	setControlRelationship(IDC_LIST_DEVICES,		IDD, CControlInfo::ACOMMON_RESIZEHEIGHT_RESIZEWIDTH);

	setControlRelationship(IDC_BUTTON_HELP,			IDD,					CControlInfo::ACOMMON_BOTTOM_LEFT);

	setControlRelationship(IDOK,					IDD,					CControlInfo::ACOMMON_BOTTOM_RIGHT);
	setControlRelationship(IDCANCEL,				IDD,					CControlInfo::ACOMMON_BOTTOM_RIGHT);

	controlAndFrameInfoIsSetup();

	setMinXToCurrent();
	setMinYToCurrent();

	UpdateData(FALSE);

	this->setRememberWindowPosition(SN_REG_PATH_WINDOWPOS, SN_REG_VALUE_WINDOWPOS_DEVADD);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CDeviceAddDlg::OnOK() 
{
	UpdateData(TRUE);

	const CCSFATDevice *pDev = NULL;

	int selIdx = m_ctrlListDevices.GetCurSel();

	if (0 <= selIdx)
	{
		void *pDevVoid = m_ctrlListDevices.GetItemDataPtr(selIdx);

		// Since when was -1 guaranteed not to be a valid pointer? But that's
		// GetItemDataPtr's way of returning errors...
		if ((NULL != pDevVoid) && ((-1) != (int)pDevVoid))
		{
			pDev = (const CCSFATDevice *)pDevVoid;
		}
	}

	guessDevice(pDev); // m_outDevice reset if pDev is NULL.

	CDeviceDlg *pSLDlg = new CDeviceDlg(&m_outDevice, NULL, m_pDeviceList, m_pAbstractTree, m_ppDeviceResult, this);
	this->openPseudoModalChild(pSLDlg, CDeviceDlg::IDD, false, true);
}

void CDeviceAddDlg::OnPseudoModalChildClose(int sldIDD, LPARAM lparam)
{
	if (CDeviceDlg::IDD == sldIDD)
	{
		CSuckLessDialog::OnOK();
	}
}

void CDeviceAddDlg::OnDblclkListDevices() 
{
	if (0 <= m_ctrlListDevices.GetCurSel())
	{
		OnOK();
	}
}

void CDeviceAddDlg::guessDevice(const CCSFATDevice *pDev)
{
	m_outDevice.Reset();

	if (NULL != pDev)
	{
		// Device Name.
		m_outDevice.GetName()->SetPronto(pDev->GetName()->GetPronto(false));

		// Jump to the first panel.
		const CCSFATPanel *pFirstPanel = pDev->GetPanels()->GetFirst();

		if (NULL != pFirstPanel)
		{
			CCSFPanelPath panelPath;
			panelPath.SetDeviceNamePronto(pDev->GetName()->GetPronto(false));
			panelPath.SetPanelNamePronto(pFirstPanel->GetName()->GetPronto(false));

			if (panelPath.IsValid(NULL))
			{
				m_outDevice.SetJumpPanel(&panelPath);
			}
		}

		// Button searches...
		CCSFButtonPath buttonPath;

		// Before button.
		if (searchDeviceForButton(&buttonPath, pDev, "Disc"))
		{
			m_outDevice.SetBeforeDiscButton(&buttonPath);
		}
		else if (searchDeviceForButton(&buttonPath, pDev, "Disk"))
		{
			m_outDevice.SetBeforeDiscButton(&buttonPath);
		}


		// After button.
		if (searchDeviceForButton(&buttonPath, pDev, "Enter"))
		{
			m_outDevice.SetAfterDiscButton(&buttonPath);
		}
		else if (searchDeviceForButton(&buttonPath, pDev, "Play"))
		{
			m_outDevice.SetAfterDiscButton(&buttonPath);
		}
		else if (searchDeviceForButton(&buttonPath, pDev, "")) // Play character.
		{
			m_outDevice.SetAfterDiscButton(&buttonPath);
		}

		// Number buttons.
		char numNameBuf[2];

		for (unsigned int i = 0; i < 10; i++)
		{
			sprintf(numNameBuf, "%u", i);

			if (searchDeviceForButton(&buttonPath, pDev, numNameBuf))
			{
				m_outDevice.SetNumberButton(i, &buttonPath);
			}
		}
	}
}

bool CDeviceAddDlg::searchDeviceForButton(CCSFButtonPath *pOutPath,
										  const CCSFATDevice *pDev, const char *szNamePronto)
{
	pOutPath->Reset();

	const CCSFATPanel *pPanel = NULL;
	const CCSFATPanel *pPanelCandidate = NULL;

	const CCSFATButton *pButton = NULL;
	const CCSFATButton *pButtonCandidate = NULL;

	// Look for a button matching the name that has an action.
	// If we find a button that contains aliases in the action, keep it as
	// a candidate but keep looking for one without aliases.

	bool bPerfectMatch = false;

	pPanel = pDev->GetPanels()->GetFirst();

	while (NULL != pPanel)
	{
		pButton = pPanel->GetButtons()->GetByNamePronto(szNamePronto, NULL);

		while (NULL != pButton)
		{
			if (pButton->IsAction() && pButton->IsUniqueName())
			{
				if (pButton->IsActionWithAlias())
				{
					// Make this the candidate if one hasn't already been found.
					if ((NULL == pButtonCandidate) && (NULL == pPanelCandidate))
					{
						pButtonCandidate = pButton;
						pPanelCandidate = pPanel;
					}
				}
				else
				{
					// This is a perfect match, we don't need to look any further.
					pButtonCandidate = pButton;
					pPanelCandidate = pPanel;
					bPerfectMatch = true;
					break;
				}
			}

			pButton = pPanel->GetButtons()->GetByNamePronto(szNamePronto, pButton);
		}

		if (bPerfectMatch)
		{
			break;
		}

		pPanel = pDev->GetPanels()->GetNext(pPanel);
	}

	if ((NULL != pButtonCandidate) && (NULL != pPanelCandidate))
	{
		pOutPath->SetDeviceNamePronto(pDev->GetName()->GetPronto(false));
		pOutPath->SetPanelNamePronto(pPanelCandidate->GetName()->GetPronto(false));
		pOutPath->SetButtonNamePronto(pButtonCandidate->GetName()->GetPronto(false));
	}

	return(pOutPath->IsValid(NULL));
}
