// DeviceDlg.cpp : implementation file
//

#include "stdafx.h"
#include "DeviceDlg.h"
#include "ATSelectDlg.h"
#include "ProntoStringDlg.h"
#include "SNConfig.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDeviceDlg dialog

CDeviceDlg::CDeviceDlg(const CSNDevice *pTemplateDevice, const CSNDevice *pUpdateDevice,
					   CSNDeviceList *pDeviceList, const CCSFAbstractTree *pAbstractTree,
					   const CSNDevice **ppDeviceResult,
					   CWnd* pParent)
	: CSuckLessDialog(CDeviceDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDeviceDlg)
	m_editName = _T("");
	m_bCheckJump = FALSE;
	m_bCheckIdenDelay = FALSE;
	m_editShortName = _T("");
	//}}AFX_DATA_INIT

	m_pNewDevice = new CSNDevice();

	if (NULL != pTemplateDevice)
	{
		m_pNewDevice->Set(pTemplateDevice);
	}

	m_pUpdateDevice = pUpdateDevice;
	m_pDeviceList = pDeviceList;
	m_pAbstractTree = pAbstractTree;

	m_ppDeviceResult = ppDeviceResult;

	if (NULL != m_ppDeviceResult)
	{
		(*m_ppDeviceResult) = NULL;
	}

	m_bUpdatingPadOrDelay = false;

	m_bInitDone = false;
}

CDeviceDlg::~CDeviceDlg()
{
	while (! m_mappingMap.empty() )
	{
		delete (m_mappingMap.begin()->second);
		m_mappingMap.erase(m_mappingMap.begin());
	}

	delete m_pNewDevice;
	m_pNewDevice = NULL;
}

void CDeviceDlg::DoDataExchange(CDataExchange* pDX)
{
	CSuckLessDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDeviceDlg)
	DDX_Control(pDX, IDC_BUTTON_SET, m_ctrlButtonSet);
	DDX_Control(pDX, IDC_BUTTON_CLEAR, m_ctrlButtonClear);
	DDX_Control(pDX, IDC_LIST_MAPPINGS, m_ctrlListMappings);
	DDX_Control(pDX, IDC_SPIN_TRACKDELAY, m_ctrlSpinTrackDelay);
	DDX_Control(pDX, IDC_EDIT_SHORTNAME, m_ctrlEditShortName);
	DDX_Control(pDX, IDC_EDIT_NAME, m_ctrlEditName);
	DDX_Control(pDX, IDC_SPIN_PAD, m_ctrlSpinPad);
	DDX_Control(pDX, IDC_SPIN_DELAY, m_ctrlSpinDelay);
	DDX_Text(pDX, IDC_EDIT_NAME, m_editName);
	DDX_Check(pDX, IDC_CHECK_JUMP, m_bCheckJump);
	DDX_Check(pDX, IDC_CHECK_IDENDELAY, m_bCheckIdenDelay);
	DDX_Text(pDX, IDC_EDIT_SHORTNAME, m_editShortName);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CDeviceDlg, CSuckLessDialog)
	//{{AFX_MSG_MAP(CDeviceDlg)
	ON_BN_CLICKED(IDC_CHECK_IDENDELAY, OnCheckIdendelay)
	ON_BN_CLICKED(IDC_CHECK_JUMP, OnCheckJump)
	ON_WM_VSCROLL()
	ON_BN_CLICKED(IDC_BUTTON_JUMP, OnButtonJump)
	ON_BN_CLICKED(IDC_BUTTON_NAME, OnButtonName)
	ON_EN_CHANGE(IDC_EDIT_PAD, OnChangeEditPad)
	ON_EN_CHANGE(IDC_EDIT_DELAY, OnChangeEditDelay)
	ON_BN_CLICKED(IDC_BUTTON_SHORTNAME, OnButtonShortname)
	ON_EN_CHANGE(IDC_EDIT_TRACKDELAY, OnChangeEditTrackdelay)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_LIST_MAPPINGS, OnItemchangedListMappings)
	ON_BN_CLICKED(IDC_BUTTON_SET, OnButtonSet)
	ON_NOTIFY(NM_DBLCLK, IDC_LIST_MAPPINGS, OnDblclkListMappings)
	ON_BN_CLICKED(IDC_BUTTON_CLEAR, OnButtonClear)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDeviceDlg message handlers

BOOL CDeviceDlg::OnInitDialog() 
{
	CSuckLessDialog::OnInitDialog();

	UpdateData(TRUE);

	this->setWindowIcon(IDR_MAINFRAME);

	CString winTitle;
	GetWindowText(winTitle);
	if (NULL != m_pUpdateDevice)
	{
		winTitle += " -- (";
		winTitle += m_pUpdateDevice->GetName()->GetPrintable(true);
		winTitle += ")";
	}
	else
	{
		winTitle += " -- <New>";
	}
	SetWindowText(winTitle);

	// -- Initial contents --

	if (NULL != g_pProntoFont)
	{
		CWnd *pCtrl;

		pCtrl = GetDlgItem(IDC_EDIT_NAME);
		if (NULL != pCtrl)
		{
			pCtrl->SetFont(g_pProntoFont);
		}

		pCtrl = GetDlgItem(IDC_EDIT_SHORTNAME);
		if (NULL != pCtrl)
		{
			pCtrl->SetFont(g_pProntoFont);
		}

		pCtrl = GetDlgItem(IDC_EDIT_JUMP);
		if (NULL != pCtrl)
		{
			pCtrl->SetFont(g_pProntoFont);
		}

		m_ctrlListMappings.SetFont(g_pProntoFont);
	}

	m_editName = m_pNewDevice->GetName()->GetPronto(true);
	m_editShortName = m_pNewDevice->GetShortName()->GetPronto(true);

	// --

	const CCSFPanelPath *pPanel = m_pNewDevice->GetJumpPanel();
	csfPanelPathToStringGadget(IDC_EDIT_JUMP, pPanel);
	m_bCheckJump = (NULL != pPanel);

	// --

	m_ctrlSpinPad.SetRange32(1, CSNDevice::MAX_DIGITSTOPADTO);
	int padDigs = m_pNewDevice->GetDigitsToPadDiscTo();
	updatePad(padDigs, true, true);

	// --

	m_ctrlSpinDelay.SetRange32(0, CSNDevice::MAX_TENTHSSECDELAYTIME);
	int tenthsSecDelayTime = m_pNewDevice->GetTenthsSecDigitDelayTime();
	updateDelay(tenthsSecDelayTime, true, true);

	// --

	m_ctrlSpinTrackDelay.SetRange32(0, CSNDevice::MAX_TENTHSSECDELAYTIME);
	tenthsSecDelayTime = m_pNewDevice->GetTenthsSecDiscToTrackDelayTime();
	updateTrackDelay(tenthsSecDelayTime, true, true);

	// --

	m_bCheckIdenDelay = m_pNewDevice->GetDelayOnlyIdentical();

	// --

	m_ctrlListMappings.SetExtendedStyle(LVS_EX_FULLROWSELECT | LVS_EX_SUBITEMIMAGES);
	m_ctrlListMappings.InsertColumn(LIST_COL_ACTION,	"Action");
	m_ctrlListMappings.InsertColumn(LIST_COL_DEVICE,	"Device");
	m_ctrlListMappings.InsertColumn(LIST_COL_PANEL,		"Panel");
	m_ctrlListMappings.InsertColumn(LIST_COL_BUTTON,	"Button");

	m_ctrlListImageList.Create(16, 16, ILC_MASK, 0, 6);
	m_ctrlListImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_LIGHTON));
	m_ctrlListImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_LIGHTOFF));
	m_ctrlListImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_DEVICE));
	m_ctrlListImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_PANEL));
	m_ctrlListImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_BUTTON));
	m_ctrlListImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_HARDBUTTONS));
	m_ctrlListImageList.Add(AfxGetApp()->LoadIcon(IDI_ICON_BLANK));
	m_ctrlListMappings.SetImageList(&m_ctrlListImageList, LVSIL_SMALL);

	int i = 1;
	m_mappingMap.insert(pair< DWORD, CButtonMapping* >(i++, new CNumberButtonMapping("Number 0", false, 0)));
	m_mappingMap.insert(pair< DWORD, CButtonMapping* >(i++, new CNumberButtonMapping("Number 1", false, 1)));
	m_mappingMap.insert(pair< DWORD, CButtonMapping* >(i++, new CNumberButtonMapping("Number 2", false, 2)));
	m_mappingMap.insert(pair< DWORD, CButtonMapping* >(i++, new CNumberButtonMapping("Number 3", false, 3)));
	m_mappingMap.insert(pair< DWORD, CButtonMapping* >(i++, new CNumberButtonMapping("Number 4", false, 4)));
	m_mappingMap.insert(pair< DWORD, CButtonMapping* >(i++, new CNumberButtonMapping("Number 5", false, 5)));
	m_mappingMap.insert(pair< DWORD, CButtonMapping* >(i++, new CNumberButtonMapping("Number 6", false, 6)));
	m_mappingMap.insert(pair< DWORD, CButtonMapping* >(i++, new CNumberButtonMapping("Number 7", false, 7)));
	m_mappingMap.insert(pair< DWORD, CButtonMapping* >(i++, new CNumberButtonMapping("Number 8", false, 8)));
	m_mappingMap.insert(pair< DWORD, CButtonMapping* >(i++, new CNumberButtonMapping("Number 9", false, 9)));

	m_mappingMap.insert(pair< DWORD, CButtonMapping* >(i++, new CNonNumberButtonMapping("Before Disc",				true, &CSNDevice::GetBeforeDiscButton,			&CSNDevice::SetBeforeDiscButton)));
	m_mappingMap.insert(pair< DWORD, CButtonMapping* >(i++, new CNonNumberButtonMapping("After Disc",				true, &CSNDevice::GetAfterDiscButton,			&CSNDevice::SetAfterDiscButton)));
	m_mappingMap.insert(pair< DWORD, CButtonMapping* >(i++, new CNonNumberButtonMapping("Before Track (1 digit)",	true, &CSNDevice::GetBeforeTrackButton1Digit,	&CSNDevice::SetBeforeTrackButton1Digit)));
	m_mappingMap.insert(pair< DWORD, CButtonMapping* >(i++, new CNonNumberButtonMapping("Before Track (2 digits)",	true, &CSNDevice::GetBeforeTrackButton2Digits,	&CSNDevice::SetBeforeTrackButton2Digits)));
	m_mappingMap.insert(pair< DWORD, CButtonMapping* >(i++, new CNonNumberButtonMapping("After Track (1 digit)",	true, &CSNDevice::GetAfterTrackButton1Digit,	&CSNDevice::SetAfterTrackButton1Digit)));
	m_mappingMap.insert(pair< DWORD, CButtonMapping* >(i++, new CNonNumberButtonMapping("After Track (2 digits)",	true, &CSNDevice::GetAfterTrackButton2Digits,	&CSNDevice::SetAfterTrackButton2Digits)));

	int idx = 0;

	for (map< DWORD, CButtonMapping* >::const_iterator miter = m_mappingMap.begin(); miter != m_mappingMap.end(); miter++)
	{
		int insIdx = m_ctrlListMappings.InsertItem(idx++, miter->second->GetDescription());

		if ((-1) == insIdx)
		{
			MessageBox("Error populating list", SUPERNUDELIST_ERROR_TITLE, MB_ICONSTOP|MB_OK);
			break;

		}
		else
		{
			m_ctrlListMappings.SetItemData(insIdx, miter->first);

			if (!updateMappingListLine(miter, false))
			{
				break;
			}
		}
	}

	updateMappingListLine(m_mappingMap.end(), true);

	// -- Control setup --

	addControlInfo(IDC_STATIC_GENERAL);
	addControlInfo(IDC_STATIC_MAPPINGS);
	addControlInfo(IDC_STATIC_LEFTHALF);
	addControlInfo(IDC_STATIC_RIGHTHALF);

	addControlInfo(IDC_STATIC_NAME);
	addControlInfo(IDC_EDIT_NAME);
	addControlInfo(IDC_BUTTON_NAME);

	addControlInfo(IDC_STATIC_SHORTNAME);
	addControlInfo(IDC_EDIT_SHORTNAME);
	addControlInfo(IDC_BUTTON_SHORTNAME);

	addControlInfo(IDC_CHECK_JUMP);
	addControlInfo(IDC_EDIT_JUMP);
	addControlInfo(IDC_BUTTON_JUMP);

	addControlInfo(IDC_STATIC_PAD_LEFT);
	addControlInfo(IDC_EDIT_PAD);
	addControlInfo(IDC_SPIN_PAD);
	addControlInfo(IDC_STATIC_PAD_RIGHT);

	addControlInfo(IDC_STATIC_DELAY_LEFT);
	addControlInfo(IDC_EDIT_DELAY);
	addControlInfo(IDC_SPIN_DELAY);
	addControlInfo(IDC_STATIC_DELAY_RIGHT);

	addControlInfo(IDC_CHECK_IDENDELAY);

	addControlInfo(IDC_STATIC_TRACKDELAY_LEFT);
	addControlInfo(IDC_EDIT_TRACKDELAY);
	addControlInfo(IDC_SPIN_TRACKDELAY);
	addControlInfo(IDC_STATIC_TRACKDELAY_RIGHT);

	addControlInfo(IDC_LIST_MAPPINGS);
	addControlInfo(IDC_BUTTON_SET);
	addControlInfo(IDC_BUTTON_CLEAR);

	addControlInfo(IDC_BUTTON_HELP);

	addControlInfo(IDOK);

	addControlInfo(IDCANCEL);

	setControlRelationship(IDC_STATIC_GENERAL,		IDD,				CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_STATIC_MAPPINGS,		IDD,				CControlInfo::ACOMMON_RESIZEHEIGHT_RESIZEWIDTH);

	setControlRelationship(IDC_STATIC_NAME,			IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_LEFT);
	setControlRelationship(IDC_EDIT_NAME,			IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_BUTTON_NAME,			IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_STATIC_SHORTNAME,	IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_LEFT);
	setControlRelationship(IDC_EDIT_SHORTNAME,		IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_BUTTON_SHORTNAME,	IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_CHECK_JUMP,			IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_LEFT);
	setControlRelationship(IDC_EDIT_JUMP,			IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_BUTTON_JUMP,			IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_STATIC_PAD_LEFT,		IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_EDIT_PAD,			IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_SPIN_PAD,			IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_STATIC_PAD_RIGHT,	IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_STATIC_DELAY_LEFT,	IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_EDIT_DELAY,			IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_SPIN_DELAY,			IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_STATIC_DELAY_RIGHT,	IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_CHECK_IDENDELAY,		IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_STATIC_TRACKDELAY_LEFT,	IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_EDIT_TRACKDELAY,			IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_SPIN_TRACKDELAY,			IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_STATIC_TRACKDELAY_RIGHT,	IDC_STATIC_GENERAL,	CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_LIST_MAPPINGS,		IDC_STATIC_MAPPINGS,	CControlInfo::ACOMMON_RESIZEHEIGHT_RESIZEWIDTH);
	setControlRelationship(IDC_BUTTON_SET,			IDC_STATIC_MAPPINGS,	CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_BUTTON_CLEAR,		IDC_STATIC_MAPPINGS,	CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_BUTTON_HELP,			IDD,					CControlInfo::ACOMMON_BOTTOM_LEFT);

	setControlRelationship(IDOK,					IDD,					CControlInfo::ACOMMON_BOTTOM_RIGHT);
	setControlRelationship(IDCANCEL,				IDD,					CControlInfo::ACOMMON_BOTTOM_RIGHT);

	controlAndFrameInfoIsSetup();

	setMinXToCurrent();
	setMinYToCurrent();

	UpdateData(FALSE);

	m_bInitDone = true;

	this->setRememberWindowPosition(SN_REG_PATH_WINDOWPOS, SN_REG_VALUE_WINDOWPOS_DEV);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CDeviceDlg::OnSizeHookEnd(void)
{
	if (m_bInitDone)
	{
		CHeaderCtrl *pHeaderCtrl;

		if (NULL != (pHeaderCtrl = m_ctrlListMappings.GetHeaderCtrl()))
		{
			int headItems = pHeaderCtrl->GetItemCount();

			if (0 != headItems)
			{
				m_ctrlListMappings.SetColumnWidth(headItems - 1, LVSCW_AUTOSIZE_USEHEADER);
			}
		}
	}
}

void CDeviceDlg::OnOK() 
{
	UpdateData(TRUE);

	// -- Put any unsaved values into the device. --
	// All changes are saved directly to m_pNewDevice except the Name and
	// the conditional Jump and Buttons.

	m_pNewDevice->GetName()->SetPronto(LPCTSTR(m_editName));
	m_pNewDevice->GetShortName()->SetPronto(LPCTSTR(m_editShortName));

	if (! m_bCheckJump )
	{
		m_pNewDevice->SetJumpPanel(NULL);
	}

	// -- Try to add to/update the list. --

	CTypedError tError;

	const CSNDevice *pAddUpdateResult = NULL;

	if (NULL != m_pUpdateDevice)
	{
		pAddUpdateResult = m_pDeviceList->Update(&tError, m_pNewDevice, m_pUpdateDevice);
	}
	else
	{
		pAddUpdateResult = m_pDeviceList->AddBelowAndOwn(&tError, m_pNewDevice, NULL);
	}

	if (NULL == pAddUpdateResult)
	{
		MessageBox(CTypedError::SafeGetFullErrorTextTempBuffer(&tError),
				   NULL != m_pUpdateDevice ? "Cannot update player" : "Cannot add player",
				   MB_OK | MB_ICONEXCLAMATION);
	}
	else
	{
		if (NULL == m_pUpdateDevice)
		{
			// We no longer own the device
			// and should not delete it on destruction.
			m_pNewDevice = NULL;
		}

		if (NULL != m_ppDeviceResult)
		{
			(*m_ppDeviceResult) = pAddUpdateResult;
		}

		CSuckLessDialog::OnOK();
	}
}

void CDeviceDlg::OnCheckIdendelay() 
{
	UpdateData(TRUE);

	m_pNewDevice->SetDelayOnlyIdentical(m_bCheckIdenDelay ? true : false);
}

void CDeviceDlg::OnCheckJump() 
{
	UpdateData(TRUE);
	
	csfPanelPathToStringGadget(IDC_EDIT_JUMP,
							m_bCheckJump ? m_pNewDevice->GetJumpPanel() : NULL);
}

void CDeviceDlg::OnButtonJump() 
{
	m_ATSelMapIter = m_mappingMap.end();

	const CCSFPanelPath *pPPath = m_pNewDevice->GetJumpPanel();

	if (NULL != pPPath)
	{
		m_ATSelDlgDev.SetPronto(pPPath->GetDeviceName()->GetPronto(false));
		m_ATSelDlgPan.SetPronto(pPPath->GetPanelName()->GetPronto(false));
	}

	m_ATSelDlgBut.Reset();

	CATSelectDlg *pSLDlg = new CATSelectDlg(CATSelectDlg::PT_PANEL, false, "Select Jump Panel", &m_ATSelDlgDev, &m_ATSelDlgPan, NULL, m_pAbstractTree, this);
	this->openPseudoModalChild(pSLDlg, CATSelectDlg::IDD, false, true);
}

void CDeviceDlg::OnButtonSet() 
{
	POSITION pos = m_ctrlListMappings.GetFirstSelectedItemPosition();

	if (NULL != pos)
	{
		int idx = m_ctrlListMappings.GetNextSelectedItem(pos);

		DWORD dwData = m_ctrlListMappings.GetItemData(idx);

		map< DWORD, CButtonMapping* >::const_iterator miter = m_mappingMap.find(dwData);

		if (miter != m_mappingMap.end())
		{
			m_ATSelMapIter = miter;

			const CCSFButtonPath *pBPath = miter->second->GetButtonPath(m_pNewDevice);

			if (NULL != pBPath)
			{
				m_ATSelDlgDev.SetPronto(pBPath->GetDeviceName()->GetPronto(false));
				m_ATSelDlgPan.SetPronto(pBPath->GetPanelName()->GetPronto(false));
				m_ATSelDlgBut.SetPronto(pBPath->GetButtonName()->GetPronto(false));
			}

			CATSelectDlg *pSLDlg = new CATSelectDlg(CATSelectDlg::PT_BUTTON, false, miter->second->GetDescription(), &m_ATSelDlgDev, &m_ATSelDlgPan, &m_ATSelDlgBut, m_pAbstractTree, this);
			this->openPseudoModalChild(pSLDlg, CATSelectDlg::IDD, false, true);
		}
	}
}

void CDeviceDlg::OnButtonClear() 
{
	POSITION pos = m_ctrlListMappings.GetFirstSelectedItemPosition();

	if (NULL != pos)
	{
		int idx = m_ctrlListMappings.GetNextSelectedItem(pos);

		DWORD dwData = m_ctrlListMappings.GetItemData(idx);

		map< DWORD, CButtonMapping* >::const_iterator miter = m_mappingMap.find(dwData);

		if ((miter != m_mappingMap.end()) && (miter->second->IsClearable()))
		{
			miter->second->SetButtonPath(m_pNewDevice, NULL);
			updateMappingListLine(miter, false);
		}
	}
}

void CDeviceDlg::OnDblclkListMappings(NMHDR* pNMHDR, LRESULT* pResult) 
{
	OnButtonSet();
	
	*pResult = 0;
}

void CDeviceDlg::OnButtonName() 
{
	CProntoStringDlg *pSLDlg = new CProntoStringDlg(&m_ctrlEditName, "Player Name", this);
	this->openPseudoModalChild(pSLDlg, CProntoStringDlg::IDD, false, true);
}

void CDeviceDlg::OnButtonShortname() 
{
	CProntoStringDlg *pSLDlg = new CProntoStringDlg(&m_ctrlEditShortName, "Short Player Name", this);
	this->openPseudoModalChild(pSLDlg, CProntoStringDlg::IDD, false, true);
}

// SuckLessDialog:
void CDeviceDlg::OnPseudoModalChildClose(int sldIDD, LPARAM lparam)
{
	UpdateData(TRUE);

	if (CATSelectDlg::IDD == sldIDD)
	{
		if(lparam)
		{
			if (m_ATSelMapIter == m_mappingMap.end())
			{
				CCSFPanelPath panel;
				panel.SetDeviceNamePronto(m_ATSelDlgDev.GetPronto(false));
				panel.SetPanelNamePronto(m_ATSelDlgPan.GetPronto(false));

				m_pNewDevice->SetJumpPanel(&panel);

				csfPanelPathToStringGadget(IDC_EDIT_JUMP, &panel);
				m_bCheckJump = TRUE;
			}
			else
			{
				CCSFButtonPath button;
				button.SetDeviceNamePronto(m_ATSelDlgDev.GetPronto(false));
				button.SetPanelNamePronto(m_ATSelDlgPan.GetPronto(false));
				button.SetButtonNamePronto(m_ATSelDlgBut.GetPronto(false));

				m_ATSelMapIter->second->SetButtonPath(m_pNewDevice, &button);

				updateMappingListLine(m_ATSelMapIter, true);
			}
		}
	}

	UpdateData(FALSE);
}

bool CDeviceDlg::updateMappingListLine(map< DWORD, CButtonMapping* >::const_iterator miter, bool bResizeColumns)
{
	bool bResult = true;

	if (miter != m_mappingMap.end())
	{
		const CCSFButtonPath *pBP = miter->second->GetButtonPath(m_pNewDevice);

		LVFINDINFO findInfo;
		ZeroMemory(&findInfo, sizeof(findInfo));
		findInfo.flags = LVFI_PARAM;
		findInfo.lParam = miter->first;

		DWORD itemIdx = m_ctrlListMappings.FindItem(&findInfo, -1);

		if (((-1) == itemIdx)
		||	(!m_ctrlListMappings.SetItemText(itemIdx, LIST_COL_DEVICE, NULL == pBP ? "" : pBP->GetDeviceName()->GetPronto(true)))\
		||	(!m_ctrlListMappings.SetItemText(itemIdx, LIST_COL_PANEL,  NULL == pBP ? "" : (pBP->GetPanelName()->IsUnassigned() ? "Hard Buttons" : pBP->GetPanelName() ->GetPronto(false))))\
		||	(!m_ctrlListMappings.SetItemText(itemIdx, LIST_COL_BUTTON, NULL == pBP ? "" : pBP->GetButtonName()->GetPronto(true))))
		{
			MessageBox("Error populating list", SUPERNUDELIST_ERROR_TITLE, MB_ICONSTOP|MB_OK);
			bResult = false;
		}
		else
		{
			if (NULL == pBP)
			{
				m_ctrlListMappings.SetItem(itemIdx, LIST_COL_ACTION,	LVIF_IMAGE, NULL, ILIDX_ICON_LIGHTOFF,		0, 0, 0);
				m_ctrlListMappings.SetItem(itemIdx, LIST_COL_DEVICE,	LVIF_IMAGE, NULL, ILIDX_ICON_BLANK,			0, 0, 0);
				m_ctrlListMappings.SetItem(itemIdx, LIST_COL_PANEL,		LVIF_IMAGE, NULL, ILIDX_ICON_BLANK,			0, 0, 0);
				m_ctrlListMappings.SetItem(itemIdx, LIST_COL_BUTTON,	LVIF_IMAGE, NULL, ILIDX_ICON_BLANK,			0, 0, 0);
			}
			else
			{
				m_ctrlListMappings.SetItem(itemIdx, LIST_COL_ACTION,	LVIF_IMAGE, NULL, ILIDX_ICON_LIGHTON,		0, 0, 0);
				m_ctrlListMappings.SetItem(itemIdx, LIST_COL_DEVICE,	LVIF_IMAGE, NULL, ILIDX_ICON_DEVICE,		0, 0, 0);

				if (pBP->GetPanelName()->IsUnassigned())
				{
					m_ctrlListMappings.SetItem(itemIdx, LIST_COL_PANEL,LVIF_IMAGE, NULL, ILIDX_ICON_HARDBUTTONS,	0, 0, 0);
				}
				else
				{
					m_ctrlListMappings.SetItem(itemIdx, LIST_COL_PANEL,	LVIF_IMAGE, NULL, ILIDX_ICON_PANEL,			0, 0, 0);
				}

				m_ctrlListMappings.SetItem(itemIdx, LIST_COL_BUTTON,LVIF_IMAGE, NULL, ILIDX_ICON_BUTTON,			0, 0, 0);
			}
		}
	}

	if (bResizeColumns)
	{
		m_ctrlListMappings.SetColumnWidth(LIST_COL_ACTION,	LVSCW_AUTOSIZE_USEHEADER);
		m_ctrlListMappings.SetColumnWidth(LIST_COL_DEVICE,	LVSCW_AUTOSIZE_USEHEADER);
		m_ctrlListMappings.SetColumnWidth(LIST_COL_PANEL,	LVSCW_AUTOSIZE_USEHEADER);
		m_ctrlListMappings.SetColumnWidth(LIST_COL_BUTTON,	LVSCW_AUTOSIZE_USEHEADER);
	}

	return(bResult);
}

void CDeviceDlg::csfPanelPathToStringGadget(int editIdent, const CCSFPanelPath *pPath)
{
	CWnd *pCtrl = GetDlgItem(editIdent);

	if (NULL != pCtrl)
	{
		if (NULL == pPath) 
		{
			// Null CCSFPath passed in means clear the string.
			pCtrl->SetWindowText("");
		}
		else
		{
			pCtrl->SetWindowText(pPath->GetName()->GetPronto(true));
		}
	}
}

void CDeviceDlg::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	if (m_ctrlSpinPad.GetDlgCtrlID() == pScrollBar->GetDlgCtrlID())
	{
		updatePad(nPos, true, false);
	}
	else if (m_ctrlSpinDelay.GetDlgCtrlID() == pScrollBar->GetDlgCtrlID())
	{
		updateDelay(nPos, true, false);
	}
	else if (m_ctrlSpinTrackDelay.GetDlgCtrlID() == pScrollBar->GetDlgCtrlID())
	{
		updateTrackDelay(nPos, true, false);
	}

	CSuckLessDialog::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CDeviceDlg::updatePad(int padDigs, bool bUpdateEdit, bool bUpdateSpin)
{
	if (!m_bUpdatingPadOrDelay)
	{
		m_bUpdatingPadOrDelay = true;

		CString csLabel;

		if (bUpdateEdit)
		{
			CWnd *pEdit = this->GetDlgItem(IDC_EDIT_PAD);

			if (NULL != pEdit)
			{
				csLabel.Format("%d", padDigs);
				pEdit->SetWindowText(csLabel);		
			}
		}

		CWnd *pStatic = this->GetDlgItem(IDC_STATIC_PAD_RIGHT);

		if (NULL != pStatic)
		{
			csLabel.Format("digit%s.", 1 == padDigs ? "" : "s");
			pStatic->SetWindowText(csLabel);
		}

		if (bUpdateSpin)
		{
			m_ctrlSpinPad.SetPos(padDigs);
		}

		m_pNewDevice->SetDigitsToPadDiscTo(padDigs);

		m_bUpdatingPadOrDelay = false;
	}
}

void CDeviceDlg::updateDelay(int tenthsSecDelayTime, bool bUpdateEdit, bool bUpdateSpin)
{
	if (!m_bUpdatingPadOrDelay)
	{
		m_bUpdatingPadOrDelay = true;

		CString csLabel;

		if (bUpdateEdit)
		{
			CWnd *pEdit = this->GetDlgItem(IDC_EDIT_DELAY);

			if (NULL != pEdit)
			{
				csLabel.Format("%d", tenthsSecDelayTime);
				pEdit->SetWindowText(csLabel);
			}
		}

		if (bUpdateSpin)
		{
			m_ctrlSpinDelay.SetPos(tenthsSecDelayTime);
		}

		m_pNewDevice->SetTenthsSecDigitDelayTime(tenthsSecDelayTime);

		m_bUpdatingPadOrDelay = false;
	}
}

void CDeviceDlg::updateTrackDelay(int tenthsSecDelayTime, bool bUpdateEdit, bool bUpdateSpin)
{
	if (!m_bUpdatingPadOrDelay)
	{
		m_bUpdatingPadOrDelay = true;

		CString csLabel;

		if (bUpdateEdit)
		{
			CWnd *pEdit = this->GetDlgItem(IDC_EDIT_TRACKDELAY);

			if (NULL != pEdit)
			{
				csLabel.Format("%d", tenthsSecDelayTime);
				pEdit->SetWindowText(csLabel);
			}
		}

		if (bUpdateSpin)
		{
			m_ctrlSpinTrackDelay.SetPos(tenthsSecDelayTime);
		}

		m_pNewDevice->SetTenthsSecDiscToTrackDelayTime(tenthsSecDelayTime);

		m_bUpdatingPadOrDelay = false;
	}
}

void CDeviceDlg::OnChangeEditPad() 
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CSuckLessDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	CWnd *pEdit = this->GetDlgItem(IDC_EDIT_PAD);

	if (NULL != pEdit)
	{
		CString csLabel;
		pEdit->GetWindowText(csLabel);

		bool bUpdateEdit = false;

		int val = 1;

		if (! csLabel.IsEmpty() )
		{
			val = atoi(LPCTSTR(csLabel));

			if (CSNDevice::MAX_DIGITSTOPADTO < val)
			{
				val = CSNDevice::MAX_DIGITSTOPADTO;
				bUpdateEdit = true;
			}
			else if (1 > val)
			{
				val = 1;
				bUpdateEdit = true;
			}
		
		}

		updatePad(val, bUpdateEdit, true);
	}
}

void CDeviceDlg::OnChangeEditDelay() 
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CSuckLessDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.
	
	CWnd *pEdit = this->GetDlgItem(IDC_EDIT_DELAY);

	if (NULL != pEdit)
	{
		CString csLabel;
		pEdit->GetWindowText(csLabel);

		bool bUpdateEdit = false;

		int val = 1;

		if (! csLabel.IsEmpty() )
		{
			val = atoi(LPCTSTR(csLabel));

			if (CSNDevice::MAX_TENTHSSECDELAYTIME < val)
			{
				val = CSNDevice::MAX_TENTHSSECDELAYTIME;
				bUpdateEdit = true;
			}
			else if (0 > val)
			{
				val = 0;
				bUpdateEdit = true;
			}
		
		}

		updateDelay(val, bUpdateEdit, true);
	}
}

void CDeviceDlg::OnChangeEditTrackdelay() 
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CSuckLessDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.
	
	CWnd *pEdit = this->GetDlgItem(IDC_EDIT_TRACKDELAY);

	if (NULL != pEdit)
	{
		CString csLabel;
		pEdit->GetWindowText(csLabel);

		bool bUpdateEdit = false;

		int val = 1;

		if (! csLabel.IsEmpty() )
		{
			val = atoi(LPCTSTR(csLabel));

			if (CSNDevice::MAX_TENTHSSECDELAYTIME < val)
			{
				val = CSNDevice::MAX_TENTHSSECDELAYTIME;
				bUpdateEdit = true;
			}
			else if (0 > val)
			{
				val = 0;
				bUpdateEdit = true;
			}
		
		}

		updateTrackDelay(val, bUpdateEdit, true);
	}
}

void CDeviceDlg::OnItemchangedListMappings(NMHDR* pNMHDR, LRESULT* pResult) 
{
	POSITION pos = m_ctrlListMappings.GetFirstSelectedItemPosition();

	if (NULL != pos)
	{
		int idx = m_ctrlListMappings.GetNextSelectedItem(pos);

		DWORD dwData = m_ctrlListMappings.GetItemData(idx);

		map< DWORD, CButtonMapping* >::const_iterator miter = m_mappingMap.find(dwData);

		if (miter == m_mappingMap.end())
		{
			m_ctrlButtonSet.EnableWindow(FALSE);
			m_ctrlButtonClear.EnableWindow(FALSE);
		}
		else
		{
			m_ctrlButtonSet.EnableWindow(TRUE);
			m_ctrlButtonClear.EnableWindow(miter->second->IsClearable() ? TRUE : FALSE);
		}
	}
	
	*pResult = 0;
}
