#if !defined(AFX_DEVICEDLG_H__E4DB7373_CC90_4BBB_AC06_4707E5C082E4__INCLUDED_)
#define AFX_DEVICEDLG_H__E4DB7373_CC90_4BBB_AC06_4707E5C082E4__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// DeviceDlg.h : header file
//
#include "SuckLessDialog.h"
#include "CSFPath.h"
#include "SNDevice.h"

/////////////////////////////////////////////////////////////////////////////
// CDeviceDlg dialog


class CSNDeviceList;
class CCSFAbstractTree;

class CDeviceDlg : public CSuckLessDialog
{
// Construction
public:
	CDeviceDlg(const CSNDevice *pTemplateDevice, const CSNDevice *pUpdateDevice,
				CSNDeviceList *pDeviceList, const CCSFAbstractTree *pAbstractTree,
				const CSNDevice **ppDeviceResult,
				CWnd* pParent);

	~CDeviceDlg();

// Dialog Data
	//{{AFX_DATA(CDeviceDlg)
	enum { IDD = IDD_DEVICE_DIALOG };
	CButton	m_ctrlButtonSet;
	CButton	m_ctrlButtonClear;
	CListCtrl	m_ctrlListMappings;
	CSpinButtonCtrl	m_ctrlSpinTrackDelay;
	CEdit	m_ctrlEditShortName;
	CEdit	m_ctrlEditName;
	CSpinButtonCtrl	m_ctrlSpinPad;
	CSpinButtonCtrl	m_ctrlSpinDelay;
	CString	m_editName;
	BOOL	m_bCheckJump;
	BOOL	m_bCheckIdenDelay;
	CString	m_editShortName;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDeviceDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	enum
	{
		LIST_COL_ACTION = 0,
		LIST_COL_DEVICE,
		LIST_COL_PANEL,
		LIST_COL_BUTTON
	};

	enum
	{
		ILIDX_ICON_LIGHTON		= 0,
		ILIDX_ICON_LIGHTOFF,
		ILIDX_ICON_DEVICE,
		ILIDX_ICON_PANEL,
		ILIDX_ICON_BUTTON,
		ILIDX_ICON_HARDBUTTONS,
		ILIDX_ICON_BLANK
	};

protected:
	class CButtonMapping
	{
	protected:
		const char *m_szDescription;
		bool m_bClearable;
	public:
		CButtonMapping(const char *szD, bool bC)
			: m_szDescription(szD),
			  m_bClearable(bC)
		{}
		const char *GetDescription(void) const			{ return(m_szDescription); }
		bool IsClearable(void) const					{ return(m_bClearable); }
		virtual const CCSFButtonPath *GetButtonPath(const CSNDevice *pDevice) const = 0;
		virtual void SetButtonPath(CSNDevice *pDevice, const CCSFButtonPath *pBP) const = 0;
	};

	class CNonNumberButtonMapping : public CButtonMapping
	{
	protected:
		const CCSFButtonPath * (CSNDevice::*m_pGetFunction)(void) const;
		void (CSNDevice::*m_pSetFunction)(const CCSFButtonPath *);

	public:
		CNonNumberButtonMapping(const char *szD, bool bC,
								const CCSFButtonPath * (CSNDevice::*pGet)(void) const,
								void (CSNDevice::*pSet)(const CCSFButtonPath *))
			:	CButtonMapping(szD, bC),
				m_pGetFunction(pGet), m_pSetFunction(pSet)
		{}
		const CCSFButtonPath *GetButtonPath(const CSNDevice *pDevice) const		{ return( (pDevice->*m_pGetFunction)() ); }
		void SetButtonPath(CSNDevice *pDevice, const CCSFButtonPath *pBP) const	{ (pDevice->*m_pSetFunction)(pBP); }
	};

	class CNumberButtonMapping : public CButtonMapping
	{
	protected:
		unsigned int m_buttonNumber;

	public:
		CNumberButtonMapping(const char *szD, bool bC, unsigned int buttonNumber)
			:	CButtonMapping(szD, bC),
				m_buttonNumber(buttonNumber)
		{}
		const CCSFButtonPath *GetButtonPath(const CSNDevice *pDevice) const		{ return( pDevice->GetNumberButton(m_buttonNumber) ); }
		void SetButtonPath(CSNDevice *pDevice, const CCSFButtonPath *pBP) const	{ pDevice->SetNumberButton(m_buttonNumber, pBP); }
	};

protected:
	bool updateMappingListLine(map< DWORD, CButtonMapping* >::const_iterator miter, bool bResizeColumns);
	void csfPanelPathToStringGadget(int editIdent, const CCSFPanelPath *pPath);
	void updatePad(int padDigs, bool bUpdateEdit, bool bUpdateSpin);
	void updateDelay(int tenthsSecDelayTime, bool bUpdateEdit, bool bUpdateSpin);
	void updateTrackDelay(int tenthsSecDelayTime, bool bUpdateEdit, bool bUpdateSpin);

protected:
	const CSNDevice *m_pUpdateDevice;
	const CCSFAbstractTree *m_pAbstractTree;
	CSNDeviceList *m_pDeviceList;

	const CSNDevice **m_ppDeviceResult;

	CSNDevice *m_pNewDevice;

	map< DWORD, CButtonMapping* >::const_iterator m_ATSelMapIter; // m_mappingMap.end if we're setting the jump.
	CProntoString m_ATSelDlgDev;
	CProntoString m_ATSelDlgPan;
	CProntoString m_ATSelDlgBut;

	bool m_bUpdatingPadOrDelay;
	bool m_bInitDone;

	CImageList m_ctrlListImageList;

	map< DWORD, CButtonMapping* > m_mappingMap;

protected:
	// SuckLessDialog:
	void virtual OnPseudoModalChildClose(int sldIDD, LPARAM lparam);
	void OnSizeHookEnd(void);

	// Generated message map functions
	//{{AFX_MSG(CDeviceDlg)
	virtual BOOL OnInitDialog();
	virtual void OnOK();
	afx_msg void OnCheckIdendelay();
	afx_msg void OnCheckJump();
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnButtonJump();
	afx_msg void OnButtonName();
	afx_msg void OnChangeEditPad();
	afx_msg void OnChangeEditDelay();
	afx_msg void OnButtonShortname();
	afx_msg void OnChangeEditTrackdelay();
	afx_msg void OnItemchangedListMappings(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnButtonSet();
	afx_msg void OnDblclkListMappings(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnButtonClear();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_DEVICEDLG_H__E4DB7373_CC90_4BBB_AC06_4707E5C082E4__INCLUDED_)
