// DevicesDlg.cpp : implementation file
//

#include "stdafx.h"
#include "SNConfig.h"
#include "DevicesDlg.h"
#include "DeviceDlg.h"
#include "DeviceAddDlg.h"
#include "CCFWrapper.h"
#include "ListingDlg.h"
#include "SNLibrary.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDevicesDlg dialog

CDevicesDlg::CDevicesDlg(CSNConfig *pConfig, CCCFWrapper *pCCFWrapper, CWnd* pParent)
	: CSuckLessDialog(CDevicesDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDevicesDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	m_pConfig = pConfig;
	m_pCCFWrapper = pCCFWrapper;

	m_bCancelAfterListing = false;
	m_bThreadResult = false;
}

void CDevicesDlg::DoDataExchange(CDataExchange* pDX)
{
	CSuckLessDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDevicesDlg)
	DDX_Control(pDX, IDC_LIST_DEVICES, m_ctrlListDevices);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDevicesDlg, CSuckLessDialog)
	//{{AFX_MSG_MAP(CDevicesDlg)
	ON_BN_CLICKED(IDC_BUTTON_ADD, OnButtonAdd)
	ON_BN_CLICKED(IDC_BUTTON_EDIT, OnButtonEdit)
	ON_BN_CLICKED(IDC_BUTTON_DELETE, OnButtonDelete)
	ON_LBN_DBLCLK(IDC_LIST_DEVICES, OnDblclkListDevices)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDevicesDlg message handlers

BOOL CDevicesDlg::OnInitDialog() 
{
	CSuckLessDialog::OnInitDialog();

	UpdateData(TRUE);

	this->setWindowIcon(IDR_MAINFRAME);

	// -- Initial contents --

	if (NULL != g_pProntoFont)
	{
		m_ctrlListDevices.SetFont(g_pProntoFont);
	}

	populateCtrlListDevices();

	// -- Control setup --

	addControlInfo(IDC_STATIC_DEVICES);

	addControlInfo(IDC_LIST_DEVICES);
	
	addControlInfo(IDC_BUTTON_ADD);
	addControlInfo(IDC_BUTTON_EDIT);
	addControlInfo(IDC_BUTTON_DELETE);

	addControlInfo(IDC_BUTTON_HELP);

	addControlInfo(IDOK);
	addControlInfo(IDCANCEL);

	setControlRelationship(IDC_STATIC_DEVICES,		IDD,					CControlInfo::ACOMMON_RESIZEHEIGHT_RESIZEWIDTH);

	setControlRelationship(IDC_LIST_DEVICES,		IDC_STATIC_DEVICES,		CControlInfo::ACOMMON_RESIZEHEIGHT_RESIZEWIDTH);

	setControlRelationship(IDC_BUTTON_ADD,			IDC_STATIC_DEVICES,		CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_BUTTON_EDIT,			IDC_STATIC_DEVICES,		CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_BUTTON_DELETE,		IDC_STATIC_DEVICES,		CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_BUTTON_HELP,			IDD,					CControlInfo::ACOMMON_BOTTOM_LEFT);

	setControlRelationship(IDOK,					IDD,					CControlInfo::ACOMMON_BOTTOM_RIGHT);
	setControlRelationship(IDCANCEL,				IDD,					CControlInfo::ACOMMON_BOTTOM_RIGHT);

	controlAndFrameInfoIsSetup();

	setMinXToCurrent();
	setMinYToCurrent();

	UpdateData(FALSE);

	this->setRememberWindowPosition(SN_REG_PATH_WINDOWPOS, SN_REG_VALUE_WINDOWPOS_DEVS);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CDevicesDlg::populateCtrlListDevices(void)
{
	m_ctrlListDevices.ResetContent();

	const CSNDeviceList *pDevList = m_pConfig->GetDevices();

	int itemIdx;

	for (const CSNDevice *pDev = pDevList->GetFirst(); pDev != NULL; pDev = pDevList->GetNext(pDev))
	{
		itemIdx = m_ctrlListDevices.AddString(pDev->GetName()->GetPronto(true));

		if (0 <= itemIdx)
		{
			m_ctrlListDevices.SetItemDataPtr(itemIdx, (void *)pDev);
		}
	}
}

void CDevicesDlg::OnButtonAdd() 
{
	CDeviceAddDlg *pSLDlg = new CDeviceAddDlg(m_pConfig->GetDevices(),
												m_pCCFWrapper->GetCSFAbstractTree(), NULL, this);
	this->openPseudoModalChild(pSLDlg, CDeviceAddDlg::IDD, false, true);
}

void CDevicesDlg::OnButtonEdit() 
{
	int selIdx = m_ctrlListDevices.GetCurSel();

	if (0 <= selIdx)
	{
		void *pDevVoid = m_ctrlListDevices.GetItemDataPtr(selIdx);

		// Since when was -1 guaranteed not to be a valid pointer? But that's
		// GetItemDataPtr's way of returning errors...
		if ((NULL != pDevVoid) && ((-1) != (int)pDevVoid))
		{
			const CSNDevice *pDev = (const CSNDevice *)pDevVoid;

			CDeviceDlg *pSLDlg = new CDeviceDlg(pDev, pDev, m_pConfig->GetDevices(),
												m_pCCFWrapper->GetCSFAbstractTree(), NULL, this);
			this->openPseudoModalChild(pSLDlg, CDeviceDlg::IDD, false, true);
		}
	}
}

void CDevicesDlg::OnDblclkListDevices() 
{
	OnButtonEdit();
}

void CDevicesDlg::OnButtonDelete() 
{
	int selIdx = m_ctrlListDevices.GetCurSel();

	if (0 <= selIdx)
	{
		void *pDevVoid = m_ctrlListDevices.GetItemDataPtr(selIdx);

		// Since when was -1 guaranteed not to be a valid pointer? But that's
		// GetItemDataPtr's way of returning errors...
		if ((NULL != pDevVoid) && ((-1) != (int)pDevVoid))
		{
			const CSNDevice *pDev = (const CSNDevice *)pDevVoid;

			m_pConfig->GetLibrary()->UnlinkDevice(pDev);
			m_pConfig->GetDevices()->RemoveAndDelete(pDev);

			populateCtrlListDevices();
		}
	}
}

void CDevicesDlg::OnOK() 
{
	if (m_pConfig->GetDevices()->IsDirty())
	{
		m_bThreadResult = true;
		CTypedError::SafeReset(&m_threadTError);
		m_threadErrorList.RemoveAll();

		CPDTDevDlgSave *pThread = new CPDTDevDlgSave(&m_bThreadResult, &m_threadErrorList,
													 &m_threadTError, m_pConfig);

		CProgressDlg *pProgDlg = new CProgressDlg(PROGD_FRAME_NOPROG, false, pThread, this);
		this->openPseudoModalChild(pProgDlg, CProgressDlg::IDD, false, true);
	}
	else
	{
		CSuckLessDialog::OnOK();
	}
}

void CDevicesDlg::OnCancel() 
{
	if (m_pConfig->GetDevices()->IsDirty())
	{
		if (IDYES == MessageBox("Discard your changes?", "Are you sure?", MB_YESNO | MB_ICONQUESTION))
		{
			m_bThreadResult = true;
			CTypedError::SafeReset(&m_threadTError);
			m_threadErrorList.RemoveAll();

			CPDTDevDlgReLoad *pThread = new CPDTDevDlgReLoad(&m_bThreadResult, &m_threadErrorList,
															 &m_threadTError, m_pConfig);

			CProgressDlg *pProgDlg = new CProgressDlg(PROGD_FRAME_NOPROG, false, pThread, this);
			this->openPseudoModalChild(pProgDlg, CProgressDlg::IDD, false, true);
		}
	}
	else
	{
		CSuckLessDialog::OnCancel();
	}
}

void CDevicesDlg::OnPseudoModalChildClose(int sldIDD, LPARAM lparam)
{
	if ((CDeviceAddDlg::IDD == sldIDD)	\
	||  (CDeviceDlg::IDD == sldIDD)		)
	{
		populateCtrlListDevices();
	}
	else if ((CProgressDlg::IDD == sldIDD)													\
		  && ((SLDTHREADID_DEVDLGSAVE == lparam) || (SLDTHREADID_DEVDLGRELOAD == lparam))	)
	{
		if (!m_bThreadResult)
		{
			if (m_threadErrorList.IsEmpty())
			{
				MessageBox(CTypedError::SafeGetFullErrorTextTempBuffer(&m_threadTError), SUPERNUDELIST_ERROR_TITLE, MB_ICONEXCLAMATION);

				if (SLDTHREADID_DEVDLGRELOAD == lparam)
				{
					CSuckLessDialog::OnCancel();
				}
			}
			else
			{
				m_bCancelAfterListing = (SLDTHREADID_DEVDLGRELOAD == lparam);

				CListingDlg *pSLDlg = new CListingDlg(SUPERNUDELIST_ERROR_TITLE,
														CTypedError::SafeGetFullErrorTextTempBuffer(&m_threadTError),
														&m_threadErrorList, false, this);
				this->openPseudoModalChild(pSLDlg, CListingDlg::IDD, false, true);
			}
		}
		else
		{
			if (SLDTHREADID_DEVDLGSAVE == lparam)
			{
				CSuckLessDialog::OnOK();
			}
			else if (SLDTHREADID_DEVDLGRELOAD == lparam)
			{
				CSuckLessDialog::OnCancel();
			}
		}
	}
	else if (CListingDlg::IDD == sldIDD)
	{
		m_threadErrorList.RemoveAll();

		if (m_bCancelAfterListing)
		{
			CSuckLessDialog::OnCancel();
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

CPDTDevDlgSave::CPDTDevDlgSave(bool *pbThreadResult, CStringList *pStrLstLog, CTypedError *pThreadTError,
									CSNConfig *pConfig) : CProgressDlgThread()
{
	m_pbThreadResult = pbThreadResult;
	m_pStrLstLog = pStrLstLog;
	m_pThreadTError = pThreadTError;

	m_pConfig = pConfig;
}

CPDTDevDlgSave::~CPDTDevDlgSave(void)
{
}

// Returns an ident for the thread.
// Typically, each implementation of this class within a project will use a unique
// ident. (-1) should not be used, but any other number is fine.
// See CSuckLessDialog::OnThreadTermination().
int CPDTDevDlgSave::GetIdent(void)
{
	return(SLDTHREADID_DEVDLGSAVE);
}

// Overriden to provide functionality.
void CPDTDevDlgSave::mainThreadSubClass(void)
{
	(*m_pbThreadResult) = false;

	SetProgressTitle("Saving changes...");

	SetProgressHeading("Saving Player List");
	SetProgressMessage("");

	if (m_pConfig->SaveDevices(m_pThreadTError))
	{
		// Save the library in case any devices were renamed.

		SetProgressHeading("Saving Library");
		SetProgressMessage("");

		if (m_pConfig->SaveLibrary(m_pThreadTError, NULL))
		{
			(*m_pbThreadResult) = true;
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

CPDTDevDlgReLoad::CPDTDevDlgReLoad(bool *pbThreadResult, CStringList *pStrLstLog, CTypedError *pThreadTError,
									CSNConfig *pConfig) : CProgressDlgThread()
{
	m_pbThreadResult = pbThreadResult;
	m_pStrLstLog = pStrLstLog;
	m_pThreadTError = pThreadTError;

	m_pConfig = pConfig;
}

CPDTDevDlgReLoad::~CPDTDevDlgReLoad(void)
{
}

// Returns an ident for the thread.
// Typically, each implementation of this class within a project will use a unique
// ident. (-1) should not be used, but any other number is fine.
// See CSuckLessDialog::OnThreadTermination().
int CPDTDevDlgReLoad::GetIdent(void)
{
	return(SLDTHREADID_DEVDLGRELOAD);
}

// Overriden to provide functionality.
void CPDTDevDlgReLoad::mainThreadSubClass(void)
{
	(*m_pbThreadResult) = false;

	SetProgressTitle("Cancelling changes...");

	SetProgressHeading("Re-loading Player List");
	SetProgressMessage("");

	if (m_pConfig->LoadDevices(m_pThreadTError, m_pStrLstLog))
	{
		(*m_pbThreadResult) = true;
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
