// DiscDlg.cpp : implementation file
//

#include "stdafx.h"
#include "DiscDlg.h"
#include "ArtistDlg.h"
#include "SNVolume.h"
#include "SNDevice.h"
#include "ProntoStringDlg.h"
#include "SNLibrary.h"
#include "TrackDlg.h"
#include "SNConfig.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDiscDlg dialog

// pParentArtist can be NULL if pUpdateDisc is NULL.

CDiscDlg::CDiscDlg(ISNUITreeDlg *pTreeDlg,
					const CSNDisc *pTemplateDisc, const CSNDisc *pUpdateDisc, const CSNDisc *pToAddBelow,
					ISNUITreeVolume *pParentVolume,
					ISNUITreeArtist *pParentArtist,
					CSNDeviceList *pDevices,
					const CSNDevice *pDevice,
					const CSNLibrary *pLibrary,
					CWnd* pParent)
	: CSuckLessDialog(CDiscDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDiscDlg)
	m_editName = _T("");
	m_editSortAs = _T("");
	m_editShort = _T("");
	//}}AFX_DATA_INIT

	m_pNewDisc = new CSNDisc();

	if (NULL != pTemplateDisc)
	{
		m_pNewDisc->Set(pTemplateDisc);
	}

	if (NULL != pDevice)
	{
		m_pNewDisc->SetDevice(pDevice);
	}

	m_pUpdateDisc = pUpdateDisc;
	m_pToAddBelow = pParentArtist ? pToAddBelow : NULL;
	m_pParentArtist = pParentArtist;
	m_pParentVolume = pParentVolume;

	m_pArtistToSelect = pParentArtist ? pParentArtist->SNUIT_GetRealArtist() : NULL;

	m_pTreeDlg = pTreeDlg;

	m_pDevices = pDevices;

	m_pLibrary = pLibrary;

	m_bUpdatingDiscNumber = false;

	m_bInitDone = false;
}

CDiscDlg::~CDiscDlg()
{
	delete m_pNewDisc;
	m_pNewDisc = NULL;
}

void CDiscDlg::DoDataExchange(CDataExchange* pDX)
{
	CSuckLessDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDiscDlg)
	DDX_Control(pDX, IDC_EDIT_SHORT, m_ctrlEditShort);
	DDX_Control(pDX, IDC_CHECK_IGNORETRACKS, m_ctrlCheckIgnoreTracks);
	DDX_Control(pDX, IDC_LIST_TRACKS, m_ctrlListTracks);
	DDX_Control(pDX, IDCANCEL, m_buttonCancel);
	DDX_Control(pDX, IDOK, m_buttonOK);
	DDX_Control(pDX, IDC_SPIN_NUMBER, m_ctrlSpinNumber);
	DDX_Control(pDX, IDC_COMBO_DEVICE, m_ctrlComboBoxDevice);
	DDX_Control(pDX, IDC_COMBO_ARTIST, m_ctrlComboBoxArtist);
	DDX_Control(pDX, IDC_EDIT_SORTAS, m_ctrlEditSortAs);
	DDX_Control(pDX, IDC_EDIT_NAME, m_ctrlEditName);
	DDX_Text(pDX, IDC_EDIT_NAME, m_editName);
	DDX_Text(pDX, IDC_EDIT_SORTAS, m_editSortAs);
	DDX_Text(pDX, IDC_EDIT_SHORT, m_editShort);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDiscDlg, CSuckLessDialog)
	//{{AFX_MSG_MAP(CDiscDlg)
	ON_BN_CLICKED(IDC_BUTTON_NAME, OnButtonName)
	ON_BN_CLICKED(IDC_BUTTON_SORTAS, OnButtonSortas)
	ON_BN_CLICKED(IDC_BUTTON_ARTISTNEW, OnButtonArtistnew)
	ON_WM_VSCROLL()
	ON_EN_CHANGE(IDC_EDIT_NUMBER, OnChangeEditNumber)
	ON_BN_CLICKED(IDC_BUTTON_ADDTRACK, OnButtonAddtrack)
	ON_BN_CLICKED(IDC_BUTTON_DELETETRACK, OnButtonDeletetrack)
	ON_BN_CLICKED(IDC_BUTTON_EDITTRACK, OnButtonEdittrack)
	ON_NOTIFY(NM_DBLCLK, IDC_LIST_TRACKS, OnDblclkListTracks)
	ON_BN_CLICKED(IDC_BUTTON_SHORT, OnButtonShort)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDiscDlg message handlers

BOOL CDiscDlg::OnInitDialog() 
{
	CSuckLessDialog::OnInitDialog();

	UpdateData(TRUE);

	this->setWindowIcon(IDR_MAINFRAME);

	CString winTitle;
	GetWindowText(winTitle);
	if (NULL != m_pUpdateDisc)
	{
		winTitle += " -- (";
		winTitle += m_pUpdateDisc->GetName()->GetPrintable(true);
		winTitle += ")";
	}
	else
	{
		winTitle += " -- <New>";
	}
	SetWindowText(winTitle);

	if (NULL == m_pUpdateDisc)
	{
		m_buttonOK.SetWindowText("Add");
		m_buttonCancel.SetWindowText("Close");
	}

	// -- Initial contents --

	if (NULL != g_pProntoFont)
	{
		m_ctrlEditName.SetFont(g_pProntoFont);
		m_ctrlEditShort.SetFont(g_pProntoFont);
		m_ctrlEditSortAs.SetFont(g_pProntoFont);
		m_ctrlComboBoxDevice.SetFont(g_pProntoFont);
		m_ctrlComboBoxArtist.SetFont(g_pProntoFont);
		m_ctrlListTracks.SetFont(g_pProntoFont);
	}

	// --

	m_ctrlListTracks.SetExtendedStyle(LVS_EX_FULLROWSELECT);
	m_ctrlListTracks.InsertColumn(LIST_COL_NUMBER,	"Number", LVCFMT_RIGHT);
	m_ctrlListTracks.InsertColumn(LIST_COL_NAME,	"Title");

	m_ctrlListTracks.NudelSortSetNumberColumn(LIST_COL_NUMBER);

	// --

	m_editName		= m_pNewDisc->GetName()->GetPronto(true);
	m_editShort		= m_pNewDisc->GetShortName()->GetPronto(true);
	m_editSortAs	= m_pNewDisc->GetSortAsName()->GetPronto(true);

	// --

	const CSNDevice *pSelDevice = NULL;

	for (const CSNDevice *pDevice = (pSelDevice = m_pDevices->GetFirst());
		 NULL != pDevice;
		 pDevice = m_pDevices->GetNext(pDevice))
	{
		int insIdx = m_ctrlComboBoxDevice.InsertString(-1, pDevice->GetName()->GetPronto(true));

		if ((0 > insIdx) \
		||	(CB_ERR == m_ctrlComboBoxDevice.SetItemDataPtr(insIdx, (void *)pDevice)))
		{
			MessageBox("Population of GUI failed (player list)",
					   SUPERNUDELIST_ERROR_TITLE,
					   MB_OK | MB_ICONSTOP);
			break;
		}
		else if (pDevice == m_pNewDisc->GetDevice())
		{
			m_ctrlComboBoxDevice.SetCurSel(insIdx);
			pSelDevice = pDevice;
		}
	}

	if ((NULL == pDevice) \
	&&	(NULL == m_pNewDisc->GetDevice()) \
	&&	(1 <= m_ctrlComboBoxDevice.GetCount()))
	{
		// If there's at least one device and one wasn't specified by the template,
		// select the first device, unless the loop was broken.
		m_ctrlComboBoxDevice.SetCurSel(0);
	}

	if ((NULL == m_pUpdateDisc) && (NULL != pSelDevice))
	{
		// If we're adding a new disc, set its number to be one greater than
		// the highest disc number in the device. (Better default than anything
		// else, most of the time.)
		m_pNewDisc->SetDiscNumber(m_pLibrary->GetHighestDiscNumber(pSelDevice) + 1);
	}

	// --

	m_ctrlSpinNumber.SetRange32(0, CSNDevice::MAX_DISCNUMBER);
	int discNumber = m_pNewDisc->GetDiscNumber();
	updateDiscNumber(discNumber, true, true);

	// --

	m_ctrlCheckIgnoreTracks.SetCheck(m_pNewDisc->GetIgnoreTracks() ? 1 : 0);

	// --

	resetAndPopulateArtists();
	resetAndPopulateTrackList();


	// -- Control setup --

	addControlInfo(IDC_STATIC_LEFTHALF);
	addControlInfo(IDC_STATIC_RIGHTHALF);

	addControlInfo(IDC_STATIC_PHYSICAL);
	addControlInfo(IDC_STATIC_TITLE);
	addControlInfo(IDC_STATIC_ARTIST);

	addControlInfo(IDC_COMBO_DEVICE);
	addControlInfo(IDC_EDIT_NUMBER);
	addControlInfo(IDC_SPIN_NUMBER);

	addControlInfo(IDC_COMBO_ARTIST);
	addControlInfo(IDC_BUTTON_ARTISTNEW);

	addControlInfo(IDC_STATIC_NAME);
	addControlInfo(IDC_EDIT_NAME);
	addControlInfo(IDC_BUTTON_NAME);

	addControlInfo(IDC_STATIC_SHORT);
	addControlInfo(IDC_EDIT_SHORT);
	addControlInfo(IDC_BUTTON_SHORT);

	addControlInfo(IDC_STATIC_SORTAS);
	addControlInfo(IDC_EDIT_SORTAS);
	addControlInfo(IDC_BUTTON_SORTAS);

	addControlInfo(IDC_STATIC_TRACKS);
	addControlInfo(IDC_LIST_TRACKS);
	addControlInfo(IDC_BUTTON_ADDTRACK);
	addControlInfo(IDC_BUTTON_EDITTRACK);
	addControlInfo(IDC_BUTTON_DELETETRACK);
	addControlInfo(IDC_CHECK_IGNORETRACKS);

	addControlInfo(IDC_BUTTON_HELP);

	addControlInfo(IDOK);
	addControlInfo(IDCANCEL);

	setControlRelationship(IDC_STATIC_LEFTHALF,		IDD,					CControlInfo::ATOP_TOP,
																			CControlInfo::ABOTTOM_TOP,
																			CControlInfo::ALEFT_LEFT,
																			CControlInfo::ARIGHT_PERCENTAGE,
																			CControlInfo::ATAG_ENDLIST);

	setControlRelationship(IDC_STATIC_RIGHTHALF,	IDD,					CControlInfo::ATOP_TOP,
																			CControlInfo::ABOTTOM_TOP,
																			CControlInfo::ALEFT_PERCENTAGE,
																			CControlInfo::ARIGHT_RIGHT,
																			CControlInfo::ATAG_ENDLIST);

	setControlRelationship(IDC_STATIC_PHYSICAL,		IDC_STATIC_LEFTHALF,	CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_STATIC_ARTIST,		IDC_STATIC_LEFTHALF,	CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_STATIC_TITLE,		IDC_STATIC_RIGHTHALF,	CControlInfo::ACOMMON_TOP_RESIZEWIDTH);

	setControlRelationship(IDC_COMBO_DEVICE,		IDC_STATIC_PHYSICAL,	CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_EDIT_NUMBER,			IDC_STATIC_PHYSICAL,	CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_SPIN_NUMBER,			IDC_STATIC_PHYSICAL,	CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_COMBO_ARTIST,		IDC_STATIC_ARTIST,		CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_BUTTON_ARTISTNEW,	IDC_STATIC_ARTIST,		CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_STATIC_NAME,			IDC_STATIC_TITLE,		CControlInfo::ACOMMON_TOP_LEFT);
	setControlRelationship(IDC_EDIT_NAME,			IDC_STATIC_TITLE,		CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_BUTTON_NAME,			IDC_STATIC_TITLE,		CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_STATIC_SHORT,		IDC_STATIC_TITLE,		CControlInfo::ACOMMON_TOP_LEFT);
	setControlRelationship(IDC_EDIT_SHORT,			IDC_STATIC_TITLE,		CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_BUTTON_SHORT,		IDC_STATIC_TITLE,		CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_STATIC_SORTAS,		IDC_STATIC_TITLE,		CControlInfo::ACOMMON_TOP_LEFT);
	setControlRelationship(IDC_EDIT_SORTAS,			IDC_STATIC_TITLE,		CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_BUTTON_SORTAS,		IDC_STATIC_TITLE,		CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_STATIC_TRACKS,		IDD,					CControlInfo::ACOMMON_RESIZEHEIGHT_RESIZEWIDTH);
	setControlRelationship(IDC_LIST_TRACKS,			IDC_STATIC_TRACKS,		CControlInfo::ACOMMON_RESIZEHEIGHT_RESIZEWIDTH);
	setControlRelationship(IDC_BUTTON_ADDTRACK,		IDC_STATIC_TRACKS,		CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_BUTTON_EDITTRACK,	IDC_STATIC_TRACKS,		CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_BUTTON_DELETETRACK,	IDC_STATIC_TRACKS,		CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_CHECK_IGNORETRACKS,	IDC_STATIC_TRACKS,		CControlInfo::ACOMMON_BOTTOM_RIGHT);

	setControlRelationship(IDC_BUTTON_HELP,			IDD,					CControlInfo::ACOMMON_BOTTOM_LEFT);

	setControlRelationship(IDOK,					IDD,					CControlInfo::ACOMMON_BOTTOM_RIGHT);
	setControlRelationship(IDCANCEL,				IDD,					CControlInfo::ACOMMON_BOTTOM_RIGHT);

	controlAndFrameInfoIsSetup();

	setMinXToCurrent();
	setMinYToCurrent();

	m_bInitDone = true;

	UpdateData(FALSE);

	this->setRememberWindowPosition(SN_REG_PATH_WINDOWPOS, SN_REG_VALUE_WINDOWPOS_DISC);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CDiscDlg::resetAndPopulateArtists(void)
{
	m_ctrlComboBoxArtist.ResetContent();

	for (ISNUITreeArtist *pArtist = m_pParentVolume->SNUIT_GetFirstArtist();
		 NULL != pArtist;
		 pArtist = m_pParentVolume->SNUIT_GetNextArtist(pArtist))
	{
		int insIdx = m_ctrlComboBoxArtist.InsertString(-1, pArtist->SNUIT_GetRealArtist()->GetName()->GetPronto(true));

		if ((0 > insIdx) \
		||	(CB_ERR == m_ctrlComboBoxArtist.SetItemDataPtr(insIdx, (void *)pArtist)))
		{
			MessageBox("Population of GUI failed (artist list)",
					   SUPERNUDELIST_ERROR_TITLE,
					   MB_OK | MB_ICONSTOP);
			break;
		}
		else if (pArtist->SNUIT_GetRealArtist() == m_pArtistToSelect)
		{
			m_ctrlComboBoxArtist.SetCurSel(insIdx);
		}
	}
}

void CDiscDlg::resetAndPopulateTrackList(void)
{
	m_ctrlListTracks.DeleteAllItems();

	const CSNTrackList *pTrackList = m_pNewDisc->GetTracks();

	int idx = 0;

	for (const CSNTrack *pTrack = pTrackList->GetFirst(); NULL != pTrack; pTrack = pTrackList->GetNext(pTrack))
	{
		CString numStr;
		numStr.Format("%d", pTrack->GetTrackNumber());

		int insIdx = m_ctrlListTracks.InsertItem(idx++, numStr);

		if (((-1) == insIdx) \
		||	(!m_ctrlListTracks.SetItemText(insIdx, LIST_COL_NAME, pTrack->GetTitle()->GetPronto(true))) )
		{
			MessageBox("Error populating list", SUPERNUDELIST_ERROR_TITLE, MB_ICONSTOP|MB_OK);
			break;
		}
		else
		{
			m_ctrlListTracks.SetItemData(insIdx, (DWORD)(pTrack));
		}
	}

	m_ctrlListTracks.SetColumnWidth(LIST_COL_NUMBER,	LVSCW_AUTOSIZE_USEHEADER);
	m_ctrlListTracks.SetColumnWidth(LIST_COL_NAME,		LVSCW_AUTOSIZE_USEHEADER);

	m_ctrlListTracks.SortItemsNudel(LIST_COL_NUMBER, false);
}

void CDiscDlg::OnSizeHookEnd(void)
{
	if (m_bInitDone)
	{
		CHeaderCtrl *pHeaderCtrl;

		if (NULL != (pHeaderCtrl = m_ctrlListTracks.GetHeaderCtrl()))
		{
			int headItems = pHeaderCtrl->GetItemCount();

			if (0 != headItems)
			{
				m_ctrlListTracks.SetColumnWidth(headItems - 1, LVSCW_AUTOSIZE_USEHEADER);
			}
		}
	}
}

void CDiscDlg::OnButtonName() 
{
	CProntoStringDlg *pSLDlg = new CProntoStringDlg(&m_ctrlEditName, "Disc / Item Name", this);
	this->openPseudoModalChild(pSLDlg, CProntoStringDlg::IDD, false, true);
}

void CDiscDlg::OnButtonShort() 
{
	CProntoStringDlg *pSLDlg = new CProntoStringDlg(&m_ctrlEditShort, "Disc / Item Short Name", this);
	this->openPseudoModalChild(pSLDlg, CProntoStringDlg::IDD, false, true);
}

void CDiscDlg::OnButtonSortas() 
{
	CProntoStringDlg *pSLDlg = new CProntoStringDlg(&m_ctrlEditSortAs, "Disc / Item Name (for sorting)", this);
	this->openPseudoModalChild(pSLDlg, CProntoStringDlg::IDD, false, true);
}

void CDiscDlg::OnButtonArtistnew() 
{
	int selIdx = m_ctrlComboBoxArtist.GetCurSel();

	if (0 <= selIdx)
	{
		void *pArtistToSelectVoid = m_ctrlComboBoxArtist.GetItemDataPtr(selIdx);

		// Since when was -1 guaranteed not to be a valid pointer? But that's
		// GetItemDataPtr's way of returning errors...
		if ((NULL != pArtistToSelectVoid) && ((-1) != (int)pArtistToSelectVoid))
		{
			m_pArtistToSelect = (const CSNArtist *)pArtistToSelectVoid;
		}
	}

	CArtistDlg *pSLDlg = new CArtistDlg(m_pTreeDlg, NULL, NULL, NULL, &m_pArtistToSelect, m_pParentVolume, this);
	this->openPseudoModalChild(pSLDlg, CArtistDlg::IDD, false, true);
}

// SuckLessDialog:
void CDiscDlg::OnPseudoModalChildClose(int sldIDD, LPARAM lparam)
{
	if (CArtistDlg::IDD == sldIDD)
	{
		if (NULL != m_pArtistToSelect)
		{
			resetAndPopulateArtists();
		}
	}
}

void CDiscDlg::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	if (m_ctrlSpinNumber.GetDlgCtrlID() == pScrollBar->GetDlgCtrlID())
	{
		updateDiscNumber(nPos, true, false);
	}

	CSuckLessDialog::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CDiscDlg::updateDiscNumber(int discNumber, bool bUpdateEdit, bool bUpdateSpin)
{
	if (!m_bUpdatingDiscNumber)
	{
		m_bUpdatingDiscNumber = true;

		if (bUpdateEdit)
		{
			CString csLabel;

			CWnd *pEdit = this->GetDlgItem(IDC_EDIT_NUMBER);

			if (NULL != pEdit)
			{
				csLabel.Format("%d", discNumber);
				pEdit->SetWindowText(csLabel);
			}
		}

		if (bUpdateSpin)
		{
			m_ctrlSpinNumber.SetPos(discNumber);
		}

		m_pNewDisc->SetDiscNumber(discNumber);

		m_bUpdatingDiscNumber = false;
	}
}

void CDiscDlg::OnChangeEditNumber() 
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CSuckLessDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	CWnd *pEdit = this->GetDlgItem(IDC_EDIT_NUMBER);

	if (NULL != pEdit)
	{
		CString csLabel;
		pEdit->GetWindowText(csLabel);

		bool bUpdateEdit = false;

		int val = 0;

		if (! csLabel.IsEmpty() )
		{
			val = atoi(LPCTSTR(csLabel));

			if (CSNDevice::MAX_DISCNUMBER < val)
			{
				val = CSNDevice::MAX_DISCNUMBER;
				bUpdateEdit = true;
			}
			else if (0 > val)
			{
				val = 1;
				bUpdateEdit = true;
			}
		
		}

		updateDiscNumber(val, bUpdateEdit, true);
	}
}

void CDiscDlg::OnOK() 
{
	UpdateData(TRUE);

	// -- Put any unsaved values into the disc. --

	m_pNewDisc->GetName()->SetPronto(LPCTSTR(m_editName));
	m_pNewDisc->GetShortName()->SetPronto(LPCTSTR(m_editShort));
	m_pNewDisc->GetSortAsName()->SetPronto(LPCTSTR(m_editSortAs));
	m_pNewDisc->SetIgnoreTracks(0 != m_ctrlCheckIgnoreTracks.GetCheck());

	//--

	{
		int selIdx = m_ctrlComboBoxDevice.GetCurSel();

		const CSNDevice *pDevice = NULL;

		if (0 <= selIdx)
		{
			void *pDeviceVoid = m_ctrlComboBoxDevice.GetItemDataPtr(selIdx);

			// Since when was -1 guaranteed not to be a valid pointer? But that's
			// GetItemDataPtr's way of returning errors...
			if ((NULL != pDeviceVoid) && ((-1) != (int)pDeviceVoid))
			{
				pDevice = (const CSNDevice *)pDeviceVoid;
			}
		}

		m_pNewDisc->SetDevice(pDevice);
	}

	//--

	ISNUITreeArtist *pArtistToAddTo = NULL;

	{
		int selIdx = m_ctrlComboBoxArtist.GetCurSel();

		if (0 <= selIdx)
		{
			void *pArtistVoid = m_ctrlComboBoxArtist.GetItemDataPtr(selIdx);

			// Since when was -1 guaranteed not to be a valid pointer? But that's
			// GetItemDataPtr's way of returning errors...
			if ((NULL != pArtistVoid) && ((-1) != (int)pArtistVoid))
			{
				pArtistToAddTo = (ISNUITreeArtist *)pArtistVoid;
			}
		}
	}

	// -- Try to add to/update the list. --

	CTypedError tError;

	const CSNDisc *pAddUpdateResult = NULL;

	if (NULL == pArtistToAddTo)
	{
		CTypedError::SafeSetMsgError(&tError, "No Artist selected.");
	}
	else if (NULL != m_pUpdateDisc)
	{
		if (pArtistToAddTo == m_pParentArtist)
		{
			// Nice, normal update 
			pAddUpdateResult = m_pParentArtist->SNUIT_UpdateDisc(m_pTreeDlg, &tError, m_pNewDisc, m_pUpdateDisc);
		}
		else
		{
			// Update includes moving the disc from one artist to another...
			pAddUpdateResult = m_pParentArtist->SNUIT_UpdateDiscAndMoveToAnotherArtist(m_pTreeDlg, &tError, pArtistToAddTo->SNUIT_GetRealArtist(), m_pNewDisc, m_pUpdateDisc);
		}
	}
	else
	{
		const CSNDisc *pToAddBelow = m_pToAddBelow;

		if (pArtistToAddTo != m_pParentArtist)
		{
			// User switched to another artist; m_pToAddBelow isn't in the same list.
			// Add to the end of the list we are adding to.
			pToAddBelow = pArtistToAddTo->SNUIT_GetLastDisc();
		}

		pAddUpdateResult = pArtistToAddTo->SNUIT_AddBelowAndOwnDisc(m_pTreeDlg, &tError, m_pNewDisc, pToAddBelow);
	}

	if (NULL == pAddUpdateResult)
	{
		MessageBox(CTypedError::SafeGetFullErrorTextTempBuffer(&tError),
				   NULL != m_pUpdateDisc ? "Cannot update disc" : "Cannot add disc",
				   MB_OK | MB_ICONEXCLAMATION);
	}
	else
	{
		if (NULL != m_pUpdateDisc)
		{
			CSuckLessDialog::OnOK();
		}
		else
		{
			// Add next disc under the last one (if same artist).
			m_pToAddBelow = pAddUpdateResult;

			// We no longer own the disc and should not delete it on destruction.
			m_pNewDisc = NULL;

			// LD 24/Feb/2001
			// Removed m_pNewDisc->Set(pAddUpdateResult) since I can't see the point in it.
			// Removed calls to clear the new disc's name since they're no longer needed.
			// Changed SetDiscNumber below to be relative to pAddUpdateResult instead of
			// m_pNewDisc (since it's not set anymore).
			// Added copying of the ignore all tracks flag.
			// Added resetAndPopulateTrackList() for new tracklist support.

			// Create a new disc which we do own (for now) and should delete on destruction.
			m_pNewDisc = new CSNDisc();

			// Doing it this way stops the number going out of range.
			m_pNewDisc->SetDiscNumber(pAddUpdateResult->GetDiscNumber() + 1);

			m_pNewDisc->SetIgnoreTracks(pAddUpdateResult->GetIgnoreTracks());

			m_editName = m_pNewDisc->GetName()->GetPronto(true);
			m_editShort = m_pNewDisc->GetShortName()->GetPronto(true);
			m_editSortAs = m_pNewDisc->GetSortAsName()->GetPronto(true);
			updateDiscNumber(m_pNewDisc->GetDiscNumber(), true, true);
			m_ctrlCheckIgnoreTracks.SetCheck(m_pNewDisc->GetIgnoreTracks() ? 1 : 0);
			resetAndPopulateTrackList();

			UpdateData(FALSE);

			m_ctrlEditName.SetFocus();
		}
	}
}

void CDiscDlg::OnButtonAddtrack() 
{
	// Give a template track with the highest track number plus one.

	int templateTrackNumber = 1;

	// Track lists are sorted by number.
	const CSNTrack *pHighestTrack = m_pNewDisc->GetTracks()->GetLast();

	if (NULL != pHighestTrack)
	{
		templateTrackNumber = pHighestTrack->GetTrackNumber() + 1;
	}

	m_templateTrack.SetTrackNumber(templateTrackNumber);

	CTrackDlg *pSLDlg = new CTrackDlg(this, &m_templateTrack, NULL, m_pNewDisc);
	this->openPseudoModalChild(pSLDlg, CTrackDlg::IDD, false, true);
}

void CDiscDlg::OnButtonDeletetrack() 
{
	POSITION pos = m_ctrlListTracks.GetFirstSelectedItemPosition();

	if (NULL != pos)
	{
		int idx = m_ctrlListTracks.GetNextSelectedItem(pos);

		CSNTrack *pTrack = (CSNTrack *)m_ctrlListTracks.GetItemData(idx);

		m_pNewDisc->GetTracks()->RemoveAndDelete(pTrack);

		resetAndPopulateTrackList();

		int trackCount = m_ctrlListTracks.GetItemCount();

		if (0 < trackCount)
		{
			if (idx >= trackCount)
			{
				idx = trackCount - 1;
			}

			m_ctrlListTracks.SetItemState(idx, LVIS_SELECTED, LVIS_SELECTED);
		}
	}
}

void CDiscDlg::OnButtonEdittrack() 
{
	POSITION pos = m_ctrlListTracks.GetFirstSelectedItemPosition();

	if (NULL != pos)
	{
		int idx = m_ctrlListTracks.GetNextSelectedItem(pos);

		CSNTrack *pTrack = (CSNTrack *)m_ctrlListTracks.GetItemData(idx);

		CTrackDlg *pSLDlg = new CTrackDlg(this, pTrack, pTrack, m_pNewDisc);
		this->openPseudoModalChild(pSLDlg, CTrackDlg::IDD, false, true);
	}
}

void CDiscDlg::OnDblclkListTracks(NMHDR* pNMHDR, LRESULT* pResult) 
{
	OnButtonEdittrack();
	
	*pResult = 0;
}
