#include "stdafx.h"

#include <lm.h>
#include <io.h>

#include "ProgressDlg.h"

// -- global variables --

CRITICAL_SECTION g_CSGlobals; // Initialized & Deleted in CSuperNudelistApp::InitInstance
CProgressDlg *g_pProgressDlg;
CStringList *g_pStringList;
bool g_bCompiling;	// True when compiling. False when not compiling. Undefined when doing neither.
int g_nLastLineNumber;
CFont *g_pProntoFont; //  May be NULL. Initialized & Deleted in CSuperNudelistApp::InitInstance
CStringList *g_pXMLErrorStringList;

// -- global utility functions --

// strncpy doesn't always null term the destination... sigh.
// This returns the index of the null terminator.
int stringCopy(char *dest, const char *source, int destSize)
{
	int idx = 0;

	if (destSize > 0)
	{
		while (('\0' != source[idx]) && (destSize > 1))
		{
			dest[idx] = source[idx];
			idx++;
			destSize--;
		}

		dest[idx] = '\0';
	}

	return(idx);
}

// _snprintf doesn't always null term the destination... sigh
// Returns the index of the null terminator.
int stringFormat(char *buffer, size_t bufferSize, const char *format, ...)
{
	va_list args;
	va_start(args, format);
	int result = vStringFormat(buffer, bufferSize, format, args);
	va_end(args);

	return(result);
}

// _vsnprintf doesn't always null term the destination... sigh
// Returns the index of the null terminator.
int vStringFormat(char *buffer, size_t bufferSize, const char *format, va_list args)
{
	int result = 0;

	if (bufferSize > 0)
	{
		result = _vsnprintf(buffer, bufferSize, format, args);

		if (0 > result)
		{
			result = bufferSize - 1; // LD 13/Dec/2000 -1 was missing (DOH DOH DOH DOH DOH!)
			buffer[result] = '\0';
		}
	}

	return(result);
}

char *TrimWhitespace(char *buffer)
{
	while(isspace(*buffer))
	{
		buffer++;
	}

	char *bufEnd = buffer + strlen(buffer);

	while( (bufEnd > buffer) && (isspace(*(--bufEnd))) )
	{
		*bufEnd = '\0';
	}

	return(buffer);
}

// You must delete[] the result of this function after use.
char *GetLowercaseCopy(const char *instr)
{
	char *result = NULL;

	if (NULL != instr)
	{
		result = new char[strlen(instr) + 1];

		const char *pi = instr;
		char *pr = result;

		while('\0' != *pi)
		{
			*pr++ = tolower(*pi++);
		}

		*pr = '\0';
	}

	return(result);
}

// You must delete[] the result of this function after use.
// Give NULL after final path.
char *JoinMultiplePaths(const char *first, ...)
{
	char *result = NULL;
	int length = 0;

	if (NULL == first)
	{
		result = new char[1];
		result[0] = '\0';
	}
	else
	{
		result = new char[strlen(first) + 1];
		strcpy(result, first);

		va_list args;
		va_start(args, first);

		char *oldResult;
		const char *next;

		for (next = va_arg(args, const char *); (NULL != next); next = va_arg(args, const char *))
		{
			oldResult = result;

			result = JoinPaths(oldResult, next);

			delete [] oldResult;
		}

		va_end(args);
	}

	return(result);
}

// You must delete[] the result of this function after use.
char *JoinPaths(const char *first,const char *second)
{
	char *result = NULL;

	int firstLen = strlen(first);
	int length = firstLen + strlen(second) + 1;

	bool addSlash = ((firstLen > 0) && ('\\' != first[firstLen - 1]) && ('/' != first[firstLen - 1]));

	if (addSlash)
	{
		length++; // Space for the extra backslash.
	}

	result = new char[length];

	strcpy(result, first);

	if (addSlash)
	{
		result[firstLen] = '\\';
		strcpy(result + firstLen + 1, second);
	}
	else
	{
		strcpy(result + firstLen, second);
	}

	return(result);
}

// You must delete[] the result of this function after use.
// A path with no slashes or the empty string returns the empty string (you must still delete the result).
char *GetParentPath(const char *path)
{
	char *result = NULL;

	if (NULL != path)
	{
		char *lastBackslash  = strrchr(path, '\\');
		char *lastFrontslash = strrchr(path, '/' );
		char *lastSlash = NULL;

		if ((NULL != lastBackslash) && (NULL != lastFrontslash))
		{
			if (lastBackslash > lastFrontslash)
			{
				lastSlash = lastBackslash;
			}
			else
			{
				lastSlash = lastFrontslash;
			}
		}
		else if (NULL != lastBackslash)
		{
			lastSlash = lastBackslash;
		}
		else if (NULL != lastFrontslash)
		{
			lastSlash = lastFrontslash;
		}

		if (NULL == lastSlash)
		{
			result = new char[1];
			result[0] = '\0';
		}
		else
		{
			size_t resSize = (lastSlash - path) + 1;
			result = new char[resSize];
			stringCopy(result, path, resSize);
		}
	}

	return(result);
}

// If fails use GetLastError().
bool RecursiveCreateDirectoryDelayedRetry(const char *dirname)
{
	int err = ERROR_SUCCESS;

	if (NULL == dirname)
	{
		err = ERROR_INVALID_PARAMETER;
	}
	else
	{
		char *dirCopy = new char[strlen(dirname) + 1];
		strcpy(dirCopy, dirname);

		char *dirPath = new char[1];
		dirPath[0] = '\0';

		char *oldDirPath;

		char *dirComponent = strtok(dirCopy, "\\/");

		while ((ERROR_SUCCESS == err) && (NULL != dirComponent))
		{
			oldDirPath = dirPath;

			dirPath = JoinPaths(oldDirPath, dirComponent);

			if (!CreateDirectoryDelayedRetry(dirPath))
			{
				err = GetLastError();
			}

			delete [] oldDirPath;

			dirComponent = strtok(NULL, "\\/");
		}

		delete [] dirCopy;
		delete [] dirPath;
	}

	SetLastError(err);

	return(ERROR_SUCCESS == err);
}

// If fails use GetLastError().
bool CreateDirectoryDelayedRetry(const char *dirname)
{
	bool result = false;

	// (Nice CODE:RANT ratio...)

	if(0 == _access(dirname, 00))
	{
		// Directory already exists. Wonderful. Let's have a party.
		result = true;
	}
	else
	{
		// Try to create more than once, waiting in between in case NT
		// is having a temporary (ha, haha) fit of lameness.
		// (Seems that if you have a dir open in explorer and try to remove
		// and then recreate it, the first remove fails but then the window
		// closes automatically, another remove succeedes, but then the create
		// might fail...
		// How whacked is that? If a dir is removed, it's removed,
		// not half there half not. The function shouldn't return until it is
		// removed. What difference does it make when creating a directory that
		// some user was looking at an old one with the same name before before
		// it was removed...
		// WHAT THE HELL PLANET DO THE PEOPLE WHO WROTE NT LIVE ON? CU...)

		for (int i=0; (i<5); i++)
		{
			if (CreateDirectory(dirname, NULL))
			{
				result = true;
				break;
			}
			else if (i<4) // Don't wait the extra second for nothing, and don't change LastError.
			{
				Sleep(10000);
			}
		}
	}

	return(result);
}

// Will remove any contents of the directory if it already exists or create it if it does not.
// If pProgressDlg is non-NULL the message line (only) will be set as each file/dir is deleted.
bool NudelEmptyDirectory(CProgressDlg *pProgressDlg, const char *dirname, CTypedError *pTError)
{
	bool bResult = true;

	if (! RecursiveCreateDirectoryDelayedRetry(dirname) )
	{
		CTypedError::SafeSetSystemError(pTError, "Error creating %s", dirname);
		bResult = false;
	}
	else
	{
		char *searchPath = JoinPaths(dirname, "*.*");

		HANDLE hFind;
		WIN32_FIND_DATA findFileData;

		if (INVALID_HANDLE_VALUE == (hFind = FindFirstFile(searchPath, &findFileData)))
		{
			if (ERROR_NO_MORE_FILES != GetLastError())
			{
				CTypedError::SafeSetSystemError(pTError, "Error finding %s", searchPath);
				bResult = false;
			}
		}
		else
		{
			char *fullPath = NULL;

			while(bResult)
			{
				// Ignore stupid "." and ".." entries.
				if ((0 != strcmp(".", findFileData.cFileName)) && (0 != strcmp("..", findFileData.cFileName)))
				{
					fullPath = JoinPaths(dirname, findFileData.cFileName);

					// Remove read-only attribute.
					if (FILE_ATTRIBUTE_READONLY & findFileData.dwFileAttributes)
					{
						if (! SetFileAttributes(fullPath, FILE_ATTRIBUTE_NORMAL) )
						{
							CTypedError::SafeSetSystemError(pTError, "Error resetting attributes of %s", fullPath);
							bResult = false;
						}
					}

					if (bResult)
					{
						if (FILE_ATTRIBUTE_DIRECTORY & findFileData.dwFileAttributes)
						{
							if (! NudelEmptyDirectory(pProgressDlg, fullPath, pTError) )
							{
								bResult = false;
							}
							else
							{
								CProgressDlg::SafeSetMessage(pProgressDlg, "Removing %s", fullPath);

								if (! RemoveDirectory(fullPath) )
								{
									CTypedError::SafeSetSystemError(pTError, "Error removing %s", fullPath);
									bResult = false;
								}
							}
						}
						else
						{
							CProgressDlg::SafeSetMessage(pProgressDlg, "Deleting %s", fullPath);

							if (! DeleteFile(fullPath) )
							{
								CTypedError::SafeSetSystemError(pTError, "Error deleting %s", fullPath);
								bResult = false;
							}
						}
					}

					delete [] fullPath;
				}

				// Get next entry
				if (bResult && (! FindNextFile(hFind, &findFileData) ))
				{
					if (ERROR_NO_MORE_FILES != GetLastError())
					{
						CTypedError::SafeSetSystemError(pTError, "Error finding %s", searchPath);
						bResult = false;
					}
					else
					{
						break;
					}
				}
			}

			FindClose(hFind);
		}

		delete [] searchPath;
	}

	return(bResult);
}

// Always test for read access.
bool NudelAccess(CTypedError *pTError, const char *path, bool bWrite)
{
	bool bResult = true;

	int iMode = bWrite ? 06 : 04; // Read and write or just read.

	if (0 != _access(path, iMode))
	{
		bResult = false;

		CTypedError::SafeSetCRuntimeError(pTError, "Cannot access %s", path);
	}

	return(bResult);
}

// pp -- Pointer to buffer to tokenize. Will be advanced to the next position string. Set to NULL after the last token is extracted.
// delimiters -- Characters which delimit the string.
// terminators -- Characters which end the string. Can be NULL.
// bSkipEmpty -- Set true to skip any empty strings. (e.g. someone typed "blah,,blah")
// bQuotes -- Set to true to handle double-quotes (") around items. A quoted empty string is NOT skipped even if bSkipEmpty is set. Terminators in quotes are ignored and included in the tokenized strings.
// bCSVQuotes -- (Ignored if bQuotes false) -- Inside quoted strings, two double-quotes next to each other will be output as one double-quote and not signify the end of the string. A string like "Blah""blah" will become blah"blah.
// buf -- Buffer to extract in to. Set to NULL not to extract.
// bufSize -- Size of buf if it is non-NULL. Will truncate string if buf too small.
// Leading and trailing spaces are always stripped.
// Returns false iff nothing could be extracted (i.e. *pp was NULL)
// pp and the contents of buf are only valid after calling if true is returns.
// Lame function should allocate us a suitably sized buffer for each string... :-)
// But hey, this isn't a public OS function and you can just give a buffer the size of the input...
bool Tokenize(const char **pp, const char *delimiters, const char *terminators, char *buf, size_t bufSize, bool bSkipEmpty, bool bQuotes, bool bCSVQuotes)
{
	bool bResult = false;

	if (NULL == terminators)
	{
		terminators = "";
	}

	if (NULL != *pp)
	{
		// Remove leading spaces.
		// Also, skip leading delimiters if we're to skip empties.
		// (Important that we skip the spaces here so that they're not included if it's the first string.)
		// (Note we must explicitly check for the terminator before checking the delimiters else we ask if
		// the terminator is in the delimiters string.)
		while ( isspace(**pp) || (('\0' != **pp) && (NULL == strchr(terminators, **pp)) && bSkipEmpty && (NULL != strchr(delimiters, **pp))) )
		{
			(*pp)++;
		}

		char *bufOrig = buf;

		bResult = true; // We got somethin'!

		bool bFindQuote = false;

		if (bQuotes && ('"' == **pp))
		{
			(*pp)++; // Skip over the starting quote.
			bFindQuote = true; // Copy string until the ending quote rather than a delimiter.
		}

		// There is some string left, extract what's next.
		// Ignore terminators and delmiters if we're looking for the end-quote.
		while ('\0' != **pp)
		{
			if (bFindQuote)
			{
				if ('"' == (*pp)[0])
				{
					if (bCSVQuotes && ('"' == (*pp)[1]))
					{
						(*pp)++; // It is a pair of quotes, skip one and copy the other.
					}
					else
					{
						break; // It is a closing quote.
					}
				}
			}
			else if ((NULL != strchr(delimiters,  **pp))
			||		 (NULL != strchr(terminators, **pp)))
			{
				// Not looking for quotes and found a delimiter or a terminator
				break;
			}

			if ((NULL != buf) && (bufSize > 1)) // Keep 1 character for the terminator.
			{
				*buf = **pp;
				bufSize--;
				buf++;
			}

			(*pp)++; // Must increment *pp here as copying to buf is optional and may also be trunctated.
		}

		// Handles the case where a zero-length buffer is passed in.
		if ((NULL != buf) && (bufSize > 0))
		{
			*buf = '\0'; // NULL-terminate it.

			// Remove trailing spaces if not quoted.
			if (! bFindQuote )
			{
				while (buf > bufOrig)
				{
					if (isspace(*(--buf)))
					{
						*buf = '\0';
					}
					else
					{
						break;
					}
				}
			}
		}

		// If quoted, move over the closing quote and skip up to the next delimiter.
		if (bFindQuote && ('\0' != **pp) && (NULL == strchr(terminators, **pp)))
		{
			do
			{
				(*pp)++;
			} while (('\0' != **pp) && (NULL == strchr(terminators, **pp)) && (NULL == strchr(delimiters, **pp)));
		}

		// Move over the delimiter (unless we're at the end of the string).
		if (('\0' != **pp) && (NULL == strchr(terminators, **pp)) && (NULL != strchr(delimiters, **pp)))
		{
			(*pp)++;

			// Skip spaces before the next token. (Useful for when buf==NULL)
			while (('\0' != **pp) && (NULL == strchr(terminators, **pp)) && isspace(**pp))
			{
				(*pp)++;
			}
		}
		else
		{
			// We're returning the last thing on the line.
			*pp = NULL;
		}
	}

	return(bResult);
}

// Returns false on failure. Use GetLastError().
// hKeyParent -- hKey of parent under which szKeyPath should be created. Usually a hive like HKEY_CURRENT_USER
// szKeyPath -- Sub key (can be a path) under hKeyHiveOrParent in which szValueName is set. Can be "" or NULL.
// szValueName -- Name of value to set. Can be "" or NULL to set the unnamed default value.
bool LeetRegDeleteValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName)
{
	LONG regRes = ERROR_SUCCESS;

	if ((NULL != szKeyPath) && (0 == _tcsicmp(szKeyPath, _T(""))))
	{
		szKeyPath = NULL;
	}

	HKEY hKey;

	regRes = RegOpenKeyEx(hKeyParent, szKeyPath, 0, KEY_SET_VALUE, &hKey);

	if (ERROR_SUCCESS == regRes)
	{
		regRes = RegDeleteValue(hKey, szValueName);

		RegCloseKey(hKey);
	}

	if ((ERROR_FILE_NOT_FOUND == regRes) || (ERROR_PATH_NOT_FOUND == regRes))
	{
		regRes = ERROR_SUCCESS;
	}

	if (ERROR_SUCCESS != regRes)
	{
		SetLastError(regRes);
	}

	return(ERROR_SUCCESS == regRes);
}

// Returns false on failure. Use GetLastError().
// hKeyParent -- hKey of parent under which szKeyPath should be created. Usually a hive like HKEY_CURRENT_USER
// szKeyPath -- Sub key (can be a path) under hKeyHiveOrParent in which szValueName is set. Can be "" or NULL.
// szValueName -- Name of value to set. Can be "" or NULL to set the unnamed, default value.
// dwType -- Type of data to set. (Same as dwType argument of RegSetValueEx())
// lpData -- Pointer to data to set. (6/Feb/2001: Can be NULL if you just want to create the key but set no values.)
// cbData -- Size of data pointed to by lpData, including null terminator(s) (Same as cbData arg of RegSetValueEx())
bool LeetRegSetValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName,
						 DWORD dwType, const void *lpData, DWORD cbData)
{
	// Documentation doesn't state that registry functions call SetLastError().
	// We need to preserve the error values they return through clean-up anyway.
	LONG regRes = ERROR_SUCCESS;

	bool bCloseKey = false;
	HKEY hKeyToCreateValueIn;

	if ((NULL == szKeyPath) || (0 == _tcsicmp(szKeyPath, _T(""))))
	{
		// Cannot give NULL as second argument to RegCreateKeyEx() as you can to RegOpenKeyEx()
		hKeyToCreateValueIn = hKeyParent;
	}
	else
	{
		DWORD disposition;

		// We don't need to create the key one level at a time, RegCreateKeyEx() does that.
		regRes = RegCreateKeyEx(hKeyParent, szKeyPath, 0, _T(""),
								REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL,
								&hKeyToCreateValueIn, &disposition);

		if (ERROR_SUCCESS == regRes)
		{
			bCloseKey = true;
		}
	}

	if ((ERROR_SUCCESS == regRes) && (NULL != lpData))
	{
		regRes = RegSetValueEx(hKeyToCreateValueIn, szValueName, 0, dwType, (CONST BYTE *)lpData, cbData);
	}

	if (bCloseKey)
	{
		RegCloseKey(hKeyToCreateValueIn);
		bCloseKey = false;
	}

	if (ERROR_SUCCESS != regRes)
	{
		SetLastError(regRes);
	}

	return(ERROR_SUCCESS == regRes);
}

// Returns false on failure. Use GetLastError().
// hKeyParent -- hKey of parent under which szKeyPath should be created. Usually a hive like HKEY_CURRENT_USER
// szKeyPath -- Sub key (can be a path) under hKeyHiveOrParent from which szValueName is read. Can be "" or NULL.
// szValueName -- Name of value to query. Can be "" or NULL to read the unnamed default value.
// pdwType -- Pointer to DWORD to receive type of data. NULL if unwanted.
// plpData -- POINTER TO POINTER to receive data buffer. NULL if data unwanted.
// pcbData -- Pointer to DWORD to receive size of data buffer. NULL if data unwanted.
// You do NOT allocate a buffer, this function does it for you, like the frigging API should.
// If the function succeeds and plpData was not NULL you must delete[] *plpData when finished with it.
// Like RegQueryValueEx, you may supply NULL plpData and non-NULL pcbData to get the required
// buffer size (except for HKEY_PERFORMANCE_DATA), but again the function always allocates a buffer for you.
// Although untested, reading HKEY_PERFORMANCE_DATA should work.
bool LeetRegQueryValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName,
					   DWORD *pdwType, void **plpData, DWORD *pcbData)
{
	// Documentation doesn't state that registry functions call SetLastError().
	// We need to preserve the error values they return through clean-up anyway.
	LONG regRes = ERROR_SUCCESS;

	HKEY hKeyToReadValueFrom;

	DWORD dwSomewhatDamaged; // Used for getting the registry size of the user doesn't care about it.

	if (NULL == pcbData)
	{
		pcbData = &dwSomewhatDamaged;
	}

	// szKeyPath may be NULL in which case another handle like hKeyParent is openned (and must be closed).
	regRes = RegOpenKeyEx(hKeyParent, szKeyPath, 0, KEY_QUERY_VALUE, &hKeyToReadValueFrom);

	if (ERROR_SUCCESS == regRes)
	{
		if (NULL == plpData)
		{
			// They just want the type and/or the data size.
			regRes = RegQueryValueEx(hKeyToReadValueFrom, szValueName, 0, pdwType, NULL, pcbData);
		}
		else
		{
			BYTE *allocatedBuffer = NULL;
			DWORD dwAllocatedBufferSize = 0; // For HKEY_PERFORMANCE_DATA, really.

			// Get the required buffer size by giving NULL for data buffer.
			regRes = RegQueryValueEx(hKeyToReadValueFrom, szValueName, 0, pdwType, NULL, pcbData);

			if (ERROR_SUCCESS == regRes)
			{
				dwAllocatedBufferSize = *pcbData;
				allocatedBuffer = new BYTE[dwAllocatedBufferSize];
			}
			else if (ERROR_MORE_DATA == regRes)
			{
				// Most likely tried to read HKEY_PERFORMANCE_DATA.
				dwAllocatedBufferSize = 1024; // Must be a number x such that (Int(x/2) > 0)
				allocatedBuffer = new BYTE[dwAllocatedBufferSize];
			}
			// else leave allocatedBuffer NULL so the while loop does not execute.

			while (NULL != allocatedBuffer)
			{
				regRes = RegQueryValueEx(hKeyToReadValueFrom, szValueName, 0, pdwType, allocatedBuffer, pcbData);

				if (ERROR_SUCCESS == regRes)
				{
					*plpData = allocatedBuffer;
					allocatedBuffer = NULL;
					break;
				}
				else if (ERROR_MORE_DATA == regRes)
				{
					// Try a larger buffer.
					delete [] allocatedBuffer;
					dwAllocatedBufferSize = dwAllocatedBufferSize + (dwAllocatedBufferSize/2);
					allocatedBuffer = new BYTE[dwAllocatedBufferSize];
				}
				else
				{
					delete [] allocatedBuffer;
					allocatedBuffer = NULL;
					break;
				}
			}
		}

		RegCloseKey(hKeyToReadValueFrom);
	}

	if (ERROR_SUCCESS != regRes)
	{
		SetLastError(regRes);
	}

	return(ERROR_SUCCESS == regRes);
}

// Returns false on failure. Use GetLastError().
// Uses LeetRegQueryValue() to get you a value of a specified type, failing if it is the wrong type.
bool LeetRegQueryTypedValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName,
							DWORD dwType, void **plpData, DWORD *pcbData)
{
	DWORD dwErrToSet = ERROR_SUCCESS;

	DWORD dwTypeFound;

	if (! LeetRegQueryValue(hKeyParent, szKeyPath, szValueName, &dwTypeFound, plpData, pcbData) )
	{
		dwErrToSet = GetLastError();
	}
	else if (dwTypeFound != dwType)
	{
		dwErrToSet = ERROR_INVALID_DATA;
		delete [] (*plpData);
	}

	if (ERROR_SUCCESS != dwErrToSet)
	{
		SetLastError(dwErrToSet);
	}

	return(ERROR_SUCCESS == dwErrToSet);
}

// Returns false on failure. Use GetLastError().
// Uses LeetRegQueryValue() to get you a DWORD without having to worry about buffers, types and so on.
bool LeetRegQueryDWORDValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName, DWORD *pdwRes)
{
	DWORD dwErrToSet = ERROR_SUCCESS;

	DWORD dwType;
	void *pData;

	*pdwRes = 0;

	if (! LeetRegQueryValue(hKeyParent, szKeyPath, szValueName, &dwType, &pData, NULL) )
	{
		dwErrToSet = GetLastError();
	}
	else
	{
		if (REG_DWORD != dwType)
		{
			dwErrToSet = ERROR_INVALID_DATA;
		}
		else
		{
			*pdwRes = *((DWORD*)pData);
		}

		delete [] pData;
	}

	if (ERROR_SUCCESS != dwErrToSet)
	{
		SetLastError(dwErrToSet);
	}

	return(ERROR_SUCCESS == dwErrToSet);
}

// Returns false on failure. Use GetLastError().
// Uses LeetRegQueryValue() to get you a TCHAR string without having to worry about buffers, types and so on.
bool LeetRegQueryStringValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName, std::basic_string<TCHAR> *pString)
{
	DWORD dwErrToSet = ERROR_SUCCESS;

	DWORD dwType;
	void *pData;

	pString->erase();

	if (! LeetRegQueryValue(hKeyParent, szKeyPath, szValueName, &dwType, &pData, NULL) )
	{
		dwErrToSet = GetLastError();
	}
	else
	{
		if (REG_SZ != dwType)
		{
			dwErrToSet = ERROR_INVALID_DATA;
		}
		else
		{
			(*pString) = reinterpret_cast<TCHAR*>(pData);
		}

		delete [] pData;
	}

	if (ERROR_SUCCESS != dwErrToSet)
	{
		SetLastError(dwErrToSet);
	}

	return(ERROR_SUCCESS == dwErrToSet);
}

bool FindNoCaseStringInList(CStringList& sl, CString& s)
{
	POSITION pos = sl.GetHeadPosition();
	while (NULL != pos)
	{
		if (0 == s.CompareNoCase(sl.GetNext(pos)))
		{
			return(true);
		}
	}

	return(false);
}

bool FindNoCaseStringInList(CStringList &sl, const char *s)
{
	POSITION pos = sl.GetHeadPosition();
	while (NULL != pos)
	{
		if (0 == stricmp(s, LPCTSTR(sl.GetNext(pos))))
		{
			return(true);
		}
	}

	return(false);
}

bool FindNoCaseStringInListBox(CListBox& lb, CString& s)
{
	bool bResult = false;

	CString stringInList;
	int itemCount = lb.GetCount();

	for (int i=0; i < itemCount; i++)
	{
		lb.GetText(i, stringInList);

		if (0 == s.CompareNoCase(stringInList))
		{
			bResult = true;
			break;
		}
	}

	return(bResult);
}

// It really pisses me off that under Windows 98 GetModuleFileName returns an upper-cased path.
// This function will take the path and correct it to what it really is on the filesystem.
// FFS!
// delete[] the result.
char *CorrectCaseOfPath(const char *path)
{
	char *result = NULL;

	if (NULL != path)
	{
		size_t subBufferSize = strlen(path) + 1;
		char *subBuffer = new char[subBufferSize];

		const char *p = path;

		char *newPathBuilderBuffer;
		char *tempSearchPath;
		char *pathBuildBuffer = new char[1];
		pathBuildBuffer[0] = '\0';

		WIN32_FIND_DATA findData;
		HANDLE hFindFile;

		while (Tokenize(&p, "\\/", NULL, subBuffer, subBufferSize, true, false, false))
		{
			tempSearchPath = JoinPaths(pathBuildBuffer, subBuffer);

			// LD 15/Oct/2000
			// Need to avoid searching for paths like "C:" because this
			// does return valid handles on Windows 2000, but doesn't work
			// how we want it to. (On Windows 98 it returns an invalid handle,
			// on Windows 2000 RC2 it returns the current directory on that
			// device or something.)

			if ((NULL == strchr(subBuffer, ':'))
			&&  (INVALID_HANDLE_VALUE != (hFindFile = FindFirstFile(tempSearchPath, &findData))))
			{
				// Add to the existing path...
				newPathBuilderBuffer = JoinPaths(pathBuildBuffer, findData.cFileName);
				delete [] pathBuildBuffer;
				pathBuildBuffer = newPathBuilderBuffer;

				FindClose(hFindFile);
			}
			else
			{
//				newPathBuilderBuffer = JoinPaths(pathBuildBuffer, subBuffer); // optimise: same as search path...
				delete [] pathBuildBuffer;
//				pathBuildBuffer = newPathBuilderBuffer;
				pathBuildBuffer = tempSearchPath;
				tempSearchPath = NULL; // stop tempSearchPath being deleted.
			}

			delete [] tempSearchPath;
			tempSearchPath = NULL;
		}

		result = pathBuildBuffer;

		delete [] subBuffer;

		// LD 15/Oct/2000
		// Sanity check
		// If we can access the input but not the output, scrap our changes.
		if ((0 == _access(path, 00)) && (0 != _access(result, 00)))
		{
			delete [] result;
			result = new char[strlen(path) + 1];
			strcpy(result, path);
		}
	}

	return(result);
}

void __cdecl NudelXmlErrorFunc(void *ctx, const char *msg, ...)
{
	if ((NULL != msg) && (NULL != g_pXMLErrorStringList))
	{
		CString lastLine;

		if (! g_pXMLErrorStringList->IsEmpty() )
		{
			lastLine = g_pXMLErrorStringList->RemoveTail();
		}

		CString newStuff;

		va_list args;
		va_start(args, msg);
		newStuff.FormatV(msg, args);
		va_end(args);

		lastLine += newStuff;

		bool bAddBlank = true;

		while (lastLine.GetLength() > 0)
		{
			newStuff = lastLine.SpanExcluding("\n\r");

			bAddBlank = (newStuff.GetLength() != lastLine.GetLength());

			g_pXMLErrorStringList->AddTail(newStuff);

			lastLine.Delete(0, newStuff.GetLength() + 1);
		}

		if (bAddBlank)
		{
			g_pXMLErrorStringList->AddTail("");
		}
	}
}

char alphabetPanelLetterGroupTable[256];

// For alphabet panel sorting. Returns '#' or an upper-case letter A-Z.
// Defined in this way to stear clear of locales and so on.
char GetAlphabetPanelLetterGroup(const char *s)
{
	if (NULL != s)
	{
		return(GetAlphabetPanelLetterGroup(*s));
	}
	else
	{
		return('#');
	}
}

char GetAlphabetPanelLetterGroup(char s)
{
	if (('\0' != s) && (0 <= s) && (s < 256))
	{
		return(alphabetPanelLetterGroupTable[s]);
	}
	else
	{
		return('#');
	}
}

// Call once before GetLetterGroup is used.
// Defined in this way to stear clear of locales and so on.
void InitAlphabetPanelLetterGroupTable(void)
{
	for (int i = 0; i < 256; i++)
	{
		alphabetPanelLetterGroupTable[i] = '#';
	}

	alphabetPanelLetterGroupTable['a'] = alphabetPanelLetterGroupTable['A'] = 'A';
	alphabetPanelLetterGroupTable['b'] = alphabetPanelLetterGroupTable['B'] = 'B';
	alphabetPanelLetterGroupTable['c'] = alphabetPanelLetterGroupTable['C'] = 'C';
	alphabetPanelLetterGroupTable['d'] = alphabetPanelLetterGroupTable['D'] = 'D';
	alphabetPanelLetterGroupTable['e'] = alphabetPanelLetterGroupTable['E'] = 'E';
	alphabetPanelLetterGroupTable['f'] = alphabetPanelLetterGroupTable['F'] = 'F';
	alphabetPanelLetterGroupTable['g'] = alphabetPanelLetterGroupTable['G'] = 'G';
	alphabetPanelLetterGroupTable['h'] = alphabetPanelLetterGroupTable['H'] = 'H';
	alphabetPanelLetterGroupTable['i'] = alphabetPanelLetterGroupTable['I'] = 'I';
	alphabetPanelLetterGroupTable['j'] = alphabetPanelLetterGroupTable['J'] = 'J';
	alphabetPanelLetterGroupTable['k'] = alphabetPanelLetterGroupTable['K'] = 'K';
	alphabetPanelLetterGroupTable['l'] = alphabetPanelLetterGroupTable['L'] = 'L';
	alphabetPanelLetterGroupTable['m'] = alphabetPanelLetterGroupTable['M'] = 'M';
	alphabetPanelLetterGroupTable['n'] = alphabetPanelLetterGroupTable['N'] = 'N';
	alphabetPanelLetterGroupTable['o'] = alphabetPanelLetterGroupTable['O'] = 'O';
	alphabetPanelLetterGroupTable['p'] = alphabetPanelLetterGroupTable['P'] = 'P';
	alphabetPanelLetterGroupTable['q'] = alphabetPanelLetterGroupTable['Q'] = 'Q';
	alphabetPanelLetterGroupTable['r'] = alphabetPanelLetterGroupTable['R'] = 'R';
	alphabetPanelLetterGroupTable['s'] = alphabetPanelLetterGroupTable['S'] = 'S';
	alphabetPanelLetterGroupTable['t'] = alphabetPanelLetterGroupTable['T'] = 'T';
	alphabetPanelLetterGroupTable['u'] = alphabetPanelLetterGroupTable['U'] = 'U';
	alphabetPanelLetterGroupTable['v'] = alphabetPanelLetterGroupTable['V'] = 'V';
	alphabetPanelLetterGroupTable['w'] = alphabetPanelLetterGroupTable['W'] = 'W';
	alphabetPanelLetterGroupTable['x'] = alphabetPanelLetterGroupTable['X'] = 'X';
	alphabetPanelLetterGroupTable['y'] = alphabetPanelLetterGroupTable['Y'] = 'Y';
	alphabetPanelLetterGroupTable['z'] = alphabetPanelLetterGroupTable['Z'] = 'Z';
}

CriticalSectionScoper::CriticalSectionScoper(CRITICAL_SECTION *pCS) : m_pCS(pCS)
{
	EnterCriticalSection(m_pCS);
}

CriticalSectionScoper::~CriticalSectionScoper(void)
{
	LeaveCriticalSection(m_pCS);
}
