#ifndef __GLOBAL_H_
#define __GLOBAL_H_

class CTypedError;
class CProgressDlg;
class CStringList;
class CFont;

// -- global variables --

extern CRITICAL_SECTION g_CSGlobals; // Initialized & Deleted in CSuperNudelistApp::InitInstance
extern CProgressDlg *g_pProgressDlg;
extern CStringList *g_pStringList;
extern bool g_bCompiling;	// True when compiling. False when not compiling. Undefined when doing neither.
extern int g_nLastLineNumber;
extern CFont *g_pProntoFont; //  May be NULL. Initialized & Deleted in CSuperNudelistApp::InitInstance
extern CStringList *g_pXMLErrorStringList;

// -- definitions --

#define SUPERNUDELIST_ERROR_TITLE	"Super-Nudelist Error"
#define SUPERNUDELIST_WARNING_TITLE	"Super-Nudelist Warning"

// SuckLessDialog Thread Idents

#define SLDTHREADID_CLEANUP				1
#define SLDTHREADID_STARTUP				2
#define SLDTHREADID_PREPFORDEVS			3
#define SLDTHREADID_PREPFORLISTS		4
#define SLDTHREADID_DEVDLGRELOAD		5
#define SLDTHREADID_DEVDLGSAVE			6
#define SLDTHREADID_LISTDLGRELOAD		7
#define SLDTHREADID_LISTDLGSAVE			8
#define SLDTHREADID_PREPFOREXECUTE		9
#define SLDTHREADID_DEVRESDLGSAVE		10
#define SLDTHREADID_RESOLVEFOREXECUTE	11
#define SLDTHREADID_FINALEXECUTE		12
#define SLDTHREADID_LISTDLGSLOTREPORT	13
#define SLDTHREADID_IMPORTDLGGENERIC	14
#define SLDTHREADID_CSVDLGGENERIC		15
#define SLDTHREADID_CSVDLGPREVIEW		16

// Default size for fixed-size buffers
#define BUFFERSIZE 1024

// -- global utility functions --

// strncpy doesn't always null term the destination... sigh.
// This returns the index of the null terminator.
int stringCopy(char *dest, const char *source, int destSize);

// _snprintf doesn't always null term the destination... sigh
// Returns the index of the null terminator.
int stringFormat(char *buffer, size_t bufferSize, const char *format, ...);

// _vsnprintf doesn't always null term the destination... sigh
// Returns the index of the null terminator.
int vStringFormat(char *buffer, size_t bufferSize, const char *format, va_list args);

char *TrimWhitespace(char *buffer);

// You must delete[] the result of this function after use.
char *GetLowercaseCopy(const char *instr);

// You must delete[] the result of this function after use.
// Give NULL after final path.
char *JoinMultiplePaths(const char *first, ...);

// You must delete[] the result of this function after use.
char *JoinPaths(const char *first,const char *second);

// You must delete[] the result of this function after use.
// A path with no slashes or the empty string returns the empty string (you must still delete the result).
char *GetParentPath(const char *path);

// If fails use GetLastError().
bool RecursiveCreateDirectoryDelayedRetry(const char *dirname);

// If fails use GetLastError().
bool CreateDirectoryDelayedRetry(const char *dirname);

// Will remove any contents of the directory if it already exists or create it if it does not.
bool NudelEmptyDirectory(CProgressDlg *pProgressDlg, const char *dirname, CTypedError *pTError);

// Always test for read access.
bool NudelAccess(CTypedError *pTError, const char *path, bool bWrite);

// pp -- Pointer to buffer to tokenize. Will be advanced to the next position string. Set to NULL after the last token is extracted.
// delimiters -- Characters which delimit the string.
// terminators -- Characters which end the string. Can be NULL.
// bSkipEmpty -- Set true to skip any empty strings. (e.g. someone typed "blah,,blah")
// bQuotes -- Set to true to handle double-quotes (") around items. A quoted empty string is NOT skipped even if bSkipEmpty is set.
// bCSVQuotes -- (Ignored if bQuotes false) -- Inside quoted strings, two double-quotes next to each other will be output as one double-quote and not signify the end of the string. A string like "Blah""blah" will become blah"blah.
// buf -- Buffer to extract in to. Set to NULL not to extract.
// bufSize -- Size of buf if it is non-NULL. Will truncate string if buf too small.
// Leading and trailing spaces are always stripped.
// Returns false iff nothing could be extracted (i.e. *pp was NULL)
// pp and the contents of buf are only valid after calling if true is returns.
// Lame function should allocate us a suitably sized buffer for each string... :-)
// But hey, this isn't a public OS function and you can just give a buffer the size of the input...
bool Tokenize(const char **pp, const char *delimiters, const char *terminators, char *buf, size_t bufSize, bool bSkipEmpty, bool bQuotes, bool bCSVQuotes);

// Returns false on failure. Use GetLastError().
// hKeyParent -- hKey of parent under which szKeyPath should be created. Usually a hive like HKEY_CURRENT_USER
// szKeyPath -- Sub key (can be a path) under hKeyHiveOrParent in which szValueName is set. Can be "" or NULL.
// szValueName -- Name of value to set. Can be "" or NULL to set the unnamed default value.
bool LeetRegDeleteValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName);

// Returns false on failure. Use GetLastError().
// hKeyParent -- hKey of parent under which szKeyPath should be created. Usually a hive like HKEY_CURRENT_USER
// szKeyPath -- Sub key (can be a path) under hKeyHiveOrParent in which szValueName is set. Can be "" or NULL.
// szValueName -- Name of value to set. Can be "" or NULL to set the unnamed, default value.
// dwType -- Type of data to set. (Same as dwType argument of RegSetValueEx())
// lpData -- Pointer to data to set. (6/Feb/2001: Can be NULL if you just want to create the key but set no values.)
// cbData -- Size of data pointed to by lpData, including null terminator(s) (Same as cbData arg of RegSetValueEx())
bool LeetRegSetValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName,
						 DWORD dwType, const void *lpData, DWORD cbData);

// Returns false on failure. Use GetLastError().
// hKeyParent -- hKey of parent under which szKeyPath should be created. Usually a hive like HKEY_CURRENT_USER
// szKeyPath -- Sub key (can be a path) under hKeyHiveOrParent from which szValueName is read. Can be "" or NULL.
// szValueName -- Name of value to query. Can be "" or NULL to read the unnamed default value.
// pdwType -- Pointer to DWORD to receive type of data. NULL if unwanted.
// plpData -- POINTER TO POINTER to receive data buffer. NULL if data unwanted.
// pcbData -- Pointer to DWORD to receive size of data buffer. NULL if data unwanted.
// You do NOT allocate a buffer, this function does it for you, like the frigging API should.
// If the function succeeds and plpData was not NULL you must delete[] *plpData when finished with it.
// Like RegQueryValueEx, you may supply NULL plpData and non-NULL pcbData to get the required
// buffer size (except for HKEY_PERFORMANCE_DATA), but again the function always allocates a buffer for you.
// Although untested, reading HKEY_PERFORMANCE_DATA should work.
bool LeetRegQueryValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName,
					   DWORD *pdwType, void **plpData, DWORD *pcbData);

// Returns false on failure. Use GetLastError().
// Uses LeetRegQueryValue() to get you a value of a specified type, failing if it is the wrong type.
bool LeetRegQueryTypedValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName,
							DWORD dwType, void **plpData, DWORD *pcbData);

// Returns false on failure. Use GetLastError().
// Uses LeetRegQueryValue() to get you a DWORD without having to worry about buffers, types and so on.
bool LeetRegQueryDWORDValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName, DWORD *pdwRes);

// Returns false on failure. Use GetLastError().
// Uses LeetRegQueryValue() to get you a TCHAR string without having to worry about buffers, types and so on.
bool LeetRegQueryStringValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName, std::basic_string<TCHAR> *pString);

bool FindNoCaseStringInList(CStringList& sl, CString& s);
bool FindNoCaseStringInList(CStringList &sl, const char *s);
bool FindNoCaseStringInListBox(CListBox& lb, CString& s);

// It really pisses me off that under Windows 98 GetModuleFileName returns an upper-cased path.
// This function will take the path and correct it to what it really is on the filesystem.
// FFS!
// delete[] the result.
char *CorrectCaseOfPath(const char *path);

// For alphabet panel sorting. Returns '#' or an upper-case letter A-Z.
// Defined in this way to stear clear of locales and so on.
char GetAlphabetPanelLetterGroup(const char *s);
char GetAlphabetPanelLetterGroup(char s);
void InitAlphabetPanelLetterGroupTable(void); // Call once before GetLetterGroup is used.

void __cdecl NudelXmlErrorFunc(void *ctx, const char *msg, ...);

class CriticalSectionScoper
{
public:
	CriticalSectionScoper(CRITICAL_SECTION *pCS);
	virtual ~CriticalSectionScoper(void);
protected:
	CRITICAL_SECTION *m_pCS;
};

#endif
