// LayoutNudelist.h: interface for the CLayoutNudelist class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LAYOUTNUDELIST_H__6BC352A3_DC13_4781_84E4_4DFFADA7BA86__INCLUDED_)
#define AFX_LAYOUTNUDELIST_H__6BC352A3_DC13_4781_84E4_4DFFADA7BA86__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CSNVolume;
class CSNArtist;
class CSNDisc;
class CSNTrack;
class CCSFSection;

#include "LayoutNudelistPanels.h"

class CLNItem;
class CSNHardButtonDefinitionList;

//*********************************************************************************************

class CLayoutNudelist  
{
friend CLNItem;

public:
	CLayoutNudelist(const char *versionString, const CSNVolume *pVolume, const char *deviceNameEscaped,
					const CSNHardButtonDefinitionList *pHardButtonDefinitionList, int nPanelCap);
	virtual ~CLayoutNudelist();

	const CActionSectionAdder *DoLayout(const CSNVolume *pVolume);

	void AddToDevicesToRootCSFSection(CCSFSection *pRootCSFSection) const;

protected:

	CCSFSection *addNewDeviceToRootCSFSection(CCSFSection *pRootCSFSection, const char *szDeviceName) const;

	int getCurrentDeviceNumber(void);

protected:
	CLNItem *m_pRootLayoutItem;

	CLNAlphabetPanel *m_pLettersPanel;
	CLNCreditsPanel *m_pCreditsPanel;

	list<CLNPLines *> m_artistPanels;
	list<CLNPLines *> m_discPanels;
	list<CLNPLines *> m_trackPanels;

	const CSNVolume *m_pVolume;
	string m_strDeviceNameEscaped;
	string m_strVersionString;
	const CSNHardButtonDefinitionList *m_pHardButtons;
	int m_nPanelCap;
};

//*********************************************************************************************

class CActionSectionAdder
{
public:
	virtual ~CActionSectionAdder() {}
	virtual void AddToActionSection(CCSFSection *pActionSection) const = 0;
};

//*********************************************************************************************

class CCompositeActionSection : public CActionSectionAdder
{
public:
	CCompositeActionSection(const CActionSectionAdder *pFirst, const CActionSectionAdder *pSecond);
	void AddToActionSection(CCSFSection *pActionSection) const;

protected:
	const CActionSectionAdder *m_pFirst;
	const CActionSectionAdder *m_pSecond;
};

//*********************************************************************************************

class CJumpActionAdder : public CActionSectionAdder
{
public:
	CJumpActionAdder(const char *szDeviceNameEscaped, const char *szPanelNameEscaped);
	virtual ~CJumpActionAdder();
	void AddToActionSection(CCSFSection *pActionSection) const;

protected:
	string m_strDeviceNameEscaped;
	string m_strPanelNameEscaped;
};

//*********************************************************************************************

class CLNItem
{
public:
	CLNItem(CLayoutNudelist *pLayout, const char *szTitleEscaped, const char *szShortTitleEscaped);
	CLNItem(CLayoutNudelist *pLayout, char charTitleEscaped);
	virtual ~CLNItem();

	const char *GetTitleEscaped(void);
	const char *GetShortTitleEscaped(void);

	virtual const char *GetButtonLabelEscaped(void)
	{
		return(""); // Default is no button label.
	}

	// Predicts whether the return of MakePanelsAndGetAction will be NULL or not.
	virtual bool IsGoingToReturnValidAction(void) = 0;

	// Returns NULL if the child list is empty and there is no appropriate default action.
	// Whether or not the return value is NULL can be predicted by calling IsGoingToReturnValidAction
	// Some items may not make any panels; it's an optional side-effect.
	// pParentPanel can be NULL if there isn't one.
	// If a panel jump is returned it will be absolute iff pParentPanel is NULL.
	virtual const CActionSectionAdder *MakePanelsAndGetAction(CLNPanel *pParentPanel) = 0;

protected:
	CLayoutNudelist *m_pLayout;

	string m_strTitleEscaped;
	string m_strShortTitleEscaped;

	const CSNVolume *getVolume(void);
	const char *getDeviceNameEscaped(void);
	int getCurrentDeviceNumber(void);
	const char *getVersionString(void);

	CLNAlphabetPanel ** getLettersPanelPointer(void);
	CLNCreditsPanel ** getCreditsPanelPointer(void);

	list<CLNPLines *> & getArtistPanels(void);
	list<CLNPLines *> & getDiscPanels(void);
	list<CLNPLines *> & getTrackPanels(void);
};

//*********************************************************************************************

class CLNItemWithChildren : public CLNItem
{
public:
	CLNItemWithChildren(CLayoutNudelist *pLayout, const char *szTitleEscaped, const char *szShortTitleEscaped);
	CLNItemWithChildren(CLayoutNudelist *pLayout, char charTitleEscaped);
	virtual ~CLNItemWithChildren();

	// Predicts whether the return of MakePanelsAndGetAction will be NULL or not.
	bool IsGoingToReturnValidAction(void);

	// Returns NULL if the child list is empty and there is no appropriate default action.
	// Whether or not the return value is NULL can be predicted by calling IsGoingToReturnValidAction
	// pParentPanel can be NULL if there isn't one.
	// If a panel jump is returned it will be absolute iff pParentPanel is NULL.
	const CActionSectionAdder *MakePanelsAndGetAction(CLNPanel *pParentPanel);

protected:
	virtual list<CLNPLines *> & getTargetPanel(void) = 0;
	virtual char getPanelNamePrefix(void) = 0;

	virtual const CActionSectionAdder *getAugmentedActionSectionAdder(const CActionSectionAdder *pPanelJumpASA)
	{
		// By default, no augmentation is done.
		return(pPanelJumpASA);
	}

	list<CLNItem *> m_children;

	int m_intCachedValidActionResult;
};

//*********************************************************************************************

class CLNItemLetters : public CLNItem
{
public:
	CLNItemLetters(CLayoutNudelist *pLayout, const CSNVolume *pVolume);
	virtual ~CLNItemLetters();

	// Predicts whether the return of MakePanelsAndGetAction will be NULL or not.
	bool IsGoingToReturnValidAction(void);

	// Returns NULL if the child list is empty and there is no appropriate default action.
	// Whether or not the return value is NULL can be predicted by calling IsGoingToReturnValidAction
	// pParentPanel can be NULL if there isn't one.
	// If a panel jump is returned it will be absolute iff pParentPanel is NULL.
	const CActionSectionAdder *MakePanelsAndGetAction(CLNPanel *pParentPanel);

protected:
	list<CLNItem *> m_children;

	int m_intCachedValidActionResult;
};

//*********************************************************************************************

class CLNItemArtists : public CLNItemWithChildren
{
public:
	CLNItemArtists(CLayoutNudelist *pLayout, const CSNVolume *pVolume, char letterEscaped);
	CLNItemArtists(CLayoutNudelist *pLayout, const CSNVolume *pVolume);

protected:
	list<CLNPLines *> & getTargetPanel(void)
	{
		return(getArtistPanels());
	}

	char getPanelNamePrefix(void)
	{
		return('A');
	}

protected:
	void addArtist(CLayoutNudelist *pLayout, const CSNArtist *pArtist);
};

//*********************************************************************************************

class CLNItemDiscs : public CLNItemWithChildren
{
public:
	CLNItemDiscs(CLayoutNudelist *pLayout, const CSNVolume *pVolume, char letterEscaped);
	CLNItemDiscs(CLayoutNudelist *pLayout, const CSNVolume *pVolume);
	CLNItemDiscs(CLayoutNudelist *pLayout, const CSNArtist *pArtist);

protected:
	list<CLNPLines *> & getTargetPanel(void)
	{
		return(getDiscPanels());
	}

	char getPanelNamePrefix(void)
	{
		return('D');
	}

protected:
	void addDisc(CLayoutNudelist *pLayout, const CSNDisc *pDisc);
};

//*********************************************************************************************

class CLNItemActionDisc : public CLNItem, public CActionSectionAdder
{
public:
	CLNItemActionDisc(CLayoutNudelist *pLayout, const CSNDisc *pDisc, bool bIncludeJump);
	virtual ~CLNItemActionDisc();

	// CLNItem
	const char *GetButtonLabelEscaped(void); // override of non-pure virtual
	bool IsGoingToReturnValidAction(void);
	const CActionSectionAdder *MakePanelsAndGetAction(CLNPanel *pParentPanel);

	// CActionSectionAdder
	void AddToActionSection(CCSFSection *pActionSection) const;

protected:
	bool m_bIncludeJump;
	const CSNDisc *m_pDisc;
	char *m_szButtonLabelCache;
};

//*********************************************************************************************

class CLNItemTracks : public CLNItemWithChildren
{
public:
	CLNItemTracks(CLayoutNudelist *pLayout, const CSNVolume *pVolume, char letterEscaped);
	CLNItemTracks(CLayoutNudelist *pLayout, const CSNVolume *pVolume);
	CLNItemTracks(CLayoutNudelist *pLayout, const CSNArtist *pArtist);
	CLNItemTracks(CLayoutNudelist *pLayout, const CSNDisc *pDisc);

	virtual ~CLNItemTracks();

	// CLNItem
	const char *GetButtonLabelEscaped(void); // override of non-pure virtual

protected:
	list<CLNPLines *> & getTargetPanel(void)
	{
		return(getTrackPanels());
	}

	char getPanelNamePrefix(void)
	{
		return('T');
	}

	const CActionSectionAdder *getAugmentedActionSectionAdder(const CActionSectionAdder *pPanelJumpASA);

protected:
	void addTrack(CLayoutNudelist *pLayout, const CSNTrack *pTrack);

	const CSNDisc *m_pDisc;
	char *m_szButtonLabelCache;
	CLayoutNudelist *m_pLayout;
	list<CActionSectionAdder *> m_actionAddersToDelete;
};

//*********************************************************************************************

class CLNItemActionTrack : public CLNItem, public CActionSectionAdder
{
public:
	CLNItemActionTrack(CLayoutNudelist *pLayout, const CSNTrack *pTrack, bool bIncludeDisc, bool bIncludeJump);
	virtual ~CLNItemActionTrack();

	// CLNItem
	const char *GetButtonLabelEscaped(void); // override of non-pure virtual
	bool IsGoingToReturnValidAction(void);
	const CActionSectionAdder *MakePanelsAndGetAction(CLNPanel *pParentPanel);

	// CActionSectionAdder
	void AddToActionSection(CCSFSection *pActionSection) const;

protected:
	bool m_bIncludeDisc;
	bool m_bIncludeJump;
	const CSNTrack *m_pTrack;
	char *m_szButtonLabelCache;
};

//*********************************************************************************************

#endif // !defined(AFX_LAYOUTNUDELIST_H__6BC352A3_DC13_4781_84E4_4DFFADA7BA86__INCLUDED_)
