// LayoutNudelistLine.h: interface for the CLayoutNudelistLine class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LAYOUTNUDELISTLINESGENERIC_H__96FCF4DA_6863_4A6A_93E1_E8CDB232768A__INCLUDED_)
#define AFX_LAYOUTNUDELISTLINESGENERIC_H__96FCF4DA_6863_4A6A_93E1_E8CDB232768A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "CSFSection.h"

class CProntoString;
class CActionSectionAdder;
class CSNArtist;
class CSNDisc;

/////////////////////////////////////////////////////////////////////////

class ILayoutNudelistLine
{
public:
	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection, unsigned int ypos,
										CCSFSection::FontType fontTypeDefault, bool bTruncate) const = 0;
};

/////////////////////////////////////////////////////////////////////////

class CLNLBlank : public ILayoutNudelistLine
{
public:
	CLNLBlank(void);
	virtual ~CLNLBlank(void);

	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection, unsigned int ypos,
										CCSFSection::FontType fontTypeDefault, bool bTruncate) const;
};

/////////////////////////////////////////////////////////////////////////

class CLNLTwoItems : public ILayoutNudelistLine
{
public:
	CLNLTwoItems(const char *szNameEscapedOne, const char *szButtonLabelEscapedOne, const CActionSectionAdder *pActionAdderOne,
				 const char *szNameEscapedTwo, const char *szButtonLabelEscapedTwo, const CActionSectionAdder *pActionAdderTwo);

	virtual ~CLNLTwoItems(void);

	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection, unsigned int ypos,
										CCSFSection::FontType fontTypeDefault, bool bTruncate) const;

protected:
	bool m_bOne;
	bool m_bTwo;
	string m_strNameEscapedOne;
	string m_strNameEscapedTwo;
	string m_strButtonLabelEscapedOne;
	string m_strButtonLabelEscapedTwo;
	const CActionSectionAdder *m_pActionAdderOne;
	const CActionSectionAdder *m_pActionAdderTwo;
};

/////////////////////////////////////////////////////////////////////////

class CLNLOneItemAndContinuation : public ILayoutNudelistLine
{
public:
	CLNLOneItemAndContinuation(const char *szNameEscapedOne, const char *szButtonLabelEscapedOne, const CActionSectionAdder *pActionAdderOne,
							   const CActionSectionAdder *pActionAdderContinuation);

	virtual ~CLNLOneItemAndContinuation(void);

	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection, unsigned int ypos,
										CCSFSection::FontType fontTypeDefault, bool bTruncate) const;

protected:
	bool m_bOne;
	string m_strNameEscapedOne;
	string m_strButtonLabelEscapedOne;
	const CActionSectionAdder *m_pActionAdderOne;

	const CActionSectionAdder *m_pActionAdderContinuation;

};

/////////////////////////////////////////////////////////////////////////

class CLNLHeading : public ILayoutNudelistLine
{
public:
	// pActionAdder may be NULL, in which case a frame is made rather than a button.
	// (This is not the general case for other classes in this file.)
	CLNLHeading(const char *szNameEscaped, const CActionSectionAdder *pActionAdder);
	virtual ~CLNLHeading(void);

	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection, unsigned int ypos,
										CCSFSection::FontType fontTypeDefault, bool bTruncate) const;

protected:
	const char *m_szNameEscaped;
	const CActionSectionAdder *m_pActionAdder;
};

/////////////////////////////////////////////////////////////////////////

class CLNLContinuedHeading : public ILayoutNudelistLine
{
public:
	CLNLContinuedHeading(const char *szNameEscaped, const CActionSectionAdder *pActionAdder);
	virtual ~CLNLContinuedHeading(void);

	virtual void AddToPanelCSFSection(CCSFSection *pPanelCSFSection, unsigned int ypos,
										CCSFSection::FontType fontTypeDefault, bool bTruncate) const;

protected:
	const char *m_szNameEscaped;
	const CActionSectionAdder *m_pActionAdder;
};

/////////////////////////////////////////////////////////////////////////

#endif // !defined(AFX_LAYOUTNUDELISTLINESGENERIC_H__96FCF4DA_6863_4A6A_93E1_E8CDB232768A__INCLUDED_)
