// LayoutNudelistPanels.cpp: implementation of the CLayoutNudelistPanels class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "LayoutNudelist.h"
#include "LayoutNudelistPanels.h"
#include "LayoutNudelistLines.h"
#include "CSFSection.h"
#include "SNVolume.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

CLNPanel::CLNPanel(const CSNVolume *pVolume, const char *deviceNameEscaped, int nDevNumber)
{
	m_bHide = pVolume->GetHidePanels();
	m_bTruncate = pVolume->GetTruncate();
	m_bLinks = pVolume->GetLinks();
	m_fontTypeDefault = pVolume->GetLargerFont() ? CCSFSection::FONT_10 : CCSFSection::FONT_8;

	if (0 >= nDevNumber)
	{
		m_nDeviceNumber = 0;
		m_csDeviceNameEscaped = deviceNameEscaped;
	}
	else
	{
		m_nDeviceNumber = nDevNumber;
		m_csDeviceNameEscaped.Format("%s.%d", deviceNameEscaped, nDevNumber);
	}

	m_pJumpActionAdderRelative = NULL;
	m_pJumpActionAdderAbsolute = NULL;

	m_pActionAdderLeft = NULL;
	m_pActionAdderRight = NULL;
}

CLNPanel::~CLNPanel(void)
{
	delete m_pJumpActionAdderRelative;
	m_pJumpActionAdderRelative = NULL;

	delete m_pJumpActionAdderAbsolute;
	m_pJumpActionAdderAbsolute = NULL;
}

void CLNPanel::AddToDeviceCSFSection(CCSFSection *pDeviceCSFSection) const
{
	CCSFSection *pOurNewPanel;

	pOurNewPanel = pDeviceCSFSection->AddSubSection();

	pOurNewPanel->SetTitleEscapedFormat("Panel(\"%s\")", LPCTSTR(m_csTitle));

	if (m_bHide)
	{
		pOurNewPanel->AddEscapedTitleOnlySubSection("IsHidden()");
	}

	pOurNewPanel->AddEscapedTitleOnlySubSection("Font(0)");
	pOurNewPanel->AddEscapedTitleOnlySubSection("BackColor(White);");
	pOurNewPanel->AddEscapedTitleOnlySubSection("TextColor(Black);");
	pOurNewPanel->AddEscapedTitleOnlySubSection("InactiveUnselectedColor(LightGray,DarkGray);");
	pOurNewPanel->AddEscapedTitleOnlySubSection("InactiveSelectedColor(DarkGray,Black);");
	pOurNewPanel->AddEscapedTitleOnlySubSection("ActiveUnselectedColor(LightGray,Black);");
	pOurNewPanel->AddEscapedTitleOnlySubSection("ActiveSelectedColor(LightGray,DarkGray);");

	addBodyToOurCSFSection(pOurNewPanel);

	unsigned int ypos = CLNPLines::MAX_LINES * 34;

	CCSFSection *pAction;

	if ((!m_bLinks) || (NULL == m_pActionAdderLeft))
	{
		pOurNewPanel->AddEscapedFrameSubSection(m_csLeftMsgEscaped,
													0, ypos, 120, 15,
													m_fontTypeDefault,
													CCSFSection::COLOUR_BLACK,
													"BMP\\FinalBarLeft.bmp");
	}
	else
	{
		pOurNewPanel->AddEscapedButtonSubSection(m_csLeftMsgEscaped,
													0, ypos, 120, 15,
													m_fontTypeDefault,
													CCSFSection::COLOUR_BLACK,
													"BMP\\FinalBarLeft.bmp",
													&pAction);

		m_pActionAdderLeft->AddToActionSection(pAction);
	}

	if ((!m_bLinks) || (NULL == m_pActionAdderRight))
	{
		pOurNewPanel->AddEscapedFrameSubSection(m_csRightMsgEscaped,
													120, ypos, 120, 15,
													m_fontTypeDefault,
													CCSFSection::COLOUR_BLACK,
													"BMP\\FinalBarRight.bmp");
	}
	else
	{
		pOurNewPanel->AddEscapedButtonSubSection(m_csRightMsgEscaped,
													120, ypos, 120, 15,
													m_fontTypeDefault,
													CCSFSection::COLOUR_BLACK,
													"BMP\\FinalBarRight.bmp",
													&pAction);

		m_pActionAdderRight->AddToActionSection(pAction);
	}
}

// Set bAbsolute to false to get a jump relative to the "this" device.
// NOTE: bAbsolute ignored currently
const CActionSectionAdder *CLNPanel::GetJumpActionAdder(bool bAbsolute) const
{
	const CActionSectionAdder *pASAResult = NULL;

//	if (bAbsolute)
	{
		if (NULL == m_pJumpActionAdderAbsolute)
		{
			m_pJumpActionAdderAbsolute = new CJumpActionAdder(m_csDeviceNameEscaped, m_csTitle);
		}

		pASAResult = m_pJumpActionAdderAbsolute;
	}
/*	else
	{
		if (NULL == m_pJumpActionAdderRelative)
		{
			m_pJumpActionAdderRelative = new CJumpActionAdder("this", m_csTitle);
		}

		pASAResult = m_pJumpActionAdderRelative;
	}
*/
	return(pASAResult);
}

const char *CLNPanel::GetTitle(void) const
{
	return(m_csTitle);
}

const char *CLNPanel::GetDeviceName(void) const
{
	return(m_csDeviceNameEscaped);
}

int CLNPanel::GetDeviceNumber(void) const
{
	return(m_nDeviceNumber);
}

void CLNPanel::SetLeftActionAndMsgEscaped(const CActionSectionAdder *pActionAdder, const char *msg, int nFollowUps)
{
	int nMaxPixelWidth = 118;

	if (0 != nFollowUps)
	{
		CString csFollowUp;
		csFollowUp.Format(" [%d]", nFollowUps);
		nMaxPixelWidth -= CProntoString::CalculateProntoStringPixelWidth(CCSFSection::FONT_10 == m_fontTypeDefault, csFollowUp);
	}

	CProntoString psLabel;

	if (NULL != msg)
	{
		psLabel.SetEscaped(msg);
		psLabel.TruncateToPixelWidth(CCSFSection::FONT_10 == m_fontTypeDefault, nMaxPixelWidth);
	}

	if (0 != nFollowUps)
	{
		m_csLeftMsgEscaped.Format("%s [%d]", psLabel.GetEscaped(true), nFollowUps);
	}
	else if (NULL != msg)
	{
		m_csLeftMsgEscaped = psLabel.GetEscaped(true);
	}
	else
	{
		m_csLeftMsgEscaped.Empty();
	}

	m_pActionAdderLeft = pActionAdder;
}

void CLNPanel::SetRightActionAndMsgEscaped(const CActionSectionAdder *pActionAdder, const char *msg, int nFollowUps)
{
	int nMaxPixelWidth = 118;

	if (0 != nFollowUps)
	{
		CString csFollowUp;
		csFollowUp.Format(" [%d]", nFollowUps);
		nMaxPixelWidth -= CProntoString::CalculateProntoStringPixelWidth(CCSFSection::FONT_10 == m_fontTypeDefault, csFollowUp);
	}

	CProntoString psLabel;

	if (NULL != msg)
	{
		psLabel.SetEscaped(msg);
		psLabel.TruncateToPixelWidth(CCSFSection::FONT_10 == m_fontTypeDefault, nMaxPixelWidth);
	}

	if (0 != nFollowUps)
	{
		m_csRightMsgEscaped.Format("%s [%d]", psLabel.GetEscaped(true), nFollowUps);
	}
	else if (NULL != msg)
	{
		m_csRightMsgEscaped = psLabel.GetEscaped(true);
	}
	else
	{
		m_csRightMsgEscaped.Empty();
	}

	m_pActionAdderRight = pActionAdder;
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

CLNPLines::CLNPLines(const CSNVolume *pVolume, const char *deviceNameEscaped, int nDevNumber, char prefixChar, unsigned int panelNumber)
	: CLNPanel(pVolume, deviceNameEscaped, nDevNumber)
{
	m_csTitle.Format("%c%u", prefixChar, panelNumber);

	m_panelNumber = panelNumber;

	m_numFreeLines = MAX_LINES;
}

CLNPLines::~CLNPLines(void)
{
	while (! m_panelLines.empty() )
	{
		delete m_panelLines.back();
		m_panelLines.pop_back();
	}
}

unsigned int CLNPLines::GetPanelNumber(void) const
{
	return(m_panelNumber);
}

unsigned int CLNPLines::FreeLines(void) const
{
	return(m_numFreeLines);
}

bool CLNPLines::IsEmpty(void) const
{
	return(CLNPLines::MAX_LINES == m_numFreeLines);
}

void CLNPLines::AddPanelLine(ILayoutNudelistLine *pPanelLine)
{
	if (0 >= m_numFreeLines)
	{
		// printf("WARNING: Internal error. Line added to full panel ignored.\n");
		delete pPanelLine;
	}
	else
	{
		m_panelLines.push_back(pPanelLine);
		m_numFreeLines--;
	}
}

void CLNPLines::SetLastShortHeadingEscaped(const char *szShortHeadingEscaped)
{
	m_strLastShortHeadingEscaped = szShortHeadingEscaped != NULL ? szShortHeadingEscaped : "";
}

const char *CLNPLines::GetLastShortHeadingEscaped(void) const
{
	return(m_strLastShortHeadingEscaped.c_str());
}

void CLNPLines::addBodyToOurCSFSection(CCSFSection *pOurCSFSection) const
{
	unsigned int ypos = 0;

	list<ILayoutNudelistLine *>::const_iterator iter;

	for (iter = m_panelLines.begin(); m_panelLines.end() != iter; iter++)
	{
		(*iter)->AddToPanelCSFSection(pOurCSFSection, ypos, m_fontTypeDefault, m_bTruncate);

		ypos += 34;
	}
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

CLNAlphabetPanel::CLNAlphabetPanel(const CSNVolume *pVolume, const char *deviceNameEscaped, int nDevNumber)
	: CLNPanel(pVolume, deviceNameEscaped, nDevNumber)
{
	m_bDoArtists = true;

	m_csTitle = "Alphabet";

	for (int i = 0; i < LETTER_ARRAY_SIZE; i++)
	{
		m_letterArray[i] = NULL;
	}
}

CLNAlphabetPanel::~CLNAlphabetPanel(void)
{
	for (int i = 0; i < LETTER_ARRAY_SIZE; i++)
	{
		delete m_letterArray[i];
		m_letterArray[i] = NULL;
	}
}

void CLNAlphabetPanel::NotePanelAction(char letterToFile, const CActionSectionAdder *pActionAdder)
{
	// If there's no alphanumeric in the name, ignore it.
	// If the action is NULL, ignore it.
	if (('\0' != letterToFile) && (NULL != pActionAdder))
	{
		// This relies on the machine using a charset with sequential letters.

		int letterArrayIndex = LETTER_ARRAY_SIZE;

		if (isupper(letterToFile))
		{
			letterArrayIndex = (letterToFile - 'A') + LETTER_ARRAY_ALPHA_OFFSET;
		}
		else if ('#' == letterToFile)
		{
			letterArrayIndex = 0;
		}

		if (LETTER_ARRAY_SIZE > letterArrayIndex)
		{
			// If the letter has already been filed, ignore it.
			if (NULL == m_letterArray[letterArrayIndex])
			{
				m_letterArray[letterArrayIndex] = new pair<char, const CActionSectionAdder *>(letterToFile, pActionAdder);
			}
		}
	}
}

void CLNAlphabetPanel::addBodyToOurCSFSection(CCSFSection *pOurCSFSection) const
{
	// Add alphabet buttons in fixed places.
	// (That way the user need not re-learn their positions if they add
	// an artist with a new letter.)

	unsigned int xpos;
	unsigned int ypos;
	char letterString[2];
	letterString[1] = '\0';

	CCSFSection *pAction;

	for (int i = 0; i < LETTER_ARRAY_SIZE; i++)
	{
		// x-pos
		if (i < 6)
		{
			xpos = 40 * i;
		}
		else if (i > 20)
		{
			xpos = 40 * (i - 21);
		}
		else
		{
			xpos = 50 * ((i - 6) % 5);
		}

		// y-pos
		if (i < 6)
		{
			ypos = 0;
		}
		else if (i < 11)
		{
			ypos = 40;
		}
		else if (i < 16)
		{
			ypos = 80;
		}
		else if (i < 21)
		{
			ypos = 120;
		}
		else
		{
			ypos = 160;
		}

		if ((NULL != m_letterArray[i]) && (NULL != m_letterArray[i]->second))
		{

			letterString[0] = m_letterArray[i]->first;

			pOurCSFSection->AddEscapedButtonSubSection(letterString,
														xpos, ypos, 38, 35,
														CCSFSection::FONT_10,
														CCSFSection::COLOUR_WHITE,
														"BMP\\SmallRoundBlackButton.bmp",
														&pAction);

			m_letterArray[i]->second->AddToActionSection(pAction);
		}
		else
		{
			// Blank button to stop things looking silly.
			pOurCSFSection->AddEscapedFrameSubSection(".",
														xpos, ypos, 38, 35,
														CCSFSection::FONT_10,
														CCSFSection::COLOUR_DARKGRAY,
														"BMP\\SmallRoundBlackButton.bmp");
		}
	}
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

CLNCreditsPanel::CLNCreditsPanel(const CSNVolume *pVolume, const char *deviceNameEscaped, int nDevNumber, const CLNPanel *pBackPanel)
	: CLNPanel(pVolume, deviceNameEscaped, nDevNumber)
{
	m_csTitle = "Credits";
}

CLNCreditsPanel::~CLNCreditsPanel(void)
{
}

void CLNCreditsPanel::addBodyToOurCSFSection(CCSFSection *pOurCSFSection) const
{
	// Nudel logo

	pOurCSFSection->AddEscapedFrameSubSection(NULL,
												37, 5, 167, 71,
												CCSFSection::FONT_0,
												CCSFSection::COLOUR_BLACK,
												"BMP\\NudelPronto.bmp");

	// Text

	pOurCSFSection->AddEscapedFrameSubSection("Super-Nudelist",
												30, 76, 181, 31,
												CCSFSection::FONT_18,
												CCSFSection::COLOUR_BLACK,
												NULL);

	pOurCSFSection->AddEscapedFrameSubSection("by Leo 'Nudel' Davidson",
												40, 108, 161, 21,
												CCSFSection::FONT_10,
												CCSFSection::COLOUR_BLACK,
												NULL);

	pOurCSFSection->AddEscapedFrameSubSection("leo@ox.compsoc.net",
												55, 130, 131, 16,
												CCSFSection::FONT_8,
												CCSFSection::COLOUR_BLACK,
												NULL);

	CProntoString psURL;

	psURL.SetPronto("www.ox.compsoc.net/~leo/nudelist");

	pOurCSFSection->AddEscapedFrameSubSection(psURL.GetEscaped(true),
												15, 148, 211, 18,
												CCSFSection::FONT_8,
												CCSFSection::COLOUR_BLACK,
												NULL);

	psURL.SetPronto("www.nudel.demon.co.uk/nudelist");

	pOurCSFSection->AddEscapedFrameSubSection(psURL.GetEscaped(true),
												15, 166, 211, 16,
												CCSFSection::FONT_8,
												CCSFSection::COLOUR_BLACK,
												NULL);

	pOurCSFSection->AddEscapedFrameSubSection("C++ source code available",
												45, 181, 151, 19,
												CCSFSection::FONT_8,
												CCSFSection::COLOUR_BLACK,
												NULL);
}

/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////

