// LayoutNudelistPanels.h: interface for the CLayoutNudelistPanels class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LAYOUTNUDELISTPANELS_H__FB92B63F_1047_4D62_93AB_B2292C5827DB__INCLUDED_)
#define AFX_LAYOUTNUDELISTPANELS_H__FB92B63F_1047_4D62_93AB_B2292C5827DB__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "CSFSection.h"

class ILayoutNudelistLine;
class CActionSectionAdder;

/////////////////////////////////////////////////////////////////////////

class CLNPanel
{
public:
	CLNPanel(const CSNVolume *pVolume, const char *deviceNameEscaped, int nDevNumber);
	virtual ~CLNPanel(void);

	void AddToDeviceCSFSection(CCSFSection *pDeviceCSFSection) const;

	// Set bAbsolute to false to get a jump relative to the "this" device.
	// NOTE: bAbsolute ignored currently
	const CActionSectionAdder *GetJumpActionAdder(bool bAbsolute) const;

	const char *GetTitle(void) const;
	const char *GetDeviceName(void) const;
	int GetDeviceNumber(void) const;

	void SetLeftActionAndMsgEscaped(const CActionSectionAdder *pActionAdder, const char *msg, int nFollowUps);
	void SetRightActionAndMsgEscaped(const CActionSectionAdder *pActionAdder, const char *msg, int nFollowUps);

protected:
	virtual void addBodyToOurCSFSection(CCSFSection *pOurCSFSection) const = 0;

protected:

	bool m_bHide;
	bool m_bTruncate;
	bool m_bLinks;
	CCSFSection::FontType m_fontTypeDefault;

	CString m_csTitle;
	CString m_csDeviceNameEscaped;
	int m_nDeviceNumber;

	mutable CActionSectionAdder *m_pJumpActionAdderRelative;
	mutable CActionSectionAdder *m_pJumpActionAdderAbsolute;

	CString m_csLeftMsgEscaped;
	CString m_csRightMsgEscaped;
	const CActionSectionAdder *m_pActionAdderLeft;
	const CActionSectionAdder *m_pActionAdderRight;
};

/////////////////////////////////////////////////////////////////////////

class CLNPLines : public CLNPanel
{
public:
	CLNPLines(const CSNVolume *pVolume, const char *deviceNameEscaped, int nDevNumber, char prefixChar, unsigned int panelNumber);
	virtual ~CLNPLines(void);

	unsigned int GetPanelNumber(void) const;

	unsigned int FreeLines(void) const;
	bool IsEmpty(void) const;

	void AddPanelLine(ILayoutNudelistLine *pPanelLine);

	void SetLastShortHeadingEscaped(const char *szShortNameEscaped);
	const char *GetLastShortHeadingEscaped(void) const;

	enum { MAX_LINES = 6 };

protected:
	virtual void addBodyToOurCSFSection(CCSFSection *pOurCSFSection) const;

	unsigned int m_panelNumber;

	int m_numFreeLines;

	list<ILayoutNudelistLine *> m_panelLines;

	string m_strLastShortHeadingEscaped;
};

/////////////////////////////////////////////////////////////////////////

class CLNAlphabetPanel : public CLNPanel
{
public:
	CLNAlphabetPanel(const CSNVolume *pVolume, const char *deviceNameEscaped, int nDevNumber);
	virtual ~CLNAlphabetPanel(void);

	void NotePanelAction(char letterToFile, const CActionSectionAdder *pActionAdder);

protected:
	virtual void addBodyToOurCSFSection(CCSFSection *pOurCSFSection) const;

	enum { LETTER_ARRAY_SIZE = 27 };
	enum { LETTER_ARRAY_ALPHA_OFFSET = 1 };

	pair<char, const CActionSectionAdder *> *m_letterArray[LETTER_ARRAY_SIZE];

	bool m_bDoArtists;
};

/////////////////////////////////////////////////////////////////////////

class CLNCreditsPanel : public CLNPanel
{
public:
	CLNCreditsPanel(const CSNVolume *pVolume, const char *deviceNameEscaped, int nDevNumber, const CLNPanel *pBackPanel);
	virtual ~CLNCreditsPanel(void);

protected:
	virtual void addBodyToOurCSFSection(CCSFSection *pOurCSFSection) const;
};

/////////////////////////////////////////////////////////////////////////

#endif // !defined(AFX_LAYOUTNUDELISTPANELS_H__FB92B63F_1047_4D62_93AB_B2292C5827DB__INCLUDED_)
