// LegacyNudelistFileFormat.cpp: implementation of the CLegacyNudelistFileFormat class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "LegacyNudelistFileFormat.h"
#include "SNLibrary.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/////////////////////////////////////////////////////////////////////////

CLegacyNudelistLine::CLegacyNudelistLine(void)
{
	m_lineBuffer = NULL;
	m_artistBuffer = NULL;
	m_artistShortBuffer = NULL;
	m_titleBuffer = NULL;

	m_number = 0;
	m_artist = NULL;
	m_title = NULL;
}

CLegacyNudelistLine::~CLegacyNudelistLine(void)
{
	SetLineBuffer(NULL);
	setArtistBuffer(NULL);
	setArtistShortBuffer(NULL);
	setTitleBuffer(NULL);
}

void CLegacyNudelistLine::SetLineBuffer(const char *buffer)
{
	if (NULL != m_lineBuffer)
	{
		delete [] m_lineBuffer;
		m_lineBuffer = NULL;
	}

	if (NULL != buffer)
	{
		m_lineBuffer = new char[strlen(buffer) + 1];
		strcpy(m_lineBuffer, buffer);
	}
}

void CLegacyNudelistLine::setArtistBuffer(const char *buffer)
{
	if (NULL != m_artistBuffer)
	{
		delete [] m_artistBuffer;
		m_artistBuffer = NULL;
	}

	if (NULL != buffer)
	{
		m_artistBuffer = new char[strlen(buffer) + 1];
		strcpy(m_artistBuffer, buffer);
	}
}

void CLegacyNudelistLine::setArtistShortBuffer(const char *buffer)
{
	if (NULL != m_artistShortBuffer)
	{
		delete [] m_artistShortBuffer;
		m_artistShortBuffer = NULL;
	}

	if (NULL != buffer)
	{
		m_artistShortBuffer = new char[strlen(buffer) + 1];
		strcpy(m_artistShortBuffer, buffer);
	}
}

void CLegacyNudelistLine::setTitleBuffer(const char *buffer)
{
	if (NULL != m_titleBuffer)
	{
		delete [] m_titleBuffer;
		m_titleBuffer = NULL;
	}

	if (NULL != buffer)
	{
		m_titleBuffer = new char[strlen(buffer) + 1];
		strcpy(m_titleBuffer, buffer);
	}
}

bool CLegacyNudelistLine::ParseBuffer(CTypedError *pTError, unsigned int previousNumber, const char *previousArtist)
{
	bool bResult = true;

	_ASSERTE(NULL != m_lineBuffer);

	const char *bufPtr = m_lineBuffer;
	const char *delimiters = "~";
	size_t tokenBufferSize = strlen(m_lineBuffer) + 1;
	char *tokenBuffer = new char[tokenBufferSize];

	// Disc Number
	if (bResult)
	{
		if (!Tokenize(&bufPtr, delimiters, NULL, tokenBuffer, tokenBufferSize, false, false, false))
		{
			CTypedError::SafeSetMsgError(pTError, "Invalid line in input file (no disc number column):\n%s", m_lineBuffer);
			bResult = false;
		}
		else if (0 == strlen(tokenBuffer))
		{
			// Empty number column, use previousNumber+1;
			m_number = previousNumber + 1;
		}
		else
		{
			m_number = atoi(tokenBuffer);
		}
	}

	// Artist
	if (bResult)
	{
		if (!Tokenize(&bufPtr, delimiters, NULL, tokenBuffer, tokenBufferSize, false, false, false))
		{
			CTypedError::SafeSetMsgError(pTError, "Invalid line in input file (no artist column):\n%s", m_lineBuffer);
			bResult = false;
		}
		else if (0 == strlen(tokenBuffer))
		{
			// Empty artist column, use previousArtist.
			m_artist = previousArtist;
		}
		else
		{
			setArtistBuffer(tokenBuffer);
			m_artist = m_artistBuffer;
		}
	}

	// Short Artist Name
	if (bResult)
	{
		if (!Tokenize(&bufPtr, delimiters, NULL, tokenBuffer, tokenBufferSize, false, false, false))
		{
			CTypedError::SafeSetMsgError(pTError, "Invalid line in input file (no short artist column):\n%s", m_lineBuffer);
			bResult = false;
		}
		else if (0 != strlen(tokenBuffer))
		{
			setArtistShortBuffer(tokenBuffer);
		}
	}

	// Title
	if (bResult)
	{
		if (!Tokenize(&bufPtr, delimiters, NULL, tokenBuffer, tokenBufferSize, false, false, false))
		{
			CTypedError::SafeSetMsgError(pTError, "Invalid line in input file (no title column):\n%s", m_lineBuffer);
			bResult = false;
		}
		else if (0 == strlen(tokenBuffer))
		{
			CTypedError::SafeSetMsgError(pTError, "Invalid line in input file (empty title column):\n%s", m_lineBuffer);
			bResult = false;
		}
		else
		{
			setTitleBuffer(tokenBuffer);
			m_title = m_titleBuffer;
		}
	}

	delete [] tokenBuffer;

	// Free line buffer.
	if (bResult)
	{
		SetLineBuffer(NULL);
	}

	return(bResult);
}

int CLegacyNudelistLine::GetNumber(void)
{
	return(m_number);
}

const char *CLegacyNudelistLine::GetArtist(void)
{
	return(m_artist);
}

const char *CLegacyNudelistLine::GetArtistShort(void)
{
	return(m_artistShortBuffer);
}

const char *CLegacyNudelistLine::GetTitle(void)
{
	return(m_title);
}

/////////////////////////////////////////////////////////////////////////

CLegacyNudelistFileFormat::CLegacyNudelistFileFormat(void)
{
}

CLegacyNudelistFileFormat::~CLegacyNudelistFileFormat(void)
{
	while (!m_lines.empty())
	{
		delete m_lines.back();
		m_lines.pop_back();
	}
}

bool CLegacyNudelistFileFormat::Read(CTypedError *pTError, FILE *pfInputList)
{
	bool bResult = true;

	// At some point, replace this fixed-size buffer and fgets() with something more intelligent.
	char lineBuffer[BUFFERSIZE];

	char *trimmedLine;
	CLegacyNudelistLine *pLine;

	while (NULL != fgets(lineBuffer, sizeof(lineBuffer), pfInputList))
	{
		trimmedLine = TrimWhitespace(lineBuffer);

		// Ignore empty lines and those starting with "//"
		if ( (0 < strlen(trimmedLine)) && (('/' != trimmedLine[0]) || ('/' != trimmedLine[1])) )
		{
			pLine = new CLegacyNudelistLine();
			pLine->SetLineBuffer(trimmedLine);

			m_lines.push_back(pLine);
		}
	}

	if (bResult && ferror(pfInputList))
	{
		CTypedError::SafeSetMsgError(pTError, "Cannot read input file");
		bResult = false;
	}

	return(bResult);
}

bool CLegacyNudelistFileFormat::Parse(CTypedError *pTError)
{
	bool bResult = true;

	unsigned int previousNumber = 0;				// CD changers seem to start at 1.
	const char *previousArtist = "Unknown Artist";	// A good default that also saves error checks.

	list<CLegacyNudelistLine *>::iterator iter;

	for(iter = m_lines.begin(); m_lines.end() != iter; iter++)
	{
		if ((*iter)->ParseBuffer(pTError, previousNumber, previousArtist))
		{
			previousNumber = (*iter)->GetNumber();
			previousArtist = (*iter)->GetArtist();
		}
		else
		{
			bResult = false;
			break;
		}
	}

	return(bResult);
}

bool CLegacyNudelistFileFormat::MakeVolume(CTypedError *pTError, CSNLibrary *pLibrary,
											ISNUITreeDlg *pTreeDlg)
{
	bool bResult = false;

	CString csVolumeName = "CD List";

	for (int i = 1; i < 101; i++)
	{
		if (NULL == pLibrary->GetByNamePronto(csVolumeName, NULL))
		{
			break;
		}

		csVolumeName.Format("CD List %d", i);
	}

	if (101 <= i)
	{
		CTypedError::SafeSetMsgError(pTError, "Could not import because there are already 100 lists named \"CD List\"");
	}
	else
	{
		CSNVolume *pVol = NULL;

		{
			CSNVolume *pVolume = new CSNVolume();

			pVolume->GetName()->SetPronto(csVolumeName);

			pVol = pLibrary->GetNonConst(pLibrary->SNUIT_AddBelowAndOwnVolume(pTreeDlg, pTError, pVolume, NULL));

			if (NULL == pVol)
			{
				delete pVolume;
			}
		}

		if (NULL != pVol)
		{
			list<CLegacyNudelistLine *>::const_iterator iter;

			bResult = true;

			for(iter = m_lines.begin(); iter != m_lines.end(); iter++)
			{
				CSNArtist *pArt = pVol->GetNonConst(pVol->GetByNamePronto((*iter)->GetArtist(), NULL));

				if (NULL == pArt)
				{
					CSNArtist *pArtist = new CSNArtist();

					pArtist->GetName()->SetPronto((*iter)->GetArtist());

					if (0 == strnicmp("The ", (*iter)->GetArtist(), 4))
					{
						char *sortName = new char[strlen((*iter)->GetArtist()) - 3];
						strcpy(sortName, (*iter)->GetArtist() + 4);

						char *trimmedSortName = TrimWhitespace(sortName);

						if ((0 != strcmp("", trimmedSortName))	\
						&&	(0 != stricmp("The", trimmedSortName)))
						{
							pArtist->GetSortAsName()->SetPronto(trimmedSortName);
						}

						delete [] sortName;
					}

					pArt = pVol->GetNonConst(pVol->SNUIT_AddBelowAndOwnArtist(pTreeDlg, pTError, pArtist, NULL));

					if (NULL == pArt)
					{
						delete pArtist;
					}
				}

				if (NULL == pArt)
				{
					bResult = false;
					break;
				}

				if (NULL != (*iter)->GetArtistShort())
				{
					pArt->GetShortName()->SetPronto((*iter)->GetArtistShort());

					pVol->SNUIT_UpdateArtist(pTreeDlg, pTError, pArt, pArt);
				}

				CSNDisc *pDisc = new CSNDisc();

				pDisc->GetName()->SetPronto((*iter)->GetTitle());

				if (0 == strnicmp("The ", (*iter)->GetTitle(), 4))
				{
					char *sortName = new char[strlen((*iter)->GetTitle()) - 3];
					strcpy(sortName, (*iter)->GetTitle() + 4);

					char *trimmedSortName = TrimWhitespace(sortName);

					if ((0 != strcmp("", trimmedSortName))	\
					&&	(0 != stricmp("The", trimmedSortName)))
					{
						pDisc->GetSortAsName()->SetPronto(trimmedSortName);
					}

					delete [] sortName;
				}

				pDisc->SetDiscNumber((*iter)->GetNumber());

				if (NULL == pArt->SNUIT_AddBelowAndOwnDisc(pTreeDlg, pTError, pDisc, NULL))
				{
					delete pDisc;
					bResult = false;
					break;
				}
			}
		}
	}

	return(bResult);
}

/*
bool CLegacyNudelistFileFormat::MakeArtistList(CArtistList *pArtistList)
{
	bool bResult = true;

	list<CLegacyNudelistLine *>::const_iterator iter;

	CArtist *pArtist;

	for(iter = m_lines.begin(); iter != m_lines.end(); iter++)
	{
		pArtist = pArtistList->FindElseAdd((*iter)->GetArtist());

		_ASSERTE(NULL != pArtist);

		if (NULL != (*iter)->GetArtistShort())
		{
			pArtist->SetShortName((*iter)->GetArtistShort());
		}

		pArtist->GetDiscList()->AddTail((*iter)->GetNumber(),
										(*iter)->GetTitle());
	}

	pArtistList->Sort();

	return(bResult);
}
*/
/////////////////////////////////////////////////////////////////////////
