// ListingDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ListingDlg.h"
#include "SNConfig.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CListingDlg dialog


CListingDlg::CListingDlg(const char *szTitle, const char *szMessage, const CStringList *pStringList, bool bProntoFont, CWnd* pParent)
	: CSuckLessDialog(CListingDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CListingDlg)
	m_staticMessage = _T("");
	//}}AFX_DATA_INIT

	m_title = szTitle;
	m_staticMessage = szMessage;
	m_pStringList = pStringList;
	m_bProntoFont = bProntoFont;
}


void CListingDlg::DoDataExchange(CDataExchange* pDX)
{
	CSuckLessDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CListingDlg)
	DDX_Control(pDX, IDC_LIST_MAIN, m_listOutputControl);
	DDX_Text(pDX, IDC_STATIC_MESSAGE, m_staticMessage);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CListingDlg, CSuckLessDialog)
	//{{AFX_MSG_MAP(CListingDlg)
	ON_BN_CLICKED(IDC_BUTTON_COPY, OnButtonCopy)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CListingDlg message handlers

BOOL CListingDlg::OnInitDialog() 
{
	CSuckLessDialog::OnInitDialog();
	
	this->setWindowIcon(IDR_MAINFRAME);

	this->SetWindowText(m_title);
	
	if (NULL != g_pProntoFont)
	{
		m_listOutputControl.SetFont(g_pProntoFont);
	}

	// -- Accellerators --

	m_hAccel = ::LoadAccelerators(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDR_ACCELERATOR_LISTING));

	// -- Initial contents --

	m_listOutputControl.InsertColumn(0, "Output");

	// Optimise item addition by specifying number in advance.
	m_listOutputControl.SetItemCount(m_pStringList->GetCount());

	int listColumnWidth = 0;
	int listWidthOffsetForBraindeadFunctionPieceOfShitMicrosoftBastards = m_listOutputControl.GetStringWidth("W");
	int listItem = 0;
	int sw;
	int idx;

	POSITION pos = m_pStringList->GetHeadPosition();

	while (NULL != pos)
	{
		CString cstr = m_pStringList->GetNext(pos);

		sw = m_listOutputControl.GetStringWidth(LPCTSTR(cstr)) + listWidthOffsetForBraindeadFunctionPieceOfShitMicrosoftBastards;

		if (sw > listColumnWidth)
		{
			m_listOutputControl.SetColumnWidth(0, sw);
			listColumnWidth = sw;
		}

		idx = m_listOutputControl.InsertItem(listItem++, LPCTSTR(cstr));
	}

	if ((-1) != idx)
	{
		m_listOutputControl.EnsureVisible(idx, FALSE);
	}

	// -- Control setup --

	addControlInfo(IDC_STATIC_MESSAGE);
	addControlInfo(IDC_LIST_MAIN);
	addControlInfo(IDC_BUTTON_COPY);
	addControlInfo(IDOK);

	setControlRelationship(IDC_STATIC_MESSAGE,	IDD, CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_LIST_MAIN,		IDD, CControlInfo::ACOMMON_RESIZEHEIGHT_RESIZEWIDTH);
	setControlRelationship(IDC_BUTTON_COPY,		IDD, CControlInfo::ACOMMON_BOTTOM_LEFT);
	setControlRelationship(IDOK,				IDD, CControlInfo::ACOMMON_BOTTOM_RIGHT);

	controlAndFrameInfoIsSetup();

	setMinXToCurrent();
	setMinYToCurrent();

	this->setRememberWindowPosition(SN_REG_PATH_WINDOWPOS, SN_REG_VALUE_WINDOWPOS_LISTING);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CListingDlg::OnButtonCopy() 
{
	this->setStatusBarText("");

	int nItem;
	int nGNIFlags;
	const char *copyMsg;

	UINT i;
	UINT uItemCount = m_listOutputControl.GetSelectedCount();

	if (0 == uItemCount)
	{
		// Annoyingly, GetItemCount returns an int rather than a UINT. God dammit.
		uItemCount = (UINT)(m_listOutputControl.GetItemCount());
		nGNIFlags = LVNI_ALL;
		copyMsg = "Copied entire log to clipboard.";
	}
	else
	{
		nGNIFlags = LVNI_SELECTED;
		copyMsg = "Copied selection to clipboard.";
	}

	// Work out required size.
	DWORD dwAllocSize = 1; // 1 for NULL.
	nItem = -1;

	for (i = 0; i < uItemCount; i++)
	{
		nItem = m_listOutputControl.GetNextItem(nItem, nGNIFlags);

		if ((-1) != nItem)
		{
			dwAllocSize += (m_listOutputControl.GetItemText(nItem, 0).GetLength());

			dwAllocSize += 2; // CR/LF
		}
	}

	bool bFreeGlobalClipMem = true;

	CTypedError tError;

	HGLOBAL hGlobalClipMem = GlobalAlloc(GMEM_MOVEABLE, dwAllocSize);

	if (NULL == hGlobalClipMem)
	{
		CTypedError::SafeSetSystemError(&tError, "GlobalAlloc failed");
	}
	else
	{
		char *szGlobalClipBuffer = (char *)GlobalLock(hGlobalClipMem);

		if (NULL == szGlobalClipBuffer)
		{
			CTypedError::SafeSetSystemError(&tError, "GlobalLock failed");
		}
		else
		{
			// Copy strings.

			CString cstr;
			nItem = -1;

			for (i = 0; i < uItemCount; i++)
			{
				nItem = m_listOutputControl.GetNextItem(nItem, nGNIFlags);

				if ((-1) != nItem)
				{
					cstr = m_listOutputControl.GetItemText(nItem, 0);

					strcpy(szGlobalClipBuffer, LPCTSTR(cstr));
					szGlobalClipBuffer += cstr.GetLength();

					strcpy(szGlobalClipBuffer, "\r\n");
					szGlobalClipBuffer += 2;
				}
			}

			// Unlock.
			GlobalUnlock(hGlobalClipMem);

			// CWnd::OpenClipboard is not documented as setting the last error.
			if (!(::OpenClipboard(this->GetSafeHwnd())))
			{
				CTypedError::SafeSetSystemError(&tError, "OpenClipboard failed");
			}
			else
			{
				if (!(::EmptyClipboard()))
				{
					CTypedError::SafeSetSystemError(&tError, "EmptyClipboard failed");
				}
				else
				{
					if (NULL == (::SetClipboardData(CF_TEXT, hGlobalClipMem)))
					{
						CTypedError::SafeSetSystemError(&tError, "SetClipboardData failed");
					}
					else
					{
						bFreeGlobalClipMem = false;
					}
				}

				if (!(::CloseClipboard()))
				{
					CTypedError::SafeSetSystemError(&tError, "CloseClipboard failed");
				}
			}
		}

		if (bFreeGlobalClipMem)
		{
			GlobalFree(hGlobalClipMem);
			hGlobalClipMem = NULL;
		}
	}

	if (CTypedError::SafeIsError(&tError))
	{
		MessageBox(CTypedError::SafeGetFullErrorTextTempBuffer(&tError), SUPERNUDELIST_ERROR_TITLE, MB_ICONEXCLAMATION);
	}
	else
	{
		this->setStatusBarText(copyMsg);
	}
}

BOOL CListingDlg::PreTranslateMessage(MSG* pMsg) 
{
	if ((WM_KEYFIRST <= pMsg->message) && (WM_KEYLAST >= pMsg->message))
	{
		// Translate using accelerator table.
		if (NULL != m_hAccel)
		{
			return(::TranslateAccelerator(this->GetSafeHwnd(), m_hAccel, pMsg));
		}
	}
	
	return CSuckLessDialog::PreTranslateMessage(pMsg);
}
