// ProgressDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ProgressDlg.h"
#include "SNConfig.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CProgressDlg dialog

// pSLDThread is deleted when the progress dialog is deleted.
CProgressDlg::CProgressDlg(int reqFrameIdent, bool bImmediatePopup,
						   CProgressDlgThread *pProgThread, CWnd* pParent)
	: CSuckLessDialog(CProgressDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CProgressDlg)
	//}}AFX_DATA_INIT

	m_progressTitle = NULL;
	m_progressHeading = NULL;
	m_progressMessage = NULL;
	m_progressValue = -1;
	m_progressRange = -1;
	m_bDoImmediatePopup = false;

	if ((PROGD_FRAME_FULL	== reqFrameIdent)\
	||  (PROGD_FRAME_NOPROG	== reqFrameIdent))
	{
		m_reqFrameIdent = reqFrameIdent;
	}
	else
	{
		ASSERT(FALSE);
		m_reqFrameIdent = PROGD_FRAME_FULL;
	}

	m_bSuccess = false;
	m_bImmediatePopup = bImmediatePopup;
	m_pProgThread = pProgThread;
	m_pProgThread->SetProgressDlg(this);
	m_pThreadIdent = m_pProgThread->GetIdent();

	m_bHaveOpenTimer = false;
	m_nOpenTimer = 0;

	InitializeCriticalSection(&m_CSProgress);
}

CProgressDlg::~CProgressDlg(void)
{
	delete [] m_progressTitle;
	m_progressTitle = NULL;

	delete [] m_progressHeading;
	m_progressHeading = NULL;

	delete [] m_progressMessage;
	m_progressMessage = NULL;

	DeleteCriticalSection(&m_CSProgress);
}

void CProgressDlg::DoDataExchange(CDataExchange* pDX)
{
	CSuckLessDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CProgressDlg)
	DDX_Control(pDX, IDC_PROGRESS, m_ctrlProgress);
	//}}AFX_DATA_MAP
}

static UINT NEAR WM_PROGRESSDLG_PROGRESS	= RegisterWindowMessage("SUCKLESSDIALOGPROGRESS_PROGRESS");

BEGIN_MESSAGE_MAP(CProgressDlg, CSuckLessDialog)
	//{{AFX_MSG_MAP(CProgressDlg)
	ON_REGISTERED_MESSAGE(WM_PROGRESSDLG_PROGRESS, OnProgressDlgProgress)
	ON_WM_TIMER()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CProgressDlg message handlers

BOOL CProgressDlg::OnInitDialog() 
{
	CSuckLessDialog::OnInitDialog();

	CWnd *pProgressHeading = this->GetDlgItem(IDC_STATIC_HEADING);
	CWnd *pProgressMessage = this->GetDlgItem(IDC_STATIC_BODY);

	if (NULL != pProgressHeading)
	{
		pProgressHeading->SetWindowText("");
	}

	if (NULL != pProgressMessage)
	{
		pProgressMessage->SetWindowText("");
	}

	this->setWindowIcon(IDR_MAINFRAME);

	setCascadeClosure(false);

	addControlInfo(IDC_STATIC_FRAMEFULL,	CControlInfo::F1_IS_FRAME, PROGD_FRAME_FULL);
	addControlInfo(IDC_STATIC_FRAMENOPROG,	CControlInfo::F1_IS_FRAME, PROGD_FRAME_NOPROG);

	addControlInfo(IDC_STATIC_PICTURE,	0, PROGD_FRAME_FULL | PROGD_FRAME_NOPROG);
	addControlInfo(IDC_STATIC_HEADING,	0, PROGD_FRAME_FULL | PROGD_FRAME_NOPROG);
	addControlInfo(IDC_STATIC_BODY,		0, PROGD_FRAME_FULL | PROGD_FRAME_NOPROG);
	addControlInfo(IDC_PROGRESS,		0, PROGD_FRAME_FULL);

	setControlRelationship(IDC_STATIC_FRAMEFULL,	IDD, CControlInfo::ACOMMON_RESIZEHEIGHT_RESIZEWIDTH);
	setControlRelationship(IDC_STATIC_FRAMENOPROG,	IDD, CControlInfo::ACOMMON_RESIZEHEIGHT_RESIZEWIDTH);

	setControlRelationship(IDC_STATIC_PICTURE,		IDD, CControlInfo::ACOMMON_TOP_LEFT);
	setControlRelationship(IDC_STATIC_HEADING,		IDD, CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_STATIC_BODY,			IDD, CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_PROGRESS,			IDD, CControlInfo::ACOMMON_TOP_RESIZEWIDTH);

	// Define the frames we're going to use.
	addFrameInfo(PROGD_FRAME_FULL,			IDC_STATIC_FRAMEFULL,		false, false);
	addFrameInfo(PROGD_FRAME_NOPROG,		IDC_STATIC_FRAMENOPROG,		false, false);

	// Must call this before using the controls we have configured.
	controlAndFrameInfoIsSetup();

	switchToFrame(m_reqFrameIdent, true, true, false, true);

	disableClosure();

	// When we close send lparam back to parent with thread's ident.
	setLParamToReturnToParent(m_pProgThread->GetIdent());

	if (m_bImmediatePopup)
	{
		ShowWindow(SW_SHOW);
	}
	else
	{
		m_nOpenTimer = SetTimer(IDTIMER_PROGRESS_OPEN, 500, NULL);

		if (0 == m_nOpenTimer)
		{
			// Just show the window now, we couldn't create a timer.
			ShowWindow(SW_SHOW);
		}
		else
		{
			m_bHaveOpenTimer = true;
		}
	}

	this->setRememberWindowPosition(SN_REG_PATH_WINDOWPOS, SN_REG_VALUE_WINDOWPOS_PROGRESS);

	// Start our thread.

	// Thread stored by the CSuckLessDialog as m_pSLDThread and is deleted automatically, whether it started or not.
	// We NULL it before starting the thread in case the thread causes us to close before OnInitDialog() finishes,
	// which can happen at least if you put up an AfxMessageBox() after the startThread(). :-)
	CProgressDlgThread *pProgThread = m_pProgThread;
	m_pProgThread = NULL;

	if (!this->startThread(pProgThread))
	{
		closeWindow(0);
	}

	// Nothing from here on should access member variables or functions.

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CProgressDlg::OnTimer(UINT nIDEvent) 
{
	if ((m_bHaveOpenTimer) && (m_nOpenTimer == nIDEvent))
	{
		KillTimer(m_nOpenTimer);
		m_bHaveOpenTimer = false;
		m_nOpenTimer = 0;

		this->ShowWindow(SW_SHOW);
	}
	else
	{
		CSuckLessDialog::OnTimer(nIDEvent);
	}
}

void CProgressDlg::OnDestroy() 
{
	// Dunno if this is really needed.
	if (m_bHaveOpenTimer)
	{
		KillTimer(m_nOpenTimer);
		m_bHaveOpenTimer = false;
		m_nOpenTimer = 0;
	}

	CSuckLessDialog::OnDestroy();
}

LRESULT CProgressDlg::OnProgressDlgProgress(WPARAM wparam, LPARAM lparam)
{
	LRESULT result = 1; // 0 for not handled, 1 for handled.

	CWnd *pProgressHeading = this->GetDlgItem(IDC_STATIC_HEADING);
	CWnd *pProgressMessage = this->GetDlgItem(IDC_STATIC_BODY);

	if (NULL != pProgressMessage)
	{
		EnterCriticalSection(&m_CSProgress);

		if (NULL != m_progressTitle)
		{
			// The window will always exist. :-)
			this->SetWindowText(m_progressTitle);

			// Delete it so we don't waste time setting the same thing again.
			delete [] m_progressTitle;
			m_progressTitle = NULL;
		}

		if (NULL != m_progressHeading)
		{
			if (NULL != pProgressHeading)
			{
				pProgressHeading->SetWindowText(m_progressHeading);
			}

			// Delete it so we don't waste time setting the same thing again.
			delete [] m_progressHeading;
			m_progressHeading = NULL;
		}

		if (NULL != m_progressMessage)
		{
			if (NULL != pProgressMessage)
			{
				pProgressMessage->SetWindowText(m_progressMessage);
			}

			// Delete it so we don't waste time setting the same thing again.
			delete [] m_progressMessage;
			m_progressMessage = NULL;
		}

		if ((-1) != m_progressValue)
		{
			m_ctrlProgress.SetPos(m_progressValue);

			m_progressValue = -1;
		}

		if ((-1) != m_progressRange)
		{
			m_ctrlProgress.SetRange32(0, m_progressRange);

			m_progressRange = -1;
		}

		if (m_bDoImmediatePopup)
		{
			if (m_bHaveOpenTimer)
			{
				KillTimer(m_nOpenTimer);
				m_bHaveOpenTimer = false;
				m_nOpenTimer = 0;

				this->ShowWindow(SW_SHOW);
			}
		}

		LeaveCriticalSection(&m_CSProgress);
	}

	return(result);
}

// To avoid changing the title, heading or message give NULL.
// To avoid changing the value or range give -1.
// Note that success is only worth setting for modal dialogs.
void CProgressDlg::SetProgress(	int progressValue,
					int progressRange,
					const char *progressTitle,
					const char *progressHeading,
					const char *progressMessage,
					int setSuccess,
					bool bDoImmediatePopup)
{
	EnterCriticalSection(&m_CSProgress);

	if (NULL != progressTitle)
	{
		if (NULL != m_progressTitle)
		{
			delete [] m_progressTitle;
			m_progressTitle = NULL;
		}

		m_progressTitle = new char[strlen(progressTitle) + 1];
		strcpy(m_progressTitle, progressTitle);
	}

	if (NULL != progressHeading)
	{
		if (NULL != m_progressHeading)
		{
			delete [] m_progressHeading;
			m_progressHeading = NULL;
		}

		m_progressHeading = new char[strlen(progressHeading) + 1];
		strcpy(m_progressHeading, progressHeading);
	}

	if (NULL != progressMessage)
	{
		if (NULL != m_progressMessage)
		{
			delete [] m_progressMessage;
			m_progressMessage = NULL;
		}

		m_progressMessage = new char[strlen(progressMessage) + 1];
		strcpy(m_progressMessage, progressMessage);
	}

	if ((-1) != progressValue)
	{
		m_progressValue = progressValue;
	}

	if ((-1) != progressRange)
	{
		m_progressRange = progressRange;
	}

	if ((-1) != setSuccess)
	{
		m_bSuccess = (0 != setSuccess ? true : false);
	}

	if (bDoImmediatePopup)
	{
		m_bDoImmediatePopup = true;
	}

	// Using SendMessage will cause deadlock since this function is to be called by a different thread.
	this->PostMessage(WM_PROGRESSDLG_PROGRESS);

	LeaveCriticalSection(&m_CSProgress);
}

// See CSuckLessDialog
void CProgressDlg::OnThreadTermination(int ident)
{
	if (ident == m_pThreadIdent)
	{
		closeWindow(m_bSuccess ? 1 : 0);
	}
}

/////////////////////////////////////////////////////////////////////////////
// Subclass of CSuckLessDialogThread

CProgressDlgThread::CProgressDlgThread(void) : CSuckLessDialogThread()
{
	m_pProgressDlg = NULL;
}

CProgressDlgThread::~CProgressDlgThread(void)
{
}

void CProgressDlgThread::SetProgressDlg(CProgressDlg *pProgDlg)
{
	m_pProgressDlg = pProgDlg;
}

// These functions check m_pProgressDlg for NULL (so you can have routines where the progress dialog
// is optional without any bother). Where useful, printf()-style formatting is done for you.

// Give progressValue of (-1) to keep existing value.
// Give progressRange of (-1) to keep existing range.
void CProgressDlgThread::SetProgressNumbers(int progressValue, int progressRange)
{
	CProgressDlg::SafeSetNumbers(m_pProgressDlg, progressValue, progressRange);
}

void CProgressDlgThread::SetProgressTitle(const char *progressTitle, ...)
{
	va_list args;
	va_start(args, progressTitle);

	CProgressDlg::vSafeSetTitle(m_pProgressDlg, progressTitle, args);

	va_end(args);
}

void CProgressDlgThread::SetProgressHeading(const char *progressHeading, ...)
{
	va_list args;
	va_start(args, progressHeading);

	CProgressDlg::vSafeSetHeading(m_pProgressDlg, progressHeading, args);

	va_end(args);
}

void CProgressDlgThread::SetProgressMessage(const char *progressMessage, ...)
{
	va_list args;
	va_start(args, progressMessage);

	CProgressDlg::vSafeSetMessage(m_pProgressDlg, progressMessage, args);

	va_end(args);
}

// Note that success is only worth setting for modal progress dialogs.
void CProgressDlgThread::SetProgressSuccess(bool bSuccess)
{
	CProgressDlg::SafeSetSuccess(m_pProgressDlg, bSuccess);
}
