#if !defined(AFX_PROGRESSDLG_H__0C811381_761A_11D4_BF8F_0020AF872188__INCLUDED_)
#define AFX_PROGRESSDLG_H__0C811381_761A_11D4_BF8F_0020AF872188__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000
// ProgressDlg.h : header file
//

#define PROGD_FRAME_FULL		(1<<0)
#define PROGD_FRAME_NOPROG		(1<<1)

#include "SuckLessDialog.h"

class CProgressDlgThread;

/////////////////////////////////////////////////////////////////////////////
// CProgressDlg dialog

class CProgressDlg : public CSuckLessDialog
{
// Construction
public:
	// pSLDThread is deleted when the progress dialog is deleted.
	CProgressDlg(int reqFrameIdent, bool bImmediatePopup, CProgressDlgThread *pSLDThread, CWnd* pParent);
	~CProgressDlg(void);

	// Note that success is only worth setting for modal dialogs.
	void SetProgress(	int progressValue,
						int progressRange,
						const char *progressTitle,
						const char *progressHeading,
						const char *progressMessage,
						int setSuccess,
						bool bDoImmediatePopup);

	void OnThreadTermination(int ident); // See CSuckLessDialog

	// These functions check pProgressDlg for NULL (so you can have routines where the progress dialog
	// is optional without any bother). Where useful, printf()-style formatting is done for you.

	// Give progressValue of (-1) to keep existing value.
	// Give progressRange of (-1) to keep existing range.
	static void SafeSetNumbers(CProgressDlg *pProgressDlg, int progressValue, int progressRange)
	{
		if (NULL != pProgressDlg)
		{
			pProgressDlg->SetProgress(progressValue, progressRange, NULL, NULL, NULL, -1, false);
		}
	}

	static void SafeSetTitle(CProgressDlg *pProgressDlg, const char *progressTitle, ...)
	{
		if (NULL != pProgressDlg)
		{
			va_list args;
			va_start(args, progressTitle);

			vSafeSetTitle(pProgressDlg, progressTitle, args);

			va_end(args);
		}
	}

	static void vSafeSetTitle(CProgressDlg *pProgressDlg, const char *progressTitle, va_list args)
	{
		if ((NULL != pProgressDlg) && (NULL != progressTitle))
		{
			char buf[BUFFERSIZE];

			vStringFormat(buf, sizeof(buf), progressTitle, args);

			pProgressDlg->SetProgress(-1, -1, buf, NULL, NULL, -1, false);
		}
	}

	static void SafeSetHeading(CProgressDlg *pProgressDlg, const char *progressHeading, ...)
	{
		if (NULL != pProgressDlg)
		{
			va_list args;
			va_start(args, progressHeading);

			vSafeSetHeading(pProgressDlg, progressHeading, args);

			va_end(args);
		}
	}

	static void vSafeSetHeading(CProgressDlg *pProgressDlg, const char *progressHeading, va_list args)
	{
		if ((NULL != pProgressDlg) && (NULL != progressHeading))
		{
			char buf[BUFFERSIZE];

			vStringFormat(buf, sizeof(buf), progressHeading, args);

			pProgressDlg->SetProgress(-1, -1, NULL, buf, NULL, -1, false);
		}
	}

	static void SafeSetMessage(CProgressDlg *pProgressDlg, const char *progressMessage, ...)
	{
		if (NULL != pProgressDlg)
		{
			va_list args;
			va_start(args, progressMessage);

			vSafeSetMessage(pProgressDlg, progressMessage, args);

			va_end(args);
		}
	}

	static void vSafeSetMessage(CProgressDlg *pProgressDlg, const char *progressMessage, va_list args)
	{
		if ((NULL != pProgressDlg) && (NULL != progressMessage))
		{
			char buf[BUFFERSIZE];

			vStringFormat(buf, sizeof(buf), progressMessage, args);

			pProgressDlg->SetProgress(-1, -1, NULL, NULL, buf, -1, false);
		}
	}

	// Note that success is only worth setting for modal dialogs.
	static void SafeSetSuccess(CProgressDlg *pProgressDlg, bool bSuccess)
	{
		if (NULL != pProgressDlg)
		{
			pProgressDlg->SetProgress(-1, -1, NULL, NULL, NULL, bSuccess ? 1 : 0, false);
		}
	}

	// If bImmediatePopup was false in the constructor you can use this to make sure the window appears
	// now rather than after the timeout.
	static void SafeDoImmediatePopup(CProgressDlg *pProgressDlg)
	{
		if (NULL != pProgressDlg)
		{
			pProgressDlg->SetProgress(-1, -1, NULL, NULL, NULL, -1, true);
		}
	}

// Dialog Data
	//{{AFX_DATA(CProgressDlg)
	enum { IDD = IDD_PROGRESS_DIALOG };
	CProgressCtrl	m_ctrlProgress;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CProgressDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	enum { IDTIMER_PROGRESS_OPEN = 1 };
	
	bool m_bHaveOpenTimer;
	UINT m_nOpenTimer;

	mutable CRITICAL_SECTION m_CSProgress;

	int m_reqFrameIdent;

	bool m_bImmediatePopup;

	CProgressDlgThread *m_pProgThread;
	int m_pThreadIdent;

	char *m_progressTitle;

	char *m_progressHeading;

	char *m_progressMessage;
	int m_progressValue;
	int m_progressRange;

	bool m_bDoImmediatePopup;

	// Note that success is only worth setting for modal dialogs.
	bool m_bSuccess;

	// Generated message map functions
	//{{AFX_MSG(CProgressDlg)
	afx_msg LRESULT OnProgressDlgProgress(WPARAM wparam, LPARAM lparam);
	virtual BOOL OnInitDialog();
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnDestroy();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


/////////////////////////////////////////////////////////////////////////////
// Subclass of CSuckLessDialogThread

class CProgressDlgThread : public CSuckLessDialogThread
{
protected:
	CProgressDlg *m_pProgressDlg;

	// Override this to provide your functionality.
	virtual void mainThreadSubClass(void) = 0;

public:
	CProgressDlgThread(void);
	virtual ~CProgressDlgThread(void);

	void SetProgressDlg(CProgressDlg *pProgDlg);

	// These functions check m_pProgressDlg for NULL (so you can have routines where the progress dialog
	// is optional without any bother). Where useful, printf()-style formatting is done for you.

	// Give progressValue of (-1) to keep existing value.
	// Give progressRange of (-1) to keep existing range.
	void SetProgressNumbers(int progressValue, int progressRange);
	void SetProgressTitle(const char *progressTitle, ...);
	void SetProgressHeading(const char *progressHeading, ...);
	void SetProgressMessage(const char *progressMessage, ...);

	// Note that success is only worth setting for modal progress dialogs.
	void SetProgressSuccess(bool bSuccess);

	// Returns an ident for the thread.
	// Typically, each implementation of this class within a project will use a unique
	// ident. (-1) should not be used, but any other number is fine.
	// See CSuckLessDialog::OnThreadTermination().
	virtual int GetIdent(void) = 0;
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_PROGRESSDLG_H__0C811381_761A_11D4_BF8F_0020AF872188__INCLUDED_)
