// ProntoString.cpp: implementation of the CProntoString class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CProntoString::CProntoString()
{
	m_szEscaped = NULL;
	m_szPronto = NULL;
	m_szPrintable = NULL;
}

CProntoString::~CProntoString()
{
	Reset();
}

void CProntoString::Reset(void)
{
	delete [] m_szEscaped;
	m_szEscaped = NULL;

	delete [] m_szPronto;
	m_szPronto = NULL;

	delete [] m_szPrintable;
	m_szPrintable = NULL;
}

bool CProntoString::IsUnassigned(void) const
{
	return((NULL == m_szEscaped) && (NULL == m_szPronto));
}

bool CProntoString::IsTheEmptyString(void) const
{
	if (NULL != m_szEscaped)
	{
		if (0 == strcmp("", m_szEscaped))
		{
			return(true);
		}
	}
	else if (NULL != m_szPronto)
	{
		if (0 == strcmp("", m_szPronto))
		{
			return(true);
		}
	}

	return(false);
}

bool CProntoString::IsANonEmptyString(void) const
{
	return((!IsUnassigned()) && (!IsTheEmptyString()));
}


const char *CProntoString::GetEscaped(bool bNeverNull) const
{
	if ((NULL == m_szEscaped) && (NULL != m_szPronto))
	{
		m_szEscaped = CopyProntoToEscaped(m_szPronto);
	}

	if (bNeverNull && (NULL == m_szEscaped))
	{
		return("");
	}
	else
	{
		return(m_szEscaped);
	}
}

const char *CProntoString::GetPronto(bool bNeverNull) const
{
	if ((NULL == m_szPronto) && (NULL != m_szEscaped))
	{
		m_szPronto = CopyEscapedToPronto(m_szEscaped);
	}

	if (bNeverNull && (NULL == m_szPronto))
	{
		return("");
	}
	else
	{
		return(m_szPronto);
	}
}

const char *CProntoString::GetPrintable(bool bNeverNull) const
{
	if ((NULL == m_szPrintable) && (NULL != GetPronto(false)))
	{
		m_szPrintable = CopyProntoToPrintable(GetPronto(true));
	}

	if (bNeverNull && (NULL == m_szPrintable))
	{
		return("");
	}
	else
	{
		return(m_szPrintable);
	}
}

// Returns <  0 if this <  pPS
// Returns == 0 if this == pPS
// Returns >  0 if this >  pPS
int CProntoString::Compare(const CProntoString *pPS, bool bCaseSensitive) const
{
	if (this->IsUnassigned())
	{
		if (pPS->IsUnassigned())
		{
			return(0);
		}
		else
		{
			return(-1);
		}
	}

	if (pPS->IsUnassigned())
	{
		return(1);
	}

	const char *stringLeft = this->GetPronto(true);
	const char *stringRight = pPS->GetPronto(true);

	if (bCaseSensitive)
	{
		return(strcmp(stringLeft, stringRight));
	}
	else
	{
		return(stricmp(stringLeft, stringRight));
	}
}

void CProntoString::SetEscaped(const char *s)
{
	Reset();

	if (NULL != s)
	{
		// Setting the pronto string means the user always gets a well-encoded
		// escaped string, even if the input has issues like unescaped '\' characters.
		// Also, most of the program works on the pronto string so having the
		// escaped one around is sometimes a waste of time.
///////	m_szPronto = CProntoString::CopyEscapedToPronto(s);

		// We run in to problems if we escape names differently to the original CSF
		// file since CCFDll (currently) compares escaped strings directly, so just
		// save the escaped string exactly as it was found.
		m_szEscaped = new char[strlen(s) + 1];
		strcpy(m_szEscaped, s);
	}
}

void CProntoString::SetPronto(const char *s)
{
	Reset();

	if (NULL != s)
	{
		m_szPronto = new char[strlen(s) + 1];
		strcpy(m_szPronto, s);
	}
}


// ****************************
// ** Static Functions Below **
// ****************************

/////////////////////////////
// -- Escaped -> Pronto -- //
/////////////////////////////

// static
bool CProntoString::IsEscapedSequence(const char *s)
{
	return(	('\\' == s[0])						\
	&&		(('x' == s[1]) || ('X' == s[1]))	\
	&&		(isxdigit(s[2]))					\
	&&		(isxdigit(s[3]))					);
}

// Get length of equivalent Pronto string.
// static
size_t CProntoString::StrLenEscapedToPronto(const char *s)
{
	size_t result = 0;
	size_t inAt = 0;

	while ('\0' != s[inAt])
	{
		if (IsEscapedSequence(s + inAt))
		{
			result += 4;
			inAt+= 4;
		}
		else
		{
			result++;
			inAt++;
		}
	}

	return(result);
}

// Undefined results if called on anything other than an escaped character sequence.
// static
char CProntoString::ConvertEscapedSequenceToProntoCharacter(const char *s)
{
	int n;

	if (1 != sscanf(s + 2, "%2x", &n))
	{
		return(',');
	}
	else
	{
		return((unsigned char)n);
	}
}

// static
void CProntoString::ConvertEscapedToPronto(char *sOut, size_t outSize, const char *sIn)
{
	if (0 < outSize)
	{
		size_t atIn = 0;
		size_t atOut = 0;
		size_t outSizeSubOne = outSize - 1;

		while ((atOut < outSizeSubOne) && ('\0' != sIn[atIn]))
		{
			if (IsEscapedSequence(sIn + atIn))
			{
				sOut[atOut++] = ConvertEscapedSequenceToProntoCharacter(sIn + atIn);
				atIn += 4;
			}
			else
			{
				sOut[atOut++] = sIn[atIn++];
			}
		}

		sOut[atOut] = '\0';
	}
}

// delete [] the result.
// static
char *CProntoString::CopyEscapedToPronto(const char *sIn)
{
	size_t outSize = StrLenEscapedToPronto(sIn) + 1;

	char *result = new char[outSize];

	ConvertEscapedToPronto(result, outSize, sIn);

	return(result);
}

/////////////////////////////
// -- Pronto -> Escaped -- //
/////////////////////////////

// static
bool CProntoString::IsProntoCharacter(char c)
{
	// Similar to printable (see below) but without doublequote, underscore or either slash.
	const char *dontEscape =	"1234567890-=qwertyuiop[]`asdfghjkl;' zxcvbnm,."\
								"!@#$%^&*()+QWERTYUIOP{}~ASDFGHJKL:ZXCVBNM<>?|";

	return(NULL == strchr(dontEscape, c));
}

// Get length of equivalent escaped string.
// static
size_t CProntoString::StrLenProntoToEscaped(const char *s)
{
	size_t result = 0;

	while ('\0' != (*s))
	{
		if (IsProntoCharacter(*s++))
		{
			result += 4;
		}
		else
		{
			result++;
		}
	}

	return(result);
}

// sOut must have five bytes of space
// static
void CProntoString::ConvertProntoCharacterToEscapedSequence(char *sOut, unsigned char cIn)
{
	// This will crash if cIn is not an UNSIGNED char.
	sprintf(sOut, "\\x%02x", (int)cIn);
}

// static
void CProntoString::ConvertProntoToEscaped(char *sOut, size_t outSize, const char *sIn)
{
	if (0 < outSize)
	{
		size_t atIn = 0;
		size_t atOut = 0;
		size_t outSizeSubOne = outSize - 1;
		size_t outSizeSubFour = outSize - 4;

		while ('\0' != sIn[atIn])
		{
			if (IsProntoCharacter(sIn[atIn]))
			{
				if (atOut >= outSizeSubFour)
				{
					break;
				}

				ConvertProntoCharacterToEscapedSequence(sOut + atOut, sIn[atIn]);
				atOut += 4;
				atIn++;
			}
			else
			{
				if (atOut >= outSizeSubOne)
				{
					break;
				}

				sOut[atOut++] = sIn[atIn++];
			}
		}

		sOut[atOut] = '\0';
	}
}

// delete [] the result.
// static
char *CProntoString::CopyProntoToEscaped(const char *sIn)
{
	size_t outSize = StrLenProntoToEscaped(sIn) + 1;

	char *result = new char[outSize];

	ConvertProntoToEscaped(result, outSize, sIn);

	return(result);
}

///////////////////////////////
// -- Pronto -> Printable -- //
///////////////////////////////

// static
bool CProntoString::IsPrintableCharacter(char c)
{
	const char *printable =	"1234567890-=qwertyuiop[]`asdfghjkl;' zxcvbnm,./\\"\
							"!@#$%^&*()_+QWERTYUIOP{}~ASDFGHJKL:\"ZXCVBNM<>?|";

	return(NULL != strchr(printable, c));
}

// delete [] the result.
// static
char *CProntoString::CopyProntoToPrintable(const char *sIn)
{
	char *sResult = new char[strlen(sIn) + 1];

	for (size_t i = 0; '\0' != sIn[i]; i++)
	{
		if (IsPrintableCharacter(sIn[i]))
		{
			sResult[i] = sIn[i];
		}
		else
		{
			sResult[i] = '*';
		}
	}

	sResult[i] = '\0';

	return(sResult);
}

////////////////////////////////
// -- Escaped -> Printable -- //
////////////////////////////////

// delete [] the result.
// static
char *CProntoString::CopyEscapedToPrintable(const char *sIn)
{
	char *sPronto = CopyEscapedToPronto(sIn);
	char *sResult = CopyProntoToPrintable(sPronto);
	delete [] sPronto;

	return(sResult);
}

///////////////////////////////////////////////////////////
// Character Widths for the 8 and 10 point Pronto fonts. //
// These were calculated by Wolfgang Scheele.			 //
///////////////////////////////////////////////////////////

int CProntoString::s_fontCharWidths8[255];
int CProntoString::s_fontCharWidths10[255];


// Must be called at the start of the program.
// static
void CProntoString::InitStatic()
{
	for (int i = 0; i < 255; i++)
	{
		s_fontCharWidths8[i] = s_fontCharWidths10[i] = 12; // Assume the worst.
	}

	s_fontCharWidths8[32] = 3;		s_fontCharWidths10[32] = 2;		// <space>
	s_fontCharWidths8[33] = 3;		s_fontCharWidths10[33] = 3;		// !
	s_fontCharWidths8[34] = 6;		s_fontCharWidths10[34] = 6;		// "
	s_fontCharWidths8[35] = 10;		s_fontCharWidths10[35] = 10;	// #
	s_fontCharWidths8[36] = 8;		s_fontCharWidths10[36] = 9;		// $
	s_fontCharWidths8[37] = 10;		s_fontCharWidths10[37] = 10;	// %
	s_fontCharWidths8[38] = 9;		s_fontCharWidths10[38] = 11;	// &
	s_fontCharWidths8[39] = 4;		s_fontCharWidths10[39] = 4;		// '
	s_fontCharWidths8[40] = 4;		s_fontCharWidths10[40] = 5;		// (
	s_fontCharWidths8[41] = 4;		s_fontCharWidths10[41] = 5;		// )
	s_fontCharWidths8[42] = 9;		s_fontCharWidths10[42] = 9;		// *
	s_fontCharWidths8[43] = 8;		s_fontCharWidths10[43] = 9;		// +
	s_fontCharWidths8[44] = 4;		s_fontCharWidths10[44] = 4;		// ,
	s_fontCharWidths8[45] = 5;		s_fontCharWidths10[45] = 6;		// -
	s_fontCharWidths8[46] = 4;		s_fontCharWidths10[46] = 4;		// .
	s_fontCharWidths8[47] = 5;		s_fontCharWidths10[47] = 4;		// /
	s_fontCharWidths8[48] = 8;		s_fontCharWidths10[48] = 9;		// 0
	s_fontCharWidths8[49] = 8;		s_fontCharWidths10[49] = 9;		// 1
	s_fontCharWidths8[50] = 8;		s_fontCharWidths10[50] = 9;		// 2
	s_fontCharWidths8[51] = 8;		s_fontCharWidths10[51] = 9;		// 3
	s_fontCharWidths8[52] = 8;		s_fontCharWidths10[52] = 9;		// 4
	s_fontCharWidths8[53] = 8;		s_fontCharWidths10[53] = 9;		// 5
	s_fontCharWidths8[54] = 8;		s_fontCharWidths10[54] = 9;		// 6
	s_fontCharWidths8[55] = 8;		s_fontCharWidths10[55] = 9;		// 7
	s_fontCharWidths8[56] = 8;		s_fontCharWidths10[56] = 9;		// 8
	s_fontCharWidths8[57] = 8;		s_fontCharWidths10[57] = 9;		// 9
	s_fontCharWidths8[58] = 4;		s_fontCharWidths10[58] = 4;		// :
	s_fontCharWidths8[59] = 4;		s_fontCharWidths10[59] = 4;		// ;
	s_fontCharWidths8[60] = 8;		s_fontCharWidths10[60] = 9;		// <
	s_fontCharWidths8[61] = 8;		s_fontCharWidths10[61] = 9;		// =
	s_fontCharWidths8[62] = 8;		s_fontCharWidths10[62] = 9;		// >
	s_fontCharWidths8[63] = 7;		s_fontCharWidths10[63] = 6;		// ?
	s_fontCharWidths8[64] = 10;		s_fontCharWidths10[64] = 10;	// @
	s_fontCharWidths8[65] = 9;		s_fontCharWidths10[65] = 9;		// A
	s_fontCharWidths8[66] = 8;		s_fontCharWidths10[66] = 8;		// B
	s_fontCharWidths8[67] = 9;		s_fontCharWidths10[67] = 8;		// C
	s_fontCharWidths8[68] = 9;		s_fontCharWidths10[68] = 10;	// D
	s_fontCharWidths8[69] = 8;		s_fontCharWidths10[69] = 7;		// E
	s_fontCharWidths8[70] = 7;		s_fontCharWidths10[70] = 7;		// F
	s_fontCharWidths8[71] = 9;		s_fontCharWidths10[71] = 10;	// G
	s_fontCharWidths8[72] = 9;		s_fontCharWidths10[72] = 10;	// H
	s_fontCharWidths8[73] = 6;		s_fontCharWidths10[73] = 6;		// I
	s_fontCharWidths8[74] = 6;		s_fontCharWidths10[74] = 5;		// J
	s_fontCharWidths8[75] = 7;		s_fontCharWidths10[75] = 9;		// K
	s_fontCharWidths8[76] = 6;		s_fontCharWidths10[76] = 7;		// L
	s_fontCharWidths8[77] = 10;		s_fontCharWidths10[77] = 12;	// M
	s_fontCharWidths8[78] = 9;		s_fontCharWidths10[78] = 10;	// N
	s_fontCharWidths8[79] = 9;		s_fontCharWidths10[79] = 10;	// O
	s_fontCharWidths8[80] = 8;		s_fontCharWidths10[80] = 8;		// P
	s_fontCharWidths8[81] = 9;		s_fontCharWidths10[81] = 10;	// Q
	s_fontCharWidths8[82] = 8;		s_fontCharWidths10[82] = 9;		// R
	s_fontCharWidths8[83] = 8;		s_fontCharWidths10[83] = 8;		// S
	s_fontCharWidths8[84] = 6;		s_fontCharWidths10[84] = 8;		// T
	s_fontCharWidths8[85] = 9;		s_fontCharWidths10[85] = 10;	// U
	s_fontCharWidths8[86] = 8;		s_fontCharWidths10[86] = 9;		// V
	s_fontCharWidths8[87] = 12;		s_fontCharWidths10[87] = 11;	// W
	s_fontCharWidths8[88] = 8;		s_fontCharWidths10[88] = 8;		// X
	s_fontCharWidths8[89] = 8;		s_fontCharWidths10[89] = 9;		// Y
	s_fontCharWidths8[90] = 9;		s_fontCharWidths10[90] = 9;		// Z
	s_fontCharWidths8[91] = 5;		s_fontCharWidths10[91] = 5;		// [
	s_fontCharWidths8[92] = 4;		s_fontCharWidths10[92] = 4;		// "\"
	s_fontCharWidths8[93] = 5;		s_fontCharWidths10[93] = 5;		// ]
	s_fontCharWidths8[94] = 6;		s_fontCharWidths10[94] = 10;	// ^
	s_fontCharWidths8[95] = 7;		s_fontCharWidths10[95] = 7;		// _
	s_fontCharWidths8[96] = 5;		s_fontCharWidths10[96] = 6;		// `
	s_fontCharWidths8[97] = 7;		s_fontCharWidths10[97] = 8;		// a
	s_fontCharWidths8[98] = 7;		s_fontCharWidths10[98] = 8;		// b
	s_fontCharWidths8[99] = 6;		s_fontCharWidths10[99] = 7;		// c
	s_fontCharWidths8[100] = 7;		s_fontCharWidths10[100] = 8;	// d
	s_fontCharWidths8[101] = 7;		s_fontCharWidths10[101] = 8;	// e
	s_fontCharWidths8[102] = 5;		s_fontCharWidths10[102] = 5;	// f
	s_fontCharWidths8[103] = 7;		s_fontCharWidths10[103] = 8;	// g
	s_fontCharWidths8[104] = 7;		s_fontCharWidths10[104] = 8;	// h
	s_fontCharWidths8[105] = 4;		s_fontCharWidths10[105] = 4;	// i
	s_fontCharWidths8[106] = 4;		s_fontCharWidths10[106] = 4;	// j
	s_fontCharWidths8[107] = 7;		s_fontCharWidths10[107] = 7;	// k
	s_fontCharWidths8[108] = 4;		s_fontCharWidths10[108] = 4;	// l
	s_fontCharWidths8[109] = 10;	s_fontCharWidths10[109] = 12;	// m
	s_fontCharWidths8[110] = 7;		s_fontCharWidths10[110] = 8;	// n
	s_fontCharWidths8[111] = 7;		s_fontCharWidths10[111] = 8;	// o
	s_fontCharWidths8[112] = 7;		s_fontCharWidths10[112] = 8;	// p
	s_fontCharWidths8[113] = 7;		s_fontCharWidths10[113] = 8;	// q
	s_fontCharWidths8[114] = 5;		s_fontCharWidths10[114] = 5;	// r
	s_fontCharWidths8[115] = 7;		s_fontCharWidths10[115] = 7;	// s
	s_fontCharWidths8[116] = 5;		s_fontCharWidths10[116] = 5;	// t
	s_fontCharWidths8[117] = 7;		s_fontCharWidths10[117] = 8;	// u
	s_fontCharWidths8[118] = 6;		s_fontCharWidths10[118] = 8;	// v
	s_fontCharWidths8[119] = 10;	s_fontCharWidths10[119] = 12;	// w
	s_fontCharWidths8[120] = 6;		s_fontCharWidths10[120] = 8;	// x
	s_fontCharWidths8[121] = 6;		s_fontCharWidths10[121] = 8;	// y
	s_fontCharWidths8[122] = 7;		s_fontCharWidths10[122] = 7;	// z
	s_fontCharWidths8[123] = 6;		s_fontCharWidths10[123] = 6;	// {
	s_fontCharWidths8[124] = 4;		s_fontCharWidths10[124] = 4;	// |
	s_fontCharWidths8[125] = 6;		s_fontCharWidths10[125] = 6;	// }
	s_fontCharWidths8[126] = 9;		s_fontCharWidths10[126] = 10;	// ~
	s_fontCharWidths8[161] = 3;		s_fontCharWidths10[161] = 4;	// 
	s_fontCharWidths8[162] = 6;		s_fontCharWidths10[162] = 8;	// 
	s_fontCharWidths8[163] = 6;		s_fontCharWidths10[163] = 8;	// 
	s_fontCharWidths8[164] = 5;		s_fontCharWidths10[164] = 6;	// 
	s_fontCharWidths8[165] = 8;		s_fontCharWidths10[165] = 9;	// 
	s_fontCharWidths8[166] = 4;		s_fontCharWidths10[166] = 4;	// 
	s_fontCharWidths8[167] = 6;		s_fontCharWidths10[167] = 7;	// 
	s_fontCharWidths8[168] = 5;		s_fontCharWidths10[168] = 5;	// 
	s_fontCharWidths8[169] = 8;		s_fontCharWidths10[169] = 10;	// 
	s_fontCharWidths8[170] = 6;		s_fontCharWidths10[170] = 7;	// 
	s_fontCharWidths8[171] = 6;		s_fontCharWidths10[171] = 7;	// 
	s_fontCharWidths8[172] = 7;		s_fontCharWidths10[172] = 7;	// 
	s_fontCharWidths8[173] = 6;		s_fontCharWidths10[173] = 6;	// 
	s_fontCharWidths8[174] = 8;		s_fontCharWidths10[174] = 10;	// 
	s_fontCharWidths8[175] = 7;		s_fontCharWidths10[175] = 7;	// 
	s_fontCharWidths8[176] = 3;		s_fontCharWidths10[176] = 4;	// 
	s_fontCharWidths8[177] = 8;		s_fontCharWidths10[177] = 9;	// 
	s_fontCharWidths8[178] = 3;		s_fontCharWidths10[178] = 4;	// 
	s_fontCharWidths8[179] = 3;		s_fontCharWidths10[179] = 4;	// 
	s_fontCharWidths8[180] = 5;		s_fontCharWidths10[180] = 6;	// 
	s_fontCharWidths8[181] = 7;		s_fontCharWidths10[181] = 8;	// 
	s_fontCharWidths8[182] = 8;		s_fontCharWidths10[182] = 9;	// 
	s_fontCharWidths8[183] = 1;		s_fontCharWidths10[183] = 2;	// 
	s_fontCharWidths8[184] = 5;		s_fontCharWidths10[184] = 6;	// 
	s_fontCharWidths8[185] = 3;		s_fontCharWidths10[185] = 4;	// 
	s_fontCharWidths8[186] = 6;		s_fontCharWidths10[186] = 7;	// 
	s_fontCharWidths8[187] = 6;		s_fontCharWidths10[187] = 7;	// 
	s_fontCharWidths8[188] = 9;		s_fontCharWidths10[188] = 11;	// 
	s_fontCharWidths8[189] = 9;		s_fontCharWidths10[189] = 11;	// 
	s_fontCharWidths8[190] = 9;		s_fontCharWidths10[190] = 11;	// 
	s_fontCharWidths8[191] = 7;		s_fontCharWidths10[191] = 6;	// 
	s_fontCharWidths8[192] = 9;		s_fontCharWidths10[192] = 9;	// 
	s_fontCharWidths8[193] = 9;		s_fontCharWidths10[193] = 9;	// 
	s_fontCharWidths8[194] = 9;		s_fontCharWidths10[194] = 9;	// 
	s_fontCharWidths8[195] = 9;		s_fontCharWidths10[195] = 9;	// 
	s_fontCharWidths8[196] = 9;		s_fontCharWidths10[196] = 9;	// 
	s_fontCharWidths8[197] = 9;		s_fontCharWidths10[197] = 9;	// 
	s_fontCharWidths8[198] = 11;	s_fontCharWidths10[198] = 12;	// 
	s_fontCharWidths8[199] = 9;		s_fontCharWidths10[199] = 8;	// 
	s_fontCharWidths8[200] = 8;		s_fontCharWidths10[200] = 7;	// 
	s_fontCharWidths8[201] = 8;		s_fontCharWidths10[201] = 7;	// 
	s_fontCharWidths8[202] = 8;		s_fontCharWidths10[202] = 7;	// 
	s_fontCharWidths8[203] = 8;		s_fontCharWidths10[203] = 7;	// 
	s_fontCharWidths8[204] = 3;		s_fontCharWidths10[204] = 3;	// 
	s_fontCharWidths8[205] = 3;		s_fontCharWidths10[205] = 3;	// 
	s_fontCharWidths8[206] = 3;		s_fontCharWidths10[206] = 3;	// 
	s_fontCharWidths8[207] = 3;		s_fontCharWidths10[207] = 3;	// 
	s_fontCharWidths8[208] = 9;		s_fontCharWidths10[208] = 10;	// 
	s_fontCharWidths8[209] = 9;		s_fontCharWidths10[209] = 10;	// 
	s_fontCharWidths8[210] = 7;		s_fontCharWidths10[210] = 8;	// 
	s_fontCharWidths8[211] = 7;		s_fontCharWidths10[211] = 8;	// 
	s_fontCharWidths8[212] = 7;		s_fontCharWidths10[212] = 8;	// 
	s_fontCharWidths8[213] = 7;		s_fontCharWidths10[213] = 8;	// 
	s_fontCharWidths8[214] = 7;		s_fontCharWidths10[214] = 8;	// 
	s_fontCharWidths8[215] = 6;		s_fontCharWidths10[215] = 8;	// 
	s_fontCharWidths8[216] = 9;		s_fontCharWidths10[216] = 10;	// 
	s_fontCharWidths8[217] = 7;		s_fontCharWidths10[217] = 8;	// 
	s_fontCharWidths8[218] = 7;		s_fontCharWidths10[218] = 8;	// 
	s_fontCharWidths8[219] = 7;		s_fontCharWidths10[219] = 8;	// 
	s_fontCharWidths8[220] = 7;		s_fontCharWidths10[220] = 8;	// 
	s_fontCharWidths8[221] = 8;		s_fontCharWidths10[221] = 9;	// 
	s_fontCharWidths8[222] = 8;		s_fontCharWidths10[222] = 8;	// 
	s_fontCharWidths8[223] = 8;		s_fontCharWidths10[223] = 9;	// 
	s_fontCharWidths8[224] = 7;		s_fontCharWidths10[224] = 8;	// 
	s_fontCharWidths8[225] = 7;		s_fontCharWidths10[225] = 8;	// 
	s_fontCharWidths8[226] = 7;		s_fontCharWidths10[226] = 8;	// 
	s_fontCharWidths8[227] = 7;		s_fontCharWidths10[227] = 8;	// 
	s_fontCharWidths8[228] = 7;		s_fontCharWidths10[228] = 8;	// 
	s_fontCharWidths8[229] = 7;		s_fontCharWidths10[229] = 8;	// 
	s_fontCharWidths8[230] = 10;	s_fontCharWidths10[230] = 12;	// 
	s_fontCharWidths8[231] = 6;		s_fontCharWidths10[231] = 7;	// 
	s_fontCharWidths8[232] = 7;		s_fontCharWidths10[232] = 8;	// 
	s_fontCharWidths8[233] = 7;		s_fontCharWidths10[233] = 8;	// 
	s_fontCharWidths8[234] = 7;		s_fontCharWidths10[234] = 8;	// 
	s_fontCharWidths8[235] = 7;		s_fontCharWidths10[235] = 8;	// 
	s_fontCharWidths8[236] = 3;		s_fontCharWidths10[236] = 3;	// 
	s_fontCharWidths8[237] = 3;		s_fontCharWidths10[237] = 3;	// 
	s_fontCharWidths8[238] = 4;		s_fontCharWidths10[238] = 4;	// 
	s_fontCharWidths8[239] = 5;		s_fontCharWidths10[239] = 5;	// 
	s_fontCharWidths8[240] = 8;		s_fontCharWidths10[240] = 9;	// 
	s_fontCharWidths8[241] = 7;		s_fontCharWidths10[241] = 8;	// 
	s_fontCharWidths8[242] = 7;		s_fontCharWidths10[242] = 8;	// 
	s_fontCharWidths8[243] = 7;		s_fontCharWidths10[243] = 8;	// 
	s_fontCharWidths8[244] = 7;		s_fontCharWidths10[244] = 8;	// 
	s_fontCharWidths8[245] = 7;		s_fontCharWidths10[245] = 8;	// 
	s_fontCharWidths8[246] = 7;		s_fontCharWidths10[246] = 8;	// 
	s_fontCharWidths8[247] = 6;		s_fontCharWidths10[247] = 7;	// 
	s_fontCharWidths8[248] = 8;		s_fontCharWidths10[248] = 9;	// 
	s_fontCharWidths8[249] = 7;		s_fontCharWidths10[249] = 8;	// 
	s_fontCharWidths8[250] = 7;		s_fontCharWidths10[250] = 8;	// 
	s_fontCharWidths8[251] = 7;		s_fontCharWidths10[251] = 8;	// 
	s_fontCharWidths8[252] = 7;		s_fontCharWidths10[252] = 8;	// 
	s_fontCharWidths8[253] = 6;		s_fontCharWidths10[253] = 8;	// 
	s_fontCharWidths8[254] = 6;		s_fontCharWidths10[254] = 8;	// 
	s_fontCharWidths8[255] = 6;		s_fontCharWidths10[255] = 8;	// 
}

// static
int CProntoString::GetProntoCharacterPixelWidth(bool bLarge, char c)
{
	return(bLarge ? s_fontCharWidths10[c] : s_fontCharWidths8[c]);
}

// static
int CProntoString::CalculateProntoStringPixelWidth(bool bLarge, const char *szString)
{
	int nWidth = 0;

	while ('\0' != *szString)
	{
		nWidth += GetProntoCharacterPixelWidth(bLarge, *szString++);
	}

	return(nWidth);
}

int CProntoString::CalculateProntoStringPixelWidth(bool bLarge) const
{
	int nWidth = 0;

	GetPronto(false);

	if (NULL != m_szPronto)
	{
		nWidth = CProntoString::CalculateProntoStringPixelWidth(bLarge, m_szPronto);
	}

	return(nWidth);
}

void CProntoString::TruncateToPixelWidth(bool bLarge, int nMaxWidth)
{
	if (nMaxWidth <= 0)
	{
		SetPronto("");
	}
	else
	{
		int nCurrentWidth = CalculateProntoStringPixelWidth(bLarge);

		if (nMaxWidth < nCurrentWidth)
		{
			int nProntoStringLength = strlen(m_szPronto);
			int nProntoBufferSize = nProntoStringLength + 2 + 1;

			char *szProntoBuffer = new char[nProntoBufferSize];
			strcpy(szProntoBuffer, m_szPronto);

			nCurrentWidth -= GetProntoCharacterPixelWidth(bLarge, szProntoBuffer[nProntoStringLength - 1]);
			nCurrentWidth += (3 * GetProntoCharacterPixelWidth(bLarge, '.'));
			szProntoBuffer[nProntoStringLength - 1] = '.';
			szProntoBuffer[nProntoStringLength    ] = '.';
			szProntoBuffer[nProntoStringLength + 1] = '.';
			szProntoBuffer[nProntoStringLength + 2] = '\0';
			nProntoStringLength += 2;

			while ((4 < nProntoStringLength) && (nMaxWidth < nCurrentWidth))
			{
				nCurrentWidth -= GetProntoCharacterPixelWidth(bLarge, szProntoBuffer[nProntoStringLength - 4]);
				szProntoBuffer[nProntoStringLength - 1] = '\0';
				szProntoBuffer[nProntoStringLength - 4] = '.';
				nProntoStringLength--;

				// Avoid adding more than three dots and avoid adding dots after a space (remove the space).
				while ((4 < nProntoStringLength)
				&&	   ((' ' == szProntoBuffer[nProntoStringLength - 4])
				||		('.' == szProntoBuffer[nProntoStringLength - 4])))
				{
					nCurrentWidth -= GetProntoCharacterPixelWidth(bLarge, szProntoBuffer[nProntoStringLength - 4]);
					szProntoBuffer[nProntoStringLength - 1] = '\0';
					szProntoBuffer[nProntoStringLength - 4] = '.';
					nProntoStringLength--;
				}
			}

			while ((0 < nProntoStringLength) && (nMaxWidth < nCurrentWidth))
			{
				nCurrentWidth -= GetProntoCharacterPixelWidth(bLarge, szProntoBuffer[nProntoStringLength - 1]);
				szProntoBuffer[nProntoStringLength - 1] = '\0';
				nProntoStringLength--;
			}

			SetPronto(szProntoBuffer);

			delete [] szProntoBuffer;
		}
	}
}
