// ProntoString.h: interface for the CProntoString class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PRONTOSTRING_H__7FD1815A_4914_493E_AFFC_81691F0D346E__INCLUDED_)
#define AFX_PRONTOSTRING_H__7FD1815A_4914_493E_AFFC_81691F0D346E__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CProntoString  
{
public:
	CProntoString();
	virtual ~CProntoString();

	static void InitStatic(); // Must be called at the start of the program.

	const char *GetEscaped(bool bNeverNull) const;
	const char *GetPronto(bool bNeverNull) const;
	const char *GetPrintable(bool bNeverNull) const;

	// Returns <  0 if this <  pPS
	// Returns == 0 if this == pPS
	// Returns >  0 if this >  pPS
	int Compare(const CProntoString *pPS, bool bCaseSensitive) const;

	void SetEscaped(const char *s);
	void SetPronto(const char *s);

	void Reset(void);

	bool IsUnassigned(void) const;
	bool IsTheEmptyString(void) const;
	bool IsANonEmptyString(void) const;

	int CalculateProntoStringPixelWidth(bool bLarge) const;
	void TruncateToPixelWidth(bool bLarge, int nMaxWidth);

protected:
	mutable char *m_szEscaped;
	mutable char *m_szPronto;
	mutable char *m_szPrintable;

protected:
	static int s_fontCharWidths8[255];
	static int s_fontCharWidths10[255];

public:
	// ****************************
	// ** Static Functions Below **
	// ****************************

	/////////////////////////////
	// -- Escaped -> Pronto -- //
	/////////////////////////////

	static bool IsEscapedSequence(const char *s);

	// Get length of equivalent Pronto string.
	static size_t StrLenEscapedToPronto(const char *s);
	
	// Undefined results if called on anything other than an escaped character sequence.
	static char ConvertEscapedSequenceToProntoCharacter(const char *s);

	static void ConvertEscapedToPronto(char *sOut, size_t outSize, const char *sIn);

	// delete [] the result.
	static char *CopyEscapedToPronto(const char *sIn);

	/////////////////////////////
	// -- Pronto -> Escaped -- //
	/////////////////////////////

	static bool IsProntoCharacter(char c);

	// Get length of equivalent escaped string.
	static size_t StrLenProntoToEscaped(const char *s);

	// sOut must have five bytes of space
	static void ConvertProntoCharacterToEscapedSequence(char *sOut, unsigned char cIn);

	static void ConvertProntoToEscaped(char *sOut, size_t outSize, const char *sIn);

	// delete [] the result.
	static char *CopyProntoToEscaped(const char *sIn);

	///////////////////////////////
	// -- Pronto -> Printable -- //
	///////////////////////////////

	static bool IsPrintableCharacter(char c);

	// delete [] the result.
	static char *CopyProntoToPrintable(const char *sIn);

	////////////////////////////////
	// -- Escaped -> Printable -- //
	////////////////////////////////

	// delete [] the result.
	static char *CopyEscapedToPrintable(const char *sIn);

	////////////////////////
	// -- Pixel Widths -- //
	////////////////////////
	static int GetProntoCharacterPixelWidth(bool bLarge, char c);
	static int CalculateProntoStringPixelWidth(bool bLarge, const char *szString);
};

#endif // !defined(AFX_PRONTOSTRING_H__7FD1815A_4914_493E_AFFC_81691F0D346E__INCLUDED_)
