// ReportDlg.cpp : implementation file
//

#include "stdafx.h"
#include "supernudelist.h"
#include "ReportDlg.h"
#include "SlotReport.h"
#include "SNVolume.h"
#include "SNConfig.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CReportDlg dialog


CReportDlg::CReportDlg(CSlotReport *pSlotReport, CWnd* pParent)
	: CSuckLessDialog(CReportDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CReportDlg)
	//}}AFX_DATA_INIT

	m_pSlotReport = pSlotReport;

	m_bInitDone = false;
}


void CReportDlg::DoDataExchange(CDataExchange* pDX)
{
	CSuckLessDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CReportDlg)
	DDX_Control(pDX, IDC_TAB_REPORTS, m_ctrlTabReports);
	DDX_Control(pDX, IDC_LIST_REPORT, m_ctrlListReport);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CReportDlg, CSuckLessDialog)
	//{{AFX_MSG_MAP(CReportDlg)
	ON_NOTIFY(TCN_SELCHANGE, IDC_TAB_REPORTS, OnSelchangeTabReports)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CReportDlg message handlers

BOOL CReportDlg::OnInitDialog() 
{
	CSuckLessDialog::OnInitDialog();
	
	this->setWindowIcon(IDR_MAINFRAME);

	// -- Initial contents --

	if (NULL != g_pProntoFont)
	{
		m_ctrlListReport.SetFont(g_pProntoFont);
	}

	m_ctrlListReport.SetExtendedStyle(LVS_EX_FULLROWSELECT);

	if ((TAB_LOWEST		!= m_ctrlTabReports.InsertItem(TAB_LOWEST,		"Lowest"	))\
	||	(TAB_HIGHEST	!= m_ctrlTabReports.InsertItem(TAB_HIGHEST,		"Highest"	))\
	||	(TAB_GAPS		!= m_ctrlTabReports.InsertItem(TAB_GAPS,		"Gaps"		))\
	||	(TAB_DUPLICATES	!= m_ctrlTabReports.InsertItem(TAB_DUPLICATES,	"Duplicates"))\
	||	(TAB_ALL		!= m_ctrlTabReports.InsertItem(TAB_ALL,			"All Discs" )))
	{
		MessageBox("Error initialising tab control.",
					SUPERNUDELIST_ERROR_TITLE, MB_OK | MB_ICONEXCLAMATION);
	}
	else
	{
		m_ctrlTabReports.SetCurSel(TAB_LOWEST);
		populateListLowest();
	}

	// -- Control setup --

	addControlInfo(IDC_TAB_REPORTS);
	addControlInfo(IDC_LIST_REPORT);
	addControlInfo(IDOK);

	setControlRelationship(IDC_TAB_REPORTS,	IDD,				CControlInfo::ACOMMON_RESIZEHEIGHT_RESIZEWIDTH);
	setControlRelationship(IDC_LIST_REPORT,	IDC_TAB_REPORTS,	CControlInfo::ACOMMON_RESIZEHEIGHT_RESIZEWIDTH);

	setControlRelationship(IDOK,			IDD,				CControlInfo::ACOMMON_BOTTOM_RIGHT);

	controlAndFrameInfoIsSetup();

	setMinXToCurrent();
	setMinYToCurrent();

	m_bInitDone = true;

	m_ctrlTabReports.SetFocus();

	this->setRememberWindowPosition(SN_REG_PATH_WINDOWPOS, SN_REG_VALUE_WINDOWPOS_REPORT);

	return FALSE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CReportDlg::OnSelchangeTabReports(NMHDR* pNMHDR, LRESULT* pResult) 
{
	switch(m_ctrlTabReports.GetCurSel())
	{
	default:
		break;
	case(TAB_LOWEST):
		populateListLowest();
		break;
	case(TAB_HIGHEST):
		populateListHighest();
		break;
	case(TAB_GAPS):
		populateListGaps();
		break;
	case(TAB_DUPLICATES):
		populateListDuplicates();
		break;
	case(TAB_ALL):
		populateListAll();
		break;
	}

	*pResult = 0;
}

void CReportDlg::OnSizeHookEnd(void)
{
	if (m_bInitDone)
	{
		CHeaderCtrl *pHeaderCtrl;

		if (NULL != (pHeaderCtrl = m_ctrlListReport.GetHeaderCtrl()))
		{
			int headItems = pHeaderCtrl->GetItemCount();

			if (0 != headItems)
			{
				m_ctrlListReport.SetColumnWidth(headItems - 1, LVSCW_AUTOSIZE_USEHEADER);
			}
		}
	}
}

void CReportDlg::populateListLowest(void)
{
	m_ctrlListReport.DeleteAllItems();
	m_ctrlListReport.NudelResetSortColumns();
	m_colDiscsMap.clear();

	int nColumnCount = m_ctrlListReport.GetHeaderCtrl()->GetItemCount();

	// Delete all of the columns.
	for (int i = 0; i < nColumnCount; i++)
	{
	   m_ctrlListReport.DeleteColumn(0);
	}

	m_ctrlListReport.InsertColumn(LIST_COL_DEVICE,	"Player");
	m_ctrlListReport.InsertColumn(LIST_COL_NUMBER,	"Number", LVCFMT_RIGHT);
	m_ctrlListReport.InsertColumn(LIST_COL_DISC1,	"Disc");

	m_ctrlListReport.NudelSortSetNumberColumn(LIST_COL_NUMBER); // cool for ranges, too.
	m_ctrlListReport.NudelSortSetCallbackColumn(LIST_COL_DISC1, nudelSLSortCallbackDisc1Column, this);

	int idx = 0;

	for (CSlotReport::const_devIter devIter = m_pSlotReport->DevBegin(); devIter != m_pSlotReport->DevEnd(); devIter++)
	{
		const vector< list<const CSNDisc *> > *pVecOfDiscLists = &(devIter->second.m_vecOfDiscLists);

		for (vector< list<const CSNDisc *> >::const_iterator discListIter = pVecOfDiscLists->begin(); discListIter != pVecOfDiscLists->end(); discListIter++)
		{
			if (discListIter->size() > 0)
			{
				if (! addDiscToList(&idx, devIter->first.c_str(), *(discListIter->begin()), NULL) )
				{
					break;
				}

				// We're only interested in the first non-empty slot.
				break; // Break out of this device and carry on with more devices.
			}
		}
	}

	m_ctrlListReport.SetColumnWidth(LIST_COL_DEVICE,	LVSCW_AUTOSIZE_USEHEADER);
	m_ctrlListReport.SetColumnWidth(LIST_COL_NUMBER,	LVSCW_AUTOSIZE_USEHEADER);
	m_ctrlListReport.SetColumnWidth(LIST_COL_DISC1,		LVSCW_AUTOSIZE_USEHEADER);

	m_ctrlListReport.SortItemsNudel(LIST_COL_DEVICE, false);

	noteEmptyList("No discs in any player.");
}

void CReportDlg::populateListHighest(void)
{
	m_ctrlListReport.DeleteAllItems();
	m_ctrlListReport.NudelResetSortColumns();
	m_colDiscsMap.clear();

	int nColumnCount = m_ctrlListReport.GetHeaderCtrl()->GetItemCount();

	// Delete all of the columns.
	for (int i = 0; i < nColumnCount; i++)
	{
	   m_ctrlListReport.DeleteColumn(0);
	}

	m_ctrlListReport.InsertColumn(LIST_COL_DEVICE,	"Player");
	m_ctrlListReport.InsertColumn(LIST_COL_NUMBER,	"Number", LVCFMT_RIGHT);
	m_ctrlListReport.InsertColumn(LIST_COL_DISC1,	"Disc");

	m_ctrlListReport.NudelSortSetNumberColumn(LIST_COL_NUMBER); // cool for ranges, too.
	m_ctrlListReport.NudelSortSetCallbackColumn(LIST_COL_DISC1, nudelSLSortCallbackDisc1Column, this);

	int idx = 0;

	for (CSlotReport::const_devIter devIter = m_pSlotReport->DevBegin(); devIter != m_pSlotReport->DevEnd(); devIter++)
	{
		const vector< list<const CSNDisc *> > *pVecOfDiscLists = &(devIter->second.m_vecOfDiscLists);

		for (vector< list<const CSNDisc *> >::const_reverse_iterator discListIterR = pVecOfDiscLists->rbegin(); discListIterR != pVecOfDiscLists->rend(); discListIterR++)
		{
			if (discListIterR->size() > 0)
			{
				if (! addDiscToList(&idx, devIter->first.c_str(), *(discListIterR->begin()), NULL) )
				{
					break;
				}

				// We're only interested in the last non-empty slot.
				break; // Break out of this device and carry on with more devices.
			}
		}
	}

	m_ctrlListReport.SetColumnWidth(LIST_COL_DEVICE,	LVSCW_AUTOSIZE_USEHEADER);
	m_ctrlListReport.SetColumnWidth(LIST_COL_NUMBER,	LVSCW_AUTOSIZE_USEHEADER);
	m_ctrlListReport.SetColumnWidth(LIST_COL_DISC1,		LVSCW_AUTOSIZE_USEHEADER);

	m_ctrlListReport.SortItemsNudel(LIST_COL_DEVICE, false);

	noteEmptyList("No discs in any player.");
}

void CReportDlg::populateListGaps(void)
{
	m_ctrlListReport.DeleteAllItems();
	m_ctrlListReport.NudelResetSortColumns();
	m_colDiscsMap.clear();

	int nColumnCount = m_ctrlListReport.GetHeaderCtrl()->GetItemCount();

	// Delete all of the columns.
	for (int i = 0; i < nColumnCount; i++)
	{
	   m_ctrlListReport.DeleteColumn(0);
	}

	m_ctrlListReport.InsertColumn(LIST_COL_DEVICE,	"Player");
	m_ctrlListReport.InsertColumn(LIST_COL_NUMBER,	"Range", LVCFMT_RIGHT);
	m_ctrlListReport.InsertColumn(LIST_COL_DISC1,	"Disc Before");
	m_ctrlListReport.InsertColumn(LIST_COL_DISC2,	"Disc After");

	m_ctrlListReport.NudelSortSetNumberColumn(LIST_COL_NUMBER); // cool for ranges, too.
	m_ctrlListReport.NudelSortSetCallbackColumn(LIST_COL_DISC1, nudelSLSortCallbackDisc1Column, this);
	m_ctrlListReport.NudelSortSetCallbackColumn(LIST_COL_DISC2, nudelSLSortCallbackDisc2Column, this);

	int idx = 0;

	for (CSlotReport::const_devIter devIter = m_pSlotReport->DevBegin(); devIter != m_pSlotReport->DevEnd(); devIter++)
	{
		const vector< list<const CSNDisc *> > *pVecOfDiscLists = &(devIter->second.m_vecOfDiscLists);

		const CSNDisc *pLastDisc = NULL;
		bool bInGap = true;

		for (vector< list<const CSNDisc *> >::const_iterator discListIter = pVecOfDiscLists->begin(); discListIter != pVecOfDiscLists->end(); discListIter++)
		{
			if (discListIter->size() > 0)
			{
				if (bInGap && (NULL != pLastDisc))
				{
					addDiscToList(&idx, devIter->first.c_str(), pLastDisc, *(discListIter->begin()));
				}

				pLastDisc = (*(discListIter->begin()));
				bInGap = false;
			}
			else
			{
				bInGap = true;
			}
		}
	}

	m_ctrlListReport.SetColumnWidth(LIST_COL_DEVICE,	LVSCW_AUTOSIZE_USEHEADER);
	m_ctrlListReport.SetColumnWidth(LIST_COL_NUMBER,	LVSCW_AUTOSIZE_USEHEADER);
	m_ctrlListReport.SetColumnWidth(LIST_COL_DISC1,		LVSCW_AUTOSIZE_USEHEADER);
	m_ctrlListReport.SetColumnWidth(LIST_COL_DISC2,		LVSCW_AUTOSIZE_USEHEADER);

	m_ctrlListReport.SortItemsNudel(LIST_COL_DEVICE, false);

	noteEmptyList("No gaps in any player.");
}

void CReportDlg::populateListDuplicates(void)
{
	m_ctrlListReport.DeleteAllItems();
	m_ctrlListReport.NudelResetSortColumns();
	m_colDiscsMap.clear();

	int nColumnCount = m_ctrlListReport.GetHeaderCtrl()->GetItemCount();

	// Delete all of the columns.
	for (int i = 0; i < nColumnCount; i++)
	{
	   m_ctrlListReport.DeleteColumn(0);
	}

	m_ctrlListReport.InsertColumn(LIST_COL_DEVICE,	"Player");
	m_ctrlListReport.InsertColumn(LIST_COL_NUMBER,	"Number", LVCFMT_RIGHT);
	m_ctrlListReport.InsertColumn(LIST_COL_DISC1,	"Disc");

	m_ctrlListReport.NudelSortSetNumberColumn(LIST_COL_NUMBER); // cool for ranges, too.
	m_ctrlListReport.NudelSortSetCallbackColumn(LIST_COL_DISC1, nudelSLSortCallbackDisc1Column, this);

	int idx = 0;

	for (CSlotReport::const_devIter devIter = m_pSlotReport->DevBegin(); devIter != m_pSlotReport->DevEnd(); devIter++)
	{
		const vector< list<const CSNDisc *> > *pVecOfDiscLists = &(devIter->second.m_vecOfDiscLists);

		for (vector< list<const CSNDisc *> >::const_iterator discListIter = pVecOfDiscLists->begin(); discListIter != pVecOfDiscLists->end(); discListIter++)
		{
			// Check for duplicates.
			if (discListIter->size() > 1)
			{
				for (list<const CSNDisc *>::const_iterator discIter = discListIter->begin(); discIter != discListIter->end(); discIter++)
				{
					if (! addDiscToList(&idx, devIter->first.c_str(), *discIter, NULL) )
					{
						break;
					}
				}
			}
		}
	}

	m_ctrlListReport.SetColumnWidth(LIST_COL_DEVICE,	LVSCW_AUTOSIZE_USEHEADER);
	m_ctrlListReport.SetColumnWidth(LIST_COL_NUMBER,	LVSCW_AUTOSIZE_USEHEADER);
	m_ctrlListReport.SetColumnWidth(LIST_COL_DISC1,		LVSCW_AUTOSIZE_USEHEADER);

	m_ctrlListReport.SortItemsNudel(LIST_COL_DEVICE, false);

	noteEmptyList("No duplicates in any player.");
}

void CReportDlg::populateListAll(void)
{
	m_ctrlListReport.DeleteAllItems();
	m_ctrlListReport.NudelResetSortColumns();
	m_colDiscsMap.clear();

	int nColumnCount = m_ctrlListReport.GetHeaderCtrl()->GetItemCount();

	// Delete all of the columns.
	for (int i = 0; i < nColumnCount; i++)
	{
	   m_ctrlListReport.DeleteColumn(0);
	}

	m_ctrlListReport.InsertColumn(LIST_COL_DEVICE,	"Player");
	m_ctrlListReport.InsertColumn(LIST_COL_NUMBER,	"Number", LVCFMT_RIGHT);
	m_ctrlListReport.InsertColumn(LIST_COL_DISC1,	"Disc");

	m_ctrlListReport.NudelSortSetNumberColumn(LIST_COL_NUMBER); // cool for ranges, too.
	m_ctrlListReport.NudelSortSetCallbackColumn(LIST_COL_DISC1, nudelSLSortCallbackDisc1Column, this);

	int idx = 0;

	for (CSlotReport::const_devIter devIter = m_pSlotReport->DevBegin(); devIter != m_pSlotReport->DevEnd(); devIter++)
	{
		const vector< list<const CSNDisc *> > *pVecOfDiscLists = &(devIter->second.m_vecOfDiscLists);

		for (vector< list<const CSNDisc *> >::const_iterator discListIter = pVecOfDiscLists->begin(); discListIter != pVecOfDiscLists->end(); discListIter++)
		{
			for (list<const CSNDisc *>::const_iterator discIter = discListIter->begin(); discIter != discListIter->end(); discIter++)
			{
				if (! addDiscToList(&idx, devIter->first.c_str(), *discIter, NULL) )
				{
					break;
				}
			}
		}
	}

	m_ctrlListReport.SetColumnWidth(LIST_COL_DEVICE,	LVSCW_AUTOSIZE_USEHEADER);
	m_ctrlListReport.SetColumnWidth(LIST_COL_NUMBER,	LVSCW_AUTOSIZE_USEHEADER);
	m_ctrlListReport.SetColumnWidth(LIST_COL_DISC1,		LVSCW_AUTOSIZE_USEHEADER);

	m_ctrlListReport.SortItemsNudel(LIST_COL_DEVICE, false);

	noteEmptyList("No discs in any player.");
}

void CReportDlg::noteEmptyList(const char *msg)
{
	if (0 == m_ctrlListReport.GetItemCount())
	{
		m_ctrlListReport.NudelResetSortColumns();

		int nColumnCount = m_ctrlListReport.GetHeaderCtrl()->GetItemCount();

		// Delete all of the columns.
		for (int i = 0; i < nColumnCount; i++)
		{
		   m_ctrlListReport.DeleteColumn(0);
		}

		m_ctrlListReport.InsertColumn(0, "Note");

		m_ctrlListReport.InsertItem(0, msg);

		m_ctrlListReport.SetColumnWidth(0, LVSCW_AUTOSIZE_USEHEADER);
	}
}

bool CReportDlg::addDiscToList(int *pIdx, const char *device, const CSNDisc *pDisc1, const CSNDisc *pDisc2)
{
	int insIdx = m_ctrlListReport.InsertItem((*pIdx)++, device);

	CString numStr;

	if (NULL == pDisc2)
	{
		numStr.Format("%d", pDisc1->GetDiscNumber());
	}
	else
	{
		int lowerNumber = pDisc1->GetDiscNumber() + 1;
		int higherNumber = pDisc2->GetDiscNumber() - 1;

		if (lowerNumber == higherNumber)
		{
			numStr.Format("%d", lowerNumber);
		}
		else
		{
			numStr.Format("%d - %d", lowerNumber, higherNumber);
		}
	}

	if (((-1) == insIdx) \
	||	(!m_ctrlListReport.SetItemText(insIdx, LIST_COL_NUMBER, numStr)) \
	||	(!m_ctrlListReport.SetItemText(insIdx, LIST_COL_DISC1, pDisc1->GetFullPathPronto())) \
	||	((NULL != pDisc2) && (!m_ctrlListReport.SetItemText(insIdx, LIST_COL_DISC2, pDisc2->GetFullPathPronto()))) )
	{
		MessageBox("Error populating list", SUPERNUDELIST_ERROR_TITLE, MB_ICONSTOP|MB_OK);
		return(false);
	}
	else
	{
		m_ctrlListReport.SetItemData(insIdx, (DWORD)(*pIdx)); // Just needs to be unique for sorting.

		m_colDiscsMap.insert(pair<int, pair<const CSNDisc *, const CSNDisc *> >(*pIdx, pair<const CSNDisc *, const CSNDisc *>(pDisc1, pDisc2)));
	}

	return(true);
}

int CALLBACK CReportDlg::nudelSLSortCallbackDisc1Column(CSortableListCtrl *pList,
															LPARAM lParam1, LPARAM lParam2,
															int idx1, int idx2, void *pVoid)
{
	CReportDlg *pThis = static_cast<CReportDlg *>(pVoid);

	map< int, pair<const CSNDisc *, const CSNDisc *> >::const_iterator mIter1;
	map< int, pair<const CSNDisc *, const CSNDisc *> >::const_iterator mIter2;

	mIter1 = pThis->m_colDiscsMap.find(lParam1);
	mIter2 = pThis->m_colDiscsMap.find(lParam2);

	if (mIter1 == pThis->m_colDiscsMap.end())
	{
		if (mIter2 == pThis->m_colDiscsMap.end())
		{
			return(0);
		}
		else
		{
			return(-1);
		}
	}
	else if (mIter2 == pThis->m_colDiscsMap.end())
	{
		return(1);
	}

	const CSNDisc *pDisc1 = mIter1->second.first;
	const CSNDisc *pDisc2 = mIter2->second.first;

	const CSNArtist *pArtist1 = pDisc1->GetParent();
	const CSNArtist *pArtist2 = pDisc2->GetParent();

	const CSNVolume *pVolume1 = pArtist1->GetParent();
	const CSNVolume *pVolume2 = pArtist2->GetParent();

	int result = pVolume1->CompareForSort(pVolume2);

	if (0 == result)
	{
		result = pArtist1->CompareForSort(pArtist2);
	}

	if (0 == result)
	{
		result = pDisc1->CompareForSort(pDisc2);
	}

	return(result);
}

int CALLBACK CReportDlg::nudelSLSortCallbackDisc2Column(CSortableListCtrl *pList,
															LPARAM lParam1, LPARAM lParam2,
															int idx1, int idx2, void *pVoid)
{
	CReportDlg *pThis = static_cast<CReportDlg *>(pVoid);

	map< int, pair<const CSNDisc *, const CSNDisc *> >::const_iterator mIter1;
	map< int, pair<const CSNDisc *, const CSNDisc *> >::const_iterator mIter2;

	mIter1 = pThis->m_colDiscsMap.find(lParam1);
	mIter2 = pThis->m_colDiscsMap.find(lParam2);

	if (mIter1 == pThis->m_colDiscsMap.end())
	{
		if (mIter2 == pThis->m_colDiscsMap.end())
		{
			return(0);
		}
		else
		{
			return(-1);
		}
	}
	else if (mIter2 == pThis->m_colDiscsMap.end())
	{
		return(1);
	}

	const CSNDisc *pDisc1 = mIter1->second.second;
	const CSNDisc *pDisc2 = mIter2->second.second;

	const CSNArtist *pArtist1 = pDisc1->GetParent();
	const CSNArtist *pArtist2 = pDisc2->GetParent();

	const CSNVolume *pVolume1 = pArtist1->GetParent();
	const CSNVolume *pVolume2 = pArtist2->GetParent();

	int result = pVolume1->CompareForSort(pVolume2);

	if (0 == result)
	{
		result = pArtist1->CompareForSort(pArtist2);
	}

	if (0 == result)
	{
		result = pDisc1->CompareForSort(pDisc2);
	}

	return(result);
}
