// SNArtist.cpp: implementation of the CSNArtist class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SNConfig.h"
#include "SNArtist.h"
#include "SNVolume.h"
#include "SNCSVLine.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSNArtist::CSNArtist()
{
	m_pParentVolume = NULL;
}

CSNArtist::~CSNArtist()
{

}

const char *CSNArtist::getTypeName(void) const
{
	return("artist");
}

bool CSNArtist::IsValid(CTypedError *pTError) const
{
	if (m_name.IsUnassigned())
	{
		CTypedError::SafeSetMsgError(pTError, "%s has no name", getTypeName());
		return(false);
	}

	if (m_name.IsTheEmptyString())
	{
		CTypedError::SafeSetMsgError(pTError, "%s has no name", getTypeName());
		return(false);
	}

	if (NULL == m_pParentVolume)
	{
		CTypedError::SafeSetMsgError(pTError, "%s not associated with a list", getTypeName());
		return(false);
	}

	return(true);
}

// Returns < 0 if this <  pArtist
// Returns   0 if this == pArtist
// Returns > 0 if this >  pArtist
int CSNArtist::CompareForSort(const CSNArtist *pArtist) const
{
	if (NULL == pArtist)
	{
		return(1);
	}

	const CProntoString *pPSLeft = this->GetSortAsNameOrName();
	const CProntoString *pPSRight = pArtist->GetSortAsNameOrName();

	const char *stringLeft = pPSLeft->GetPronto(true);
	const char *stringRight = pPSRight->GetPronto(true);

	char alphaGroupLeft = GetAlphabetPanelLetterGroup(stringLeft);
	char alphaGroupRight = GetAlphabetPanelLetterGroup(stringRight);

	if (('#' == alphaGroupLeft) && ('#' != alphaGroupRight))
	{
		return(-1);
	}
	else if (('#' != alphaGroupLeft) && ('#' == alphaGroupRight))
	{
		return(1);
	}

	return(pPSLeft->Compare(pPSRight, false));
}

// Set does NOT copy/set the child list of discs.
void CSNArtist::Set(const CSNArtist *pArtist)
{
	if (this != pArtist)
	{
		m_name.SetPronto(pArtist->m_name.GetPronto(false));
		m_shortName.SetPronto(pArtist->m_shortName.GetPronto(false));
		m_sortAsName.SetPronto(pArtist->m_sortAsName.GetPronto(false));

//		m_pParentVolume = pArtist->m_pParentVolume;
	}
}

CSNVolume *CSNArtist::GetParent(void)
{
	return(m_pParentVolume);
}

const CSNVolume *CSNArtist::GetParent(void) const
{
	return(m_pParentVolume);
}

CProntoString *CSNArtist::GetName(void)
{
	return(&m_name);
}

const CProntoString *CSNArtist::GetName(void) const
{
	return(&m_name);
}

CProntoString *CSNArtist::GetShortName(void)
{
	return(&m_shortName);
}

const CProntoString *CSNArtist::GetShortName(void) const
{
	return(&m_shortName);
}

const CProntoString *CSNArtist::GetShortNameOrName(void) const
{
	if (m_shortName.IsANonEmptyString())
	{
		return(&m_shortName);
	}
	else
	{
		return(&m_name);
	}
}

CProntoString *CSNArtist::GetSortAsName(void)
{
	return(&m_sortAsName);
}

const CProntoString *CSNArtist::GetSortAsName(void) const
{
	return(&m_sortAsName);
}

const CProntoString *CSNArtist::GetSortAsNameOrName(void) const
{
	if (m_sortAsName.IsANonEmptyString())
	{
		return(&m_sortAsName);
	}
	else
	{
		return(&m_name);
	}
}

void CSNArtist::ResolveDevices(set<CSNDisc *> *pDiscSet, const CSNDeviceList *pDeviceList)
{
	for (CSNDisc *pDisc = GetNonConst(GetFirst()); NULL != pDisc; pDisc = GetNonConst(GetNext(pDisc)))
	{
		pDisc->ResolveDevice(pDiscSet, pDeviceList);
	}
}

void CSNArtist::UnlinkDevice(const CSNDevice *pDevice)
{
	for (CSNDisc *pDisc = GetNonConst(GetFirst()); NULL != pDisc; pDisc = GetNonConst(GetNext(pDisc)))
	{
		pDisc->UnlinkDevice(pDevice);
	}
}

int CSNArtist::GetHighestDiscNumber(const CSNDevice *pDevice) const
{
	int result = 0;
	int candidate;

	for (const CSNDisc *pDisc = GetFirst(); NULL != pDisc; pDisc = GetNext(pDisc))
	{
		if (pDevice == pDisc->GetDevice())
		{
			candidate = pDisc->GetDiscNumber();

			if (result < candidate)
			{
				result = candidate;
			}
		}
	}

	return(result);
}

unsigned int CSNArtist::GetNumElements(void) const
{
	return(CSNList<CSNDisc>::GetNumElements());
}

const CSNDisc *CSNArtist::GetFirst(void) const
{
	return(CSNList<CSNDisc>::GetFirst());
}

const CSNDisc *CSNArtist::GetNext(const CSNDisc *pCurrent) const
{
	return(CSNList<CSNDisc>::GetNext(pCurrent));
}

CSNDisc *CSNArtist::GetNonConst(const CSNDisc *pDisc)
{
	return(CSNList<CSNDisc>::GetNonConst(pDisc));
}

const CSNDisc *CSNArtist::GetByNamePronto(const char *szNamePronto, const CSNDisc *pFindAfter) const
{
	return(CSNList<CSNDisc>::GetByNamePronto(szNamePronto, pFindAfter));
}

CString CSNArtist::SNUIT_GetLabel(void) const
{
	CString csResult = m_name.GetPronto(true);

	if (! (m_shortName.IsUnassigned() || m_shortName.IsTheEmptyString()) )
	{
		csResult += " (";
		csResult += m_shortName.GetPronto(true);
		csResult += ")";
	}

	if (! (m_sortAsName.IsUnassigned() || m_sortAsName.IsTheEmptyString()) )
	{
		csResult += " (Sort As: ";
		csResult += m_sortAsName.GetPronto(true);
		csResult += ")";
	}

	return(csResult);
}

int CSNArtist::SNUIT_GetImageIdx(ISNUITreeDlg *pTreeDlg) const
{
	return(pTreeDlg->SNUIT_GetImageIdxArtist());
}

CString CSNArtist::SNUIT_GetDeleteConfirmMessage(void) const
{
	CString msg;

	unsigned int discCount = GetNumElements();

	if (0 != discCount)
	{
		msg.Format("Delete artist \"%s\" containing %u disc%s?",
					GetName()->GetPrintable(true),
					discCount, 1 == discCount   ? "" : "s");
	}
	// else empty message.

	return(msg);
}

bool CSNArtist::SNUIT_AddChildrenToTree(ISNUITreeDlg *pTreeDlg)
{
	bool bResult = true;

	for (CSNDisc *pDisc = GetNonConst(GetLast()); pDisc != NULL; pDisc = GetNonConst(GetPrevious(pDisc)))
	{
		// Insert at the top of the list.
		if (! pTreeDlg->SNUIT_AddTreeItem(pDisc, this, NULL) )
		{
			bResult = false;
			break;
		}
	}

	return(bResult);
}

const char *CSNArtist::SNUIT_GetAddButtonName(void) const
{
	return("Add Disc");
}

const char *CSNArtist::SNUIT_GetEditButtonName(void) const
{
	return("Edit Artist");
}

const char *CSNArtist::SNUIT_GetDeleteButtonName(void) const
{
	return("Delete Artist");
}

const char *CSNArtist::SNUIT_GetSortButtonName(void) const
{
	return("Sort Artists");
}

bool CSNArtist::SNUIT_GetKeepSorted(void) const
{
	return(m_pParentVolume->GetSort());
}

void CSNArtist::SNUIT_SetKeepSorted(ISNUITreeDlg *pTreeDlg, bool bKeepSorted)
{
	m_pParentVolume->SetSort(bKeepSorted);

	if ((bKeepSorted) && (NULL != pTreeDlg))
	{
		set<ISNUITreeItem *> *pExpandSet = pTreeDlg->SNUIT_GetExpandedItems(m_pParentVolume);

		// Remove from the tree the children of our parent..
		pTreeDlg->SNUIT_RemoveTreeItems(m_pParentVolume, true);

		// Add children back again, this time sorted.
		m_pParentVolume->SNUIT_AddChildrenToTree(pTreeDlg);

		if (NULL != pExpandSet)
		{
			pTreeDlg->SNUIT_ExpandedItems(pExpandSet);
			delete pExpandSet;
		}
	}

	if (NULL != pTreeDlg)
	{
		pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(this);
	}
}

void CSNArtist::SNUIT_PressedSortButton(ISNUITreeDlg *pTreeDlg)
{
	if (! SNUIT_GetKeepSorted() )
	{
		SNUIT_SetKeepSorted(pTreeDlg, true);
		SNUIT_SetKeepSorted(pTreeDlg, false);
	}
}

bool CSNArtist::SNUIT_GetEditButtonActivation(void) const
{
	return(true);
}

bool CSNArtist::SNUIT_GetDeleteButtonActivation(void) const
{
	return(true);
}

bool CSNArtist::SNUIT_GetUpButtonActivation(void) const
{
	return(SNUIT_GetSortButtonActivation() && (this != m_pParentVolume->GetFirst()));
}

bool CSNArtist::SNUIT_GetDownButtonActivation(void) const
{
	return(SNUIT_GetSortButtonActivation() && (this != m_pParentVolume->GetLast()));
}

bool CSNArtist::SNUIT_GetSortButtonActivation(void) const
{
	return(! SNUIT_GetKeepSorted() );
}

bool CSNArtist::SNUIT_GetKeepSortedCheckActivation(void) const
{
	return(true);
}

void CSNArtist::SNUIT_PressedUpButton(ISNUITreeDlg *pTreeDlg)
{
	set<ISNUITreeItem *> *pExpandSet = NULL;

	if (NULL != pTreeDlg)
	{
		pExpandSet = pTreeDlg->SNUIT_GetExpandedItems(m_pParentVolume);

		if (! pTreeDlg->SNUIT_RemoveTreeItems(this, false) )
		{
			return;
		}
	}

	m_pParentVolume->MoveUp(this);

	if (NULL != pTreeDlg)
	{
		if (pTreeDlg->SNUIT_AddTreeItem(this, m_pParentVolume, m_pParentVolume->GetPrevious(this)))
		{
			this->SNUIT_AddChildrenToTree(pTreeDlg);

			if (NULL != pExpandSet)
			{
				pTreeDlg->SNUIT_ExpandedItems(pExpandSet);
			}

			pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(this);
		}
	}

	delete pExpandSet;
}

void CSNArtist::SNUIT_PressedDownButton(ISNUITreeDlg *pTreeDlg)
{
	set<ISNUITreeItem *> *pExpandSet = NULL;

	if (NULL != pTreeDlg)
	{
		pExpandSet = pTreeDlg->SNUIT_GetExpandedItems(m_pParentVolume);

		if (! pTreeDlg->SNUIT_RemoveTreeItems(this, false) )
		{
			return;
		}
	}

	m_pParentVolume->MoveDown(this);

	if (NULL != pTreeDlg)
	{
		if (pTreeDlg->SNUIT_AddTreeItem(this, m_pParentVolume, m_pParentVolume->GetPrevious(this)))
		{
			this->SNUIT_AddChildrenToTree(pTreeDlg);

			if (NULL != pExpandSet)
			{
				pTreeDlg->SNUIT_ExpandedItems(pExpandSet);
			}

			pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(this);
		}
	}

	delete pExpandSet;
}

void CSNArtist::SNUIT_PressedDeleteButton(ISNUITreeDlg *pTreeDlg)
{
	if (NULL != pTreeDlg)
	{
		if (! pTreeDlg->SNUIT_RemoveTreeItems(this, false) )
		{
			return;
		}
		else
		{
			const ISNUITreeItem *tiSelect = m_pParentVolume->GetPrevious(this);

			if (NULL == tiSelect)
			{
				tiSelect = m_pParentVolume->GetNext(this);
			}

			if (NULL == tiSelect)
			{
				tiSelect = m_pParentVolume;
			}

			pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(tiSelect);
		}
	}

	m_pParentVolume->RemoveAndDelete(this);

	// NOTE: This object is now deleted and we must not access members.
}

void CSNArtist::SNUIT_PressedAddButton(ISNUITreeDlg *pTreeDlg)
{
	if (NULL != pTreeDlg)
	{
		const CSNDisc *pLastDisc = GetLast();

		const CSNDevice *pDevice = NULL;

		if (NULL != pLastDisc)
		{
			// Set device to the same as that used by the last disc in this artist.
			pDevice = pLastDisc->GetDevice();
		}
		else
		{
			// Get any device used by a disc in this artist's volume,
			// or leave it NULL if no devices used.
			for (const CSNArtist *pArtist = m_pParentVolume->GetFirst();
				 (NULL == pDevice) && (NULL != pArtist);
				 pArtist = m_pParentVolume->GetNext(pArtist))
			{
				for (const CSNDisc *pDisc = pArtist->GetFirst(); NULL != pDisc; pDisc = pArtist->GetNext(pDisc))
				{
					if (NULL != (pDevice = pDisc->GetDevice()))
					{
						break;
					}
				}
			}
		}

		pTreeDlg->SNUIT_AsyncAddDisc(m_pParentVolume->m_pParentLibrary, m_pParentVolume,
										this, pLastDisc, pDevice);
	}
}

void CSNArtist::SNUIT_PressedEditButton(ISNUITreeDlg *pTreeDlg)
{
	if (NULL != pTreeDlg)
	{
		pTreeDlg->SNUIT_AsyncEditArtist(m_pParentVolume, this);
	}
}

bool CSNArtist::SNUIT_GetExpandParentInitial(void) const
{
	return(true);
}

// You can have a CONST artist from a ISNUITreeArtist, but not a non-const one.
const CSNArtist *CSNArtist::SNUIT_GetRealArtist(void) const
{
	return(this);
}

const CSNDisc *CSNArtist::SNUIT_GetLastDisc(void)
{
	return(GetLast());
}

const CSNDisc *CSNArtist::SNUIT_AddBelowAndOwnDisc(ISNUITreeDlg *pTreeDlg, CTypedError *pTError,
													CSNDisc *pInDisc,
													const CSNDisc *pToAddBelow)
{
	pInDisc->m_pParentArtist = this;

	CSNDisc *pResult = GetNonConst(AddBelowAndOwn(pTError, pInDisc, pToAddBelow));

	if (NULL != pResult)
	{
		// Do NOT null out pResult if anything goes wrong after now.

		if (NULL != pTreeDlg)
		{
			// GetPrevious call needed since the list may be sorted and pAddBelow ignored.
			if (pTreeDlg->SNUIT_AddTreeItem(pResult, this, GetPrevious(pResult)))
			{
				pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(pResult);
			}
		}
	}

	return(pResult);
}

const CSNDisc *CSNArtist::SNUIT_UpdateDisc(ISNUITreeDlg *pTreeDlg, CTypedError *pTError,
											CSNDisc *pInDisc,
											const CSNDisc *pToUpdate)
{
	pInDisc->m_pParentArtist = this;

	const CSNDisc *pOldPrevious = GetPrevious(pToUpdate);

	CSNDisc *pResult = GetNonConst(Update(pTError, pInDisc, pToUpdate));

	if (NULL != pResult)
	{
		// Do NOT null out pResult if anything goes wrong after now.

		if (NULL != pTreeDlg)
		{
			const CSNDisc *pNewPrevious = GetPrevious(pResult);

			if ((!pResult->SNUIT_GetKeepSorted()) \
			||	(pNewPrevious == pOldPrevious))
			{
				// Item's position in the list hasn't changed (fast).
				pTreeDlg->SNUIT_UpdateLabel(pResult);
			}
			else
			{
				// Item's position in the list has changed (slow).
				if (pTreeDlg->SNUIT_RemoveTreeItems(pResult, false))
				{
					if (pTreeDlg->SNUIT_AddTreeItem(pResult, this, GetPrevious(pResult)))
					{
						pResult->SNUIT_AddChildrenToTree(pTreeDlg);

						pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(pResult);
					}
				}
			}
		}
	}

	return(pResult);
}

const CSNDisc *CSNArtist::SNUIT_UpdateDiscAndMoveToAnotherArtist(ISNUITreeDlg *pTreeDlg,
														CTypedError *pTError,
														const CSNArtist *pTargetArtist,
														CSNDisc *pInDisc,
														const CSNDisc *pToUpdate)
{
	CSNDisc *pResult = NULL;

	CSNArtist *pNonConstTargetArtist = m_pParentVolume->GetNonConst(pTargetArtist);

	if (NULL == pNonConstTargetArtist)
	{
		CTypedError::SafeSetMsgError(pTError, "Can only move discs between sibling artists");
	}
	else
	{
		pInDisc->m_pParentArtist = this;

		pResult = pNonConstTargetArtist->GetNonConst(UpdateAndMove(pTError, pNonConstTargetArtist, pInDisc,
														pToUpdate, pTargetArtist->GetLast()));

		if (NULL != pResult)
		{
			pResult->m_pParentArtist = pNonConstTargetArtist;

			// Do NOT null out pResult if anything goes wrong after now.

			if (NULL != pTreeDlg)
			{
				if (pTreeDlg->SNUIT_RemoveTreeItems(pResult, false))
				{
					if (pTreeDlg->SNUIT_AddTreeItem(pResult, pNonConstTargetArtist, pTargetArtist->GetPrevious(pResult)))
					{
						pResult->SNUIT_AddChildrenToTree(pTreeDlg);

						pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(pResult);
					}
				}
			}
		}
	}

	return(pResult);
}

xmlNodePtr CSNArtist::AddToXML(xmlNodePtr parentNode) const
{
	xmlNodePtr artistNode = xmlNewChild(parentNode, NULL, SNXML_NODE_ARTIST, NULL);

	xmlSetProp(artistNode, SNXML_PROP_NAME,	(const xmlChar *)GetName()->GetEscaped(true));

	if (GetShortName()->IsANonEmptyString())
	{
		xmlSetProp(artistNode, SNXML_PROP_SHORTNAME, (const xmlChar *)GetShortName()->GetEscaped(true));
	}

	if (GetSortAsName()->IsANonEmptyString())
	{
		xmlSetProp(artistNode, SNXML_PROP_SORTASNAME, (const xmlChar *)GetSortAsName()->GetEscaped(true));
	}

	xmlSetProp(artistNode, SNXML_PROP_KEEPCHILDRENSORTED, GetSort() ? SNXML_VALUE_1 : SNXML_VALUE_0);

	for (const CSNDisc *pDisc = GetFirst(); NULL != pDisc; pDisc = GetNext(pDisc))
	{
		pDisc->AddToXML(artistNode);
	}

	return(artistNode);
}

bool CSNArtist::BuildFromXML(CTypedError *pTError, CStringList *pSLErrorDetails, xmlNodePtr pNode)
{
	bool bResult = true;

	xmlChar *pXC;

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_NAME)))
	{
		GetName()->SetEscaped((const char *)pXC);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_SHORTNAME)))
	{
		GetShortName()->SetEscaped((const char *)pXC);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_SORTASNAME)))
	{
		GetSortAsName()->SetEscaped((const char *)pXC);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_KEEPCHILDRENSORTED)))
	{
		int i = atoi((const char *)pXC);
		SetSort(0 != i);
		xmlFree(pXC);
	}

	for (xmlNodePtr pNodeChild = pNode->children; NULL != pNodeChild; pNodeChild = pNodeChild->next)
	{
		if ((!xmlIsBlankNode(pNodeChild)) && (NULL != pNodeChild->name))
		{
			if (0 == strcmp((const char *)SNXML_NODE_DISC, (const char *)pNodeChild->name))
			{
				CSNDisc *pDisc = new CSNDisc();

				pDisc->m_pParentArtist = this;

				if ((pDisc->BuildFromXML(pTError, pSLErrorDetails, pNodeChild))	\
				&&	(NULL != AddBelowAndOwn(pTError, pDisc, NULL))				)
				{
					pDisc = NULL; // Owned.
				}
				else
				{
					delete pDisc;
					bResult = false;
					break;
				}
			}
		}
	}

	return(bResult);
}

bool CSNArtist::AddToCSVFile(CSNCSVLine *pSNCSVLine, CCSVFile *pCSVFile) const
{
	bool bAddedAnyStuff = false;
	
	if (pSNCSVLine->SetArtistName(      m_name.GetPronto(true)))		{ bAddedAnyStuff = true; }
	if (pSNCSVLine->SetArtistShortName( m_shortName.GetPronto(true)))	{ bAddedAnyStuff = true; }
	if (pSNCSVLine->SetArtistSortAsName(m_sortAsName.GetPronto(true)))	{ bAddedAnyStuff = true; }

	bool bChildrenAddedAnyStuff = false;

	if (!pSNCSVLine->IsReady())
	{
		for (const CSNDisc *pDisc = GetFirst(); NULL != pDisc; pDisc = GetNext(pDisc))
		{
			CSNCSVLine lineCopy = *pSNCSVLine;
		
			if (pDisc->AddToCSVFile(&lineCopy, pCSVFile))
			{
				bChildrenAddedAnyStuff = true;
			}
		}
	}

	if ((!bChildrenAddedAnyStuff) && (bAddedAnyStuff))
	{
		pSNCSVLine->AddToCSVFile(pCSVFile);
	}

	return(bAddedAnyStuff || bChildrenAddedAnyStuff);
}

bool CSNArtist::BuildFromCSVLine(CTypedError *pTError, const CSNCSVLine *pLineCopy)
{
	bool bResult = true;

	if (pLineCopy->GetArtistShortName())	m_shortName.SetPronto(pLineCopy->GetArtistShortName());
	if (pLineCopy->GetArtistSortAsName())	m_sortAsName.SetPronto(pLineCopy->GetArtistSortAsName());

	if (pLineCopy->GetDiscName())
	{
		CSNDisc *pDisc = this->GetNonConst(this->GetByNamePronto(pLineCopy->GetDiscName(), NULL));

		if (NULL == pDisc)
		{
			CSNDisc *pDiscNew = new CSNDisc();
			pDiscNew->GetName()->SetPronto(pLineCopy->GetDiscName());

			pDisc = this->GetNonConst(this->SNUIT_AddBelowAndOwnDisc(NULL, pTError, pDiscNew, NULL));

			if (NULL == pDisc)
			{
				delete pDiscNew;
				bResult = false;
			}
		}

		if (bResult && (!pDisc->BuildFromCSVLine(pTError, pLineCopy)))
		{
			bResult = false;
		}
	}

	return(bResult);
}
