// SNArtist.h: interface for the CSNArtist class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SNARTIST_H__0111C6FB_4638_458C_88AD_47C0AAAF9304__INCLUDED_)
#define AFX_SNARTIST_H__0111C6FB_4638_458C_88AD_47C0AAAF9304__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SNDisc.h"

class CSNVolume;
class CSNCSVLine;
class CCSVFile;

class CSNArtist : public ISNUITreeItem, public ISNUITreeArtist, public CSNListObject, protected CSNUpdatableList<CSNDisc>
{
	friend class CSNList<CSNArtist>;
	friend class CSNUpdatableList<CSNArtist>;
	friend class CSNVolume;
	friend class CSNDisc;
	friend class CSNLibrary;

public:
	CSNArtist();
	virtual ~CSNArtist();

	bool IsValid(CTypedError *pTError) const;

	// Returns < 0 if this <  pArtist
	// Returns   0 if this == pArtist
	// Returns > 0 if this >  pArtist
	int CompareForSort(const CSNArtist *pArtist) const;

	// Set does NOT copy/set the child list of discs.
	void Set(const CSNArtist *pArtist);

	CProntoString *GetName(void);
	const CProntoString *GetName(void) const;
	CProntoString *GetShortName(void);
	const CProntoString *GetShortName(void) const;
	CProntoString *GetSortAsName(void);
	const CProntoString *GetSortAsName(void) const;

	const CProntoString *GetShortNameOrName(void) const;
	const CProntoString *GetSortAsNameOrName(void) const;

	xmlNodePtr AddToXML(xmlNodePtr parentNode) const;
	bool BuildFromXML(CTypedError *pTError, CStringList *pSLErrorDetails, xmlNodePtr pNode);

	bool AddToCSVFile(CSNCSVLine *pSNCSVLine, CCSVFile *pCSVFile) const;
	bool BuildFromCSVLine(CTypedError *pTError, const CSNCSVLine *pLineCopy);

	unsigned int GetNumElements(void) const;
	const CSNDisc *GetFirst(void) const;
	const CSNDisc *GetNext(const CSNDisc *pCurrent) const;
	CSNDisc *GetNonConst(const CSNDisc *pDisc);
	const CSNDisc *GetByNamePronto(const char *szNamePronto, const CSNDisc *pFindAfter) const;

	void ResolveDevices(set<CSNDisc *> *pDiscSet, const CSNDeviceList *pDeviceList);
	void UnlinkDevice(const CSNDevice *pDevice);

	int GetHighestDiscNumber(const CSNDevice *pDevice) const;

	CSNVolume *GetParent(void);
	const CSNVolume *GetParent(void) const;

protected:
	// Type name should be lowercase, singular.
	const char *getTypeName(void) const;

public:
	// You can have a CONST artist from a ISNUITreeArtist, but not a non-const one.
	const CSNArtist *SNUIT_GetRealArtist(void) const;

	const CSNDisc *SNUIT_GetLastDisc(void);

	const CSNDisc *SNUIT_AddBelowAndOwnDisc(ISNUITreeDlg *pTreeDlg, CTypedError *pTError,
												CSNDisc *pInDisc,
												const CSNDisc *pToAddBelow);

	const CSNDisc *SNUIT_UpdateDisc(ISNUITreeDlg *pTreeDlg, CTypedError *pTError,
										CSNDisc *pInDisc,
										const CSNDisc *pToUpdate);

	const CSNDisc *SNUIT_UpdateDiscAndMoveToAnotherArtist(ISNUITreeDlg *pTreeDlg,
															CTypedError *pTError,
															const CSNArtist *pTargetArtist,
															CSNDisc *pInDisc,
															const CSNDisc *pToUpdate);

public:
	bool SNUIT_AddChildrenToTree(ISNUITreeDlg *pTreeDlg);
	CString SNUIT_GetLabel(void) const;
	int SNUIT_GetImageIdx(ISNUITreeDlg *pTreeDlg) const;
	CString SNUIT_GetDeleteConfirmMessage(void) const;
	const char *SNUIT_GetAddButtonName(void)	const;
	const char *SNUIT_GetEditButtonName(void)	const;
	const char *SNUIT_GetDeleteButtonName(void)	const;
	const char *SNUIT_GetSortButtonName(void)	const;
	bool SNUIT_GetKeepSorted(void) const;
	void SNUIT_SetKeepSorted(ISNUITreeDlg *pTreeDlg, bool bKeepSorted);
	void SNUIT_PressedSortButton(ISNUITreeDlg *pTreeDlg);
	bool SNUIT_GetEditButtonActivation(void)		const;
	bool SNUIT_GetDeleteButtonActivation(void)		const;
	bool SNUIT_GetUpButtonActivation(void)			const;
	bool SNUIT_GetDownButtonActivation(void)		const;
	bool SNUIT_GetSortButtonActivation(void)		const;
	bool SNUIT_GetKeepSortedCheckActivation(void)	const;
	void SNUIT_PressedUpButton(ISNUITreeDlg *pTreeDlg);
	void SNUIT_PressedDownButton(ISNUITreeDlg *pTreeDlg);
	void SNUIT_PressedDeleteButton(ISNUITreeDlg *pTreeDlg);
	void SNUIT_PressedAddButton(ISNUITreeDlg *pTreeDlg);
	void SNUIT_PressedEditButton(ISNUITreeDlg *pTreeDlg);
	bool SNUIT_GetExpandParentInitial(void) const;

private:
	CProntoString m_name;
	CProntoString m_shortName;
	CProntoString m_sortAsName;

	CSNVolume *m_pParentVolume;
};

#endif // !defined(AFX_SNARTIST_H__0111C6FB_4638_458C_88AD_47C0AAAF9304__INCLUDED_)
