// SNDevice.cpp: implementation of the CSNDevice class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SNDevice.h"
#include "SNConfig.h"
#include "UnsignedBaseTenNumber.h"
#include "CSFSection.h"
#include "CSFAbstractTree.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CSNDevice::CSNDevice()
{
	init();
}

void CSNDevice::init(void)
{
	m_pJumpPanel = NULL;

	m_pBeforeDiscButton = NULL;
	m_pAfterDiscButton = NULL;
	m_pBeforeTrackButton1Digit = NULL;
	m_pBeforeTrackButton2Digits = NULL;
	m_pAfterTrackButton1Digit = NULL;
	m_pAfterTrackButton2Digits = NULL;

	for (int i = 0; i < 10; i++)
	{
		m_pNumberButtons[i] = NULL;
	}

	Reset();
}

CSNDevice::~CSNDevice()
{
	Reset();
}

void CSNDevice::Reset(void)
{
	m_name.Reset();
	m_shortName.Reset();

	m_digitsToPadDiscTo = 1;
	m_tenthsSecDigitDelayTime = 2;
	m_bDelayOnlyIdentical = true;

	m_tenthsSecDiscToTrackDelayTime = 0;

	SetJumpPanel(NULL);
	SetBeforeDiscButton(NULL);
	SetAfterDiscButton(NULL);
	SetBeforeTrackButton1Digit(NULL);
	SetBeforeTrackButton2Digits(NULL);
	SetAfterTrackButton1Digit(NULL);
	SetAfterTrackButton2Digits(NULL);

	for (int i = 0; i < 10; i++)
	{
		SetNumberButton(i, NULL);
	}
}

void CSNDevice::Set(const CSNDevice *pDevice)
{
	if (this != pDevice)
	{
		GetName()->SetPronto(pDevice->GetName()->GetPronto(false));
		GetShortName()->SetPronto(pDevice->GetShortName()->GetPronto(false));
		SetJumpPanel(pDevice->GetJumpPanel());
		SetDigitsToPadDiscTo(pDevice->GetDigitsToPadDiscTo());
		SetTenthsSecDigitDelayTime(pDevice->GetTenthsSecDigitDelayTime());
		SetDelayOnlyIdentical(pDevice->GetDelayOnlyIdentical());
		SetTenthsSecDiscToTrackDelayTime(pDevice->GetTenthsSecDiscToTrackDelayTime());
		SetBeforeDiscButton(pDevice->GetBeforeDiscButton());
		SetAfterDiscButton(pDevice->GetAfterDiscButton());
		SetBeforeTrackButton1Digit(pDevice->GetBeforeTrackButton1Digit());
		SetBeforeTrackButton2Digits(pDevice->GetBeforeTrackButton2Digits());
		SetAfterTrackButton1Digit(pDevice->GetAfterTrackButton1Digit());
		SetAfterTrackButton2Digits(pDevice->GetAfterTrackButton2Digits());

		for (int i=0; i < 10; i++)
		{
			SetNumberButton(i, pDevice->GetNumberButton(i));
		}
	}
}

const char *CSNDevice::getTypeName(void) const
{
	return("player");
}

bool CSNDevice::ResolveCSFPaths(CTypedError *pTError, const CCSFAbstractTree *pAT)
{
	for (int i=0; i < 10; i++)
	{
		if (NULL != m_pNumberButtons[i])
		{
			CString csPathName;
			csPathName.Format("#%d button", i);

			if (! m_pNumberButtons[i]->ResolveCSFPath(pTError, true, pAT, getTypeName(),
													GetName()->GetPrintable(true), csPathName) )
			{
				return(false);
			}
		}
	}

	if (NULL != m_pJumpPanel)
	{
		if (! m_pJumpPanel->ResolveCSFPath(pTError, true, pAT, getTypeName(),
										GetName()->GetPrintable(true), "jump panel") )
		{
			return(false);
		}
	}

	if (NULL != m_pBeforeDiscButton)
	{
		if (! m_pBeforeDiscButton->ResolveCSFPath(pTError, true, pAT, getTypeName(),
											GetName()->GetPrintable(true), "before disc button") )
		{
			return(false);
		}
	}

	if (NULL != m_pAfterDiscButton)
	{
		if (! m_pAfterDiscButton->ResolveCSFPath(pTError, true, pAT, getTypeName(),
											GetName()->GetPrintable(true), "after disc button") )
		{
			return(false);
		}
	}

	if (NULL != m_pBeforeTrackButton1Digit)
	{
		if (! m_pBeforeTrackButton1Digit->ResolveCSFPath(pTError, true, pAT, getTypeName(),
											GetName()->GetPrintable(true), "before track button (1 digit)") )
		{
			return(false);
		}
	}

	if (NULL != m_pBeforeTrackButton2Digits)
	{
		if (! m_pBeforeTrackButton2Digits->ResolveCSFPath(pTError, true, pAT, getTypeName(),
											GetName()->GetPrintable(true), "before track button (2 digits)") )
		{
			return(false);
		}
	}

	if (NULL != m_pAfterTrackButton1Digit)
	{
		if (! m_pAfterTrackButton1Digit->ResolveCSFPath(pTError, true, pAT, getTypeName(),
											GetName()->GetPrintable(true), "after track button (1 digit)") )
		{
			return(false);
		}
	}

	if (NULL != m_pAfterTrackButton2Digits)
	{
		if (! m_pAfterTrackButton2Digits->ResolveCSFPath(pTError, true, pAT, getTypeName(),
											GetName()->GetPrintable(true), "after track button (2 digits)") )
		{
			return(false);
		}
	}

	return(true);
}

bool CSNDevice::IsValid(CTypedError *pTError) const
{
	if (m_name.IsUnassigned())
	{
		CTypedError::SafeSetMsgError(pTError, "%s has no name", getTypeName());
		return(false);
	}

	if (m_name.IsTheEmptyString())
	{
		CTypedError::SafeSetMsgError(pTError, "%s has no name", getTypeName());
		return(false);
	}

	for (int i=0; i < 10; i++)
	{
		if (NULL == m_pNumberButtons[i])
		{
			CTypedError::SafeSetMsgError(pTError, "%s \"%s\" has no #%d button", getTypeName(), GetName()->GetPrintable(true), i);
			return(false);
		}
		else if (!m_pNumberButtons[i]->IsValid(NULL))
		{
			CTypedError::SafeSetMsgError(pTError, "%s \"%s\" has invalid #%d button", getTypeName(), GetName()->GetPrintable(true), i);
			return(false);
		}
	}

	return(true);
}

// Returns < 0 if this <  pObject
// Returns   0 if this == pObject
// Returns > 0 if this >  pObject
int CSNDevice::CompareForSort(const CSNDevice *pDevice) const
{
	if (NULL == pDevice)
	{
		return(1);
	}
	else
	{
		return(m_name.Compare(pDevice->GetName(), false));
	}
}

void CSNDevice::SetJumpPanel(const CCSFPanelPath *pJumpPanel)
{
	delete m_pJumpPanel;
	m_pJumpPanel = NULL;

	if (NULL != pJumpPanel)
	{
		m_pJumpPanel = new CCSFPanelPath();
		m_pJumpPanel->Set(pJumpPanel);
	}
}

void CSNDevice::SetDigitsToPadDiscTo(int padTo)
{
	if (MAX_DIGITSTOPADTO < padTo)
	{
		padTo = MAX_DIGITSTOPADTO;
	}

	if (0 >= padTo)
	{
		padTo = 1;
	}

	m_digitsToPadDiscTo = padTo;
}

void CSNDevice::SetTenthsSecDigitDelayTime(int delayTime)
{
	if (MAX_TENTHSSECDELAYTIME < delayTime)
	{
		delayTime = MAX_TENTHSSECDELAYTIME;
	}
	else if (0 > delayTime)
	{
		delayTime = 0;
	}

	m_tenthsSecDigitDelayTime = delayTime;
}

void CSNDevice::SetTenthsSecDiscToTrackDelayTime(int delayTime)
{
	if (MAX_TENTHSSECDELAYTIME < delayTime)
	{
		delayTime = MAX_TENTHSSECDELAYTIME;
	}
	else if (0 > delayTime)
	{
		delayTime = 0;
	}

	m_tenthsSecDiscToTrackDelayTime = delayTime;
}

void CSNDevice::SetDelayOnlyIdentical(bool bOnlyIdentical)
{
	m_bDelayOnlyIdentical = bOnlyIdentical;
}

void CSNDevice::SetBeforeDiscButton(const CCSFButtonPath *pBeforeButton)
{
	delete m_pBeforeDiscButton;
	m_pBeforeDiscButton = NULL;

	if (NULL != pBeforeButton)
	{
		m_pBeforeDiscButton = new CCSFButtonPath();
		m_pBeforeDiscButton->Set(pBeforeButton);
	}
}

void CSNDevice::SetAfterDiscButton(const CCSFButtonPath *pAfterButton)
{
	delete m_pAfterDiscButton;
	m_pAfterDiscButton = NULL;

	if (NULL != pAfterButton)
	{
		m_pAfterDiscButton = new CCSFButtonPath();
		m_pAfterDiscButton->Set(pAfterButton);
	}
}

void CSNDevice::SetBeforeTrackButton1Digit(const CCSFButtonPath *pBeforeButton)
{
	delete m_pBeforeTrackButton1Digit;
	m_pBeforeTrackButton1Digit = NULL;

	if (NULL != pBeforeButton)
	{
		m_pBeforeTrackButton1Digit = new CCSFButtonPath();
		m_pBeforeTrackButton1Digit->Set(pBeforeButton);
	}
}

void CSNDevice::SetBeforeTrackButton2Digits(const CCSFButtonPath *pBeforeButton)
{
	delete m_pBeforeTrackButton2Digits;
	m_pBeforeTrackButton2Digits = NULL;

	if (NULL != pBeforeButton)
	{
		m_pBeforeTrackButton2Digits = new CCSFButtonPath();
		m_pBeforeTrackButton2Digits->Set(pBeforeButton);
	}
}

void CSNDevice::SetAfterTrackButton1Digit(const CCSFButtonPath *pAfterButton)
{
	delete m_pAfterTrackButton1Digit;
	m_pAfterTrackButton1Digit = NULL;

	if (NULL != pAfterButton)
	{
		m_pAfterTrackButton1Digit = new CCSFButtonPath();
		m_pAfterTrackButton1Digit->Set(pAfterButton);
	}
}

void CSNDevice::SetAfterTrackButton2Digits(const CCSFButtonPath *pAfterButton)
{
	delete m_pAfterTrackButton2Digits;
	m_pAfterTrackButton2Digits = NULL;

	if (NULL != pAfterButton)
	{
		m_pAfterTrackButton2Digits = new CCSFButtonPath();
		m_pAfterTrackButton2Digits->Set(pAfterButton);
	}
}

void CSNDevice::SetNumberButton(unsigned int n, const CCSFButtonPath *pNumberButton)
{
	if (n < 10)
	{
		delete m_pNumberButtons[n];
		m_pNumberButtons[n] = NULL;

		if (NULL != pNumberButton)
		{
			m_pNumberButtons[n] = new CCSFButtonPath();
			m_pNumberButtons[n]->Set(pNumberButton);
		}
	}
}

CProntoString *CSNDevice::GetName(void)
{
	return(&m_name);
}

const CProntoString *CSNDevice::GetName(void) const
{
	return(&m_name);
}

CProntoString *CSNDevice::GetShortName(void)
{
	return(&m_shortName);
}

const CProntoString *CSNDevice::GetShortName(void) const
{
	return(&m_shortName);
}

const CCSFPanelPath *CSNDevice::GetJumpPanel(void) const
{
	return(m_pJumpPanel);
}

int CSNDevice::GetDigitsToPadDiscTo(void) const
{
	return(m_digitsToPadDiscTo);
}

int CSNDevice::GetTenthsSecDigitDelayTime(void) const
{
	return(m_tenthsSecDigitDelayTime);
}

int CSNDevice::GetTenthsSecDiscToTrackDelayTime(void) const
{
	return(m_tenthsSecDiscToTrackDelayTime);
}

bool CSNDevice::GetDelayOnlyIdentical(void) const
{
	return(m_bDelayOnlyIdentical);
}

const CCSFButtonPath *CSNDevice::GetBeforeDiscButton(void) const
{
	return(m_pBeforeDiscButton);
}

const CCSFButtonPath *CSNDevice::GetAfterDiscButton(void) const
{
	return(m_pAfterDiscButton);
}

const CCSFButtonPath *CSNDevice::GetBeforeTrackButton1Digit(void) const
{
	return(m_pBeforeTrackButton1Digit);
}

const CCSFButtonPath *CSNDevice::GetBeforeTrackButton2Digits(void) const
{
	return(m_pBeforeTrackButton2Digits);
}

const CCSFButtonPath *CSNDevice::GetAfterTrackButton1Digit(void) const
{
	return(m_pAfterTrackButton1Digit);
}

const CCSFButtonPath *CSNDevice::GetAfterTrackButton2Digits(void) const
{
	return(m_pAfterTrackButton2Digits);
}

const CCSFButtonPath *CSNDevice::GetNumberButton(unsigned int n) const
{
	if (n < 10)
	{
		return(m_pNumberButtons[n]);
	}
	else
	{
		return(NULL);
	}
}

xmlNodePtr CSNDevice::AddToXML(xmlNodePtr parentNode) const
{
	xmlNodePtr deviceNode = xmlNewChild(parentNode, NULL, SNXML_NODE_DEVICE, NULL);
	xmlSetProp(deviceNode, SNXML_PROP_NAME, (const xmlChar *)GetName()->GetEscaped(true));

	if (m_shortName.IsANonEmptyString())
	{
		xmlSetProp(deviceNode, SNXML_PROP_SHORTNAME, (const xmlChar *)GetShortName()->GetEscaped(true));
	}
	
	CUnsignedBaseTenNumber ubtnDigitsToPadTo(m_digitsToPadDiscTo);
	xmlSetProp(deviceNode, SNXML_PROP_DIGITSTOPADTO, (const xmlChar *)ubtnDigitsToPadTo.GetAsciiRepresentation());

	CUnsignedBaseTenNumber ubtnTenthsSecDigitDelayTime(m_tenthsSecDigitDelayTime);
	xmlSetProp(deviceNode, SNXML_PROP_TENTHSSECDELAYTIME, (const xmlChar *)ubtnTenthsSecDigitDelayTime.GetAsciiRepresentation());

	xmlSetProp(deviceNode, SNXML_PROP_DELAYONLYIDENTICAL, m_bDelayOnlyIdentical ? SNXML_VALUE_1 : SNXML_VALUE_0);

	CUnsignedBaseTenNumber ubtnTenthsSecDiscToTrackDelayTime(m_tenthsSecDiscToTrackDelayTime);
	xmlSetProp(deviceNode, SNXML_PROP_TENTHSSECDISCTOTRACKDELAYTIME, (const xmlChar *)ubtnTenthsSecDiscToTrackDelayTime.GetAsciiRepresentation());

	if (NULL != m_pJumpPanel)
	{
		xmlNodePtr jpNode = m_pJumpPanel->AddToXML(deviceNode);
		xmlSetProp(jpNode, SNXML_PROP_NAME, SNXML_VALUE_JUMPPANEL);
	}

	if (NULL != m_pBeforeDiscButton)
	{
		xmlNodePtr bbNode = m_pBeforeDiscButton->AddToXML(deviceNode);
		xmlSetProp(bbNode, SNXML_PROP_NAME, SNXML_VALUE_BEFOREBUTTON);
	}

	if (NULL != m_pAfterDiscButton)
	{
		xmlNodePtr abNode = m_pAfterDiscButton->AddToXML(deviceNode);
		xmlSetProp(abNode, SNXML_PROP_NAME, SNXML_VALUE_AFTERBUTTON);
	}

	if (NULL != m_pBeforeTrackButton1Digit)
	{
		xmlNodePtr bbNode = m_pBeforeTrackButton1Digit->AddToXML(deviceNode);
		xmlSetProp(bbNode, SNXML_PROP_NAME, SNXML_VALUE_BEFORETRACKBUTTON1D);
	}

	if (NULL != m_pBeforeTrackButton2Digits)
	{
		xmlNodePtr bbNode = m_pBeforeTrackButton2Digits->AddToXML(deviceNode);
		xmlSetProp(bbNode, SNXML_PROP_NAME, SNXML_VALUE_BEFORETRACKBUTTON2D);
	}

	if (NULL != m_pAfterTrackButton1Digit)
	{
		xmlNodePtr abNode = m_pAfterTrackButton1Digit->AddToXML(deviceNode);
		xmlSetProp(abNode, SNXML_PROP_NAME, SNXML_VALUE_AFTERTRACKBUTTON1D);
	}

	if (NULL != m_pAfterTrackButton2Digits)
	{
		xmlNodePtr abNode = m_pAfterTrackButton2Digits->AddToXML(deviceNode);
		xmlSetProp(abNode, SNXML_PROP_NAME, SNXML_VALUE_AFTERTRACKBUTTON2D);
	}

	for (int i = 0; i < 10; i++)
	{
		xmlNodePtr nbNode;
		xmlChar xmlNumber[2];

		if (NULL != m_pNumberButtons[i])
		{
			stringFormat((char *)xmlNumber, sizeof(xmlNumber), "%d", i);

			nbNode = m_pNumberButtons[i]->AddToXML(deviceNode);
			xmlSetProp(nbNode, SNXML_PROP_NAME, SNXML_VALUE_NUMBERBUTTON);
			xmlSetProp(nbNode, SNXML_PROP_NUMBER, xmlNumber);
		}
	}

	return(deviceNode);
}

bool CSNDevice::BuildFromXML(CTypedError *pTError, CStringList *pSLErrorDetails, xmlNodePtr pNode)
{
	bool bResult = true;

	xmlChar *pXC;
	xmlChar *pXCType;
	xmlChar *pXCName;
	xmlChar *pXCNumber;

	// Name attribute.

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_NAME)))
	{
		GetName()->SetEscaped((const char *)pXC);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_SHORTNAME)))
	{
		GetShortName()->SetEscaped((const char *)pXC);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_DIGITSTOPADTO)))
	{
		int i = atoi((const char *)pXC);
		SetDigitsToPadDiscTo(i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_TENTHSSECDELAYTIME)))
	{
		int i = atoi((const char *)pXC);
		SetTenthsSecDigitDelayTime(i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_TENTHSSECDISCTOTRACKDELAYTIME)))
	{
		int i = atoi((const char *)pXC);
		SetTenthsSecDiscToTrackDelayTime(i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_DELAYONLYIDENTICAL)))
	{
		int i = atoi((const char *)pXC);
		SetDelayOnlyIdentical(0 != i);
		xmlFree(pXC);
	}

	for (xmlNodePtr pNodeChild = pNode->children; NULL != pNodeChild; pNodeChild = pNodeChild->next)
	{
		if ((!xmlIsBlankNode(pNodeChild)) && (NULL != pNodeChild->name))
		{
			if (0 == strcmp((const char *)SNXML_NODE_CSFPATH, (const char *)pNodeChild->name))
			{
				if (NULL != (pXCType = xmlGetProp(pNodeChild, SNXML_PROP_TYPE)))
				{
					if (NULL != (pXCName = xmlGetProp(pNodeChild, SNXML_PROP_NAME)))
					{
						if (0 == strcmp((const char *)SNXML_VALUE_PANEL, (const char *)pXCType))
						{
							CCSFPanelPath *pPanelPath = new CCSFPanelPath();

							if (! pPanelPath->BuildFromXML(pTError, pSLErrorDetails, pNodeChild) )
							{
								bResult = false;
							}
							else if (0 == strcmp((const char *)SNXML_VALUE_JUMPPANEL, (const char *)pXCName))
							{
								SetJumpPanel(pPanelPath);
							}

							delete pPanelPath;
						}
						else if (0 == strcmp((const char *)SNXML_VALUE_BUTTON, (const char *)pXCType))
						{
							CCSFButtonPath *pButtonPath = new CCSFButtonPath();

							if (! pButtonPath->BuildFromXML(pTError, pSLErrorDetails, pNodeChild) )
							{
								bResult = false;
							}
							else if (0 == strcmp((const char *)SNXML_VALUE_BEFOREBUTTON, (const char *)pXCName))
							{
								SetBeforeDiscButton(pButtonPath);
							}
							else if (0 == strcmp((const char *)SNXML_VALUE_AFTERBUTTON, (const char *)pXCName))
							{
								SetAfterDiscButton(pButtonPath);
							}
							else if (0 == strcmp((const char *)SNXML_VALUE_BEFORETRACKBUTTON1D, (const char *)pXCName))
							{
								SetBeforeTrackButton1Digit(pButtonPath);
							}
							else if (0 == strcmp((const char *)SNXML_VALUE_BEFORETRACKBUTTON2D, (const char *)pXCName))
							{
								SetBeforeTrackButton2Digits(pButtonPath);
							}
							else if (0 == strcmp((const char *)SNXML_VALUE_AFTERTRACKBUTTON1D, (const char *)pXCName))
							{
								SetAfterTrackButton1Digit(pButtonPath);
							}
							else if (0 == strcmp((const char *)SNXML_VALUE_AFTERTRACKBUTTON2D, (const char *)pXCName))
							{
								SetAfterTrackButton2Digits(pButtonPath);
							}
							else if (0 == strcmp((const char *)SNXML_VALUE_AFTERTRACKBUTTON, (const char *)pXCName))
							{
								SetAfterTrackButton1Digit(pButtonPath);
								SetAfterTrackButton2Digits(pButtonPath);
							}
							else if (0 == strcmp((const char *)SNXML_VALUE_NUMBERBUTTON, (const char *)pXCName))
							{
								if (NULL != (pXCNumber = xmlGetProp(pNodeChild, SNXML_PROP_NUMBER)))
								{
									int i = atoi((const char *)pXCNumber);
									SetNumberButton((unsigned int)i, pButtonPath);
									xmlFree(pXCNumber);
								}
							}

							delete pButtonPath;
						}

						xmlFree(pXCName);
					}

					xmlFree(pXCType);
				}
			}
		}
	}

//	if (! this->IsValid(pTError) )
//	{
//		bResult = false;
//	}

	return(bResult);
}

// Only call after a successful call of ResolveCSFPaths.
void CSNDevice::AddDiscCallToButtonAction(CCSFSection *pActionSection, int discNumber,
									  bool bIncludeJump, const CCSFPanelPath *pJumpOverride) const
{
	if (NULL != m_pBeforeDiscButton)
	{
		pActionSection->AddEscapedTitleOnlySubSection((*(m_pBeforeDiscButton->GetATButtons()->begin()))->GetCSFAliasString());
	}

	if ((!m_bDelayOnlyIdentical) && (0 != m_tenthsSecDigitDelayTime))
	{
		pActionSection->AddEscapedTitleOnlySubSectionFormat("Delay(%d00);", m_tenthsSecDigitDelayTime);
	}

	CUnsignedBaseTenNumber ubtnDiscNumber(discNumber, m_digitsToPadDiscTo);
	int numDigits = ubtnDiscNumber.GetNumberOfDigits();
	int currentDigit;
	int previousDigit;

	previousDigit = currentDigit = ubtnDiscNumber.GetDigit(0);
	pActionSection->AddEscapedTitleOnlySubSection((*(m_pNumberButtons[currentDigit]->GetATButtons()->begin()))->GetCSFAliasString());

	for (int i = 1; i < numDigits; i++)
	{
		currentDigit = ubtnDiscNumber.GetDigit(i);

		if (((!m_bDelayOnlyIdentical) || (currentDigit == previousDigit)) && (0 != m_tenthsSecDigitDelayTime))
		{
			pActionSection->AddEscapedTitleOnlySubSectionFormat("Delay(%d00);", m_tenthsSecDigitDelayTime);
		}

		pActionSection->AddEscapedTitleOnlySubSection((*(m_pNumberButtons[currentDigit]->GetATButtons()->begin()))->GetCSFAliasString());

		previousDigit = currentDigit;
	}

	if ((!m_bDelayOnlyIdentical) && (0 != m_tenthsSecDigitDelayTime))
	{
		pActionSection->AddEscapedTitleOnlySubSectionFormat("Delay(%d00);", m_tenthsSecDigitDelayTime);
	}

	if (NULL != m_pAfterDiscButton)
	{
		pActionSection->AddEscapedTitleOnlySubSection((*(m_pAfterDiscButton->GetATButtons()->begin()))->GetCSFAliasString());
	}

	// Jump back to the main CD panel.
	if (bIncludeJump)
	{
		if (NULL != pJumpOverride)
		{
			pActionSection->AddEscapedTitleOnlySubSection((*(pJumpOverride->GetATPanels()->begin()))->GetCSFJumpString());
		}
		else if (NULL != m_pJumpPanel)
		{
			pActionSection->AddEscapedTitleOnlySubSection((*(m_pJumpPanel->GetATPanels()->begin()))->GetCSFJumpString());
		}
	}
}

// Only call after a successful call of ResolveCSFPaths.
void CSNDevice::AddTrackCallToButtonAction(CCSFSection *pActionSection, int trackNumber,
									  bool bIncludeJump, const CCSFPanelPath *pJumpOverride,
									  bool bIncludeDiscToTrackDelay) const
{
	CUnsignedBaseTenNumber ubtnTrackNumber(trackNumber/*, m_digitsToPadDiscTo*/);
	int numDigits = ubtnTrackNumber.GetNumberOfDigits();

	if ((bIncludeDiscToTrackDelay) && (0 != m_tenthsSecDiscToTrackDelayTime))
	{
		pActionSection->AddEscapedTitleOnlySubSectionFormat("Delay(%d00);", m_tenthsSecDiscToTrackDelayTime);
	}

	if ((1 == numDigits) && (NULL != m_pBeforeTrackButton1Digit))
	{
		pActionSection->AddEscapedTitleOnlySubSection((*(m_pBeforeTrackButton1Digit->GetATButtons()->begin()))->GetCSFAliasString());
	}

	if ((2 == numDigits) && (NULL != m_pBeforeTrackButton2Digits))
	{
		pActionSection->AddEscapedTitleOnlySubSection((*(m_pBeforeTrackButton2Digits->GetATButtons()->begin()))->GetCSFAliasString());
	}

	if ((!m_bDelayOnlyIdentical) && (0 != m_tenthsSecDigitDelayTime))
	{
		pActionSection->AddEscapedTitleOnlySubSectionFormat("Delay(%d00);", m_tenthsSecDigitDelayTime);
	}

	int currentDigit;
	int previousDigit;

	previousDigit = currentDigit = ubtnTrackNumber.GetDigit(0);
	pActionSection->AddEscapedTitleOnlySubSection((*(m_pNumberButtons[currentDigit]->GetATButtons()->begin()))->GetCSFAliasString());

	for (int i = 1; i < numDigits; i++)
	{
		currentDigit = ubtnTrackNumber.GetDigit(i);

		if (((!m_bDelayOnlyIdentical) || (currentDigit == previousDigit)) && (0 != m_tenthsSecDigitDelayTime))
		{
			pActionSection->AddEscapedTitleOnlySubSectionFormat("Delay(%d00);", m_tenthsSecDigitDelayTime);
		}

		pActionSection->AddEscapedTitleOnlySubSection((*(m_pNumberButtons[currentDigit]->GetATButtons()->begin()))->GetCSFAliasString());

		previousDigit = currentDigit;
	}

	if ((!m_bDelayOnlyIdentical) && (0 != m_tenthsSecDigitDelayTime))
	{
		pActionSection->AddEscapedTitleOnlySubSectionFormat("Delay(%d00);", m_tenthsSecDigitDelayTime);
	}

	if ((1 == numDigits) && (NULL != m_pAfterTrackButton1Digit))
	{
		pActionSection->AddEscapedTitleOnlySubSection((*(m_pAfterTrackButton1Digit->GetATButtons()->begin()))->GetCSFAliasString());
	}

	if ((2 == numDigits) && (NULL != m_pAfterTrackButton2Digits))
	{
		pActionSection->AddEscapedTitleOnlySubSection((*(m_pAfterTrackButton2Digits->GetATButtons()->begin()))->GetCSFAliasString());
	}

	// Jump back to the main CD panel.
	if (bIncludeJump)
	{
		if (NULL != pJumpOverride)
		{
			pActionSection->AddEscapedTitleOnlySubSection((*(pJumpOverride->GetATPanels()->begin()))->GetCSFJumpString());
		}
		else if (NULL != m_pJumpPanel)
		{
			pActionSection->AddEscapedTitleOnlySubSection((*(m_pJumpPanel->GetATPanels()->begin()))->GetCSFJumpString());
		}
	}
}

//================================================================================
//================================================================================
//================================================================================

xmlNodePtr CSNDeviceList::AddToXML(xmlNodePtr parentNode) const
{
	xmlNodePtr devicesNode = xmlNewChild(parentNode, NULL, SNXML_NODE_DEVICES, NULL);

	for (const CSNDevice *pDev = GetFirst(); NULL != pDev; pDev = GetNext(pDev))
	{
		pDev->AddToXML(devicesNode);
	}

	return(devicesNode);
}

bool CSNDeviceList::BuildFromXML(CTypedError *pTError, CStringList *pSLErrorDetails, xmlNodePtr pNode)
{
	bool bResult = true;

	CSNDevice *pDev;

	for (xmlNodePtr pNodeChild = pNode->children; NULL != pNodeChild; pNodeChild = pNodeChild->next)
	{
		if ((!xmlIsBlankNode(pNodeChild)) && (NULL != pNodeChild->name))
		{
			if (0 == strcmp((const char *)SNXML_NODE_DEVICE, (const char *)pNodeChild->name))
			{
				pDev = new CSNDevice();

				if ((!pDev->BuildFromXML(pTError, pSLErrorDetails, pNodeChild))	\
				||	(NULL == AddBelowAndOwn(pTError, pDev, NULL))				)
				{
					delete pDev;

					bResult = false;

					break;
				}
			}
		}
	}

	return(bResult);
}

bool CSNDeviceList::ResolveCSFPaths(CTypedError *pTError, const CCSFAbstractTree *pAT)
{
	for (CSNDevice *pDev = GetNonConst(GetFirst()); NULL != pDev; pDev = GetNonConst(GetNext(pDev)))
	{
		if (! pDev->ResolveCSFPaths(pTError, pAT) )
		{
			return(false);
		}
	}

	return(true);
}

//================================================================================
//================================================================================
//================================================================================
