// SNDevice.h: interface for the CSNDevice class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SNDEVICE_H__DE28AF00_8408_11D4_BF8F_0020AF872188__INCLUDED_)
#define AFX_SNDEVICE_H__DE28AF00_8408_11D4_BF8F_0020AF872188__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "CSFPath.h"
#include "SNList.h"

class CCSFSection;
class CSFButtonPath;

class CSNDevice : public CSNListObject
{
	friend class CSNList<CSNDevice>;
	friend class CSNUpdatableList<CSNDevice>;

public:
	CSNDevice();
	virtual ~CSNDevice();

	void Reset(void);

	enum { MAX_DISCNUMBER = 32767 }; // Piece of shit spin control seems to flake above this.
	enum { MAX_TRACKNUMBER = 99 };
	enum { MAX_DIGITSTOPADTO = 5 };
	enum { MAX_TENTHSSECDELAYTIME = 999 }; // 99.9 seconds.

	bool IsValid(CTypedError *pTError) const;

	bool ResolveCSFPaths(CTypedError *pTError, const CCSFAbstractTree *pAT);

	// Returns < 0 if this <  pDevice
	// Returns   0 if this == pDevice
	// Returns > 0 if this >  pDevice
	int CompareForSort(const CSNDevice *pDevice) const;

	void Set(const CSNDevice *pDevice);

	void SetJumpPanel(const CCSFPanelPath *pJumpPanel);
	void SetDigitsToPadDiscTo(int padTo);
	void SetTenthsSecDigitDelayTime(int delayTime);
	void SetTenthsSecDiscToTrackDelayTime(int delayTime);
	void SetDelayOnlyIdentical(bool bOnlyIdentical);
	void SetBeforeDiscButton(const CCSFButtonPath *pBeforeButton);
	void SetAfterDiscButton(const CCSFButtonPath *pAfterButton);
	void SetBeforeTrackButton1Digit(const CCSFButtonPath *pBeforeButton);
	void SetBeforeTrackButton2Digits(const CCSFButtonPath *pBeforeButton);
	void SetAfterTrackButton1Digit(const CCSFButtonPath *pAfterButton);
	void SetAfterTrackButton2Digits(const CCSFButtonPath *pAfterButton);
	void SetNumberButton(unsigned int n, const CCSFButtonPath *pNumberButton);

	// All Get routines returning pointers may return NULL.
	// Pointer results can be invalidated by later Set calls.
		  CProntoString *	GetName(void);
	const CProntoString *	GetName(void)							const;
		  CProntoString *	GetShortName(void);
	const CProntoString *	GetShortName(void)						const;
	const CCSFPanelPath *	GetJumpPanel(void)						const;
	int						GetDigitsToPadDiscTo(void)				const;
	int						GetTenthsSecDigitDelayTime(void)		const;
	int						GetTenthsSecDiscToTrackDelayTime(void)	const;
	bool					GetDelayOnlyIdentical(void)				const;
	const CCSFButtonPath *	GetBeforeDiscButton(void)				const;
	const CCSFButtonPath *	GetAfterDiscButton(void)				const;
	const CCSFButtonPath *	GetBeforeTrackButton1Digit(void)		const;
	const CCSFButtonPath *	GetBeforeTrackButton2Digits(void)		const;
	const CCSFButtonPath *	GetAfterTrackButton1Digit(void)			const;
	const CCSFButtonPath *	GetAfterTrackButton2Digits(void)		const;
	const CCSFButtonPath *	GetNumberButton(unsigned int n)			const;

	xmlNodePtr AddToXML(xmlNodePtr parentNode) const;
	bool BuildFromXML(CTypedError *pTError, CStringList *pSLErrorDetails, xmlNodePtr pNode);

	// Only call after a successful call of ResolveCSFPaths.
	void AddDiscCallToButtonAction(CCSFSection *pActionSection, int discNumber,
									bool bIncludeJump, const CCSFPanelPath *pJumpOverride) const;

	// Only call after a successful call of ResolveCSFPaths.
	void AddTrackCallToButtonAction(CCSFSection *pActionSection, int trackNumber,
									bool bIncludeJump, const CCSFPanelPath *pJumpOverride,
									bool bIncludeDiscToTrackDelay) const;

protected:
	void init(void);
	const char *getTypeName(void) const;

protected:
	CProntoString m_name;
	CProntoString m_shortName;

	CCSFPanelPath *m_pJumpPanel;

	int m_digitsToPadDiscTo;

	int m_tenthsSecDigitDelayTime;
	bool m_bDelayOnlyIdentical;

	int m_tenthsSecDiscToTrackDelayTime;

	CCSFButtonPath *m_pBeforeDiscButton;
	CCSFButtonPath *m_pAfterDiscButton;
	CCSFButtonPath *m_pBeforeTrackButton1Digit;
	CCSFButtonPath *m_pBeforeTrackButton2Digits;
	CCSFButtonPath *m_pAfterTrackButton1Digit;
	CCSFButtonPath *m_pAfterTrackButton2Digits;
	CCSFButtonPath *m_pNumberButtons[10];
};

class CSNDeviceList : public CSNUpdatableList<CSNDevice>
{
public:
	CSNDeviceList()
	{
		SetSort(true);
	}

	virtual ~CSNDeviceList()
	{
	}

	xmlNodePtr AddToXML(xmlNodePtr parentNode) const;
	bool BuildFromXML(CTypedError *pTError, CStringList *pSLErrorDetails, xmlNodePtr pNode);

	bool ResolveCSFPaths(CTypedError *pTError, const CCSFAbstractTree *pAT);
};

#endif // !defined(AFX_SNDEVICE_H__DE28AF00_8408_11D4_BF8F_0020AF872188__INCLUDED_)
