// SNDisc.cpp: implementation of the CSNDisc class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SNConfig.h"
#include "UnsignedBaseTenNumber.h"
#include "SNArtist.h"
#include "SNDisc.h"
#include "SNDevice.h"
#include "SNVolume.h"
#include "SNCSVLine.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSNDisc::CSNDisc()
{
	m_discNumber = 0;
	m_pDevice = NULL;
	m_pParentArtist = NULL;
	m_bIgnoreTracks = false;
}

CSNDisc::~CSNDisc()
{

}

const char *CSNDisc::getTypeName(void) const
{
	return("disc");
}

bool CSNDisc::IsValid(CTypedError *pTError) const
{
	if (m_name.IsUnassigned())
	{
		CTypedError::SafeSetMsgError(pTError, "%s has no name", getTypeName());
		return(false);
	}

	if (m_name.IsTheEmptyString())
	{
		CTypedError::SafeSetMsgError(pTError, "%s has no name", getTypeName());
		return(false);
	}

//	if ((NULL == m_pDevice) && (m_deviceNameBeforeResolution.IsUnassigned()))
//	{
//		CTypedError::SafeSetMsgError(pTError, "%s not associated with a player", getTypeName());
//		return(false);
//	}

	if (NULL == m_pParentArtist)
	{
		CTypedError::SafeSetMsgError(pTError, "%s not associated with an artist", getTypeName());
		return(false);
	}

	return(true);
}

// Returns < 0 if this <  pDisc
// Returns   0 if this == pDisc
// Returns > 0 if this >  pDisc
int CSNDisc::CompareForSort(const CSNDisc *pDisc) const
{
	if (NULL == pDisc)
	{
		return(1);
	}

	const CProntoString *pPSLeft = this->GetSortAsNameOrName();
	const CProntoString *pPSRight = pDisc->GetSortAsNameOrName();

	const char *stringLeft = pPSLeft->GetPronto(true);
	const char *stringRight = pPSRight->GetPronto(true);

	char alphaGroupLeft = GetAlphabetPanelLetterGroup(stringLeft);
	char alphaGroupRight = GetAlphabetPanelLetterGroup(stringRight);

	if (('#' == alphaGroupLeft) && ('#' != alphaGroupRight))
	{
		return(-1);
	}
	else if (('#' != alphaGroupLeft) && ('#' == alphaGroupRight))
	{
		return(1);
	}

	return(pPSLeft->Compare(pPSRight, false));
}

void CSNDisc::Set(const CSNDisc *pDisc)
{
	if (this != pDisc)
	{
		m_name.SetPronto(pDisc->m_name.GetPronto(false));
		m_shortName.SetPronto(pDisc->m_shortName.GetPronto(false));
		m_sortAsName.SetPronto(pDisc->m_sortAsName.GetPronto(false));
		m_discNumber = pDisc->m_discNumber;
		m_pDevice = pDisc->m_pDevice;

		m_tracks.Set(pDisc->m_tracks);

		for (CSNTrack *pTrack = m_tracks.GetNonConst(m_tracks.GetFirst()); NULL != pTrack; pTrack = m_tracks.GetNonConst(m_tracks.GetNext(pTrack)))
		{
			pTrack->m_pParentDisc = this;
		}

		m_bIgnoreTracks = pDisc->m_bIgnoreTracks;

		if (NULL == m_pDevice)
		{
			m_deviceNameBeforeResolution.SetPronto(pDisc->m_deviceNameBeforeResolution.GetPronto(false));
		}
		else
		{
			m_deviceNameBeforeResolution.Reset();
		}

//		m_pParentArtist = pDisc->m_pParentArtist;
	}
}

CSNArtist *CSNDisc::GetParent(void)
{
	return(m_pParentArtist);
}

const CSNArtist *CSNDisc::GetParent(void) const
{
	return(m_pParentArtist);
}

const CProntoString *CSNDisc::GetUnresolvedDeviceName(void) const
{
	return(&m_deviceNameBeforeResolution);
}

CProntoString *CSNDisc::GetName(void)
{
	return(&m_name);
}

const CProntoString *CSNDisc::GetName(void) const
{
	return(&m_name);
}

CProntoString *CSNDisc::GetSortAsName(void)
{
	return(&m_sortAsName);
}

CProntoString *CSNDisc::GetShortName(void)
{
	return(&m_shortName);
}

const CProntoString *CSNDisc::GetShortName(void) const
{
	return(&m_shortName);
}

const CProntoString *CSNDisc::GetSortAsName(void) const
{
	return(&m_sortAsName);
}

const CProntoString *CSNDisc::GetShortNameOrName(void) const
{
	if (m_shortName.IsANonEmptyString())
	{
		return(&m_shortName);
	}
	else
	{
		return(&m_name);
	}
}

const CProntoString *CSNDisc::GetSortAsNameOrName(void) const
{
	if (m_sortAsName.IsANonEmptyString())
	{
		return(&m_sortAsName);
	}
	else
	{
		return(&m_name);
	}
}

CString CSNDisc::GetFullPathPronto(void) const
{
	CString csResult;

	csResult.Format("%s: %s: %s", GetParent()->GetParent()->GetName()->GetPronto(true),
											   GetParent()->GetName()->GetPronto(true),
															GetName()->GetPronto(true));

	return(csResult);
}

CString CSNDisc::GetDiscNumberLabelEscaped(void) const
{
	CString csResult;

	csResult.Format("%s%d", m_pDevice->GetShortName()->GetEscaped(true), GetDiscNumber());

	return(csResult);
}

CSNTrackList *CSNDisc::GetTracks(void)
{
	return(&m_tracks);
}

bool CSNDisc::GetIgnoreTracks(void) const
{
	return(m_bIgnoreTracks);
}

void CSNDisc::SetIgnoreTracks(bool bIgnoreTracks)
{
	m_bIgnoreTracks = bIgnoreTracks;
}

const CSNTrackList *CSNDisc::GetTracks(void) const
{
	return(&m_tracks);
}

int CSNDisc::GetDiscNumber(void) const
{
	return(m_discNumber);
}

void CSNDisc::SetDiscNumber(int discNumber)
{
	if (CSNDevice::MAX_DISCNUMBER < discNumber)
	{
		discNumber = CSNDevice::MAX_DISCNUMBER;
	}
	else if (discNumber < 0)
	{
		discNumber = 0;
	}

	m_discNumber = discNumber;
}

// GetDevice may return NULL even though the CSNDisc is valid if
// device resolution has not yet been done.
const CSNDevice *CSNDisc::GetDevice(void) const
{
	return(m_pDevice);
}

// Also clears m_deviceNameBeforeResolution.
void CSNDisc::SetDevice(const CSNDevice *pDevice)
{
	m_pDevice = pDevice;

	m_deviceNameBeforeResolution.Reset();
}

void CSNDisc::ResolveDevice(set<CSNDisc *> *pDiscSet, const CSNDeviceList *pDeviceList)
{
	if ((NULL == m_pDevice) && m_deviceNameBeforeResolution.IsANonEmptyString())
	{
		const CSNDevice *pDevice = pDeviceList->GetByNamePronto(m_deviceNameBeforeResolution.GetPronto(true), NULL);

		if (NULL != pDevice)
		{
			SetDevice(pDevice);
		}
	}

	if ((NULL == m_pDevice) && (NULL != pDiscSet))
	{
		pDiscSet->insert(this);
	}
}

void CSNDisc::UnlinkDevice(const CSNDevice *pDevice)
{
	if ((NULL != pDevice) && (pDevice == m_pDevice))
	{
		m_deviceNameBeforeResolution.SetPronto(m_pDevice->GetName()->GetPronto(false));

		m_pDevice = NULL;
	}
}

CString CSNDisc::SNUIT_GetLabel(void) const
{
	CString csResult = m_name.GetPronto(true);

	csResult += " (";

	int padDigits = 1;

	if (NULL != m_pDevice)
	{
		csResult += m_pDevice->GetName()->GetPronto(true);
		csResult += ", ";

		padDigits = m_pDevice->GetDigitsToPadDiscTo();
	}
	else if (m_deviceNameBeforeResolution.IsANonEmptyString())
	{
		csResult += "UNRESOLVED PLAYER: ";
		csResult += m_deviceNameBeforeResolution.GetPronto(true);
		csResult += ", ";
	}
	else
	{
		csResult += "NO PLAYER, ";
	}

	CString numStr;
	numStr.Format("%0*d", padDigits, m_discNumber);
	csResult += numStr;

	csResult += ")";

	if (! (m_shortName.IsUnassigned() || m_shortName.IsTheEmptyString()) )
	{
		csResult += " (";
		csResult += m_shortName.GetPronto(true);
		csResult += ")";
	}

	if (! (m_sortAsName.IsUnassigned() || m_sortAsName.IsTheEmptyString()) )
	{
		csResult += " (Sort As: ";
		csResult += m_sortAsName.GetPronto(true);
		csResult += ")";
	}

	return(csResult);
}

int CSNDisc::SNUIT_GetImageIdx(ISNUITreeDlg *pTreeDlg) const
{
	return(pTreeDlg->SNUIT_GetImageIdxDisc());
}

CString CSNDisc::SNUIT_GetDeleteConfirmMessage(void) const
{
	return("");
}

bool CSNDisc::SNUIT_AddChildrenToTree(ISNUITreeDlg *pTreeDlg)
{
	return(true);
}

const char *CSNDisc::SNUIT_GetAddButtonName(void) const
{
	return("Add Disc");
}

const char *CSNDisc::SNUIT_GetEditButtonName(void) const
{
	return("Edit Disc");
}

const char *CSNDisc::SNUIT_GetDeleteButtonName(void) const
{
	return("Delete Disc");
}

const char *CSNDisc::SNUIT_GetSortButtonName(void) const
{
	return("Sort Discs");
}

bool CSNDisc::SNUIT_GetKeepSorted(void) const
{
	return(m_pParentArtist->GetSort());
}

void CSNDisc::SNUIT_SetKeepSorted(ISNUITreeDlg *pTreeDlg, bool bKeepSorted)
{
	m_pParentArtist->SetSort(bKeepSorted);

	if ((bKeepSorted) && (NULL != pTreeDlg))
	{
		set<ISNUITreeItem *> *pExpandSet = pTreeDlg->SNUIT_GetExpandedItems(m_pParentArtist);

		// Remove from the tree the children of our parent..
		pTreeDlg->SNUIT_RemoveTreeItems(m_pParentArtist, true);

		// Add children back again, this time sorted.
		m_pParentArtist->SNUIT_AddChildrenToTree(pTreeDlg);

		if (NULL != pExpandSet)
		{
			pTreeDlg->SNUIT_ExpandedItems(pExpandSet);
			delete pExpandSet;
		}
	}

	if (NULL != pTreeDlg)
	{
		pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(this);
	}
}

void CSNDisc::SNUIT_PressedSortButton(ISNUITreeDlg *pTreeDlg)
{
	if (! SNUIT_GetKeepSorted() )
	{
		SNUIT_SetKeepSorted(pTreeDlg, true);
		SNUIT_SetKeepSorted(pTreeDlg, false);
	}
}

bool CSNDisc::SNUIT_GetEditButtonActivation(void) const
{
	return(true);
}

bool CSNDisc::SNUIT_GetDeleteButtonActivation(void) const
{
	return(true);
}

bool CSNDisc::SNUIT_GetUpButtonActivation(void) const
{
	return(SNUIT_GetSortButtonActivation() && (this != m_pParentArtist->GetFirst()));
}

bool CSNDisc::SNUIT_GetDownButtonActivation(void) const
{
	return(SNUIT_GetSortButtonActivation() && (this != m_pParentArtist->GetLast()));
}

bool CSNDisc::SNUIT_GetSortButtonActivation(void) const
{
	return(! SNUIT_GetKeepSorted() );
}

bool CSNDisc::SNUIT_GetKeepSortedCheckActivation(void) const
{
	return(true);
}

void CSNDisc::SNUIT_PressedUpButton(ISNUITreeDlg *pTreeDlg)
{
	set<ISNUITreeItem *> *pExpandSet = NULL;

	if (NULL != pTreeDlg)
	{
		pExpandSet = pTreeDlg->SNUIT_GetExpandedItems(m_pParentArtist);

		if (! pTreeDlg->SNUIT_RemoveTreeItems(this, false) )
		{
			return;
		}
	}

	m_pParentArtist->MoveUp(this);

	if (NULL != pTreeDlg)
	{
		if (pTreeDlg->SNUIT_AddTreeItem(this, m_pParentArtist, m_pParentArtist->GetPrevious(this)))
		{
			if (NULL != pExpandSet)
			{
				pTreeDlg->SNUIT_ExpandedItems(pExpandSet);
			}

			pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(this);
		}
	}

	delete pExpandSet;
}

void CSNDisc::SNUIT_PressedDownButton(ISNUITreeDlg *pTreeDlg)
{
	set<ISNUITreeItem *> *pExpandSet = NULL;

	if (NULL != pTreeDlg)
	{
		pExpandSet = pTreeDlg->SNUIT_GetExpandedItems(m_pParentArtist);

		if (! pTreeDlg->SNUIT_RemoveTreeItems(this, false) )
		{
			return;
		}
	}

	m_pParentArtist->MoveDown(this);

	if (NULL != pTreeDlg)
	{
		if (pTreeDlg->SNUIT_AddTreeItem(this, m_pParentArtist, m_pParentArtist->GetPrevious(this)))
		{
			if (NULL != pExpandSet)
			{
				pTreeDlg->SNUIT_ExpandedItems(pExpandSet);
			}

			pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(this);
		}
	}

	delete pExpandSet;
}

void CSNDisc::SNUIT_PressedDeleteButton(ISNUITreeDlg *pTreeDlg)
{
	if (NULL != pTreeDlg)
	{
		if (! pTreeDlg->SNUIT_RemoveTreeItems(this, false) )
		{
			return;
		}
		else
		{
			const ISNUITreeItem *tiSelect = m_pParentArtist->GetPrevious(this);

			if (NULL == tiSelect)
			{
				tiSelect = m_pParentArtist->GetNext(this);
			}

			if (NULL == tiSelect)
			{
				tiSelect = m_pParentArtist;
			}

			pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(tiSelect);
		}
	}

	m_pParentArtist->RemoveAndDelete(this);

	// NOTE: This object is now deleted and we must not access members.
}

void CSNDisc::SNUIT_PressedAddButton(ISNUITreeDlg *pTreeDlg)
{
	if (NULL != pTreeDlg)
	{
		pTreeDlg->SNUIT_AsyncAddDisc(m_pParentArtist->GetParent()->GetParent(),
										m_pParentArtist->m_pParentVolume, m_pParentArtist,
										this, this->GetDevice());
	}
}

void CSNDisc::SNUIT_PressedEditButton(ISNUITreeDlg *pTreeDlg)
{
	if (NULL != pTreeDlg)
	{
		pTreeDlg->SNUIT_AsyncEditDisc(m_pParentArtist->GetParent()->GetParent(),
										m_pParentArtist->GetParent(), m_pParentArtist, this);
	}
}

bool CSNDisc::SNUIT_GetExpandParentInitial(void) const
{
	return(false);
}

xmlNodePtr CSNDisc::AddToXML(xmlNodePtr parentNode) const
{
	xmlNodePtr discNode = xmlNewChild(parentNode, NULL, SNXML_NODE_DISC, NULL);

	xmlSetProp(discNode, SNXML_PROP_NAME,		(const xmlChar *)GetName()->GetEscaped(true));

	if (GetShortName()->IsANonEmptyString())
	{
		xmlSetProp(discNode, SNXML_PROP_SHORTNAME, (const xmlChar *)GetShortName()->GetEscaped(true));
	}

	if (GetSortAsName()->IsANonEmptyString())
	{
		xmlSetProp(discNode, SNXML_PROP_SORTASNAME,	(const xmlChar *)GetSortAsName()->GetEscaped(true));
	}

	CUnsignedBaseTenNumber ubtnDiscNumber(m_discNumber);
	xmlSetProp(discNode, SNXML_PROP_NUMBER, (const xmlChar *)ubtnDiscNumber.GetAsciiRepresentation());

	if (NULL != m_pDevice)
	{
		xmlSetProp(discNode, SNXML_PROP_DEVICENAME, (const xmlChar *)m_pDevice->GetName()->GetEscaped(true));
	}
	else if (m_deviceNameBeforeResolution.IsANonEmptyString())
	{
		xmlSetProp(discNode, SNXML_PROP_DEVICENAME, (const xmlChar *)m_deviceNameBeforeResolution.GetEscaped(true));
	}

	xmlSetProp(discNode, SNXML_PROP_IGNORETRACKS, GetIgnoreTracks() ? SNXML_VALUE_1 : SNXML_VALUE_0);

	for (const CSNTrack *pTrack = m_tracks.GetFirst(); NULL != pTrack; pTrack = m_tracks.GetNext(pTrack))
	{
		pTrack->AddToXML(discNode);
	}

	return(discNode);
}

bool CSNDisc::BuildFromXML(CTypedError *pTError, CStringList *pSLErrorDetails, xmlNodePtr pNode)
{
	bool bResult = true;

	xmlChar *pXC;

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_NAME)))
	{
		GetName()->SetEscaped((const char *)pXC);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_SHORTNAME)))
	{
		GetShortName()->SetEscaped((const char *)pXC);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_SORTASNAME)))
	{
		GetSortAsName()->SetEscaped((const char *)pXC);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_NUMBER)))
	{
		int i = atoi((const char *)pXC);
		SetDiscNumber(i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_DEVICENAME)))
	{
		m_pDevice = NULL; // We need device resolution.
		m_deviceNameBeforeResolution.SetEscaped((const char *)pXC);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_IGNORETRACKS)))
	{
		int i = atoi((const char *)pXC);
		SetIgnoreTracks(0 != i);
		xmlFree(pXC);
	}

	for (xmlNodePtr pNodeChild = pNode->children; NULL != pNodeChild; pNodeChild = pNodeChild->next)
	{
		if ((!xmlIsBlankNode(pNodeChild)) && (NULL != pNodeChild->name))
		{
			if (0 == strcmp((const char *)SNXML_NODE_TRACK, (const char *)pNodeChild->name))
			{
				CSNTrack *pTrack = new CSNTrack();

				pTrack->m_pParentDisc = this;

				if ((pTrack->BuildFromXML(pTError, pSLErrorDetails, pNodeChild))	\
				&&	(NULL != m_tracks.AddBelowAndOwn(pTError, pTrack, NULL))		)
				{
					pTrack = NULL; // Owned.
				}
				else
				{
					delete pTrack;
					bResult = false;
					break;
				}
			}
		}
	}

	return(bResult);
}

bool CSNDisc::AddToCSVFile(CSNCSVLine *pSNCSVLine, CCSVFile *pCSVFile) const
{
	bool bAddedAnyStuff = false;

	CString csDiscNumber;
	csDiscNumber.Format("%d", GetDiscNumber());
	
	CProntoString prstrDeviceName;
	const CSNDevice *pDevice = GetDevice();
	if (NULL != pDevice)
	{
		prstrDeviceName.SetPronto(pDevice->GetName()->GetPronto(false));
	}
	else
	{
		prstrDeviceName.SetPronto(GetUnresolvedDeviceName()->GetPronto(false));
	}

	if (pSNCSVLine->SetSNDevName(     prstrDeviceName.GetPronto(true)))	{ bAddedAnyStuff = true; }
	if (pSNCSVLine->SetDiscName(      m_name.GetPronto(true)))			{ bAddedAnyStuff = true; }
	if (pSNCSVLine->SetDiscShortName( m_shortName.GetPronto(true)))		{ bAddedAnyStuff = true; }
	if (pSNCSVLine->SetDiscSortAsName(m_sortAsName.GetPronto(true)))	{ bAddedAnyStuff = true; }
	if (pSNCSVLine->SetDiscNumber(LPCTSTR(csDiscNumber)))				{ bAddedAnyStuff = true; }

	bool bChildrenAddedAnyStuff = false;

	if (!pSNCSVLine->IsReady())
	{
		for (const CSNTrack *pTrack = m_tracks.GetFirst(); NULL != pTrack; pTrack = m_tracks.GetNext(pTrack))
		{
			CSNCSVLine lineCopy = *pSNCSVLine;
		
			if (pTrack->AddToCSVFile(&lineCopy, pCSVFile))
			{
				bChildrenAddedAnyStuff = true;
			}
		}
	}

	if ((!bChildrenAddedAnyStuff) && (bAddedAnyStuff))
	{
		pSNCSVLine->AddToCSVFile(pCSVFile);
	}

	return(bAddedAnyStuff || bChildrenAddedAnyStuff);
}

bool CSNDisc::BuildFromCSVLine(CTypedError *pTError, const CSNCSVLine *pLineCopy)
{
	bool bResult = true;

	if (pLineCopy->GetSNDevName())
	{
		m_pDevice = NULL; // We need device resolution.
		m_deviceNameBeforeResolution.SetPronto(pLineCopy->GetSNDevName());
	}

	if (pLineCopy->GetDiscShortName())	m_shortName.SetPronto(pLineCopy->GetDiscShortName());
	if (pLineCopy->GetDiscSortAsName())	m_sortAsName.SetPronto(pLineCopy->GetDiscSortAsName());
	if (pLineCopy->GetDiscNumber())		this->SetDiscNumber(atoi(pLineCopy->GetDiscNumber()));

	if (pLineCopy->GetTrackNumber())
	{
		CSNTrack *pTrack = this->GetTracks()->GetNonConst(this->GetTracks()->GetByNamePronto(pLineCopy->GetTrackNumber(), NULL));

		if (NULL == pTrack)
		{
			CSNTrack *pTrackNew = new CSNTrack();
			pTrackNew->m_pParentDisc = this;
			pTrackNew->SetTrackNumber(atoi(pLineCopy->GetTrackNumber()));

			pTrack = this->GetTracks()->GetNonConst(this->GetTracks()->AddBelowAndOwn(pTError, pTrackNew, NULL));

			if (NULL == pTrack)
			{
				delete pTrackNew;
				bResult = false;
			}
		}

		if (bResult && (!pTrack->BuildFromCSVLine(pTError, pLineCopy)))
		{
			bResult = false;
		}
	}

	return(bResult);
}

// Only call after a successful call of ResolveCSFPaths.
void CSNDisc::AddDiscCallToButtonAction(CCSFSection *pActionSection, bool bIncludeJump) const
{
	if (NULL != m_pDevice)
	{
		m_pDevice->AddDiscCallToButtonAction(pActionSection, m_discNumber, bIncludeJump,
												m_pParentArtist->GetParent()->GetJumpPanel());
	}
}
