// SNDisc.h: interface for the CSNDisc class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SNDISC_H__89B2B1DB_AB5A_4522_A2A6_36344DC8604D__INCLUDED_)
#define AFX_SNDISC_H__89B2B1DB_AB5A_4522_A2A6_36344DC8604D__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SNList.h"
#include "SNUITree.h"
#include "SNTrack.h"

class CSNDevice;
class CSNArtist;
class CSNDeviceList;
class CCSFSection;
class CSNCSVLine;
class CCSVFile;

class CSNDisc : public ISNUITreeItem, public CSNListObject
{
	friend class CSNList<CSNDisc>;
	friend class CSNUpdatableList<CSNDisc>;
	friend class CSNArtist;
	friend class CSNLibrary;

public:
	CSNDisc();
	virtual ~CSNDisc();

	bool IsValid(CTypedError *pTError) const;

	// Returns < 0 if this <  pDisc
	// Returns   0 if this == pDisc
	// Returns > 0 if this >  pDisc
	int CompareForSort(const CSNDisc *pDisc) const;

	void Set(const CSNDisc *pDisc);

		  CProntoString *GetName(void);
	const CProntoString *GetName(void) const;
		  CProntoString *GetShortName(void);
	const CProntoString *GetShortName(void) const;
		  CProntoString *GetSortAsName(void);
	const CProntoString *GetSortAsName(void) const;

	const CProntoString *GetShortNameOrName(void) const;
	const CProntoString *GetSortAsNameOrName(void) const;

	int GetDiscNumber(void) const;
	void SetDiscNumber(int discNumber);

	// GetDevice may return NULL even though the CSNDisc is valid if
	// device resolution has not yet been done.
	const CSNDevice *GetDevice(void) const;
	// Also clears m_deviceNameBeforeResolution.
	void SetDevice(const CSNDevice *pDevice);

	CString GetFullPathPronto(void) const;

	CString GetDiscNumberLabelEscaped(void) const;

	bool GetIgnoreTracks(void) const;
	void SetIgnoreTracks(bool bIgnoreTracks);

	CSNTrackList *GetTracks(void);
	const CSNTrackList *GetTracks(void) const;

	xmlNodePtr AddToXML(xmlNodePtr parentNode) const;
	bool BuildFromXML(CTypedError *pTError, CStringList *pSLErrorDetails, xmlNodePtr pNode);

	bool AddToCSVFile(CSNCSVLine *pSNCSVLine, CCSVFile *pCSVFile) const;
	bool BuildFromCSVLine(CTypedError *pTError, const CSNCSVLine *pLineCopy);

	void ResolveDevice(set<CSNDisc *> *pDiscSet, const CSNDeviceList *pDeviceList);
	void UnlinkDevice(const CSNDevice *pDevice);

	CSNArtist *GetParent(void);
	const CSNArtist *GetParent(void) const;

	const CProntoString *GetUnresolvedDeviceName(void) const;

	// Only call after a successful call of ResolveCSFPaths.
	void AddDiscCallToButtonAction(CCSFSection *pActionSection, bool bIncludeJump) const;

protected:
	// Type name should be lowercase, singular.
	const char *getTypeName(void) const;

public:
	bool SNUIT_AddChildrenToTree(ISNUITreeDlg *pTreeDlg);
	CString SNUIT_GetLabel(void) const;
	int SNUIT_GetImageIdx(ISNUITreeDlg *pTreeDlg) const;
	CString SNUIT_GetDeleteConfirmMessage(void) const;
	const char *SNUIT_GetAddButtonName(void)	const;
	const char *SNUIT_GetEditButtonName(void)	const;
	const char *SNUIT_GetDeleteButtonName(void)	const;
	const char *SNUIT_GetSortButtonName(void)	const;
	bool SNUIT_GetKeepSorted(void) const;
	void SNUIT_SetKeepSorted(ISNUITreeDlg *pTreeDlg, bool bKeepSorted);
	void SNUIT_PressedSortButton(ISNUITreeDlg *pTreeDlg);
	bool SNUIT_GetEditButtonActivation(void)		const;
	bool SNUIT_GetDeleteButtonActivation(void)		const;
	bool SNUIT_GetUpButtonActivation(void)			const;
	bool SNUIT_GetDownButtonActivation(void)		const;
	bool SNUIT_GetSortButtonActivation(void)		const;
	bool SNUIT_GetKeepSortedCheckActivation(void)	const;
	void SNUIT_PressedUpButton(ISNUITreeDlg *pTreeDlg);
	void SNUIT_PressedDownButton(ISNUITreeDlg *pTreeDlg);
	void SNUIT_PressedDeleteButton(ISNUITreeDlg *pTreeDlg);
	void SNUIT_PressedAddButton(ISNUITreeDlg *pTreeDlg);
	void SNUIT_PressedEditButton(ISNUITreeDlg *pTreeDlg);
	bool SNUIT_GetExpandParentInitial(void) const;

private:
	CProntoString m_name;
	CProntoString m_shortName;
	CProntoString m_sortAsName;
	int m_discNumber;
	const CSNDevice *m_pDevice;

	CProntoString m_deviceNameBeforeResolution;

	bool m_bIgnoreTracks;
	CSNTrackList m_tracks;

	CSNArtist *m_pParentArtist;
};

#endif // !defined(AFX_SNDISC_H__89B2B1DB_AB5A_4522_A2A6_36344DC8604D__INCLUDED_)
