// SNLibrary.cpp: implementation of the CSNLibrary class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SNConfig.h"
#include "SNLibrary.h"
#include "CSFSection.h"
#include "SNCSVLine.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSNLibrary::CSNLibrary()
{
}

CSNLibrary::~CSNLibrary()
{

}

void CSNLibrary::ResolveDevices(set<CSNDisc *> *pDiscSet, const CSNDeviceList *pDeviceList)
{
	for (CSNVolume *pVol = GetNonConst(GetFirst()); NULL != pVol; pVol = GetNonConst(GetNext(pVol)))
	{
		pVol->ResolveDevices(pDiscSet, pDeviceList);
	}
}

void CSNLibrary::UnlinkDevice(const CSNDevice *pDevice)
{
	for (CSNVolume *pVol = GetNonConst(GetFirst()); NULL != pVol; pVol = GetNonConst(GetNext(pVol)))
	{
		pVol->UnlinkDevice(pDevice);
	}
}

int CSNLibrary::GetHighestDiscNumber(const CSNDevice *pDevice) const
{
	int result = 0;
	int candidate;

	for (const CSNVolume *pVol = GetFirst(); NULL != pVol; pVol = GetNext(pVol))
	{
		candidate = pVol->GetHighestDiscNumber(pDevice);

		if (result < candidate)
		{
			result = candidate;
		}
	}

	return(result);
}

void CSNLibrary::ClearDirty(void)
{
	CSNList<CSNVolume>::ClearDirty();

	for (CSNVolume *pVol = GetNonConst(GetFirst());
		 NULL != pVol;
		 pVol = GetNonConst(GetNext(pVol)))
	{
		pVol->ClearDirty();
	}
}

bool CSNLibrary::IsDirty(void) const
{
	bool bDirty = CSNList<CSNVolume>::IsDirty();

	for (const CSNVolume *pVol = GetFirst();
		 (!bDirty) && (NULL != pVol);
		 pVol = GetNext(pVol))
	{
		bDirty = pVol->IsDirty();
	}

	return(bDirty);
}

unsigned int CSNLibrary::GetNumElements(void) const
{
	return(CSNList<CSNVolume>::GetNumElements());
}

const CSNVolume *CSNLibrary::GetFirst(void) const
{
	return(CSNList<CSNVolume>::GetFirst());
}

const CSNVolume *CSNLibrary::GetNext(const CSNVolume *pCurrent) const
{
	return(CSNList<CSNVolume>::GetNext(pCurrent));
}

CSNVolume *CSNLibrary::GetNonConst(const CSNVolume *pVolume)
{
	return(CSNList<CSNVolume>::GetNonConst(pVolume));
}

const CSNVolume *CSNLibrary::GetByNamePronto(const char *szNamePronto, const CSNVolume *pFindAfter) const
{
	return(CSNList<CSNVolume>::GetByNamePronto(szNamePronto, pFindAfter));
}

void CSNLibrary::RemoveAndDeleteAll(void)
{
	CSNList<CSNVolume>::RemoveAndDeleteAll();
}

xmlNodePtr CSNLibrary::AddToXML(xmlNodePtr parentNode) const
{
	xmlNodePtr libraryNode = xmlNewChild(parentNode, NULL, SNXML_NODE_LIBRARY, NULL);
	xmlSetProp(libraryNode, SNXML_PROP_KEEPCHILDRENSORTED, GetSort() ? SNXML_VALUE_1 : SNXML_VALUE_0);

	for (const CSNVolume *pVol = GetFirst(); NULL != pVol; pVol = GetNext(pVol))
	{
		pVol->AddToXML(libraryNode);
	}

	return(libraryNode);
}

CString CSNLibrary::SNUIT_GetLabel(void) const
{
	return("Lists");
}

int CSNLibrary::SNUIT_GetImageIdx(ISNUITreeDlg *pTreeDlg) const
{
	return(pTreeDlg->SNUIT_GetImageIdxRoot());
}

CString CSNLibrary::SNUIT_GetDeleteConfirmMessage(void) const
{
	return("");
}

bool CSNLibrary::SNUIT_AddChildrenToTree(ISNUITreeDlg *pTreeDlg)
{
	bool bResult = true;

	for (CSNVolume *pVolume = GetNonConst(GetLast()); pVolume != NULL; pVolume = GetNonConst(GetPrevious(pVolume)))
	{
		// Insert at the top of the list.
		if ((! pTreeDlg->SNUIT_AddTreeItem(pVolume, this, NULL) ) \
		||	(! pVolume->SNUIT_AddChildrenToTree(pTreeDlg) )		  )
		{
			bResult = false;
			break;
		}
	}

	return(bResult);
}

const char *CSNLibrary::SNUIT_GetAddButtonName(void) const
{
	return("Add List");
}

const char *CSNLibrary::SNUIT_GetEditButtonName(void) const
{
	return("Edit");
}

const char *CSNLibrary::SNUIT_GetDeleteButtonName(void) const
{
	return("Delete");
}

const char *CSNLibrary::SNUIT_GetSortButtonName(void) const
{
	return("Sort");
}

bool CSNLibrary::SNUIT_GetKeepSorted(void) const
{
	return(false);
}

void CSNLibrary::SNUIT_SetKeepSorted(ISNUITreeDlg *pTreeDlg, bool bKeepSorted)
{
}

void CSNLibrary::SNUIT_PressedSortButton(ISNUITreeDlg *pTreeDlg)
{
}

bool CSNLibrary::SNUIT_GetEditButtonActivation(void) const
{
	return(false);
}

bool CSNLibrary::SNUIT_GetDeleteButtonActivation(void) const
{
	return(false);
}

bool CSNLibrary::SNUIT_GetUpButtonActivation(void) const
{
	return(false);
}

bool CSNLibrary::SNUIT_GetDownButtonActivation(void) const
{
	return(false);
}

bool CSNLibrary::SNUIT_GetSortButtonActivation(void) const
{
	return(false);
}

bool CSNLibrary::SNUIT_GetKeepSortedCheckActivation(void) const
{
	return(false);
}

void CSNLibrary::SNUIT_PressedUpButton(ISNUITreeDlg *pTreeDlg)
{
}

void CSNLibrary::SNUIT_PressedDownButton(ISNUITreeDlg *pTreeDlg)
{
}

void CSNLibrary::SNUIT_PressedDeleteButton(ISNUITreeDlg *pTreeDlg)
{
}

void CSNLibrary::SNUIT_PressedAddButton(ISNUITreeDlg *pTreeDlg)
{
	if (NULL != pTreeDlg)
	{
		pTreeDlg->SNUIT_AsyncAddList(this, GetLast());
	}
}

void CSNLibrary::SNUIT_PressedEditButton(ISNUITreeDlg *pTreeDlg)
{
}

bool CSNLibrary::SNUIT_GetExpandParentInitial(void) const
{
	return(true);
}

const CSNVolume *CSNLibrary::SNUIT_AddBelowAndOwnVolume(ISNUITreeDlg *pTreeDlg, CTypedError *pTError,
											CSNVolume *pInVolume,
											const CSNVolume *pToAddBelow)
{
	pInVolume->m_pParentLibrary = this;

	CSNVolume *pResult = GetNonConst(AddBelowAndOwn(pTError, pInVolume, pToAddBelow));

	if (NULL != pResult)
	{
		// Do NOT null out pResult if anything goes wrong after now.

		if (NULL != pTreeDlg)
		{
			// GetPrevious call needed since the list may be sorted and pAddBelow ignored.
			if (pTreeDlg->SNUIT_AddTreeItem(pResult, this, GetPrevious(pResult)))
			{
				pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(pResult);
			}
		}
	}

	return(pResult);
}

const CSNVolume *CSNLibrary::SNUIT_UpdateVolume(ISNUITreeDlg *pTreeDlg, CTypedError *pTError,
									CSNVolume *pInVolume,
									const CSNVolume *pToUpdate)
{
	pInVolume->m_pParentLibrary = this;

	const CSNVolume *pOldPrevious = GetPrevious(pToUpdate);

	CSNVolume *pResult = GetNonConst(Update(pTError, pInVolume, pToUpdate));

	if (NULL != pResult)
	{
		// Do NOT null out pResult if anything goes wrong after now.

		if (NULL != pTreeDlg)
		{
			const CSNVolume *pNewPrevious = GetPrevious(pResult);

			if ((!pResult->SNUIT_GetKeepSorted()) \
			||	(pNewPrevious == pOldPrevious))
			{
				// Item's position in the list hasn't changed (fast).
				pTreeDlg->SNUIT_UpdateLabel(pResult);
			}
			else
			{
				// Item's position in the list has changed (slow).
				if (pTreeDlg->SNUIT_RemoveTreeItems(pResult, false))
				{
					if (pTreeDlg->SNUIT_AddTreeItem(pResult, this, GetPrevious(pResult)))
					{
						pResult->SNUIT_AddChildrenToTree(pTreeDlg);

						pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(pResult);
					}
				}
			}
		}
	}

	return(pResult);
}

bool CSNLibrary::BuildFromXML(CTypedError *pTError, CStringList *pSLErrorDetails, xmlNodePtr pNode)
{
	bool bResult = true;

	xmlChar *pXC;

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_KEEPCHILDRENSORTED)))
	{
		int i = atoi((const char *)pXC);
		SetSort(0 != i);
		xmlFree(pXC);
	}

	CSNVolume *pVol;

	for (xmlNodePtr pNodeChild = pNode->children; NULL != pNodeChild; pNodeChild = pNodeChild->next)
	{
		if ((!xmlIsBlankNode(pNodeChild)) && (NULL != pNodeChild->name))
		{
			if (0 == strcmp((const char *)SNXML_NODE_VOLUME, (const char *)pNodeChild->name))
			{
				pVol = new CSNVolume();

				pVol->m_pParentLibrary = this;

				if ((pVol->BuildFromXML(pTError, pSLErrorDetails, pNodeChild))	\
				&&	(NULL != AddBelowAndOwn(pTError, pVol, NULL))				)
				{
					pVol = NULL; // Owned.
				}
				else
				{
					delete pVol;
					bResult = false;
					break;
				}
			}
		}
	}

	return(bResult);
}

bool CSNLibrary::AddToCSVFile(CSNCSVLine *pSNCSVLine, CCSVFile *pCSVFile) const
{
	bool bAddedAnyStuff = false; // Nothing to add at this level.
	bool bChildrenAddedAnyStuff = false;

	if (!pSNCSVLine->IsReady())
	{
		for (const CSNVolume *pVol = GetFirst(); NULL != pVol; pVol = GetNext(pVol))
		{
			CSNCSVLine lineCopy = *pSNCSVLine;
		
			if (pVol->AddToCSVFile(&lineCopy, pCSVFile))
			{
				bChildrenAddedAnyStuff = true;
			}
		}
	}

	if ((!bChildrenAddedAnyStuff) && (bAddedAnyStuff))
	{
		pSNCSVLine->AddToCSVFile(pCSVFile);
	}

	return(bAddedAnyStuff || bChildrenAddedAnyStuff);
}

bool CSNLibrary::BuildFromCSVFile(CTypedError *pTError, const CSNCSVLine *pSNCSVLine, const CCSVFile *pCSVFile)
{
	bool bResult = true;

	// Name for list to place stuff without a list specified.

	CString csDefVolumeName = "CD List";

	for (int i = 1; i < 101; i++)
	{
		if (NULL == this->GetByNamePronto(csDefVolumeName, NULL))
		{
			break;
		}

		csDefVolumeName.Format("CD List %d", i);

		// If there are 100 imported lists (heh), just merge with CD List 100...
	}

	if (101 <= i)
	{
		CTypedError::SafeSetMsgError(pTError, "Could not import because there are already 100 lists named \"CD List\"");
		bResult = false;
	}
	else
	{
		for (CCSVFile::const_lineIter liter = pCSVFile->Begin(); liter != pCSVFile->End(); liter++)
		{
			CSNCSVLine lineCopy = *pSNCSVLine;

			lineCopy.GetFromCSVFile(pCSVFile, liter);

			CString csVolName = (NULL != lineCopy.GetListName() ? lineCopy.GetListName() : csDefVolumeName);

			CSNVolume *pVol = this->GetNonConst(this->GetByNamePronto(csVolName, NULL));

			if (NULL == pVol)
			{
				CSNVolume *pVolNew = new CSNVolume();
				pVolNew->GetName()->SetPronto(csVolName);

				pVol = this->GetNonConst(this->SNUIT_AddBelowAndOwnVolume(NULL, pTError, pVolNew, NULL));

				if (NULL == pVol)
				{
					delete pVolNew;
					bResult = false;
					break;
				}
			}

			if (!pVol->BuildFromCSVLine(pTError, &lineCopy))
			{
				bResult = false;
				break;
			}
		}
	}

	return(bResult);
}


bool CSNLibrary::ResolveCSFPaths(CTypedError *pTError, const CCSFAbstractTree *pAT)
{
	for (CSNVolume *pVol = GetNonConst(GetFirst()); NULL != pVol; pVol = GetNonConst(GetNext(pVol)))
	{
		if (! pVol->ResolveCSFPaths(pTError, pAT) )
		{
			return(false);
		}
	}

	return(true);
}

void CSNLibrary::AddToAndUpdateRootCSFSection(CSNConfig *pConfig, CCSFSection *pRootCSFSection, CStringList *pStrLstEmpties) const
{
	for (const CSNVolume *pVol = GetFirst(); NULL != pVol; pVol = GetNext(pVol))
	{
		pVol->AddToAndUpdateRootCSFSection(pConfig, pRootCSFSection, pStrLstEmpties);
	}
}
