// SNLibrary.h: interface for the CSNLibrary class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SNLIBRARY_H__0DFFF765_3A0F_42A7_8F43_CA7E61930FCC__INCLUDED_)
#define AFX_SNLIBRARY_H__0DFFF765_3A0F_42A7_8F43_CA7E61930FCC__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SNVolume.h"

class CSNDevice;
class CCSFSection;
class CSNConfig;
class CSNCSVLine;
class CCSVFile;

class CSNLibrary : public ISNUITreeItem, public ISNUITreeLibrary, protected CSNUpdatableList<CSNVolume>
{
	friend class CSNVolume;

public:
	CSNLibrary();
	virtual ~CSNLibrary();

	xmlNodePtr AddToXML(xmlNodePtr parentNode) const;
	bool BuildFromXML(CTypedError *pTError, CStringList *pSLErrorDetails, xmlNodePtr pNode);

	bool AddToCSVFile(CSNCSVLine *pSNCSVLine, CCSVFile *pCSVFile) const;
	bool BuildFromCSVFile(CTypedError *pTError, const CSNCSVLine *pSNCSVLine, const CCSVFile *pCSVFile);

	void ClearDirty(void);
	bool IsDirty(void) const;

	void ResolveDevices(set<CSNDisc *> *pDiscSet, const CSNDeviceList *pDeviceList);
	void UnlinkDevice(const CSNDevice *pDevice);

	int GetHighestDiscNumber(const CSNDevice *pDevice) const;

	void RemoveAndDeleteAll(void);

	bool ResolveCSFPaths(CTypedError *pTError, const CCSFAbstractTree *pAT);

	unsigned int GetNumElements(void) const;
	const CSNVolume *GetFirst(void) const;
	const CSNVolume *GetNext(const CSNVolume *pCurrent) const;
	CSNVolume *GetNonConst(const CSNVolume *pVolume);
	const CSNVolume *GetByNamePronto(const char *szNamePronto, const CSNVolume *pFindAfter) const;

	void AddToAndUpdateRootCSFSection(CSNConfig *pConfig, CCSFSection *pRootCSFSection, CStringList *pStrLstEmpties) const;

public:
	const CSNVolume *SNUIT_AddBelowAndOwnVolume(ISNUITreeDlg *pTreeDlg, CTypedError *pTError,
												CSNVolume *pInVolume,
												const CSNVolume *pToAddBelow);

	const CSNVolume *SNUIT_UpdateVolume(ISNUITreeDlg *pTreeDlg, CTypedError *pTError,
										CSNVolume *pInVolume,
										const CSNVolume *pToUpdate);

public:
	bool SNUIT_AddChildrenToTree(ISNUITreeDlg *pTreeDlg);
	CString SNUIT_GetLabel(void) const;
	int SNUIT_GetImageIdx(ISNUITreeDlg *pTreeDlg) const;
	CString SNUIT_GetDeleteConfirmMessage(void) const;
	const char *SNUIT_GetAddButtonName(void)	const;
	const char *SNUIT_GetEditButtonName(void)	const;
	const char *SNUIT_GetDeleteButtonName(void)	const;
	const char *SNUIT_GetSortButtonName(void)	const;
	bool SNUIT_GetKeepSorted(void) const;
	void SNUIT_SetKeepSorted(ISNUITreeDlg *pTreeDlg, bool bKeepSorted);
	void SNUIT_PressedSortButton(ISNUITreeDlg *pTreeDlg);
	bool SNUIT_GetEditButtonActivation(void)		const;
	bool SNUIT_GetDeleteButtonActivation(void)		const;
	bool SNUIT_GetUpButtonActivation(void)			const;
	bool SNUIT_GetDownButtonActivation(void)		const;
	bool SNUIT_GetSortButtonActivation(void)		const;
	bool SNUIT_GetKeepSortedCheckActivation(void)	const;
	void SNUIT_PressedUpButton(ISNUITreeDlg *pTreeDlg);
	void SNUIT_PressedDownButton(ISNUITreeDlg *pTreeDlg);
	void SNUIT_PressedDeleteButton(ISNUITreeDlg *pTreeDlg);
	void SNUIT_PressedAddButton(ISNUITreeDlg *pTreeDlg);
	void SNUIT_PressedEditButton(ISNUITreeDlg *pTreeDlg);
	bool SNUIT_GetExpandParentInitial(void) const;
};

#endif // !defined(AFX_SNLIBRARY_H__0DFFF765_3A0F_42A7_8F43_CA7E61930FCC__INCLUDED_)
