// SNList.cpp: implementation of the CSNList class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SNList.h"
#include <algorithm>
using namespace std;;

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//----------------------------------------------------------------------------------------------
// Stupid crap to make the template code get generated without having
// the implementation in the header file.
// Microsoft should get their C++ compiler[1] to work before starting on
// new languages like C#.
// [1] Not to mention their crappy Operating Systems and APIs.
//
// See "PRB: LNK2001 on Template Member Functions":
// http://support.microsoft.com/support/kb/articles/Q239/4/36.ASP
//----------------------------------------------------------------------------------------------
#include "SNDevice.h"
#include "SNLibrary.h"
#include "CSFAbstractTree.h"
#include "SNHardButtonDefinition.h"
#pragma warning(disable: 4660) // It's not instantiated though, is it?
template class CSNList<CSNDevice>;
template class CSNUpdatableList<CSNDevice>;
template class CSNList<CSNDisc>;
template class CSNUpdatableList<CSNDisc>;
template class CSNList<CSNArtist>;
template class CSNUpdatableList<CSNArtist>;
template class CSNList<CSNVolume>;
template class CSNUpdatableList<CSNVolume>;
template class CSNList<CCSFATButton>;
template class CSNList<CCSFATPanel>;
template class CSNList<CCSFATDevice>;
template class CSNList<CCSFButtonPath>;
template class CSNUpdatableList<CCSFButtonPath>;
template class CSNSetableList<CCSFButtonPath>;
template class CSNList<CSNHardButtonDefinition>;
template class CSNUpdatableList<CSNHardButtonDefinition>;
template class CSNSetableList<CSNHardButtonDefinition>;
template class CSNList<CSNTrack>;
template class CSNUpdatableList<CSNTrack>;
template class CSNSetableList<CSNTrack>;
#pragma warning(default: 4660)
//----------------------------------------------------------------------------------------------
// End stupid crap.
//----------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------

template<class TListObject> CSNList<TListObject>::CSNList()
{
	m_bDirty = false;

	m_bSort = false;
	m_bRequireUniqueNames = true;

	// By default, add space for 10 elements when we run out and need more.
	m_growType = GROW_ADD;
	m_growSizeMultipliedByTen = 100;
	m_recommendedMinSize = 0;

	m_allocatedSize = 0;
	m_numElements = 0;

	m_array = NULL;
}

template<class TListObject> CSNList<TListObject>::~CSNList()
{
	RemoveAndDeleteAll();
}

// Will not shrink the array too small to hold all current elements.
template<class TListObject> bool CSNList<TListObject>::resizeArray(unsigned int newSize)
{
	bool bResult = false;

	if (m_numElements > newSize)
	{
		newSize = m_numElements;
	}

	if (newSize != m_allocatedSize)
	{
		if (0 == newSize)
		{
			if (NULL != m_array)
			{
				free(m_array);
				m_array = NULL;
			}

			m_allocatedSize = newSize;
			bResult = true;
		}
		else
		{
			// Note that realloc is happy if m_array is NULL.

			TListObject **newArray;
			newArray = (TListObject **)realloc(m_array, sizeof(TListObject *) * newSize);

			// On failure, leave m_array and m_allocatedSize the same as before,
			// since when realloc fails the old array is not freed.

			if (NULL != newArray)
			{
				// Success
				m_array = newArray;
				m_allocatedSize = newSize;
				bResult = true;
			}
		}
	}

	return(bResult);
}

// To improve performance, reserve space before large insertions.
// If bAdditional is true, readies the list for allocSize new items to
// be added, else makes sure the list is ready for at least allocSize
// elements in total.
template<class TListObject> void CSNList<TListObject>::ReserveAllocation(unsigned int allocSize, bool bAdditional)
{
	if (bAdditional)
	{
		// allocSize could be zero...
		allocSize += m_allocatedSize;
	}

	if (allocSize > m_allocatedSize)
	{
		resizeArray(allocSize);
	}
}

template<class TListObject> void CSNList<TListObject>::FreeExcessAllocation(void)
{
	resizeArray(0);
}

// Allows you to change the way the list grows and shrinks automatially.
// If you set GROW_MULTIPLY you may want to use specify a non-zero
// recommendedMinSize to avoid lots of reallocations when adding and removing
// small lists. You can always call FreeExcessAllocation when the list contents are
// more stable.
template<class TListObject> void CSNList<TListObject>::SetAllocationRule(GrowType growType,
								unsigned int growSizeMultipliedByTen,
								unsigned int recommendedMinSize)
{
	// calcSize will make sure the combination of grow variables and current sizes do not
	// result in zero growth. We should still reject changes to completely wrong values
	// here, though. (Multiplcation by 1 or less, addition of less than 1, and so on.)

	// Just store recommendedMinSize (note that m_recommendedMinSize is NOT used
	// by calcSize but, optionally, by those functions which use calcSize.
	m_recommendedMinSize = recommendedMinSize;

	if (GROW_ADD == growType)
	{
		if (10 <= growSizeMultipliedByTen)
		{
			m_growType = growType;
			m_growSizeMultipliedByTen = growSizeMultipliedByTen;
		}
	}
	else if (GROW_MULTIPLY == growType)
	{
		if (10 < growSizeMultipliedByTen)
		{
			m_growType = growType;
			m_growSizeMultipliedByTen = growSizeMultipliedByTen;
		}
	}
}

// Must always increase / decrease the size by at least one.
// Does not actually change the size, just calculates a new one.
// When shrinking, it does not check against the element count.
template<class TListObject> unsigned int CSNList<TListObject>::calcSize(unsigned int oldSize, bool bGrow) const
{
	unsigned int newSize = oldSize;

	if (GROW_ADD == m_growType)
	{
		unsigned int intGrowSize = (m_growSizeMultipliedByTen / 10);

		if (bGrow)
		{
			newSize = oldSize + intGrowSize;
		}
		else if (newSize >= intGrowSize)
		{
			newSize = oldSize - intGrowSize;
		}
		else
		{
			newSize = 0;
		}
	}
	else if (GROW_MULTIPLY == m_growType)
	{
		// Rounding here may mean we don't always remove the same amount we previously
		// added. Aside from a few spare element slots, it shouldn't be a problem.

		if (bGrow)
		{
			newSize = (oldSize * m_growSizeMultipliedByTen) / 10;
		}
		else
		{
			newSize = (oldSize * 10) / m_growSizeMultipliedByTen;
		}
	}
	else
	{
		// Do something, Mutley! Pretend GROW_ADD, 100 (i.e. 10)
		if (bGrow)
		{
			newSize = oldSize + 10;
		}
		else if (newSize >= 10)
		{
			newSize = oldSize - 10;
		}
		else
		{
			newSize = 0;
		}
	}

	// If the result was rounded to zero, make sure we at least add/subtract one element.
	if ((bGrow) && (newSize <= oldSize))
	{
		newSize = oldSize + 1;
	}
	else if ((!bGrow) && (newSize >= oldSize))
	{
		if (oldSize >= 1)
		{
			newSize = oldSize - 1;
		}
		else
		{
			newSize = 0;
		}
	}

	return(newSize);
}

// static
template<class TListObject> int CSNList<TListObject>::qSortCompare(const void *arg1, const void *arg2)
{
	const TListObject **pLeft  = (const TListObject **)arg1;
	const TListObject **pRight = (const TListObject **)arg2;

	if (NULL == (*pLeft))
	{
		if (NULL == (*pRight))
		{
			return(0);
		}
		else
		{
			return(-1);
		}
	}
	else if (NULL == (*pRight))
	{
		return(1);
	}
	else
	{
		return((*pLeft)->CompareForSort(*pRight));
	}
}

// Sorting is disabled by default.
template<class TListObject> void CSNList<TListObject>::SetSort(bool bSort)
{
	m_bDirty = true;

	if ((!m_bSort) && bSort && (NULL != m_array) && (0 < m_numElements))
	{
		qsort(m_array, m_numElements, sizeof(TListObject *), qSortCompare);

		// Fix the index values in the objects.
		for (unsigned int i = 0; i < m_numElements; i++)
		{
			m_array[i]->m_indexInOwnerList = i;
		}
	}

	m_bSort = bSort;
}

template<class TListObject> bool CSNList<TListObject>::GetSort(void) const
{
	return(m_bSort);
}

// Unique names are required by default.
// Any duplicate names in the list remain when you turn unique names back on.
template<class TListObject> void CSNList<TListObject>::SetRequireUniqueNames(bool bReqUniqueNames)
{
	m_bRequireUniqueNames = bReqUniqueNames;
}

template<class TListObject> bool CSNList<TListObject>::GetRequireUniqueNames(void) const
{
	return(m_bRequireUniqueNames);
}

template<class TListObject> void CSNList<TListObject>::ClearDirty(void)
{
	m_bDirty = false;
}

template<class TListObject> bool CSNList<TListObject>::IsDirty(void) const
{
	return(m_bDirty);
}

// pToExclude may be NULL.
// Set startIdx to zero to search entire list.
// If is safe to call with startIdx indicating past the end of the list.
template<class TListObject> TListObject *CSNList<TListObject>::getByNameExcluding(const CProntoString *pName,
											const TListObject *pToExclude,
											unsigned int startIdx) const
{
	// When the list is sorted we could do a binary search here.
	// But the list isn't always sorted and everything else is linear time.

	TListObject *pResult = NULL;

	for (unsigned int i = startIdx; i < m_numElements; i++)
	{
		if (pToExclude == m_array[i])
		{
			continue;
		}

		if (0 == pName->Compare(m_array[i]->GetName(), false))
		{
			pResult = m_array[i];
			break;
		}
	}

	return(pResult);
}

// Returns NULL on failure.
template<class TListObject> TListObject *CSNList<TListObject>::GetNonConst(const TListObject *pConst)
{
	TListObject *pResult = NULL;

	if (NULL != pConst)
	{
		unsigned int idx = pConst->m_indexInOwnerList;

		if ((idx < m_numElements) && (pConst == m_array[idx]))
		{
			pResult = m_array[idx];
		}
	}

	return(pResult);
}


// Doesn't make sense to call when the list is not sorted.
// Will return index one past the end of the buffer if no item is bigger.
template<class TListObject> unsigned int CSNList<TListObject>::getIndexFirstGreater(const TListObject *pToOwn)
{
	// Could do a binary search to make things a bit faster, but all the
	// shuffling around of pointers to do insertions means we're stuck
	// in linear time anyway. Need to use a tree instead of an array. :)

	unsigned int idxResult = 0;

	for (idxResult = 0; idxResult < m_numElements; idxResult++)
	{
		if (0 > pToOwn->CompareForSort(m_array[idxResult]))
		{
			break;
		}
	}

	return(idxResult);
}

template<class TListObject>
void CSNList<TListObject>::removeElement(unsigned int idx, bool bDeleteElement, bool bAllowArrayShrink)
{
	m_bDirty = true;

	if (idx < m_numElements)
	{
		// If we are removing an object which does not have a unique name,
		// check if there will remain only one other object with the same
		// name and if so mark that object as now being uniquely named.
		if ((!m_bRequireUniqueNames) && (!m_array[idx]->m_bUniqueName))
		{
			TListObject *pFirstDupe;
			TListObject *pSecondDupe;

			pFirstDupe = getByNameExcluding(m_array[idx]->GetName(), m_array[idx], 0);

			if (NULL != pFirstDupe) // Shouldn't happen, but just in case.
			{
				// Safe to add one to the start index even if pFirstDupe is the last object.
				pSecondDupe = getByNameExcluding(m_array[idx]->GetName(), m_array[idx],
													pFirstDupe->m_indexInOwnerList + 1);

				if (NULL == pSecondDupe)
				{
					pFirstDupe->m_bUniqueName = true;
				}
			}
		}

		if (bDeleteElement)
		{
			// Delete the object.
			delete m_array[idx];
			m_array[idx] = NULL;
		}

		// Update the number of elements.
		m_numElements--;

		// Shuffle up those elements which were after the object, updating their
		// index values.
		for (unsigned int i = idx; i < m_numElements; i++)
		{
			m_array[i] = m_array[i+1];
			m_array[i]->m_indexInOwnerList = i;
		}

		if (bAllowArrayShrink)
		{
			// If we shrunk the array twice, would it still be large enough?
			// If so, shrink it, but only once, to avoid repeated growing and shrinking.
			unsigned int shrinkOnceSize = calcSize(m_allocatedSize, false);
			unsigned int shrinkTwiceSize = calcSize(shrinkOnceSize, false);

			if (shrinkTwiceSize >= m_numElements)
			{
				if (shrinkOnceSize < m_recommendedMinSize)
				{
					// resizeArray checks if we're asking for the same size
					// that we already have. No need to worry about resizing
					// to what we already have all the time.
					shrinkOnceSize = m_recommendedMinSize;
				}

				// If it fails to make it smaller, don't care. :-)
				resizeArray(shrinkOnceSize);
			}
		}
	}
}

template<class TListObject> bool CSNList<TListObject>::insertElement(CTypedError *pTError, TListObject *pToOwn, unsigned int idx)
{
	bool bResult = true;

	m_bDirty = true;

	// Sanity check.
	if (idx > m_numElements)
	{
		idx = m_numElements;
	}

	// Resize the array if it's full.
	if (m_numElements >= m_allocatedSize)
	{
		unsigned int growOnceSize = calcSize(m_allocatedSize, true);

		if (growOnceSize < m_recommendedMinSize)
		{
			growOnceSize = m_recommendedMinSize;
		}

		if (! resizeArray(growOnceSize) )
		{
			CTypedError::SafeSetMsgError(pTError, "Memory allocation failed");
			bResult = false;
		}
	}

	if (bResult)
	{
		pToOwn->m_bUniqueName = true;

		// If unique names are not required, mark new object if its
		// name is not unique, as well as first object found with the
		// same name, in case it was previously unique.
		if (!m_bRequireUniqueNames)
		{
			TListObject *pDupe = getByNameExcluding(pToOwn->GetName(), NULL, 0);

			if (NULL != pDupe)
			{
				pToOwn->m_bUniqueName = false;
				pDupe->m_bUniqueName = false;
			}
		}

		// Shift along those elements that are in the way.
		for (unsigned int i = m_numElements; i > idx; i--)
		{
			m_array[i] = m_array[i-1];
			m_array[i]->m_indexInOwnerList = i;
		}

		m_array[idx] = pToOwn;
		m_array[idx]->m_indexInOwnerList = idx;

		// Update the number of elements.
		m_numElements++;
	}

	return(bResult);
}

// Does nothing when the list is sorted.
template<class TListObject> void CSNList<TListObject>::MoveUp(const TListObject *pToMove)
{
	m_bDirty = true;

	if ((!m_bSort) && (NULL != pToMove))
	{
		unsigned int idx = pToMove->m_indexInOwnerList;

		if ((0 < idx) && (0 < m_numElements) && (idx < m_numElements))
		{
			TListObject *pTemp = m_array[idx];
			m_array[idx] = m_array[idx - 1];
			m_array[idx - 1] = pTemp;

			m_array[idx - 1]->m_indexInOwnerList = idx - 1;
			m_array[idx]->m_indexInOwnerList = idx;
		}
	}
}

// Does nothing when the list is sorted.
template<class TListObject> void CSNList<TListObject>::MoveDown(const TListObject *pToMove)
{
	m_bDirty = true;

	if ((!m_bSort) && (NULL != pToMove))
	{
		unsigned int idx = pToMove->m_indexInOwnerList;

		if ((0 < m_numElements) && ((idx+1) < m_numElements))
		{
			TListObject *pTemp = m_array[idx];
			m_array[idx] = m_array[idx + 1];
			m_array[idx + 1] = pTemp;

			m_array[idx]->m_indexInOwnerList = idx;
			m_array[idx + 1]->m_indexInOwnerList = idx + 1;
		}
	}
}

// Removing may cause (some) unused allocated space to be freed.
template<class TListObject> void CSNList<TListObject>::RemoveAndDelete(const TListObject *pToRemove)
{
	TListObject *pResult = GetNonConst(pToRemove);

	if (NULL != pResult)
	{
		// Remove the old element and allow the array to resize if it wants.
		removeElement(pResult->m_indexInOwnerList, true, true);
	}
}

// Returns NULL on failure.
template<class TListObject> TListObject *CSNList<TListObject>::RemoveAndReleaseOwnership(const TListObject *pToRemove)
{
	TListObject *pResult = GetNonConst(pToRemove);

	if (NULL != pResult)
	{
		// Remove the old element and allow the array to resize if it wants.
		removeElement(pResult->m_indexInOwnerList, false, true);
	}

	return(pResult);
}

template<class TListObject> void CSNList<TListObject>::RemoveAndDeleteAll(void)
{
	m_bDirty = true;

	while (0 < m_numElements)
	{
		delete m_array[--m_numElements];
		m_array[m_numElements] = NULL;
	}

	// This MUST free the array because it's called by the destructor.
	resizeArray(0);
}

// Empties this list and copies all elements from the other list to this one.
// Also copies the sort and unique-names flags and memory allocation parameters.
// Sets dirty flag.
template<class TListObject> void CSNSetableList<TListObject>::Set(const CSNSetableList<TListObject>&listToSet)
{
	m_bDirty = true;

	while (0 < m_numElements)
	{
		delete m_array[--m_numElements];
		m_array[m_numElements] = NULL;
	}

	m_numElements = 0;
	resizeArray(listToSet.m_allocatedSize);
	m_numElements = listToSet.m_numElements;

	m_bSort = listToSet.m_bSort;
	m_bRequireUniqueNames = listToSet.m_bRequireUniqueNames;

	m_growType = listToSet.m_growType;
	m_growSizeMultipliedByTen = listToSet.m_growSizeMultipliedByTen;
	m_recommendedMinSize = listToSet.m_recommendedMinSize;

	for (int i = 0; i < m_numElements; i++)
	{
		m_array[i] = new TListObject();
		m_array[i]->Set(listToSet.m_array[i]);
		m_array[i]->m_indexInOwnerList = listToSet.m_array[i]->m_indexInOwnerList;
		m_array[i]->m_bUniqueName = listToSet.m_array[i]->m_bUniqueName;
	}
}

template<class TListObject> unsigned int CSNList<TListObject>::GetNumElements(void) const
{
	return(m_numElements);
}

// Keeps the list sorted if it is, in which case pToAddBelow is ignored.
// Give NULL pToAddBelow to add to the bottom of the list.
// Returns NULL on error, else a pointer to the added object.
// Iff fails the caller must delete pInObject themselves.
template<class TListObject>
const TListObject *CSNList<TListObject>::AddBelowAndOwn(CTypedError *pTError,
														TListObject *pInObject,
														const TListObject *pToAddBelow)
{
	const TListObject *pResult = NULL;

	bool bPrependErrors = CTypedError::SafeIsNoError(pTError);

	if (NULL == pInObject)
	{
		CTypedError::SafeSetMsgError(pTError, "Cannot add NULL object");
	}
	else if (! pInObject->IsValid(pTError) )
	{
		if (bPrependErrors)
		{
			CTypedError::SafePrefixMsgError(pTError, "Cannot add invalid %s",
												pInObject->getTypeName());
		}
	}
	else if (m_bRequireUniqueNames && (NULL != getByNameExcluding(pInObject->GetName(), NULL, 0)))
	{
		CTypedError::SafeSetMsgError(pTError, "Another %s is already named \"%s\"",
												pInObject->getTypeName(), pInObject->GetName()->GetPrintable(true));
	}
	else
	{
		unsigned int insertIndex;

		if (m_bSort)
		{
			insertIndex = getIndexFirstGreater(pInObject);
		}
		else if (NULL != pToAddBelow)
		{
			insertIndex = pToAddBelow->m_indexInOwnerList + 1;
		}
		else
		{
			insertIndex = m_numElements;
		}

		if (insertIndex > m_numElements)
		{
			insertIndex = m_numElements;
		}

		if (! insertElement(pTError, pInObject, insertIndex) )
		{
			if (bPrependErrors)
			{
				CTypedError::SafePrefixMsgError(pTError, "Error adding %s \"%s\"",
													pInObject->getTypeName(), pInObject->GetName()->GetPrintable(true));
			}
		}
		else
		{
			pResult = pInObject;
		}
	}

	return(pResult);
}

// Updating is guarnateed not to change the object's address.
// Resorts the list if needed, else the element's position is unchanged.
// Returns NULL on error, else a pointer to the updated object.
// (Will *NOT* fail or detect if pInObject is valid but pToUpdate->Set(pInObject)
// makes pToUpdate invalid.)
template<class TListObject>
const TListObject *CSNUpdatableList<TListObject>::Update(CTypedError *pTError,
												const TListObject *pInObject,
												const TListObject *pToUpdate)
{
	const TListObject *pResult = NULL;

	TListObject *pToUpdateNonConst;

	bool bPrependErrors = CTypedError::SafeIsNoError(pTError);

	if ((NULL == pInObject) || (NULL == pToUpdate))
	{
		CTypedError::SafeSetMsgError(pTError, "Cannot update with NULL object");
	}
	else if (! pInObject->IsValid(pTError) )
	{
		if (bPrependErrors)
		{
			CTypedError::SafePrefixMsgError(pTError, "Cannot update with invalid %s",
												pInObject->getTypeName());
		}
	}
	else if (m_bRequireUniqueNames && (NULL != getByNameExcluding(pInObject->GetName(), pToUpdate, 0)))
	{
		CTypedError::SafeSetMsgError(pTError, "Another %s is already named \"%s\"",
												pInObject->getTypeName(), pInObject->GetName()->GetPrintable(true));
	}
	else if (NULL == (pToUpdateNonConst = GetNonConst(pToUpdate)))
	{
		CTypedError::SafeSetMsgError(pTError, "Cannot find old %s named \"%s\"",
												pInObject->getTypeName(), pInObject->GetName()->GetPrintable(true));
	}
	else
	{
		unsigned int insertIndex = pToUpdateNonConst->m_indexInOwnerList;

		// Remove the old element but do not resize the array.
		// (Since the array does not resize, the insert should always succeed.
		// It would suck to remove the old element and then fail to insert the new one.)
		// (It would also suck to undo any explicit array size allocation because
		// of an update.)
		// Do not delete the old element! We're going to update it and reinsert it.
		removeElement(insertIndex, false, false);

		pToUpdateNonConst->Set(pInObject);

		// If the list is sorted, forget about where the old element was,
		// fine the proper place for the new one.
		if (m_bSort)
		{
			insertIndex = getIndexFirstGreater(pToUpdateNonConst);
		}

		if (insertIndex > m_numElements)
		{
			insertIndex = m_numElements;
		}

		if (! insertElement(pTError, pToUpdateNonConst, insertIndex) )
		{
			if (bPrependErrors)
			{
				CTypedError::SafePrefixMsgError(pTError, "Abnormal failure updating %s \"%s\"; object orphaned",
													pToUpdateNonConst->getTypeName(), pToUpdateNonConst->GetName()->GetPrintable(true));
			}

//			delete pToUpdateNonConst;
//			pToUpdateNonConst = NULL;
		}
		else
		{
			pResult = pToUpdateNonConst;
		}
	}

	return(pResult);
}

// UpdateAndMove does what you could do with RemoveAndTakeOwnership an update
// if the item and then AddBelowAndOwn, except as one atomic operation which
// will either succeed or do nothing, exceptional situations (low-memory, etc.) aside.
// (Will *NOT* fail or detect if pInObject is valid but pToUpdateInSource->Set(pInObject)
// makes pToUpdateInSource invalid.)
// Safe to call with pTargetList the same as this list.
// Call with InObject NULL to move without updating the element.
// Call with pTargetList NULL to move within the same list.
// Call with pToAddBelowInTarget NULL to add to the bottom of the target list.
// If the target list is sorted, pToAddBelowInTarget is ignored.
template<class TListObject>
const TListObject *CSNUpdatableList<TListObject>::UpdateAndMove(CTypedError *pTError,
													   CSNList<TListObject> *pTargetList,
													   const TListObject *pInObject,
													   const TListObject *pToUpdateInSource,
													   const TListObject *pToAddBelowInTarget)
{
	TListObject *pResult = NULL;

	TListObject *pToUpdateNonConst;

	bool bPrependErrors = CTypedError::SafeIsNoError(pTError);

	if (NULL == pTargetList)
	{
		pTargetList = this;
	}

	const TListObject *pExcludeInNameChecks = NULL;

	if (this == pTargetList)
	{
		pExcludeInNameChecks = pToUpdateInSource;
	}

	if (NULL == pToUpdateInSource)
	{
		CTypedError::SafeSetMsgError(pTError, "Cannot update with NULL object");
	}
	else if ((NULL != pInObject) \
		 &&  (!pInObject->IsValid(pTError)))
	{
		if (bPrependErrors)
		{
			CTypedError::SafePrefixMsgError(pTError, "Cannot update with invalid %s",
												pInObject->getTypeName());
		}
	}
	else if ((NULL != pInObject) \
		 &&  pTargetList->m_bRequireUniqueNames \
		 &&  (NULL != pTargetList->getByNameExcluding(pInObject->GetName(), pExcludeInNameChecks, 0)))
	{
		CTypedError::SafeSetMsgError(pTError, "Target already has another %s named \"%s\"",
												pInObject->getTypeName(), pInObject->GetName()->GetPrintable(true));
	}
	else if ((NULL == pInObject) \
		 &&  pTargetList->m_bRequireUniqueNames \
		 &&  (NULL != pTargetList->getByNameExcluding(pToUpdateInSource->GetName(), pExcludeInNameChecks, 0)))
	{
		CTypedError::SafeSetMsgError(pTError, "Target already has another %s named \"%s\"",
												pToUpdateInSource->getTypeName(), pToUpdateInSource->GetName()->GetPrintable(true));
	}
	else if (NULL == (pToUpdateNonConst = GetNonConst(pToUpdateInSource)))
	{
		CTypedError::SafeSetMsgError(pTError, "Cannot find old %s named \"%s\"",
												pInObject->getTypeName(), pInObject->GetName()->GetPrintable(true));
	}
	else
	{
		unsigned int updateSourceIndex = pToUpdateNonConst->m_indexInOwnerList;

		// Remove the old element but do not resize the array if we're moving to ourself.
		// (See Update for reasoning.)
		// Do not delete the old element! We're going to update it and reinsert it.
		removeElement(updateSourceIndex, false, this!=pTargetList);

		if (NULL != pInObject)
		{
			pToUpdateNonConst->Set(pInObject);
		}

		unsigned int insertIndex;

		// If the list is sorted, forget about where the old element was,
		// fine the proper place for the new one.
		if (pTargetList->m_bSort)
		{
			insertIndex = pTargetList->getIndexFirstGreater(pToUpdateNonConst);
		}
		else if (NULL != pToAddBelowInTarget)
		{
			insertIndex = pToAddBelowInTarget->m_indexInOwnerList + 1;
		}
		else
		{
			insertIndex = pTargetList->m_numElements;
		}

		if (insertIndex > pTargetList->m_numElements)
		{
			insertIndex = pTargetList->m_numElements;
		}

		if (! pTargetList->insertElement(pTError, pToUpdateNonConst, insertIndex) )
		{
			if (bPrependErrors)
			{
				CTypedError::SafePrefixMsgError(pTError, "Failure updating %s \"%s\" in target; object orphaned",
													pToUpdateNonConst->getTypeName(), pToUpdateNonConst->GetName()->GetPrintable(true));
			}

			pToUpdateNonConst->m_indexInOwnerList = 0;
			pToUpdateNonConst->m_bUniqueName = true;

//			delete pToUpdateNonConst;
//			pToUpdateNonConst = NULL;
		}
		else
		{
			pResult = pToUpdateNonConst;
		}
	}

	return(pResult);
}

// Returns NULL if not found.
// If pFindAfter is NULL the search starts with the first element.
template<class TListObject> const TListObject *CSNList<TListObject>::GetByNamePronto(const char *szNamePronto, const TListObject *pFindAfter) const
{
	unsigned int startIdx = 0;

	if (NULL != pFindAfter)
	{
		// Note that if pFindAfter is the last element it is
		// still safe to pass startIdx++.
		startIdx = pFindAfter->m_indexInOwnerList;
		startIdx++;
	}

	CProntoString prontoNamePronto;
	prontoNamePronto.SetPronto(szNamePronto);

	return(getByNameExcluding(&prontoNamePronto, NULL, startIdx));
}


// Returns NULL if not found.
template<class TListObject> const TListObject *CSNList<TListObject>::GetFirst(void) const
{
	if (0 < m_numElements)
	{
		return(m_array[0]);
	}
	else
	{
		return(NULL);
	}
}

// Returns NULL if not found.
template<class TListObject> const TListObject *CSNList<TListObject>::GetLast(void) const
{
	if (0 < m_numElements)
	{
		return(m_array[m_numElements - 1]);
	}
	else
	{
		return(NULL);
	}
}

// Returns NULL if not found.
template<class TListObject> const TListObject *CSNList<TListObject>::GetNext(const TListObject *pCurrent) const
{
	const TListObject *pResult = NULL;

	if (NULL != pCurrent)
	{
		unsigned int idx = pCurrent->m_indexInOwnerList + 1;

		if (idx < m_numElements)
		{
			pResult = m_array[idx];
		}
	}

	return(pResult);
}

// Returns NULL if not found.
template<class TListObject> const TListObject *CSNList<TListObject>::GetPrevious(const TListObject *pCurrent) const
{
	const TListObject *pResult = NULL;

	if (NULL != pCurrent)
	{
		unsigned int idx = pCurrent->m_indexInOwnerList - 1;

		if (idx < m_numElements)
		{
			pResult = m_array[idx];
		}
	}

	return(pResult);
}

//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------
//----------------------------------------------------------------------------------------------

