// SNTrack.cpp: implementation of the CSNTrack class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SNConfig.h"
#include "UnsignedBaseTenNumber.h"
#include "SNVolume.h"
#include "SNArtist.h"
#include "SNDisc.h"
#include "SNTrack.h"
#include "SNDevice.h"
#include "SNCSVLine.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CSNTrack::CSNTrack()
{
	SetTrackNumber(0);
	m_pParentDisc = NULL;
}

CSNTrack::~CSNTrack()
{
}

// Type name should be lowercase, singular.
const char *CSNTrack::getTypeName(void) const
{
	return("track");
}

bool CSNTrack::IsValid(CTypedError *pTError) const
{
	// Tracks with no name are allowed.

	if (NULL == m_pParentDisc)
	{
		CTypedError::SafeSetMsgError(pTError, "%s not associated with a disc", getTypeName());
		return(false);
	}

	return(true);
}

// Returns < 0 if this <  pTrack
// Returns   0 if this == pTrack
// Returns > 0 if this >  pTrack
int CSNTrack::CompareForSort(const CSNTrack *pTrack) const
{
	if (NULL == pTrack)
	{
		return(1);
	}

	return(m_trackNumber - pTrack->m_trackNumber);
}

void CSNTrack::Set(const CSNTrack *pTrack)
{
	if (this != pTrack)
	{
		m_nameNumber.SetPronto(pTrack->m_nameNumber.GetPronto(false));
		m_title.SetPronto(pTrack->m_title.GetPronto(false));
		m_trackNumber = pTrack->m_trackNumber;

//		m_pParentDisc = pTrack->m_pParentDisc;
	}
}

// Returns the track number as a string.
const CProntoString *CSNTrack::GetName(void) const
{
	return(&m_nameNumber);
}

CProntoString *CSNTrack::GetTitle(void)
{
	return(&m_title);
}

const CProntoString *CSNTrack::GetTitle(void) const
{
	return(&m_title);
}


int CSNTrack::GetTrackNumber(void) const
{
	return(m_trackNumber);
}

void CSNTrack::SetTrackNumber(int trackNumber)
{
	if (CSNDevice::MAX_TRACKNUMBER < trackNumber)
	{
		trackNumber = CSNDevice::MAX_TRACKNUMBER;
	}
	else if (trackNumber < 0)
	{
		trackNumber = 0;
	}

	m_trackNumber = trackNumber;

	CString csNum;
	csNum.Format("%d", m_trackNumber);

	m_nameNumber.SetPronto(csNum);
}

CString CSNTrack::GetTrackNumberLabelEscaped(void) const
{
	CString csResult;

	csResult.Format("%d", GetTrackNumber());

	return(csResult);
}

xmlNodePtr CSNTrack::AddToXML(xmlNodePtr parentNode) const
{
	xmlNodePtr trackNode = xmlNewChild(parentNode, NULL, SNXML_NODE_TRACK, NULL);

	xmlSetProp(trackNode, SNXML_PROP_NAME, (const xmlChar *)GetTitle()->GetEscaped(true));

	CUnsignedBaseTenNumber ubtnTrackNumber(m_trackNumber);
	xmlSetProp(trackNode, SNXML_PROP_NUMBER, (const xmlChar *)ubtnTrackNumber.GetAsciiRepresentation());

	return(trackNode);
}

bool CSNTrack::BuildFromXML(CTypedError *pTError, CStringList *pSLErrorDetails, xmlNodePtr pNode)
{
	bool bResult = true;

	xmlChar *pXC;

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_NAME)))
	{
		GetTitle()->SetEscaped((const char *)pXC);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_NUMBER)))
	{
		int i = atoi((const char *)pXC);
		SetTrackNumber(i);
		xmlFree(pXC);
	}

	return(bResult);
}

bool CSNTrack::AddToCSVFile(CSNCSVLine *pSNCSVLine, CCSVFile *pCSVFile) const
{
	bool bAddedAnyStuff = false; // Nothing to add at this level.

	if (pSNCSVLine->SetTrackName(  m_title.GetPronto(true)))			{ bAddedAnyStuff = true; }
	if (pSNCSVLine->SetTrackNumber(m_nameNumber.GetPronto(true)))		{ bAddedAnyStuff = true; }

	if (bAddedAnyStuff)
	{
		pSNCSVLine->AddToCSVFile(pCSVFile);
	}

	return(bAddedAnyStuff);
}

bool CSNTrack::BuildFromCSVLine(CTypedError *pTError, const CSNCSVLine *pLineCopy)
{
	bool bResult = true;

	if (pLineCopy->GetTrackName())		m_title.SetPronto(pLineCopy->GetTrackName());

	return(bResult);
}

CSNDisc *CSNTrack::GetParent(void)
{
	return(m_pParentDisc);
}

const CSNDisc *CSNTrack::GetParent(void) const
{
	return(m_pParentDisc);
}

// Only call after a successful call of ResolveCSFPaths.
void CSNTrack::AddTrackCallToButtonAction(CCSFSection *pActionSection,
										  bool bIncludeDisc, bool bIncludeJump) const
{
	if ((NULL != m_pParentDisc) && (NULL != m_pParentDisc->GetDevice()))
	{
		if (bIncludeDisc)
		{
			m_pParentDisc->GetDevice()->AddDiscCallToButtonAction(pActionSection,
												m_pParentDisc->GetDiscNumber(), false, NULL);
		}

		m_pParentDisc->GetDevice()->AddTrackCallToButtonAction(pActionSection, m_trackNumber,
					bIncludeJump, m_pParentDisc->GetParent()->GetParent()->GetJumpPanel(),
					bIncludeDisc);
	}
}
