// SNTrack.h: interface for the CSNTrack class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SNTRACK_H__6E763954_AD88_43EB_B858_489B46FD18DE__INCLUDED_)
#define AFX_SNTRACK_H__6E763954_AD88_43EB_B858_489B46FD18DE__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SNList.h"
#include "SNUITree.h"

class CCSFSection;
class CSNDisc;
class CSNCSVLine;
class CCSVFile;

class CSNTrack : public CSNListObject
{
	friend class CSNList<CSNTrack>;
	friend class CSNUpdatableList<CSNTrack>;
	friend class CSNSetableList<CSNTrack>;
	friend class CSNDisc;
	friend class CTrackDlg; // Sets parent disc. (Sigh, bit of a hack.)

public:
	CSNTrack();
	virtual ~CSNTrack();

	bool IsValid(CTypedError *pTError) const;

	// Returns < 0 if this <  pTrack
	// Returns   0 if this == pTrack
	// Returns > 0 if this >  pTrack
	int CompareForSort(const CSNTrack *pTrack) const;

	void Set(const CSNTrack *pTrack);

protected: // Because if you call if you probably meant to call GetTitle. It's used by CSNList.
	// Returns the track number as a string.
	const CProntoString *GetName(void) const;

public:
	int GetTrackNumber(void) const;
	void SetTrackNumber(int trackNumber);

		  CProntoString *GetTitle(void);
	const CProntoString *GetTitle(void) const;

	CString GetTrackNumberLabelEscaped(void) const;

	xmlNodePtr AddToXML(xmlNodePtr parentNode) const;
	bool BuildFromXML(CTypedError *pTError, CStringList *pSLErrorDetails, xmlNodePtr pNode);

	bool AddToCSVFile(CSNCSVLine *pSNCSVLine, CCSVFile *pCSVFile) const;
	bool BuildFromCSVLine(CTypedError *pTError, const CSNCSVLine *pLineCopy);

	CSNDisc *GetParent(void);
	const CSNDisc *GetParent(void) const;

	// Only call after a successful call of ResolveCSFPaths.
	void AddTrackCallToButtonAction(CCSFSection *pActionSection,
									bool bIncludeDisc, bool bIncludeJump) const;

protected:
	// Type name should be lowercase, singular.
	const char *getTypeName(void) const;

private:
	CProntoString m_title;
	CProntoString m_nameNumber;
	int m_trackNumber;

	CSNDisc *m_pParentDisc;
};

class CSNTrackList : public CSNSetableList<CSNTrack>
{
public:
	CSNTrackList()
	{
		this->SetSort(true);
	}

	virtual ~CSNTrackList()
	{
	}
};

#endif // !defined(AFX_SNTRACK_H__6E763954_AD88_43EB_B858_489B46FD18DE__INCLUDED_)
