// SNUITree.h: interfaces for the Tree UI.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SNUITREE_H__88713905_24B1_4424_9211_B8A220C8C671__INCLUDED_)
#define AFX_SNUITREE_H__88713905_24B1_4424_9211_B8A220C8C671__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SNList.h"

class ISNUITreeDlg;

class CSNLibrary;
class CSNVolume;
class CSNArtist;
class CSNDisc;
class CSNDevice;

class ISNUITreeItem
{
public:
	virtual bool SNUIT_AddChildrenToTree(ISNUITreeDlg *pTreeDlg) = 0;

	virtual CString SNUIT_GetLabel(void)					const = 0;
	virtual int SNUIT_GetImageIdx(ISNUITreeDlg *pTreeDlg)	const = 0;

	// Returns empty string if no message is appropriate.
	virtual CString SNUIT_GetDeleteConfirmMessage(void)		const = 0;

	virtual const char *SNUIT_GetAddButtonName(void)	const = 0;
	virtual const char *SNUIT_GetEditButtonName(void)	const = 0;
	virtual const char *SNUIT_GetDeleteButtonName(void)	const = 0;
	virtual const char *SNUIT_GetSortButtonName(void)	const = 0;

	virtual bool SNUIT_GetEditButtonActivation(void)		const = 0;
	virtual bool SNUIT_GetDeleteButtonActivation(void)		const = 0;
	virtual bool SNUIT_GetUpButtonActivation(void)			const = 0;
	virtual bool SNUIT_GetDownButtonActivation(void)		const = 0;
	virtual bool SNUIT_GetSortButtonActivation(void)		const = 0;
	virtual bool SNUIT_GetKeepSortedCheckActivation(void)	const = 0;

	virtual void SNUIT_PressedAddButton(ISNUITreeDlg *pTreeDlg)		= 0;
	virtual void SNUIT_PressedEditButton(ISNUITreeDlg *pTreeDlg)	= 0;
	virtual void SNUIT_PressedDeleteButton(ISNUITreeDlg *pTreeDlg)	= 0;
	virtual void SNUIT_PressedUpButton(ISNUITreeDlg *pTreeDlg)		= 0;
	virtual void SNUIT_PressedDownButton(ISNUITreeDlg *pTreeDlg)	= 0;
	virtual void SNUIT_PressedSortButton(ISNUITreeDlg *pTreeDlg)	= 0;

	virtual bool SNUIT_GetKeepSorted(void) const = 0;
	virtual void SNUIT_SetKeepSorted(ISNUITreeDlg *pTreeDlg, bool bKeepSorted) = 0;

	virtual bool SNUIT_GetExpandParentInitial(void) const = 0;
};

class ISNUITreeArtist
{
public:
	// You can have a CONST artist from a ISNUITreeArtist, but not a non-const one.
	virtual const CSNArtist *SNUIT_GetRealArtist(void) const = 0;

	virtual const CSNDisc *SNUIT_GetLastDisc(void) = 0;

	virtual const CSNDisc *SNUIT_AddBelowAndOwnDisc(ISNUITreeDlg *pTreeDlg, CTypedError *pTError,
													CSNDisc *pInDisc,
													const CSNDisc *pToAddBelow) = 0;

	virtual const CSNDisc *SNUIT_UpdateDisc(ISNUITreeDlg *pTreeDlg, CTypedError *pTError,
											CSNDisc *pInDisc,
											const CSNDisc *pToUpdate) = 0;

	virtual const CSNDisc *SNUIT_UpdateDiscAndMoveToAnotherArtist(ISNUITreeDlg *pTreeDlg,
																	CTypedError *pTError,
																	const CSNArtist *pTargetArtist,
																	CSNDisc *pInDisc,
																	const CSNDisc *pToUpdate) = 0;
};

class ISNUITreeVolume
{
public:
	virtual ISNUITreeArtist *SNUIT_GetFirstArtist(void) = 0;
	virtual ISNUITreeArtist *SNUIT_GetNextArtist(ISNUITreeArtist *pPrevious) = 0;

	virtual const CSNArtist *SNUIT_AddBelowAndOwnArtist(ISNUITreeDlg *pTreeDlg, CTypedError *pTError,
														CSNArtist *pInArtist,
														const CSNArtist *pToAddBelow) = 0;

	virtual const CSNArtist *SNUIT_UpdateArtist(ISNUITreeDlg *pTreeDlg, CTypedError *pTError,
												CSNArtist *pInArtist,
												const CSNArtist *pToUpdate) = 0;
};

class ISNUITreeLibrary
{
public:
	virtual const CSNVolume *SNUIT_AddBelowAndOwnVolume(ISNUITreeDlg *pTreeDlg,
														CTypedError *pTError,
														CSNVolume *pInVolume,
														const CSNVolume *pToAddBelow) = 0;

	virtual const CSNVolume *SNUIT_UpdateVolume(ISNUITreeDlg *pTreeDlg,
												CTypedError *pTError,
												CSNVolume *pInVolume,
												const CSNVolume *pToUpdate) = 0;
};

class ISNUITreeDlg
{
public:
	virtual void SNUIT_AsyncAddList(ISNUITreeLibrary *pParent, const CSNVolume *pAddBelow) = 0;
	virtual void SNUIT_AsyncAddArtist(ISNUITreeVolume *pParent, const CSNArtist *pAddBelow) = 0;
	virtual void SNUIT_AsyncAddDisc(const CSNLibrary *pLibrary, ISNUITreeVolume *pParentVolume, ISNUITreeArtist *pParentArtist, const CSNDisc *pAddBelow, const CSNDevice *pDevice) = 0;

	virtual void SNUIT_AsyncEditList(ISNUITreeLibrary *pParent, const CSNVolume *pList) = 0;
	virtual void SNUIT_AsyncEditArtist(ISNUITreeVolume *pParent, const CSNArtist *pArtist) = 0;
	virtual void SNUIT_AsyncEditDisc(const CSNLibrary *pLibrary, ISNUITreeVolume *pParentVolume, ISNUITreeArtist *pParentArtist, const CSNDisc *pDisc) = 0;

	// Callback
	// Note that this is faster than Remove/Add but only suitable when the list isn't sorted.
	virtual bool SNUIT_UpdateLabel(ISNUITreeItem *pTItem) = 0;

	// Callback
	// NULL parent to insert at the root.
	// NULL insertAfter to insert at the top.
	virtual bool SNUIT_AddTreeItem(ISNUITreeItem *pTItem,
									ISNUITreeItem *pTIParent, const ISNUITreeItem *pTIInsertAfter) = 0;

	// Callback
	virtual bool SNUIT_RemoveTreeItems(ISNUITreeItem *pTIParent, bool bKeepParent) = 0;

	// Callback
	virtual bool SNUIT_SelectAndMakeVisibleTreeItem(const ISNUITreeItem *pTItem) = 0;

	virtual set<ISNUITreeItem *> *SNUIT_GetExpandedItems(const ISNUITreeItem *pTIParent) = 0;
	virtual void SNUIT_ExpandedItems(set<ISNUITreeItem *> *pExpandSet) = 0;

	virtual int SNUIT_GetImageIdxRoot(void) const = 0;
	virtual int SNUIT_GetImageIdxList(void) const = 0;
	virtual int SNUIT_GetImageIdxArtist(void) const = 0;
	virtual int SNUIT_GetImageIdxDisc(void) const = 0;
};


#endif // !defined(AFX_SNUITREE_H__88713905_24B1_4424_9211_B8A220C8C671__INCLUDED_)
