// SNVolume.cpp: implementation of the CSNVolume class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SNConfig.h"
#include "SNVolume.h"
#include "SNLibrary.h"
#include "CSFSection.h"
#include "LayoutNudelist.h"
#include "CSFAbstractTree.h"
#include "SNCSVLine.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSNVolume::CSNVolume()
{
	SetSort(true);

	m_pJumpPanel = NULL;
	m_pParentLibrary = NULL;
	m_bShowNumbers = false;
	m_bGapBreakAlways = true;
	m_bGapSpace = true;
	m_bGapFit = false;
	m_bShowLetters = true;
	m_bShowArtists = true;
	m_bShowDiscs = true;
	m_bShowTracks = true;
	m_bHidePanels = false;
	m_bLargerFont = false;
	m_bTruncate = true;
	m_bLinks = true;
	m_bDelayDiscUntilTrack = false;
	m_bJumpAfterTrack = false;

	m_hardButtons.AddBelowAndOwn(NULL, new CSNHardButtonDefinition("Left",	true), NULL);
	m_hardButtons.AddBelowAndOwn(NULL, new CSNHardButtonDefinition("Right",	true), NULL);
	m_hardButtons.AddBelowAndOwn(NULL, new CSNHardButtonDefinition("Vol-",	false), NULL);
	m_hardButtons.AddBelowAndOwn(NULL, new CSNHardButtonDefinition("Vol+",	false), NULL);
	m_hardButtons.AddBelowAndOwn(NULL, new CSNHardButtonDefinition("Chan-",	false), NULL);
	m_hardButtons.AddBelowAndOwn(NULL, new CSNHardButtonDefinition("Chan+",	false), NULL);
	m_hardButtons.AddBelowAndOwn(NULL, new CSNHardButtonDefinition("Mute",	false), NULL);
}

CSNVolume::~CSNVolume()
{
	SetJumpPanel(NULL);
}

const char *CSNVolume::getTypeName(void) const
{
	return("list");
}

bool CSNVolume::IsValid(CTypedError *pTError) const
{
	if (m_name.IsUnassigned())
	{
		CTypedError::SafeSetMsgError(pTError, "%s has no name", getTypeName());
		return(false);
	}

	if (m_name.IsTheEmptyString())
	{
		CTypedError::SafeSetMsgError(pTError, "%s has no name", getTypeName());
		return(false);
	}

	if (NULL == m_pParentLibrary)
	{
		CTypedError::SafeSetMsgError(pTError, "%s not associated with root library", getTypeName());
		return(false);
	}

	return(true);
}

// Returns < 0 if this <  pVolume
// Returns   0 if this == pVolume
// Returns > 0 if this >  pVolume
int CSNVolume::CompareForSort(const CSNVolume *pVolume) const
{
	if (NULL == pVolume)
	{
		return(1);
	}
	else
	{
		return(m_name.Compare(pVolume->GetName(), false));
	}
}

// Set does NOT copy/set the child list of artists.
// Set DOES copy/set the child list of link button paths.
void CSNVolume::Set(const CSNVolume *pVolume)
{
	if (this != pVolume)
	{
		m_name.SetPronto(pVolume->m_name.GetPronto(false));
		this->SetJumpPanel(pVolume->m_pJumpPanel);

		m_hardButtons.Set(pVolume->m_hardButtons);

		m_linkButtonPaths.Set(pVolume->m_linkButtonPaths);

		m_bShowNumbers = pVolume->m_bShowNumbers;
		m_bGapBreakAlways = pVolume->m_bGapBreakAlways;
		m_bGapSpace = pVolume->m_bGapSpace;
		m_bGapFit = pVolume->m_bGapFit;
		m_bShowLetters = pVolume->m_bShowLetters;
		m_bShowArtists = pVolume->m_bShowArtists;
		m_bShowDiscs = pVolume->m_bShowDiscs;
		m_bShowTracks = pVolume->m_bShowTracks;
		m_bHidePanels = pVolume->m_bHidePanels;
		m_bLargerFont = pVolume->m_bLargerFont;
		m_bTruncate = pVolume->m_bTruncate;
		m_bLinks = pVolume->m_bLinks;
		m_bDelayDiscUntilTrack = pVolume->m_bDelayDiscUntilTrack;
		m_bJumpAfterTrack = pVolume->m_bJumpAfterTrack;

//		m_pParentLibrary = pVolume->m_pParentLibrary;
	}
}

CProntoString *CSNVolume::GetName(void)
{
	return(&m_name);
}

const CProntoString *CSNVolume::GetName(void) const
{
	return(&m_name);
}

const CCSFPanelPath *CSNVolume::GetJumpPanel(void) const
{
	return(m_pJumpPanel);
}

void CSNVolume::SetJumpPanel(const CCSFPanelPath *pJumpPanel)
{
	delete m_pJumpPanel;
	m_pJumpPanel = NULL;

	if (NULL != pJumpPanel)
	{
		m_pJumpPanel = new CCSFPanelPath();
		m_pJumpPanel->Set(pJumpPanel);
	}
}

CSNHardButtonDefinitionList *CSNVolume::GetHardButtons(void)
{
	return(&m_hardButtons);
}

const CSNHardButtonDefinitionList *CSNVolume::GetHardButtons(void) const
{
	return(&m_hardButtons);
}

CCSFButtonPathList *CSNVolume::GetLinkButtonPaths(void)
{
	return(&m_linkButtonPaths);
}

const CCSFButtonPathList *CSNVolume::GetLinkButtonPaths(void) const
{
	return(&m_linkButtonPaths);
}

bool CSNVolume::GetShowNumbers(void) const
{
	return(m_bShowNumbers);
}

void CSNVolume::SetShowNumbers(bool bShowNumbers)
{
	m_bShowNumbers = bShowNumbers;
}

bool CSNVolume::GetGapBreakAlways(void) const
{
	return(m_bGapBreakAlways);
}

void CSNVolume::SetGapBreakAlways(bool bGapBreakAlways)
{
	m_bGapBreakAlways = bGapBreakAlways;
}

bool CSNVolume::GetGapSpace(void) const
{
	return(m_bGapSpace);
}

void CSNVolume::SetGapSpace(bool bGapSpace)
{
	m_bGapSpace = bGapSpace;
}

bool CSNVolume::GetGapFit(void) const
{
	return(m_bGapFit);
}

void CSNVolume::SetGapFit(bool bGapFit)
{
	m_bGapFit = bGapFit;
}

bool CSNVolume::GetShowLetters(void) const
{
	return(m_bShowLetters);
}

void CSNVolume::SetShowLetters(bool bShowLetters)
{
	m_bShowLetters = bShowLetters;
}

bool CSNVolume::GetShowArtists(void) const
{
	return(m_bShowArtists);
}

void CSNVolume::SetShowArtists(bool bShowArtists)
{
	m_bShowArtists = bShowArtists;
}

bool CSNVolume::GetShowDiscs(void) const
{
	return(m_bShowDiscs);
}

void CSNVolume::SetShowDiscs(bool bShowDiscs)
{
	m_bShowDiscs = bShowDiscs;
}

bool CSNVolume::GetShowTracks(void) const
{
	return(m_bShowTracks);
}

void CSNVolume::SetShowTracks(bool bShowTracks)
{
	m_bShowTracks = bShowTracks;
}

bool CSNVolume::GetHidePanels(void) const
{
	return(m_bHidePanels);
}

void CSNVolume::SetHidePanels(bool bHidePanels)
{
	m_bHidePanels = bHidePanels;
}

bool CSNVolume::GetLargerFont(void) const
{
	return(m_bLargerFont);
}

void CSNVolume::SetLargerFont(bool bLargerFont)
{
	m_bLargerFont = bLargerFont;
}

bool CSNVolume::GetTruncate(void) const
{
	return(m_bTruncate);
}

void CSNVolume::SetTruncate(bool bTruncate)
{
	m_bTruncate = bTruncate;
}

bool CSNVolume::GetLinks(void) const
{
	return(m_bLinks);
}

void CSNVolume::SetLinks(bool bLinks)
{
	m_bLinks = bLinks;
}

bool CSNVolume::GetDelayDiscUntilTrack(void) const
{
	return(m_bDelayDiscUntilTrack);
}

void CSNVolume::SetDelayDiscUntilTrack(bool bDelayDiscUntilTrack)
{
	m_bDelayDiscUntilTrack = bDelayDiscUntilTrack;
}

bool CSNVolume::GetJumpAfterTrack(void) const
{
	return(m_bJumpAfterTrack);
}

void CSNVolume::SetJumpAfterTrack(bool bJumpAfterTrack)
{
	m_bJumpAfterTrack = bJumpAfterTrack;
}

void CSNVolume::ResolveDevices(set<CSNDisc *> *pDiscSet, const CSNDeviceList *pDeviceList)
{
	for (CSNArtist *pArtist = GetNonConst(GetFirst()); NULL != pArtist; pArtist = GetNonConst(GetNext(pArtist)))
	{
		pArtist->ResolveDevices(pDiscSet, pDeviceList);
	}
}

void CSNVolume::UnlinkDevice(const CSNDevice *pDevice)
{
	for (CSNArtist *pArtist = GetNonConst(GetFirst()); NULL != pArtist; pArtist = GetNonConst(GetNext(pArtist)))
	{
		pArtist->UnlinkDevice(pDevice);
	}
}

int CSNVolume::GetHighestDiscNumber(const CSNDevice *pDevice) const
{
	int result = 0;
	int candidate;

	for (const CSNArtist *pArtist = GetFirst(); NULL != pArtist; pArtist = GetNext(pArtist))
	{
		candidate = pArtist->GetHighestDiscNumber(pDevice);

		if (result < candidate)
		{
			result = candidate;
		}
	}

	return(result);
}

unsigned int CSNVolume::GetNumElements(void) const
{
	return(CSNList<CSNArtist>::GetNumElements());
}

const CSNArtist *CSNVolume::GetFirst(void) const
{
	return(CSNList<CSNArtist>::GetFirst());
}

const CSNArtist *CSNVolume::GetNext(const CSNArtist *pCurrent) const
{
	return(CSNList<CSNArtist>::GetNext(pCurrent));
}

CSNArtist *CSNVolume::GetNonConst(const CSNArtist *pArtist)
{
	return(CSNList<CSNArtist>::GetNonConst(pArtist));
}

const CSNArtist *CSNVolume::GetByNamePronto(const char *szNamePronto, const CSNArtist *pFindAfter) const
{
	return(CSNList<CSNArtist>::GetByNamePronto(szNamePronto, pFindAfter));
}

CString CSNVolume::SNUIT_GetLabel(void) const
{
	CString csResult = m_name.GetPronto(true);
	return(csResult);
}

int CSNVolume::SNUIT_GetImageIdx(ISNUITreeDlg *pTreeDlg) const
{
	return(pTreeDlg->SNUIT_GetImageIdxList());
}

CString CSNVolume::SNUIT_GetDeleteConfirmMessage(void) const
{
	CString msg;

	unsigned int artistCount = GetNumElements();

	unsigned int discCount = 0;

	for (const CSNArtist *pArtist = GetFirst(); NULL != pArtist; pArtist = GetNext(pArtist))
	{
		discCount += pArtist->GetNumElements();
	}

	if (0 != discCount)
	{
		msg.Format("Delete list \"%s\" containing %u artist%s and %u disc%s?",
					GetName()->GetPrintable(true),
					artistCount, 1 == artistCount ? "" : "s",
					discCount,   1 == discCount   ? "" : "s");
	}
	else if (0 != artistCount)
	{
		msg.Format("Delete list \"%s\" containing %u artist%s?",
					GetName()->GetPrintable(true),
					artistCount, 1 == artistCount ? "" : "s");
	}
	// else empty message.

	return(msg);
}

bool CSNVolume::SNUIT_AddChildrenToTree(ISNUITreeDlg *pTreeDlg)
{
	bool bResult = true;

	for (CSNArtist *pArtist = GetNonConst(GetLast()); pArtist != NULL; pArtist = GetNonConst(GetPrevious(pArtist)))
	{
		// Insert at the top of the list.
		if ((! pTreeDlg->SNUIT_AddTreeItem(pArtist, this, NULL) ) \
		||	(! pArtist->SNUIT_AddChildrenToTree(pTreeDlg) )		  )
		{
			bResult = false;
			break;
		}
	}

	return(bResult);
}

const char *CSNVolume::SNUIT_GetAddButtonName(void) const
{
	return("Add Disc");
}

const char *CSNVolume::SNUIT_GetEditButtonName(void) const
{
	return("Edit List");
}

const char *CSNVolume::SNUIT_GetDeleteButtonName(void) const
{
	return("Delete List");
}

const char *CSNVolume::SNUIT_GetSortButtonName(void) const
{
	return("Sort Lists");
}

bool CSNVolume::SNUIT_GetKeepSorted(void) const
{
	return(m_pParentLibrary->GetSort());
}

void CSNVolume::SNUIT_SetKeepSorted(ISNUITreeDlg *pTreeDlg, bool bKeepSorted)
{
	m_pParentLibrary->SetSort(bKeepSorted);

	if ((bKeepSorted) && (NULL != pTreeDlg))
	{
		set<ISNUITreeItem *> *pExpandSet = pTreeDlg->SNUIT_GetExpandedItems(m_pParentLibrary);

		// Remove from the tree the children of our parent..
		pTreeDlg->SNUIT_RemoveTreeItems(m_pParentLibrary, true);

		// Add children back again, this time sorted.
		m_pParentLibrary->SNUIT_AddChildrenToTree(pTreeDlg);

		if (NULL != pExpandSet)
		{
			pTreeDlg->SNUIT_ExpandedItems(pExpandSet);
			delete pExpandSet;
		}
	}

	if (NULL != pTreeDlg)
	{
		pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(this);
	}
}

void CSNVolume::SNUIT_PressedSortButton(ISNUITreeDlg *pTreeDlg)
{
	if (! SNUIT_GetKeepSorted() )
	{
		SNUIT_SetKeepSorted(pTreeDlg, true);
		SNUIT_SetKeepSorted(pTreeDlg, false);
	}
}

bool CSNVolume::SNUIT_GetEditButtonActivation(void) const
{
	return(true);
}

bool CSNVolume::SNUIT_GetDeleteButtonActivation(void) const
{
	return(true);
}

bool CSNVolume::SNUIT_GetUpButtonActivation(void) const
{
	return(SNUIT_GetSortButtonActivation() && (this != m_pParentLibrary->GetFirst()));
}

bool CSNVolume::SNUIT_GetDownButtonActivation(void) const
{
	return(SNUIT_GetSortButtonActivation() && (this != m_pParentLibrary->GetLast()));
}

bool CSNVolume::SNUIT_GetSortButtonActivation(void) const
{
	return(! SNUIT_GetKeepSorted() );
}

bool CSNVolume::SNUIT_GetKeepSortedCheckActivation(void) const
{
	return(true);
}

void CSNVolume::SNUIT_PressedUpButton(ISNUITreeDlg *pTreeDlg)
{
	set<ISNUITreeItem *> *pExpandSet = NULL;

	if (NULL != pTreeDlg)
	{
		pExpandSet = pTreeDlg->SNUIT_GetExpandedItems(m_pParentLibrary);

		if (! pTreeDlg->SNUIT_RemoveTreeItems(this, false) )
		{
			return;
		}
	}

	m_pParentLibrary->MoveUp(this);

	if (NULL != pTreeDlg)
	{
		if (pTreeDlg->SNUIT_AddTreeItem(this, m_pParentLibrary, m_pParentLibrary->GetPrevious(this)))
		{
			this->SNUIT_AddChildrenToTree(pTreeDlg);

			if (NULL != pExpandSet)
			{
				pTreeDlg->SNUIT_ExpandedItems(pExpandSet);
			}

			pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(this);
		}
	}

	delete pExpandSet;
}

void CSNVolume::SNUIT_PressedDownButton(ISNUITreeDlg *pTreeDlg)
{
	set<ISNUITreeItem *> *pExpandSet = NULL;

	if (NULL != pTreeDlg)
	{
		pExpandSet = pTreeDlg->SNUIT_GetExpandedItems(m_pParentLibrary);

		if (! pTreeDlg->SNUIT_RemoveTreeItems(this, false) )
		{
			return;
		}
	}

	m_pParentLibrary->MoveDown(this);

	if (NULL != pTreeDlg)
	{
		if (pTreeDlg->SNUIT_AddTreeItem(this, m_pParentLibrary, m_pParentLibrary->GetPrevious(this)))
		{
			this->SNUIT_AddChildrenToTree(pTreeDlg);

			if (NULL != pExpandSet)
			{
				pTreeDlg->SNUIT_ExpandedItems(pExpandSet);
			}

			pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(this);
		}
	}

	delete pExpandSet;
}

void CSNVolume::SNUIT_PressedDeleteButton(ISNUITreeDlg *pTreeDlg)
{
	if (NULL != pTreeDlg)
	{
		if (! pTreeDlg->SNUIT_RemoveTreeItems(this, false) )
		{
			return;
		}
		else
		{
			const ISNUITreeItem *tiSelect = m_pParentLibrary->GetPrevious(this);

			if (NULL == tiSelect)
			{
				tiSelect = m_pParentLibrary->GetNext(this);
			}

			if (NULL == tiSelect)
			{
				tiSelect = m_pParentLibrary;
			}

			pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(tiSelect);
		}
	}

	m_pParentLibrary->RemoveAndDelete(this);

	// NOTE: This object is now deleted and we must not access members.
}

void CSNVolume::SNUIT_PressedAddButton(ISNUITreeDlg *pTreeDlg)
{
	if (NULL != pTreeDlg)
	{
		CSNArtist *pArtist = NULL;
		const CSNDevice *pDevice = NULL;
		const CSNDisc *pLastDisc = NULL;

		// Get any artist and device for a disc in this volume,
		// or leave them NULL if none found.
		for (pArtist = GetNonConst(GetFirst());
			 NULL != pArtist;
			 pArtist = GetNonConst(GetNext(pArtist)))
		{
			for (const CSNDisc *pDisc = pArtist->GetFirst(); NULL != pDisc; pDisc = pArtist->GetNext(pDisc))
			{
				if (NULL != (pDevice = pDisc->GetDevice()))
				{
					pLastDisc = pArtist->GetLast(); // Should never be NULL in this situation.

					break;
				}
			}

			if (NULL != pDevice)
			{
				break; // Before incrementing pArtist
			}
		}

		if (NULL == pDevice)
		{
			// Couldn't find a device. Try to find a default artist.

			pArtist = GetNonConst(GetFirst()); // pArtist may be NULL after this.

			if (NULL == pArtist)
			{
				pLastDisc = NULL;
			}
			else
			{
				pLastDisc = pArtist->GetLast(); // pLastDisc may be NULL after this.
			}
		}

		pTreeDlg->SNUIT_AsyncAddDisc(m_pParentLibrary, this, pArtist, pLastDisc, pDevice);
	}
}

void CSNVolume::SNUIT_PressedEditButton(ISNUITreeDlg *pTreeDlg)
{
	if (NULL != pTreeDlg)
	{
		pTreeDlg->SNUIT_AsyncEditList(m_pParentLibrary, this);
	}
}

bool CSNVolume::SNUIT_GetExpandParentInitial(void) const
{
	return(true);
}

ISNUITreeArtist *CSNVolume::SNUIT_GetFirstArtist(void)
{
	return(GetNonConst(GetFirst()));
}

ISNUITreeArtist *CSNVolume::SNUIT_GetNextArtist(ISNUITreeArtist *pPrevious)
{
	return(GetNonConst(GetNext(pPrevious->SNUIT_GetRealArtist())));
}

const CSNArtist *CSNVolume::SNUIT_AddBelowAndOwnArtist(ISNUITreeDlg *pTreeDlg, CTypedError *pTError,
														CSNArtist *pInArtist,
														const CSNArtist *pToAddBelow)
{
	pInArtist->m_pParentVolume = this;

	CSNArtist *pResult = GetNonConst(AddBelowAndOwn(pTError, pInArtist, pToAddBelow));

	if (NULL != pResult)
	{
		// Do NOT null out pResult if anything goes wrong after now.

		if (NULL != pTreeDlg)
		{
			// GetPrevious call needed since the list may be sorted and pAddBelow ignored.
			if (pTreeDlg->SNUIT_AddTreeItem(pResult, this, GetPrevious(pResult)))
			{
				pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(pResult);
			}
		}
	}

	return(pResult);
}

const CSNArtist *CSNVolume::SNUIT_UpdateArtist(ISNUITreeDlg *pTreeDlg, CTypedError *pTError,
												CSNArtist *pInArtist,
												const CSNArtist *pToUpdate)
{
	pInArtist->m_pParentVolume = this;

	const CSNArtist *pOldPrevious = GetPrevious(pToUpdate);

	CSNArtist *pResult = GetNonConst(Update(pTError, pInArtist, pToUpdate));

	if (NULL != pResult)
	{
		// Do NOT null out pResult if anything goes wrong after now.

		if (NULL != pTreeDlg)
		{
			const CSNArtist *pNewPrevious = GetPrevious(pResult);

			if ((!pResult->SNUIT_GetKeepSorted()) \
			||	(pNewPrevious == pOldPrevious))
			{
				// Item's position in the list hasn't changed (fast).
				pTreeDlg->SNUIT_UpdateLabel(pResult);
			}
			else
			{
				// Item's position in the list has changed (slow).
				if (pTreeDlg->SNUIT_RemoveTreeItems(pResult, false))
				{
					if (pTreeDlg->SNUIT_AddTreeItem(pResult, this, GetPrevious(pResult)))
					{
						pResult->SNUIT_AddChildrenToTree(pTreeDlg);

						pTreeDlg->SNUIT_SelectAndMakeVisibleTreeItem(pResult);
					}
				}
			}
		}
	}

	return(pResult);
}

void CSNVolume::ClearDirty(void)
{
	CSNList<CSNArtist>::ClearDirty();

	for (CSNArtist *pArtist = GetNonConst(GetFirst());
		 NULL != pArtist;
		 pArtist = GetNonConst(GetNext(pArtist)))
	{
		pArtist->ClearDirty();
	}
}

bool CSNVolume::IsDirty(void) const
{
	bool bDirty = CSNList<CSNArtist>::IsDirty();

	for (const CSNArtist *pArtist = GetFirst();
		 (!bDirty) && (NULL != pArtist);
		 pArtist = GetNext(pArtist))
	{
		bDirty = pArtist->IsDirty();
	}

	return(bDirty);
}

xmlNodePtr CSNVolume::AddToXML(xmlNodePtr parentNode) const
{
	xmlNodePtr volumeNode = xmlNewChild(parentNode, NULL, SNXML_NODE_VOLUME, NULL);
	xmlSetProp(volumeNode, SNXML_PROP_NAME, (const xmlChar *)GetName()->GetEscaped(true));
	xmlSetProp(volumeNode, SNXML_PROP_KEEPCHILDRENSORTED, GetSort() ? SNXML_VALUE_1 : SNXML_VALUE_0);
	xmlSetProp(volumeNode, SNXML_PROP_SHOWNUMBERS, GetShowNumbers() ? SNXML_VALUE_1 : SNXML_VALUE_0);
	xmlSetProp(volumeNode, SNXML_PROP_GAPBREAKALWAYS, GetGapBreakAlways() ? SNXML_VALUE_1 : SNXML_VALUE_0);
	xmlSetProp(volumeNode, SNXML_PROP_GAPSPACE, GetGapSpace() ? SNXML_VALUE_1 : SNXML_VALUE_0);
	xmlSetProp(volumeNode, SNXML_PROP_GAPFIT, GetGapFit() ? SNXML_VALUE_1 : SNXML_VALUE_0);
	xmlSetProp(volumeNode, SNXML_PROP_SHOWLETTERS, GetShowLetters() ? SNXML_VALUE_1 : SNXML_VALUE_0);
	xmlSetProp(volumeNode, SNXML_PROP_SHOWARTISTS, GetShowArtists() ? SNXML_VALUE_1 : SNXML_VALUE_0);
	xmlSetProp(volumeNode, SNXML_PROP_SHOWDISCS, GetShowDiscs() ? SNXML_VALUE_1 : SNXML_VALUE_0);
	xmlSetProp(volumeNode, SNXML_PROP_SHOWTRACKS, GetShowTracks() ? SNXML_VALUE_1 : SNXML_VALUE_0);
	xmlSetProp(volumeNode, SNXML_PROP_HIDEPANELS, GetHidePanels() ? SNXML_VALUE_1 : SNXML_VALUE_0);
	xmlSetProp(volumeNode, SNXML_PROP_LARGERFONT, GetLargerFont() ? SNXML_VALUE_1 : SNXML_VALUE_0);
	xmlSetProp(volumeNode, SNXML_PROP_TRUNCATE, GetTruncate() ? SNXML_VALUE_1 : SNXML_VALUE_0);
	xmlSetProp(volumeNode, SNXML_PROP_LINKS, GetLinks() ? SNXML_VALUE_1 : SNXML_VALUE_0);
	xmlSetProp(volumeNode, SNXML_PROP_DELAYDISCUNTILTRACK, GetDelayDiscUntilTrack() ? SNXML_VALUE_1 : SNXML_VALUE_0);
	xmlSetProp(volumeNode, SNXML_PROP_JUMPAFTERTRACK, GetJumpAfterTrack() ? SNXML_VALUE_1 : SNXML_VALUE_0);

	if (NULL != m_pJumpPanel)
	{
		xmlNodePtr jpNode = m_pJumpPanel->AddToXML(volumeNode);
		xmlSetProp(jpNode, SNXML_PROP_NAME, SNXML_VALUE_JUMPPANEL);
	}

	for (const CCSFButtonPath *pBut = m_linkButtonPaths.GetFirst(); NULL != pBut; pBut = m_linkButtonPaths.GetNext(pBut))
	{
		xmlNodePtr lbNode = pBut->AddToXML(volumeNode);
		xmlSetProp(lbNode, SNXML_PROP_NAME, SNXML_VALUE_LINKBUTTON);
	}

	for (const CSNHardButtonDefinition *pHBD = m_hardButtons.GetFirst(); NULL != pHBD; pHBD = m_hardButtons.GetNext(pHBD))
	{
		if (pHBD->IsNonEmpty())
		{
			pHBD->AddToXML(volumeNode);
		}
	}

	for (const CSNArtist *pArtist = GetFirst(); NULL != pArtist; pArtist = GetNext(pArtist))
	{
		pArtist->AddToXML(volumeNode);
	}

	return(volumeNode);
}

bool CSNVolume::BuildFromXML(CTypedError *pTError, CStringList *pSLErrorDetails, xmlNodePtr pNode)
{
	bool bResult = true;

	xmlChar *pXC;
	xmlChar *pXCType;
	xmlChar *pXCName;

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_NAME)))
	{
		GetName()->SetEscaped((const char *)pXC);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_KEEPCHILDRENSORTED)))
	{
		int i = atoi((const char *)pXC);
		SetSort(0 != i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_SHOWNUMBERS)))
	{
		int i = atoi((const char *)pXC);
		SetShowNumbers(0 != i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_GAPBREAKALWAYS)))
	{
		int i = atoi((const char *)pXC);
		SetGapBreakAlways(0 != i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_GAPSPACE)))
	{
		int i = atoi((const char *)pXC);
		SetGapSpace(0 != i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_GAPFIT)))
	{
		int i = atoi((const char *)pXC);
		SetGapFit(0 != i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_SHOWLETTERS)))
	{
		int i = atoi((const char *)pXC);
		SetShowLetters(0 != i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_SHOWARTISTS)))
	{
		int i = atoi((const char *)pXC);
		SetShowArtists(0 != i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_SHOWDISCS)))
	{
		int i = atoi((const char *)pXC);
		SetShowDiscs(0 != i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_SHOWTRACKS)))
	{
		int i = atoi((const char *)pXC);
		SetShowTracks(0 != i);
		xmlFree(pXC);
	}

	// Sanity check
	if (! (GetShowDiscs() || GetShowTracks()) )
	{
		SetShowLetters(true);
		SetShowArtists(true);
		SetShowDiscs(true);
		SetShowTracks(true);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_HIDEPANELS)))
	{
		int i = atoi((const char *)pXC);
		SetHidePanels(0 != i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_LARGERFONT)))
	{
		int i = atoi((const char *)pXC);
		SetLargerFont(0 != i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_TRUNCATE)))
	{
		int i = atoi((const char*)pXC);
		SetTruncate(0 != i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_LINKS)))
	{
		int i = atoi((const char*)pXC);
		SetLinks(0 != i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_DELAYDISCUNTILTRACK)))
	{
		int i = atoi((const char *)pXC);
		SetDelayDiscUntilTrack(0 != i);
		xmlFree(pXC);
	}

	if (NULL != (pXC = xmlGetProp(pNode, SNXML_PROP_JUMPAFTERTRACK)))
	{
		int i = atoi((const char *)pXC);
		SetJumpAfterTrack(0 != i);
		xmlFree(pXC);
	}

	for (xmlNodePtr pNodeChild = pNode->children; NULL != pNodeChild; pNodeChild = pNodeChild->next)
	{
		if ((!xmlIsBlankNode(pNodeChild)) && (NULL != pNodeChild->name))
		{
			if (0 == strcmp((const char *)SNXML_NODE_ARTIST, (const char *)pNodeChild->name))
			{
				CSNArtist *pArtist = new CSNArtist();

				pArtist->m_pParentVolume = this;

				if ((pArtist->BuildFromXML(pTError, pSLErrorDetails, pNodeChild))	\
				&&	(NULL != AddBelowAndOwn(pTError, pArtist, NULL))				)
				{
					pArtist = NULL; // Owned.
				}
				else
				{
					delete pArtist;
					bResult = false;
					break;
				}
			}
			else if (0 == strcmp((const char *)SNXML_NODE_CSFPATH, (const char *)pNodeChild->name))
			{
				if (NULL != (pXCType = xmlGetProp(pNodeChild, SNXML_PROP_TYPE)))
				{
					if (NULL != (pXCName = xmlGetProp(pNodeChild, SNXML_PROP_NAME)))
					{
						if (0 == strcmp((const char *)SNXML_VALUE_PANEL, (const char *)pXCType))
						{
							CCSFPanelPath *pPanelPath = new CCSFPanelPath();

							if (! pPanelPath->BuildFromXML(pTError, pSLErrorDetails, pNodeChild) )
							{
								bResult = false;
							}
							else if (0 == strcmp((const char *)SNXML_VALUE_JUMPPANEL, (const char *)pXCName))
							{
								SetJumpPanel(pPanelPath);
							}

							delete pPanelPath;
						}
						else if (0 == strcmp((const char *)SNXML_VALUE_BUTTON, (const char *)pXCType))
						{
							CCSFButtonPath *pButtonPath = new CCSFButtonPath();

							if (! pButtonPath->BuildFromXML(pTError, pSLErrorDetails, pNodeChild) )
							{
								bResult = false;
							}
							else if (0 == strcmp((const char *)SNXML_VALUE_LINKBUTTON, (const char *)pXCName))
							{
								if (NULL != m_linkButtonPaths.AddBelowAndOwn(pTError, pButtonPath, NULL))
								{
									pButtonPath = NULL; // Owned.
								}
								else
								{
									bResult = false;
								}
							}

							delete pButtonPath;
						}

						xmlFree(pXCName);
					}

					xmlFree(pXCType);
				}
			}
			else if (0 == strcmp((const char *)SNXML_NODE_HARDBUTTONDEFINITION, (const char *)pNodeChild->name))
			{
				if (NULL != (pXCName = xmlGetProp(pNodeChild, SNXML_PROP_NAME)))
				{
					// Assumes names of hardbuttons have no escaped characters.
					const CSNHardButtonDefinition *pHBD = m_hardButtons.GetByNamePronto((const char *)pXCName, NULL);

					// Case-sensitive comparison.
					if ((NULL != pHBD) && (0 == strcmp((const char *)pXCName, pHBD->GetName()->GetPronto(true))))
					{
						CSNHardButtonDefinition *pHBDNew = new CSNHardButtonDefinition();
						pHBDNew->Set(pHBD);

						if ((!pHBDNew->BuildFromXML(pTError, pSLErrorDetails, pNodeChild, true))
						||	(NULL == m_hardButtons.Update(pTError, pHBDNew, pHBD)))
						{
							bResult = false;
						}

						delete pHBDNew;
					}

					xmlFree(pXCName);
				}
			}
		}
	}

	return(bResult);
}

bool CSNVolume::AddToCSVFile(CSNCSVLine *pSNCSVLine, CCSVFile *pCSVFile) const
{
	bool bAddedAnyStuff = false;
	
	if (pSNCSVLine->SetListName(m_name.GetPronto(true)))
	{
		bAddedAnyStuff = true;
	}

	bool bChildrenAddedAnyStuff = false;

	if (!pSNCSVLine->IsReady())
	{
		for (const CSNArtist *pArtist = GetFirst(); NULL != pArtist; pArtist = GetNext(pArtist))
		{
			CSNCSVLine lineCopy = *pSNCSVLine;
		
			if (pArtist->AddToCSVFile(&lineCopy, pCSVFile))
			{
				bChildrenAddedAnyStuff = true;
			}
		}
	}

	if ((!bChildrenAddedAnyStuff) && (bAddedAnyStuff))
	{
		pSNCSVLine->AddToCSVFile(pCSVFile);
	}

	return(bAddedAnyStuff || bChildrenAddedAnyStuff);
}

bool CSNVolume::BuildFromCSVLine(CTypedError *pTError, const CSNCSVLine *pLineCopy)
{
	bool bResult = true;

	if (pLineCopy->GetArtistName())
	{
		CSNArtist *pArt = this->GetNonConst(this->GetByNamePronto(pLineCopy->GetArtistName(), NULL));

		if (NULL == pArt)
		{
			CSNArtist *pArtNew = new CSNArtist();
			pArtNew->GetName()->SetPronto(pLineCopy->GetArtistName());

			pArt = this->GetNonConst(this->SNUIT_AddBelowAndOwnArtist(NULL, pTError, pArtNew, NULL));

			if (NULL == pArt)
			{
				delete pArtNew;
				bResult = false;
			}
		}

		if (bResult && (!pArt->BuildFromCSVLine(pTError, pLineCopy)))
		{
			bResult = false;
		}
	}

	return(bResult);
}

bool CSNVolume::ResolveCSFPaths(CTypedError *pTError, const CCSFAbstractTree *pAT)
{
	if (NULL != m_pJumpPanel)
	{
		if (! m_pJumpPanel->ResolveCSFPath(pTError, true, pAT, getTypeName(),
										GetName()->GetPrintable(true), "jump panel") )
		{
			return(false);
		}
	}

	for (CCSFButtonPath *pBut = m_linkButtonPaths.GetNonConst(m_linkButtonPaths.GetFirst()); NULL != pBut; pBut = m_linkButtonPaths.GetNonConst(m_linkButtonPaths.GetNext(pBut)))
	{
		if (! pBut->ResolveCSFPath(pTError, false, pAT, getTypeName(),
										GetName()->GetPrintable(true), "link button") )
		{
			return(false);
		}
	}

	for (CSNHardButtonDefinition *pHBD = m_hardButtons.GetNonConst(m_hardButtons.GetFirst()); NULL != pHBD; pHBD = m_hardButtons.GetNonConst(m_hardButtons.GetNext(pHBD)))
	{
		if (! pHBD->ResolveCSFPaths(pTError, pAT, getTypeName(), GetName()->GetPrintable(true)) )
		{
			return(false);
		}
	}

	return(true);
}

CSNLibrary *CSNVolume::GetParent(void)
{
	return(m_pParentLibrary);
}

const CSNLibrary *CSNVolume::GetParent(void) const
{
	return(m_pParentLibrary);
}

void CSNVolume::AddToAndUpdateRootCSFSection(CSNConfig *pConfig, CCSFSection *pRootCSFSection, CStringList *pStrLstEmpties) const
{
	// Select layout (only one for now).

	CLayoutNudelist *pLayout = new CLayoutNudelist(pConfig->GetExeVersionString(NULL),
												   this, m_name.GetEscaped(true), &m_hardButtons,
												   pConfig->GetPanelCap());

	// Do layout

	const CActionSectionAdder *pRootActionSectionAdder = pLayout->DoLayout(this);

	if (NULL == pRootActionSectionAdder)
	{
		if (NULL != pStrLstEmpties)
		{
			pStrLstEmpties->AddTail(m_name.GetPronto(true));
		}
	}
	else if (NULL != pRootActionSectionAdder)
	{
		// Add devices if there's any point.

		pLayout->AddToDevicesToRootCSFSection(pRootCSFSection);

		// Link buttons to the main list device.

		for (const CCSFButtonPath *pButPath = m_linkButtonPaths.GetFirst(); NULL != pButPath; pButPath = m_linkButtonPaths.GetNext(pButPath))
		{
			const list<const CCSFATButton *> *pATButs = pButPath->GetATButtons();

			for (list<const CCSFATButton *>::const_iterator iter = pATButs->begin(); iter != pATButs->end(); iter++)
			{
				CCSFSection *pButton = (*iter)->GetCSFSection();

				CCSFSection *pAction = pButton->FindSubSectionByNameEscaped("Action()");

				if (NULL == pAction)
				{
					pAction = pButton->AddEscapedTitleOnlySubSection("Action()");
				}

				CCSFSection *pOldJump = pAction->FindSubSectionByNamePrefixEscaped("JumpDevicePanel(");

				if (NULL != pOldJump)
				{
					pAction->RemoveAndDeleteSubSectionByPointer(pOldJump);
					pOldJump = NULL;
				}

				pRootActionSectionAdder->AddToActionSection(pAction);
			}
		}
	}

	// Done.

	delete pLayout;
}
