// SNVolume.h: interface for the CSNVolume class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SNVOLUME_H__76872B65_04F3_40B6_A086_2D8F9DE9CB9F__INCLUDED_)
#define AFX_SNVOLUME_H__76872B65_04F3_40B6_A086_2D8F9DE9CB9F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SNArtist.h"
#include "CSFPath.h"
#include "SNHardButtonDefinition.h"

class CSNLibrary;
class CCSFSection;
class CSNConfig;

class CSNVolume : public ISNUITreeItem, public ISNUITreeVolume, public CSNListObject, protected CSNUpdatableList<CSNArtist>
{
	friend class CSNList<CSNVolume>;
	friend class CSNUpdatableList<CSNVolume>;
	friend class CSNLibrary;
	friend class CSNArtist;

public:
	CSNVolume();
	virtual ~CSNVolume();

	bool IsValid(CTypedError *pTError) const;

	// Returns < 0 if this <  pVolume
	// Returns   0 if this == pVolume
	// Returns > 0 if this >  pVolume
	int CompareForSort(const CSNVolume *pVolume) const;

	// Set does NOT copy/set the child list of artists.
	// Set DOES copy/set the child list of link button paths.
	void Set(const CSNVolume *pVolume);

	CProntoString *GetName(void);
	const CProntoString *GetName(void) const;

	const CCSFPanelPath *GetJumpPanel(void) const;
	void SetJumpPanel(const CCSFPanelPath *pJumpPanel);

	CSNHardButtonDefinitionList *GetHardButtons(void);
	const CSNHardButtonDefinitionList *GetHardButtons(void) const;

	CCSFButtonPathList *GetLinkButtonPaths(void);
	const CCSFButtonPathList *GetLinkButtonPaths(void) const;

	bool GetShowNumbers(void) const;
	void SetShowNumbers(bool bShowNumbers);

	bool GetGapBreakAlways(void) const;
	void SetGapBreakAlways(bool bGapBreakAlways);
	bool GetGapSpace(void) const;
	void SetGapSpace(bool bGapSpace);
	bool GetGapFit(void) const;
	void SetGapFit(bool bGapFit);

	bool GetShowLetters(void) const;
	void SetShowLetters(bool bShowLetters);
	bool GetShowArtists(void) const;
	void SetShowArtists(bool bShowArtists);
	bool GetShowDiscs(void) const;
	void SetShowDiscs(bool bShowDiscs);
	bool GetShowTracks(void) const;
	void SetShowTracks(bool bShowTracks);
	bool GetDelayDiscUntilTrack(void) const;
	void SetDelayDiscUntilTrack(bool bDelayDiscUntilTrack);
	bool GetJumpAfterTrack(void) const;
	void SetJumpAfterTrack(bool bJumpAfterTrack);

	bool GetHidePanels(void) const;
	void SetHidePanels(bool bHidePanels);

	bool GetLargerFont(void) const;
	void SetLargerFont(bool bLargerFont);

	bool GetTruncate(void) const;
	void SetTruncate(bool bTruncate);

	bool GetLinks(void) const;
	void SetLinks(bool bLinks);

	xmlNodePtr AddToXML(xmlNodePtr parentNode) const;
	bool BuildFromXML(CTypedError *pTError, CStringList *pSLErrorDetails, xmlNodePtr pNode);

	bool AddToCSVFile(CSNCSVLine *pSNCSVLine, CCSVFile *pCSVFile) const;
	bool BuildFromCSVLine(CTypedError *pTError, const CSNCSVLine *pLineCopy);

	void ClearDirty(void);
	bool IsDirty(void) const;

	unsigned int GetNumElements(void) const;
	const CSNArtist *GetFirst(void) const;
	const CSNArtist *GetNext(const CSNArtist *pCurrent) const;
	CSNArtist *GetNonConst(const CSNArtist *pArtist);
	const CSNArtist *GetByNamePronto(const char *szNamePronto, const CSNArtist *pFindAfter) const;

	void ResolveDevices(set<CSNDisc *> *pDiscSet, const CSNDeviceList *pDeviceList);
	void UnlinkDevice(const CSNDevice *pDevice);

	int GetHighestDiscNumber(const CSNDevice *pDevice) const;

	bool ResolveCSFPaths(CTypedError *pTError, const CCSFAbstractTree *pAT);

	CSNLibrary *GetParent(void);
	const CSNLibrary *GetParent(void) const;

	void AddToAndUpdateRootCSFSection(CSNConfig *pConfig, CCSFSection *pRootCSFSection, CStringList *pStrLstEmpties) const;

protected:
	// Type name should be lowercase, singular.
	const char *getTypeName(void) const;

public:
	ISNUITreeArtist *SNUIT_GetFirstArtist(void);
	ISNUITreeArtist *SNUIT_GetNextArtist(ISNUITreeArtist *pPrevious);

	const CSNArtist *SNUIT_AddBelowAndOwnArtist(ISNUITreeDlg *pTreeDlg, CTypedError *pTError,
												CSNArtist *pInArtist,
												const CSNArtist *pToAddBelow);

	const CSNArtist *SNUIT_UpdateArtist(ISNUITreeDlg *pTreeDlg, CTypedError *pTError,
										CSNArtist *pInArtist,
										const CSNArtist *pToUpdate);

public:
	bool SNUIT_AddChildrenToTree(ISNUITreeDlg *pTreeDlg);
	CString SNUIT_GetLabel(void) const;
	int SNUIT_GetImageIdx(ISNUITreeDlg *pTreeDlg) const;
	CString SNUIT_GetDeleteConfirmMessage(void) const;
	const char *SNUIT_GetAddButtonName(void)	const;
	const char *SNUIT_GetEditButtonName(void)	const;
	const char *SNUIT_GetDeleteButtonName(void)	const;
	const char *SNUIT_GetSortButtonName(void)	const;
	bool SNUIT_GetKeepSorted(void) const;
	void SNUIT_SetKeepSorted(ISNUITreeDlg *pTreeDlg, bool bKeepSorted);
	void SNUIT_PressedSortButton(ISNUITreeDlg *pTreeDlg);
	bool SNUIT_GetEditButtonActivation(void)		const;
	bool SNUIT_GetDeleteButtonActivation(void)		const;
	bool SNUIT_GetUpButtonActivation(void)			const;
	bool SNUIT_GetDownButtonActivation(void)		const;
	bool SNUIT_GetSortButtonActivation(void)		const;
	bool SNUIT_GetKeepSortedCheckActivation(void)	const;
	void SNUIT_PressedUpButton(ISNUITreeDlg *pTreeDlg);
	void SNUIT_PressedDownButton(ISNUITreeDlg *pTreeDlg);
	void SNUIT_PressedDeleteButton(ISNUITreeDlg *pTreeDlg);
	void SNUIT_PressedAddButton(ISNUITreeDlg *pTreeDlg);
	void SNUIT_PressedEditButton(ISNUITreeDlg *pTreeDlg);
	bool SNUIT_GetExpandParentInitial(void) const;

private:
	CProntoString m_name;
	CCSFPanelPath *m_pJumpPanel;
	CCSFButtonPathList m_linkButtonPaths;

	CSNHardButtonDefinitionList m_hardButtons;

	bool m_bShowNumbers;
	bool m_bGapBreakAlways;
	bool m_bGapSpace;
	bool m_bGapFit;
	bool m_bShowLetters;
	bool m_bShowArtists;
	bool m_bShowDiscs;
	bool m_bShowTracks;
	bool m_bHidePanels;
	bool m_bLargerFont;
	bool m_bTruncate;
	bool m_bLinks;
	bool m_bDelayDiscUntilTrack;
	bool m_bJumpAfterTrack;

	CSNLibrary *m_pParentLibrary;
};

#endif // !defined(AFX_SNVOLUME_H__76872B65_04F3_40B6_A086_2D8F9DE9CB9F__INCLUDED_)
