// SlotReport.cpp: implementation of the CSlotReport class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "SNConfig.h"
#include "SNLibrary.h"
#include "SNDevice.h"
#include "SlotReport.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSlotReport::CSlotReport(const CSNConfig *pConfig)
{
	m_pConfig = pConfig;
}

CSlotReport::~CSlotReport()
{
	Reset(); // Not strictly needed at time of writing.
}

// Implicitly calls Reset before doing anything.
void CSlotReport::Generate(void)
{
	Reset();

	for (const CSNVolume *pVol = m_pConfig->GetLibrary()->GetFirst(); NULL != pVol; pVol = m_pConfig->GetLibrary()->GetNext(pVol))
	{
		for (const CSNArtist *pArt = pVol->GetFirst(); pArt != NULL; pArt = pVol->GetNext(pArt))
		{
			for (const CSNDisc *pDisc = pArt->GetFirst(); pDisc != NULL; pDisc = pArt->GetNext(pDisc))
			{
				string deviceName;
				deviceName.erase();

				if (NULL != pDisc->GetDevice())
				{
					deviceName = pDisc->GetDevice()->GetName()->GetPronto(true);
				}
				else
				{
					deviceName = pDisc->GetUnresolvedDeviceName()->GetPronto(true);
				}

				if (! deviceName.empty() )
				{
					pair<string, VecOfDiscLists> ip;
					ip.first = deviceName;

					// Find existing item for deviceName or insert a new one.
					pair<devIter, bool> iIterPair = m_mapDevs.insert(ip);

					vector< list<const CSNDisc *> > *pVecOfDiscLists = &((iIterPair.first)->second.m_vecOfDiscLists);

					if (pDisc->GetDiscNumber() >= pVecOfDiscLists->size())
					{
						pVecOfDiscLists->resize(pDisc->GetDiscNumber() + 1);
					}

					(*pVecOfDiscLists)[pDisc->GetDiscNumber()].push_back(pDisc);
				}
			}
		}
	}
}

void CSlotReport::Reset(void)
{
	m_mapDevs.clear();
}

CSlotReport::const_devIter CSlotReport::DevBegin(void)
{
	return(m_mapDevs.begin());
}

CSlotReport::const_devIter CSlotReport::DevEnd(void)
{
	return(m_mapDevs.end());
}
