// SortableListCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "SortableListCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSortableListCtrl

CSortableListCtrl::CSortableListCtrl()
{
	m_sortCol = -1;
	m_bSortReverse = false;
	m_sortNumElements = 0;
}

CSortableListCtrl::~CSortableListCtrl()
{
}


BEGIN_MESSAGE_MAP(CSortableListCtrl, CListCtrl)
	//{{AFX_MSG_MAP(CSortableListCtrl)
	ON_NOTIFY_REFLECT(LVN_COLUMNCLICK, OnColumnclick)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSortableListCtrl message handlers

void CSortableListCtrl::OnColumnclick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// The iItem member is -1, and the iSubItem member identifies the column. All other members are zero. 
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

	bool bReverse = false;

	if ((m_sortCol == pNMListView->iSubItem) && (GetItemCount() == m_sortNumElements))
	{
		bReverse = m_bSortReverse ? false : true;
	}

	SortItemsNudel(pNMListView->iSubItem, bReverse);

	*pResult = 0;
}

void CSortableListCtrl::NudelResetSortColumns(void)
{
	m_numberColumns.clear();
	m_iconColumns.clear();
	m_callbackColumns.clear();
}

void CSortableListCtrl::NudelSortSetNumberColumn(int column)
{
	m_numberColumns.insert(column);
}

void CSortableListCtrl::NudelSortSetIconColumn(int column)
{
	m_iconColumns.insert(column);
}

void CSortableListCtrl::NudelSortSetCallbackColumn(int column, PFNNUDELLVCOMPARE fn, void *pVoid)
{
	// Erase existing entry, if any.
	callbackMapIter iter = m_callbackColumns.find(column);

	if (iter != m_callbackColumns.end())
	{
		m_callbackColumns.erase(iter);
	}

	m_callbackColumns.insert(pair< int, pair<PFNNUDELLVCOMPARE, void *> >(column, pair<PFNNUDELLVCOMPARE, void *>(fn, pVoid)));
}

void CSortableListCtrl::SortItemsNudel(int sortColumn, bool bReverse)
{
	m_sortCol = sortColumn;
	m_bSortReverse = bReverse;
	m_sortNumElements = GetItemCount();

	// DWORD... sigh...
	SortItems(nudelSLSortCallback, (DWORD)this);
}

int CALLBACK CSortableListCtrl::nudelSLSortCallback(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort)
{
	int result = 0;

	CSortableListCtrl *pThis = (CSortableListCtrl*)lParamSort;

	LVFINDINFO findInfo1;
	ZeroMemory(&findInfo1, sizeof(findInfo1));
	findInfo1.flags = LVFI_PARAM;
	findInfo1.lParam = lParam1;

	LVFINDINFO findInfo2;
	ZeroMemory(&findInfo2, sizeof(findInfo2));
	findInfo2.flags = LVFI_PARAM;
	findInfo2.lParam = lParam2;

	int itemIdx1 = pThis->FindItem(&findInfo1, -1);
	int itemIdx2 = pThis->FindItem(&findInfo2, -1);

	CHeaderCtrl *pHeaderCtrl;

	int numColumns = 0;

	if (NULL != (pHeaderCtrl = pThis->GetHeaderCtrl()))
	{
		numColumns = pHeaderCtrl->GetItemCount();
	}

	if (((-1) != itemIdx1) && ((-1) != itemIdx2))
	{
		result = nudelSLSSCColumn(pThis, lParam1, lParam2, itemIdx1, itemIdx2, pThis->m_sortCol);

		if (0 == result)
		{
			// When the column we're sorting by is identical for the two items, compare the
			// other columns in order for a slightly nicer sort.
			for(int i=0; (i < numColumns); i++)
			{
				if (i != pThis->m_sortCol)
				{
					result = nudelSLSSCColumn(pThis, lParam1, lParam2, itemIdx1, itemIdx2, i);

					if (0 != result)
					{
						// If sorting by other columns, do not reverse. (i.e. reverse twice.)
						if (pThis->m_bSortReverse)
						{
							if (0 > result)
							{
								result = 1;
							}
							else if (0 < result)
							{
								result = (-1);
							}
						}

						break;
					}
				}
			}
		}
	}

	if (pThis->m_bSortReverse)
	{
		if (0 > result)
		{
			result = 1;
		}
		else if (0 < result)
		{
			result = (-1);
		}
	}

	return(result);
}

int CSortableListCtrl::nudelSLSSCColumn(CSortableListCtrl *pThis,
										LPARAM lParam1, LPARAM lParam2,
										int itemIdx1, int itemIdx2,
										int sortColumn)
{
	int result = 0;

	callbackMapIter cmIter = pThis->m_callbackColumns.find(sortColumn);

	if (cmIter != pThis->m_callbackColumns.end())
	{
		// column is sorted by callback.

		result = (cmIter->second.first)(pThis, lParam1, lParam2, itemIdx1, itemIdx2, cmIter->second.second);
	}
	else if (pThis->m_numberColumns.find(sortColumn) != pThis->m_numberColumns.end())
	{
		// column is numeric text.

		CString csItemText1 = pThis->GetItemText(itemIdx1, sortColumn);
		CString csItemText2 = pThis->GetItemText(itemIdx2, sortColumn);

		int item1 = atoi(csItemText1);
		int item2 = atoi(csItemText2);

		result = item1 - item2;
	}
	else if (pThis->m_iconColumns.find(sortColumn) != pThis->m_iconColumns.end())
	{
		// column is sorted by icon.

		LVITEM lvItem1;
		ZeroMemory(&lvItem1, sizeof(lvItem1));
		lvItem1.iItem = itemIdx1;
		lvItem1.mask = LVIF_IMAGE;
		lvItem1.iSubItem = sortColumn;

		LVITEM lvItem2;
		ZeroMemory(&lvItem2, sizeof(lvItem2));
		lvItem2.iItem = itemIdx2;
		lvItem2.mask = LVIF_IMAGE;
		lvItem2.iSubItem = sortColumn;

		if (pThis->GetItem(&lvItem1)\
		&&  pThis->GetItem(&lvItem2))
		{
			if (lvItem1.iImage > lvItem2.iImage)
			{
				result = 1;
			}
			else if (lvItem1.iImage < lvItem2.iImage)
			{
				result = (-1);
			}
		}
	}
	else
	{
		// column is text.

		CString csItemText1 = pThis->GetItemText(itemIdx1, sortColumn);
		CString csItemText2 = pThis->GetItemText(itemIdx2, sortColumn);

		result = csItemText1.CompareNoCase(csItemText2);
	}

	return(result);
}
