#include "stdafx.h"

#include "SuckLessDialog.h"
#include <process.h>

static UINT NEAR WM_SLDLG_THREADTERMINATING	= RegisterWindowMessage("SUCKLESSDIALOG_THREADTERMINATING");
static UINT NEAR WM_SLDLG_PMCHILDCLOSE		= RegisterWindowMessage("SUCKLESSDIALOG_PMCHILDCLOSE");

// Adds itself to the head of the chain pointed to by ppFrameInfoNext, which may be NULL.
// frameIdent is the identifier of the frame. Usually an int with one bit set.
// frameIdent of 0 is a special case which is the default frame containing all controls, equal size to the window.
// pMainWindow is the window containing the frame control, not the frame control itself.
// frameControlIdent is the ident of the associated control, typically a picture frame.
CFrameInfo::CFrameInfo(CFrameInfo **ppFrameInfoNext, DWORD frameIdent, CWnd *pMainWindow, int frameControlIdent,
						bool bHorizPercentage, bool bVertPercentage)
{
	// Add ourselves to the head of the list, updating the caller's pointer.
	m_pNext = *ppFrameInfoNext;
	*ppFrameInfoNext = this;

	m_frameIdent = frameIdent;
	m_controlIdent = frameControlIdent;

	m_bHorizPercentage = bHorizPercentage;
	m_bVertPercentage = bVertPercentage;

	ZeroMemory(&m_originalRect, sizeof(RECT)); // In case there is an error getting the original rect.

	if (NULL != pMainWindow)
	{
		if (0 == m_frameIdent)
		{
			// The main window is a special case.
			pMainWindow->GetClientRect(&m_originalRect);
		}
		else
		{
			// Find the control.
			CWnd *pControlCWnd = pMainWindow->GetDlgItem(frameControlIdent);

			if (NULL != pControlCWnd)
			{
				pControlCWnd->GetWindowRect(&m_originalRect);
				pMainWindow->ScreenToClient(&m_originalRect);
			}
		}
	}
}

// Deletes the chain.
CFrameInfo::~CFrameInfo(void)
{
	delete m_pNext;
	m_pNext = NULL;
}

// Should be called on the root.
// destRect filled in if returns true.
// Based on the relative sizes of the desired and active frames, works out how large the window
// should be to switch to the desired frame.
// Expects top/bottom/left/right border values in *pBorderRect.
bool CFrameInfo::GetNewWindowSize(DWORD desiredFrameIdent, DWORD activeFrameIdent, CWnd *pMainWindow,
									RECT *pBorderRect, POINT &destSize)
{
	bool result = false;

	if (NULL != pMainWindow)
	{
		// Get the current window width and height, minus the borders.
		RECT currentRect;
		POINT currentSize;

		pMainWindow->GetClientRect(&currentRect);

		currentRect.top += pBorderRect->top;
		currentRect.left += pBorderRect->left;
		currentRect.right -= pBorderRect->right;
		currentRect.bottom -= pBorderRect->bottom;

		currentSize.x = currentRect.right - currentRect.left;
		currentSize.y = currentRect.bottom - currentRect.top;

		// Get the rects and percentage bools for the original versions of the frames.
		// (The initial frame is both percentage (100%) and offset (0), so we ignore its bools.)

		RECT desiredOrigRect;
		POINT desiredOrigSize;
		bool bDesiredHorizPercentage;
		bool bDesiredVertPercentage;

		RECT activeOrigRect;
		POINT activeOrigSize;
		bool bActiveHorizPercentage;
		bool bActiveVertPercentage;

		RECT initialOrigRect;
		POINT initialOrigSize;

		if (GetOriginalRect(desiredFrameIdent,	desiredOrigRect,&bDesiredHorizPercentage,	&bDesiredVertPercentage	) \
		&&	GetOriginalRect(activeFrameIdent,	activeOrigRect,	&bActiveHorizPercentage,	&bActiveVertPercentage	) \
		&&	GetOriginalRect(0,					initialOrigRect,NULL,						NULL					) )
		{
			desiredOrigSize.x = desiredOrigRect.right - desiredOrigRect.left;
			desiredOrigSize.y = desiredOrigRect.bottom - desiredOrigRect.top;

			activeOrigSize.x = activeOrigRect.right - activeOrigRect.left;
			activeOrigSize.y = activeOrigRect.bottom - activeOrigRect.top;

			initialOrigSize.x = initialOrigRect.right - initialOrigRect.left;
			initialOrigSize.y = initialOrigRect.bottom - initialOrigRect.top;

			// First, work out how large the default frame would be based on the current size of the active frame.
			POINT effectiveCurrentDefaultSize;

			if (bActiveHorizPercentage)
			{
				effectiveCurrentDefaultSize.x = (currentSize.x * initialOrigSize.x) / activeOrigSize.x;
			}
			else
			{
				effectiveCurrentDefaultSize.x = currentSize.x + (initialOrigSize.x - activeOrigSize.x);
			}

			if (bActiveVertPercentage)
			{
				effectiveCurrentDefaultSize.y = (currentSize.y * initialOrigSize.y) / activeOrigSize.y;
			}
			else
			{
				effectiveCurrentDefaultSize.y = currentSize.y + (initialOrigSize.y - activeOrigSize.y);
			}

			// Finally, go from the default to the desired.
			if (bDesiredHorizPercentage)
			{
				destSize.x = (effectiveCurrentDefaultSize.x * desiredOrigSize.x) / initialOrigSize.x;
			}
			else
			{
				destSize.x = effectiveCurrentDefaultSize.x - (initialOrigSize.x - desiredOrigSize.x);
			}

			if (bDesiredVertPercentage)
			{
				destSize.y = (effectiveCurrentDefaultSize.y * desiredOrigSize.y) / initialOrigSize.y;
			}
			else
			{
				destSize.y = effectiveCurrentDefaultSize.y - (initialOrigSize.y - desiredOrigSize.y);
			}

			result = true;
		
		}
	}

	return(result);
}

// Should be called on the root.
// destRect filled in and *pbXXXPercentage set if returns true.
// Set pbXXXPercentage to NULL if you are not interested in them.
bool CFrameInfo::GetOriginalRect(DWORD frameIdent, RECT &destRect, bool *pbHorizPercentage, bool *pbVertPercentage)
{
	bool result = false;

	if (frameIdent == m_frameIdent)
	{
		destRect.top	= m_originalRect.top;
		destRect.bottom	= m_originalRect.bottom;
		destRect.left	= m_originalRect.left;
		destRect.right	= m_originalRect.right;

		if (NULL != pbHorizPercentage)
		{
			*pbHorizPercentage = m_bHorizPercentage;
		}

		if (NULL != pbVertPercentage)
		{
			*pbVertPercentage = m_bVertPercentage;
		}

		result = true;
	}
	else if (NULL != m_pNext)
	{
		result = m_pNext->GetOriginalRect(frameIdent, destRect, pbHorizPercentage, pbVertPercentage);
	}

	return(result);
}

// Should be called on the root.
// Removes the given border from the default frame (0).
// Expects top/bottom/left/right border values in *pBorderRect.
void CFrameInfo::RemoveBorderFromDefaultFrame(RECT *pBorderRect)
{
	if (0 == m_frameIdent)
	{
		m_originalRect.top += pBorderRect->top;
		m_originalRect.bottom -= pBorderRect->bottom;
		m_originalRect.left += pBorderRect->left;
		m_originalRect.right -= pBorderRect->right;
	}
	else if (NULL != m_pNext)
	{
		m_pNext->RemoveBorderFromDefaultFrame(pBorderRect);
	}
}

// Returns the ident for the frame control corresponding to the given frame ident.
int CFrameInfo::GetFrameCtrlIdent(DWORD frameIdent)
{
	int result = 0;

	if (frameIdent == m_frameIdent)
	{
		result = m_controlIdent;		
	}
	else if (NULL != m_pNext)
	{
		result = m_pNext->GetFrameCtrlIdent(frameIdent);
	}

	return(result);
}


//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================

CControlInfo::CControlInfo(CControlInfo **ppControlInfoNext, CWnd *pMainWindow, int windowIdent, int controlIdent,
						   DWORD flagOne, DWORD frameMembership)
{
	// Add ourselves to the head of the list, updating the caller's pointer.
	m_pNext = *ppControlInfoNext;
	*ppControlInfoNext = this;

	m_ident = controlIdent;
	m_windowIdent = windowIdent;

	m_topAlign = ATOP_NONE;
	m_topRelativeIdent = m_windowIdent;
	m_topOffset = 0;
	m_topPercentage = 0;
	m_topIsSet = false;
	m_topValue = 0;

	m_bottomAlign = ABOTTOM_NONE;
	m_bottomRelativeIdent = m_windowIdent;
	m_bottomOffset = 0;
	m_bottomPercentage = 0;
	m_bottomIsSet = false;
	m_bottomValue = 0;

	m_leftAlign = ALEFT_NONE;
	m_leftRelativeIdent = m_windowIdent;
	m_leftOffset = 0;
	m_leftPercentage = 0;
	m_leftIsSet = false;
	m_leftValue = 0;

	m_rightAlign = ARIGHT_NONE;
	m_rightRelativeIdent = m_windowIdent;
	m_rightOffset = 0;
	m_rightPercentage = 0;
	m_rightIsSet = false;
	m_rightValue = 0;

	// Flags
	m_flagOne = flagOne;

	m_frameMembership = frameMembership;

	// Private flags...
	m_privFlagOne = 0;

	if (NULL != pMainWindow)
	{
		CWnd *pControlCWnd = pMainWindow->GetDlgItem(controlIdent);

		if (NULL != pControlCWnd)
		{
			if (! (WS_VISIBLE & pControlCWnd->GetStyle()) )
			{
				m_privFlagOne |= PF1_INVISIBLE;
			}
		}
	}
}

// Should be called on the root CControlInfo and it'll delete the list.
CControlInfo::~CControlInfo(void)
{
	delete m_pNext;
	m_pNext = NULL;
}

// Should be called on the root CControlInfo
void CControlInfo::SetControlRelationship(int identSubject, int identRelative, va_list args)
{
	if (identSubject != m_ident)
	{
		if (NULL != m_pNext)
		{
			m_pNext->SetControlRelationship(identSubject, identRelative, args);
		}
	}
	else
	{
		// Set up the relationships.

		AlignType alignType;
		bool bKeepGoing = true;

		while (bKeepGoing && (ATAG_ENDLIST != (alignType = va_arg(args, AlignType))))
		{
			switch(alignType)
			{
			default:
				break;

			case(ATOP_NONE):
			case(ATOP_TOP):
			case(ATOP_TOP_ACTIVEFRAME):
			case(ATOP_TOP_NOOFFSET):
			case(ATOP_BOTTOM):
			case(ATOP_BOTTOM_ACTIVEFRAME):
			case(ATOP_BOTTOM_NOOFFSET):
				m_topAlign = alignType;
				m_topRelativeIdent = identRelative;
				break;

			case(ATOP_PERCENTAGE):
				m_topAlign = ATOP_PERCENTAGE;
				m_topRelativeIdent = identRelative;
				break;

			case(ATOP_CENTER):
				m_topAlign = ATOP_CENTER;
				m_topRelativeIdent = identRelative;
				break;

			case(ABOTTOM_NONE):
			case(ABOTTOM_TOP):
			case(ABOTTOM_TOP_ACTIVEFRAME):
			case(ABOTTOM_TOP_NOOFFSET):
			case(ABOTTOM_BOTTOM):
			case(ABOTTOM_BOTTOM_ACTIVEFRAME):
			case(ABOTTOM_BOTTOM_NOOFFSET):
				m_bottomAlign = alignType;
				m_bottomRelativeIdent = identRelative;
				break;

			case(ABOTTOM_PERCENTAGE):
				m_bottomAlign = ABOTTOM_PERCENTAGE;
				m_bottomRelativeIdent = identRelative;
				break;

			case(ABOTTOM_CENTER):
				m_bottomAlign = ABOTTOM_CENTER;
				m_bottomRelativeIdent = identRelative;
				break;

			case(ATOPBOTTOM_CENTER):
				m_topAlign = ATOPBOTTOM_CENTER;
				m_bottomAlign = ATOPBOTTOM_CENTER;
				m_topRelativeIdent = identRelative;
				m_bottomRelativeIdent = identRelative;
				break;

			case(ALEFT_NONE):
			case(ALEFT_LEFT):
			case(ALEFT_LEFT_ACTIVEFRAME):
			case(ALEFT_LEFT_NOOFFSET):
			case(ALEFT_RIGHT):
			case(ALEFT_RIGHT_ACTIVEFRAME):
			case(ALEFT_RIGHT_NOOFFSET):
				m_leftAlign = alignType;
				m_leftRelativeIdent = identRelative;
				break;
				
			case(ALEFT_PERCENTAGE):
				m_leftAlign = ALEFT_PERCENTAGE;
				m_leftRelativeIdent = identRelative;
				break;

			case(ALEFT_CENTER):
				m_leftAlign = ALEFT_CENTER;
				m_leftRelativeIdent = identRelative;
				break;

			case(ARIGHT_NONE):
			case(ARIGHT_LEFT):
			case(ARIGHT_LEFT_ACTIVEFRAME):
			case(ARIGHT_LEFT_NOOFFSET):
			case(ARIGHT_RIGHT):
			case(ARIGHT_RIGHT_ACTIVEFRAME):
			case(ARIGHT_RIGHT_NOOFFSET):
				m_rightAlign = alignType;
				m_rightRelativeIdent = identRelative;
				break;

			case(ARIGHT_PERCENTAGE):
				m_rightAlign = ARIGHT_PERCENTAGE;
				m_rightRelativeIdent = identRelative;
				break;

			case(ARIGHT_CENTER):
				m_rightAlign = ARIGHT_CENTER;
				m_rightRelativeIdent = identRelative;
				break;

			case(ALEFTRIGHT_CENTER):
				m_leftAlign = ALEFTRIGHT_CENTER;
				m_rightAlign = ALEFTRIGHT_CENTER;
				m_leftRelativeIdent = identRelative;
				m_rightRelativeIdent = identRelative;
				break;

			case(ACOMMON_TOP_LEFT):
				bKeepGoing = false;
				m_topAlign		= ATOP_TOP;
				m_bottomAlign	= ABOTTOM_TOP;
				m_leftAlign		= ALEFT_LEFT;
				m_rightAlign	= ARIGHT_LEFT;
				m_topRelativeIdent		= identRelative;
				m_bottomRelativeIdent	= identRelative;
				m_leftRelativeIdent		= identRelative;
				m_rightRelativeIdent	= identRelative;
				break;

			case(ACOMMON_TOP_RIGHT):
				bKeepGoing = false;
				m_topAlign		= ATOP_TOP;
				m_bottomAlign	= ABOTTOM_TOP;
				m_leftAlign		= ALEFT_RIGHT;
				m_rightAlign	= ARIGHT_RIGHT;
				m_topRelativeIdent		= identRelative;
				m_bottomRelativeIdent	= identRelative;
				m_leftRelativeIdent		= identRelative;
				m_rightRelativeIdent	= identRelative;
				break;

			case(ACOMMON_BOTTOM_LEFT):
				bKeepGoing = false;
				m_topAlign		= ATOP_BOTTOM;
				m_bottomAlign	= ABOTTOM_BOTTOM;
				m_leftAlign		= ALEFT_LEFT;
				m_rightAlign	= ARIGHT_LEFT;
				m_topRelativeIdent		= identRelative;
				m_bottomRelativeIdent	= identRelative;
				m_leftRelativeIdent		= identRelative;
				m_rightRelativeIdent	= identRelative;
				break;

			case(ACOMMON_BOTTOM_RIGHT):
				bKeepGoing = false;
				m_topAlign		= ATOP_BOTTOM;
				m_bottomAlign	= ABOTTOM_BOTTOM;
				m_leftAlign		= ALEFT_RIGHT;
				m_rightAlign	= ARIGHT_RIGHT;
				m_topRelativeIdent		= identRelative;
				m_bottomRelativeIdent	= identRelative;
				m_leftRelativeIdent		= identRelative;
				m_rightRelativeIdent	= identRelative;
				break;

			case(ACOMMON_TOP_RESIZEWIDTH):
				bKeepGoing = false;
				m_topAlign		= ATOP_TOP;
				m_bottomAlign	= ABOTTOM_TOP;
				m_leftAlign		= ALEFT_LEFT;
				m_rightAlign	= ARIGHT_RIGHT;
				m_topRelativeIdent		= identRelative;
				m_bottomRelativeIdent	= identRelative;
				m_leftRelativeIdent		= identRelative;
				m_rightRelativeIdent	= identRelative;
				break;

			case(ACOMMON_BOTTOM_RESIZEWIDTH):
				bKeepGoing = false;
				m_topAlign		= ATOP_BOTTOM;
				m_bottomAlign	= ABOTTOM_BOTTOM;
				m_leftAlign		= ALEFT_LEFT;
				m_rightAlign	= ARIGHT_RIGHT;
				m_topRelativeIdent		= identRelative;
				m_bottomRelativeIdent	= identRelative;
				m_leftRelativeIdent		= identRelative;
				m_rightRelativeIdent	= identRelative;
				break;

			case(ACOMMON_RESIZEHEIGHT_LEFT):
				bKeepGoing = false;
				m_topAlign		= ATOP_TOP;
				m_bottomAlign	= ABOTTOM_BOTTOM;
				m_leftAlign		= ALEFT_LEFT;
				m_rightAlign	= ARIGHT_LEFT;
				m_topRelativeIdent		= identRelative;
				m_bottomRelativeIdent	= identRelative;
				m_leftRelativeIdent		= identRelative;
				m_rightRelativeIdent	= identRelative;
				break;

			case(ACOMMON_RESIZEHEIGHT_RIGHT):
				bKeepGoing = false;
				m_topAlign		= ATOP_TOP;
				m_bottomAlign	= ABOTTOM_BOTTOM;
				m_leftAlign		= ALEFT_RIGHT;
				m_rightAlign	= ARIGHT_RIGHT;
				m_topRelativeIdent		= identRelative;
				m_bottomRelativeIdent	= identRelative;
				m_leftRelativeIdent		= identRelative;
				m_rightRelativeIdent	= identRelative;
				break;

			case(ACOMMON_RESIZEHEIGHT_RESIZEWIDTH):
				bKeepGoing = false;
				m_topAlign		= ATOP_TOP;
				m_bottomAlign	= ABOTTOM_BOTTOM;
				m_leftAlign		= ALEFT_LEFT;
				m_rightAlign	= ARIGHT_RIGHT;
				m_topRelativeIdent		= identRelative;
				m_bottomRelativeIdent	= identRelative;
				m_leftRelativeIdent		= identRelative;
				m_rightRelativeIdent	= identRelative;
				break;
			}
		}
	}
}

bool CControlInfo::getCtrlRect(CWnd *pMainWindow, RECT *pMainWindowExcludingBorders, int ctrlIdent, RECT *pOutRect)
{
	bool result = false;

	if ((NULL != pMainWindow) && (NULL != pMainWindowExcludingBorders) && (NULL != pOutRect))
	{
		if (m_windowIdent == ctrlIdent)
		{
			// The main window is a special case.
			pOutRect->top = pMainWindowExcludingBorders->top;
			pOutRect->bottom = pMainWindowExcludingBorders->bottom;
			pOutRect->left = pMainWindowExcludingBorders->left;
			pOutRect->right = pMainWindowExcludingBorders->right;

			result = true;
		}
		else
		{
			// Find the control.
			CWnd *pRelativeCWnd = pMainWindow->GetDlgItem(ctrlIdent);

			if (NULL != pRelativeCWnd)
			{
				pRelativeCWnd->GetWindowRect(pOutRect);
				pMainWindow->ScreenToClient(pOutRect);

				result = true;
			}
		}
	}

	return(result);
}

// Should be called on the root CControlInfo
void CControlInfo::CalculateInitialControlRelationshipValues(CWnd *pMainWindow, RECT *pMainWindowExcludingBorders)
{
	if ((NULL != pMainWindow) && (NULL != pMainWindowExcludingBorders))
	{
		// Get the rectangle we're relative to.
		RECT relativeRect;

		bool noErr = true;

		// Get our rectangle.
		RECT ourRect;
		
		CWnd *pOurCWnd = pMainWindow->GetDlgItem(m_ident);

		if (NULL == pOurCWnd)
		{
			noErr = false;
		}
		else
		{
			pOurCWnd->GetWindowRect(&ourRect);
			pMainWindow->ScreenToClient(&ourRect);
		}

		// Set up the relationships.

		if (noErr && getCtrlRect(pMainWindow, pMainWindowExcludingBorders, m_topRelativeIdent, &relativeRect))
		{
			switch(m_topAlign)
			{
			default:
			case(ATOP_NONE):
			case(ATOP_TOP_NOOFFSET):
			case(ATOP_BOTTOM_NOOFFSET):
				m_topOffset = 0;
				break;

			case(ATOP_TOP):
			case(ATOP_TOP_ACTIVEFRAME):
				m_topOffset = ourRect.top - relativeRect.top;
				break;
			
			case(ATOP_BOTTOM):
			case(ATOP_BOTTOM_ACTIVEFRAME):
				m_topOffset = ourRect.top - relativeRect.bottom;
				break;

			case(ATOP_PERCENTAGE):
				m_topPercentage = (PERCENTAGE_SCALE_FACTOR * (ourRect.top - relativeRect.top)) / (relativeRect.bottom - relativeRect.top);
				break;

			case(ATOP_CENTER):
				m_topOffset = ourRect.top - (relativeRect.top + ((relativeRect.bottom - relativeRect.top) / 2));
				break;

			case(ATOPBOTTOM_CENTER):
				m_topOffset = ourRect.bottom - ourRect.top;
				break;
			}
		}

		if (noErr && getCtrlRect(pMainWindow, pMainWindowExcludingBorders, m_bottomRelativeIdent, &relativeRect))
		{
			switch(m_bottomAlign)
			{
			default:
			case(ABOTTOM_NONE):
			case(ABOTTOM_TOP_NOOFFSET):
			case(ABOTTOM_BOTTOM_NOOFFSET):
				m_bottomOffset = 0;
				break;

			case(ABOTTOM_TOP):
			case(ABOTTOM_TOP_ACTIVEFRAME):
				m_bottomOffset = ourRect.bottom - relativeRect.top;
				break;

			case(ABOTTOM_BOTTOM):
			case(ABOTTOM_BOTTOM_ACTIVEFRAME):
				m_bottomOffset = ourRect.bottom - relativeRect.bottom;
				break;

			case(ABOTTOM_PERCENTAGE):
				m_bottomPercentage = (PERCENTAGE_SCALE_FACTOR * (ourRect.bottom - relativeRect.top)) / (relativeRect.bottom - relativeRect.top);
				break;

			case(ABOTTOM_CENTER):
				m_bottomOffset = ourRect.bottom - (relativeRect.top + ((relativeRect.bottom - relativeRect.top) / 2));
				break;

			case(ATOPBOTTOM_CENTER):
				m_bottomOffset = ourRect.bottom - ourRect.top;
				break;
			}
		}

		if (noErr && getCtrlRect(pMainWindow, pMainWindowExcludingBorders, m_leftRelativeIdent, &relativeRect))
		{
			switch(m_leftAlign)
			{
			default:
			case(ALEFT_NONE):
			case(ALEFT_LEFT_NOOFFSET):
			case(ALEFT_RIGHT_NOOFFSET):
				m_leftOffset = 0;
				break;

			case(ALEFT_LEFT):
			case(ALEFT_LEFT_ACTIVEFRAME):
				m_leftOffset = ourRect.left - relativeRect.left;
				break;
			
			case(ALEFT_RIGHT):
			case(ALEFT_RIGHT_ACTIVEFRAME):
				m_leftOffset = ourRect.left - relativeRect.right;
				break;
			
			case(ALEFT_PERCENTAGE):
				m_leftPercentage = (PERCENTAGE_SCALE_FACTOR * (ourRect.left - relativeRect.left)) / (relativeRect.right - relativeRect.left);
				break;

			case(ALEFT_CENTER):
				m_leftOffset = ourRect.left - (relativeRect.left + ((relativeRect.right - relativeRect.left) / 2));
				break;

			case(ALEFTRIGHT_CENTER):
				m_leftOffset = ourRect.right - ourRect.left;
				break;
			}
		}

		if (noErr && getCtrlRect(pMainWindow, pMainWindowExcludingBorders, m_rightRelativeIdent, &relativeRect))
		{
			switch(m_rightAlign)
			{
			default:
			case(ARIGHT_NONE):
			case(ARIGHT_LEFT_NOOFFSET):
			case(ARIGHT_RIGHT_NOOFFSET):
				m_rightOffset = 0;
				break;
			
			case(ARIGHT_LEFT):
			case(ARIGHT_LEFT_ACTIVEFRAME):
				m_rightOffset = ourRect.right - relativeRect.left;
				break;
			
			case(ARIGHT_RIGHT):
			case(ARIGHT_RIGHT_ACTIVEFRAME):
				m_rightOffset = ourRect.right - relativeRect.right;
				break;

			case(ARIGHT_PERCENTAGE):
				m_rightPercentage = (PERCENTAGE_SCALE_FACTOR * (ourRect.right - relativeRect.left)) / (relativeRect.right - relativeRect.left);
				break;

			case(ARIGHT_CENTER):
				m_rightOffset = ourRect.right - (relativeRect.left + ((relativeRect.right - relativeRect.left) / 2));
				break;

			case(ALEFTRIGHT_CENTER):
				m_rightOffset = ourRect.right - ourRect.left;
				break;
			}
		}

		if (NULL != m_pNext)
		{
			m_pNext->CalculateInitialControlRelationshipValues(pMainWindow, pMainWindowExcludingBorders);
		}
	}
}

// Should be called on the root CControlInfo
void CControlInfo::DoResize(CWnd *pMainWindow, RECT *pMainWindowRect, int activeFrameCtrlIdent)
{
	prepareForResize(activeFrameCtrlIdent);

	calculateAndResize(this, pMainWindow, pMainWindowRect);
}

// Should be called on the root CControlInfo
void CControlInfo::prepareForResize(int activeFrameIdent)
{
	m_topIsSet = false;
	m_bottomIsSet = false;
	m_leftIsSet = false;
	m_rightIsSet = false;

	if ((ATOP_TOP_ACTIVEFRAME == m_topAlign) || (ATOP_BOTTOM_ACTIVEFRAME == m_topAlign))
	{
		m_topRelativeIdent = activeFrameIdent;
	}

	if ((ABOTTOM_TOP_ACTIVEFRAME == m_bottomAlign) || (ABOTTOM_BOTTOM_ACTIVEFRAME == m_bottomAlign))
	{
		m_bottomRelativeIdent = activeFrameIdent;
	}

	if ((ALEFT_LEFT_ACTIVEFRAME == m_leftAlign) || (ALEFT_RIGHT_ACTIVEFRAME == m_leftAlign))
	{
		m_leftRelativeIdent = activeFrameIdent;
	}

	if ((ARIGHT_LEFT_ACTIVEFRAME == m_rightAlign) || (ARIGHT_RIGHT_ACTIVEFRAME == m_rightAlign))
	{
		m_rightRelativeIdent = activeFrameIdent;
	}

	if (NULL != m_pNext)
	{
		m_pNext->prepareForResize(activeFrameIdent);
	}
}

void CControlInfo::calculateAndResize(CControlInfo *pRootControlInfo, CWnd *pMainWindow, RECT *pMainWindowRect)
{
	CWnd *pCWnd = pMainWindow->GetDlgItem(m_ident);

	if (NULL != pCWnd)
	{
		LONG top	= calculateTop(		m_ident, pRootControlInfo, pMainWindow, pMainWindowRect);
		LONG bottom	= calculateBottom(	m_ident, pRootControlInfo, pMainWindow, pMainWindowRect);
		LONG left	= calculateLeft(	m_ident, pRootControlInfo, pMainWindow, pMainWindowRect);
		LONG right	= calculateRight(	m_ident, pRootControlInfo, pMainWindow, pMainWindowRect);

		if (((-1) != top) && ((-1) != bottom) && ((-1) != left) && ((-1) != right))
		{
			RECT ourWindowRect;
			ourWindowRect.top = top;
			ourWindowRect.bottom = bottom;
			ourWindowRect.left = left;
			ourWindowRect.right = right;

			pCWnd->MoveWindow(&ourWindowRect, FALSE); // Don't repaint.
		}
	}

	if (NULL != m_pNext)
	{
		m_pNext->calculateAndResize(pRootControlInfo, pMainWindow, pMainWindowRect);
	}
}

// calculate functions
// -------------------
// Return -1 if the value cannot be calculated.
// ident is the ident of the control / window which you want a value for.
// pRootControlInfo should point to the first CControlInfo in the list. (Used for finding the CControlInfo for ident.)
// pMainWindow and pMainWindowRect should be the CWnd and RECT of the main window and should not be NULL.

LONG CControlInfo::calculateTop(int ident, CControlInfo *pRootControlInfo, CWnd *pMainWindow, RECT *pMainWindowRect)
{
	LONG result = (-1);

	if (m_windowIdent == ident)
	{
		if (NULL != pMainWindowRect)
		{
			result = pMainWindowRect->top;
		}
	}
	else if (m_ident != ident)
	{
		if (NULL != m_pNext)
		{
			result = m_pNext->calculateTop(ident, pRootControlInfo, pMainWindow, pMainWindowRect);
		}
	}
	else
	{
		if (!m_topIsSet)
		{
			switch(m_topAlign)
			{
			case(ATOP_TOP):
			case(ATOP_TOP_ACTIVEFRAME):
			case(ATOP_TOP_NOOFFSET):
				{
					LONG relTop = pRootControlInfo->calculateTop(m_topRelativeIdent,
																	pRootControlInfo, pMainWindow, pMainWindowRect);

					if ((-1) != relTop)
					{
						m_topValue = relTop + m_topOffset;
						m_topIsSet = true;
					}
				}
				break;

			case(ATOP_BOTTOM):
			case(ATOP_BOTTOM_ACTIVEFRAME):
			case(ATOP_BOTTOM_NOOFFSET):
				{
					LONG relBottom = pRootControlInfo->calculateBottom(m_topRelativeIdent,
																		pRootControlInfo, pMainWindow, pMainWindowRect);

					if ((-1) != relBottom)
					{
						m_topValue = relBottom + m_topOffset;
						m_topIsSet = true;
					}
				}
				break;

			case(ATOP_PERCENTAGE):
				{
					LONG relHeight = pRootControlInfo->calculateHeight(m_topRelativeIdent,
																		pRootControlInfo, pMainWindow,
																		pMainWindowRect);

					LONG relTop = pRootControlInfo->calculateTop(m_topRelativeIdent,
																	pRootControlInfo, pMainWindow, pMainWindowRect);

					if (((-1) != relHeight) && ((-1) != relTop))
					{
						m_topValue = relTop + ((m_topPercentage * relHeight) / PERCENTAGE_SCALE_FACTOR);
						m_topIsSet = true;
					}
				}
				break;

			case(ATOP_CENTER):
				{
					LONG relHeight = pRootControlInfo->calculateHeight(m_topRelativeIdent,
																		pRootControlInfo, pMainWindow,
																		pMainWindowRect);

					LONG relTop = pRootControlInfo->calculateTop(m_topRelativeIdent,
																	pRootControlInfo, pMainWindow, pMainWindowRect);

					if (((-1) != relHeight) && ((-1) != relTop))
					{
						m_topValue = relTop + relHeight/2 + m_topOffset;
						m_topIsSet = true;
					}
				}
				break;

			case(ATOPBOTTOM_CENTER):
				{
					LONG relHeight = pRootControlInfo->calculateHeight(m_topRelativeIdent,
																		pRootControlInfo, pMainWindow,
																		pMainWindowRect);

					LONG relTop = pRootControlInfo->calculateTop(m_topRelativeIdent,
																	pRootControlInfo, pMainWindow, pMainWindowRect);

					if (((-1) != relHeight) && ((-1) != relTop))
					{
						// m_topOffset is our constant height in this case.
						m_topValue = relTop + ((relHeight - m_topOffset) / 2);
						m_topIsSet = true;
					}
				}
				break;

			default:
				break;
			}
		}

		if (m_topIsSet)
		{
			result = m_topValue;
		}
	}

	return(result);
}

LONG CControlInfo::calculateBottom(int ident, CControlInfo *pRootControlInfo, CWnd *pMainWindow, RECT *pMainWindowRect)
{
	LONG result = (-1);

	if (m_windowIdent == ident)
	{
		if (NULL != pMainWindowRect)
		{
			result = pMainWindowRect->bottom;
		}
	}
	else if (m_ident != ident)
	{
		if (NULL != m_pNext)
		{
			result = m_pNext->calculateBottom(ident, pRootControlInfo, pMainWindow, pMainWindowRect);
		}
	}
	else
	{
		if (!m_bottomIsSet)
		{
			switch(m_bottomAlign)
			{
			case(ABOTTOM_TOP):
			case(ABOTTOM_TOP_ACTIVEFRAME):
			case(ABOTTOM_TOP_NOOFFSET):
				{
					LONG relTop = pRootControlInfo->calculateTop(m_bottomRelativeIdent,
																	pRootControlInfo, pMainWindow, pMainWindowRect);

					if ((-1) != relTop)
					{
						m_bottomValue = relTop + m_bottomOffset;
						m_bottomIsSet = true;
					}
				}
				break;

			case(ABOTTOM_BOTTOM):
			case(ABOTTOM_BOTTOM_ACTIVEFRAME):
			case(ABOTTOM_BOTTOM_NOOFFSET):
				{
					LONG relBottom = pRootControlInfo->calculateBottom(m_bottomRelativeIdent,
																		pRootControlInfo, pMainWindow, pMainWindowRect);

					if ((-1) != relBottom)
					{
						m_bottomValue = relBottom + m_bottomOffset;
						m_bottomIsSet = true;
					}
				}
				break;

			case(ABOTTOM_PERCENTAGE):
				{
					LONG relHeight = pRootControlInfo->calculateHeight(m_bottomRelativeIdent,
																		pRootControlInfo, pMainWindow,
																		pMainWindowRect);

					LONG relTop = pRootControlInfo->calculateTop(m_bottomRelativeIdent,
																	pRootControlInfo, pMainWindow, pMainWindowRect);

					if (((-1) != relHeight) && ((-1) != relTop))
					{
						m_bottomValue = relTop + ((m_bottomPercentage * relHeight) / PERCENTAGE_SCALE_FACTOR);
						m_bottomIsSet = true;
					}
				}
				break;

			case(ABOTTOM_CENTER):
				{
					LONG relHeight = pRootControlInfo->calculateHeight(m_topRelativeIdent,
																		pRootControlInfo, pMainWindow,
																		pMainWindowRect);

					LONG relTop = pRootControlInfo->calculateTop(m_topRelativeIdent,
																	pRootControlInfo, pMainWindow, pMainWindowRect);

					if (((-1) != relHeight) && ((-1) != relTop))
					{
						m_bottomValue = relTop + relHeight/2 + m_bottomOffset;
						m_bottomIsSet = true;
					}
				}
				break;

			case(ATOPBOTTOM_CENTER):
				{
					LONG relHeight = pRootControlInfo->calculateHeight(m_bottomRelativeIdent,
																		pRootControlInfo, pMainWindow,
																		pMainWindowRect);

					LONG relTop = pRootControlInfo->calculateTop(m_bottomRelativeIdent,
																	pRootControlInfo, pMainWindow, pMainWindowRect);

					if (((-1) != relHeight) && ((-1) != relTop))
					{
						// m_bottomOffset is our constant height in this case.
						m_bottomValue = relTop + ((relHeight - m_bottomOffset) / 2) + m_bottomOffset;
						m_bottomIsSet = true;
					}
				}
				break;

			default:
				break;
			}
		}

		if (m_bottomIsSet)
		{
			result = m_bottomValue;
		}
	}

	return(result);
}

LONG CControlInfo::calculateLeft(int ident, CControlInfo *pRootControlInfo, CWnd *pMainWindow, RECT *pMainWindowRect)
{
	LONG result = (-1);

	if (m_windowIdent == ident)
	{
		if (NULL != pMainWindowRect)
		{
			result = pMainWindowRect->left;
		}
	}
	else if (m_ident != ident)
	{
		if (NULL != m_pNext)
		{
			result = m_pNext->calculateLeft(ident, pRootControlInfo, pMainWindow, pMainWindowRect);
		}
	}
	else
	{
		if (!m_leftIsSet)
		{
			switch(m_leftAlign)
			{
			case(ALEFT_LEFT):
			case(ALEFT_LEFT_ACTIVEFRAME):
			case(ALEFT_LEFT_NOOFFSET):
				{
					LONG relLeft = pRootControlInfo->calculateLeft(m_leftRelativeIdent,
																	pRootControlInfo, pMainWindow, pMainWindowRect);

					if ((-1) != relLeft)
					{
						m_leftValue = relLeft + m_leftOffset;
						m_leftIsSet = true;
					}
				}
				break;

			case(ALEFT_RIGHT):
			case(ALEFT_RIGHT_ACTIVEFRAME):
			case(ALEFT_RIGHT_NOOFFSET):
				{
					LONG relRight = pRootControlInfo->calculateRight(m_leftRelativeIdent,
																		pRootControlInfo, pMainWindow, pMainWindowRect);

					if ((-1) != relRight)
					{
						m_leftValue = relRight + m_leftOffset;
						m_leftIsSet = true;
					}
				}
				break;

			case(ALEFT_PERCENTAGE):
				{
					LONG relWidth = pRootControlInfo->calculateWidth(m_leftRelativeIdent,
																		pRootControlInfo, pMainWindow,
																		pMainWindowRect);

					LONG relLeft = pRootControlInfo->calculateLeft(m_leftRelativeIdent,
																	pRootControlInfo, pMainWindow, pMainWindowRect);

					if (((-1) != relWidth) && ((-1) != relLeft))
					{
						m_leftValue = relLeft + ((m_leftPercentage * relWidth) / PERCENTAGE_SCALE_FACTOR);
						m_leftIsSet = true;
					}
				}
				break;

			case(ALEFT_CENTER):
				{
					LONG relWidth = pRootControlInfo->calculateWidth(m_leftRelativeIdent,
																		pRootControlInfo, pMainWindow,
																		pMainWindowRect);

					LONG relLeft = pRootControlInfo->calculateLeft(m_leftRelativeIdent,
																	pRootControlInfo, pMainWindow, pMainWindowRect);

					if (((-1) != relWidth) && ((-1) != relLeft))
					{
						m_leftValue = relLeft + relWidth/2 + m_leftOffset;
						m_leftIsSet = true;
					}
				}
				break;

			case(ALEFTRIGHT_CENTER):
				{
					LONG relWidth = pRootControlInfo->calculateWidth(m_leftRelativeIdent,
																		pRootControlInfo, pMainWindow,
																		pMainWindowRect);

					LONG relLeft = pRootControlInfo->calculateLeft(m_leftRelativeIdent,
																	pRootControlInfo, pMainWindow, pMainWindowRect);

					if (((-1) != relWidth) && ((-1) != relLeft))
					{
						// m_leftOffset is our constant width in this case.
						m_leftValue = relLeft + ((relWidth - m_leftOffset) / 2);
						m_leftIsSet = true;
					}
				}
				break;

			default:
				break;
			}
		}

		if (m_leftIsSet)
		{
			result = m_leftValue;
		}
	}

	return(result);
}

LONG CControlInfo::calculateRight(int ident, CControlInfo *pRootControlInfo, CWnd *pMainWindow, RECT *pMainWindowRect)
{
	LONG result = (-1);

	if (m_windowIdent == ident)
	{
		if (NULL != pMainWindowRect)
		{
			result = pMainWindowRect->right;
		}
	}
	else if (m_ident != ident)
	{
		if (NULL != m_pNext)
		{
			result = m_pNext->calculateRight(ident, pRootControlInfo, pMainWindow, pMainWindowRect);
		}
	}
	else
	{
		if (!m_rightIsSet)
		{
			switch(m_rightAlign)
			{
			case(ARIGHT_LEFT):
			case(ARIGHT_LEFT_ACTIVEFRAME):
			case(ARIGHT_LEFT_NOOFFSET):
				{
					LONG relLeft = pRootControlInfo->calculateLeft(m_rightRelativeIdent,
																	pRootControlInfo, pMainWindow, pMainWindowRect);

					if ((-1) != relLeft)
					{
						m_rightValue = relLeft + m_rightOffset;
						m_rightIsSet = true;
					}
				}
				break;

			case(ARIGHT_RIGHT):
			case(ARIGHT_RIGHT_ACTIVEFRAME):
			case(ARIGHT_RIGHT_NOOFFSET):
				{
					LONG relRight = pRootControlInfo->calculateRight(m_rightRelativeIdent,
																		pRootControlInfo, pMainWindow, pMainWindowRect);

					if ((-1) != relRight)
					{
						m_rightValue = relRight + m_rightOffset;
						m_rightIsSet = true;
					}
				}
				break;

			case(ARIGHT_PERCENTAGE):
				{
					LONG relWidth = pRootControlInfo->calculateWidth(m_rightRelativeIdent,
																		pRootControlInfo, pMainWindow,
																		pMainWindowRect);

					LONG relLeft = pRootControlInfo->calculateLeft(m_rightRelativeIdent,
																	pRootControlInfo, pMainWindow, pMainWindowRect);

					if (((-1) != relWidth) && ((-1) != relLeft))
					{
						m_rightValue = relLeft + ((m_rightPercentage * relWidth) / PERCENTAGE_SCALE_FACTOR);
						m_rightIsSet = true;
					}
				}
				break;

			case(ARIGHT_CENTER):
				{
					LONG relWidth = pRootControlInfo->calculateWidth(m_leftRelativeIdent,
																		pRootControlInfo, pMainWindow,
																		pMainWindowRect);

					LONG relLeft = pRootControlInfo->calculateLeft(m_leftRelativeIdent,
																	pRootControlInfo, pMainWindow, pMainWindowRect);

					if (((-1) != relWidth) && ((-1) != relLeft))
					{
						m_rightValue = relLeft + relWidth/2 + m_rightOffset;
						m_rightIsSet = true;
					}
				}
				break;

			case(ALEFTRIGHT_CENTER):
				{
					LONG relWidth = pRootControlInfo->calculateWidth(m_rightRelativeIdent,
																		pRootControlInfo, pMainWindow,
																		pMainWindowRect);

					LONG relLeft = pRootControlInfo->calculateLeft(m_rightRelativeIdent,
																	pRootControlInfo, pMainWindow, pMainWindowRect);

					if (((-1) != relWidth) && ((-1) != relLeft))
					{
						// m_rightOffset is our constant width in this case.
						m_rightValue = relLeft + ((relWidth - m_rightOffset) / 2) + m_rightOffset;
						m_rightIsSet = true;
					}
				}
				break;

			default:
				break;
			}
		}

		if (m_rightIsSet)
		{
			result = m_rightValue;
		}
	}

	return(result);
}

LONG CControlInfo::calculateHeight(int ident, CControlInfo *pRootControlInfo, CWnd *pMainWindow, RECT *pMainWindowRect)
{
	LONG result = (-1);

	if (m_windowIdent == ident)
	{
		if (NULL != pMainWindowRect)
		{
			result = pMainWindowRect->bottom - pMainWindowRect->top;
		}
	}
	else if (m_ident != ident)
	{
		if (NULL != m_pNext)
		{
			result = m_pNext->calculateHeight(ident, pRootControlInfo, pMainWindow, pMainWindowRect);
		}
	}
	else
	{
		LONG top	= calculateTop(		ident, pRootControlInfo, pMainWindow, pMainWindowRect);
		LONG bottom	= calculateBottom(	ident, pRootControlInfo, pMainWindow, pMainWindowRect);

		if (((-1) != top) && ((-1) != bottom))
		{
			result = bottom - top;
		}
	}

	return(result);
}

LONG CControlInfo::calculateWidth(int ident, CControlInfo *pRootControlInfo, CWnd *pMainWindow, RECT *pMainWindowRect)
{
	LONG result = (-1);

	if (m_windowIdent == ident)
	{
		if (NULL != pMainWindowRect)
		{
			result = pMainWindowRect->right - pMainWindowRect->left;
		}
	}
	else if (m_ident != ident)
	{
		if (NULL != m_pNext)
		{
			result = m_pNext->calculateWidth(ident, pRootControlInfo, pMainWindow, pMainWindowRect);
		}
	}
	else
	{
		LONG left	= calculateLeft(	ident, pRootControlInfo, pMainWindow, pMainWindowRect);
		LONG right	= calculateRight(	ident, pRootControlInfo, pMainWindow, pMainWindowRect);

		if (((-1) != left) && ((-1) != right))
		{
			result = right - left;
		}
	}

	return(result);
}

// Should be called on the root CControlInfo
void CControlInfo::DisableBeforePseudoModalChild(CWnd *pMainWindow, bool bCascadeClosure)
{
	if (!( (m_flagOne & F1_DISALLOW_DISABLE)											\
	||    ((m_flagOne & F1_DISALLOW_DISABLE_WHEN_CASCADECLOSURE) && bCascadeClosure))	)
	{
		CWnd *pCWnd = pMainWindow->GetDlgItem(m_ident);

		if ((NULL != pCWnd) && (pCWnd->IsWindowEnabled()))
		{
			m_privFlagOne |= PF1_REENABLE;		// Enable the window when the child closes.
			pCWnd->EnableWindow(FALSE);
		}
		else
		{
			m_privFlagOne &= (~PF1_REENABLE);	// Leave the window disabled when the child closes.
		}
	}

	if (NULL != m_pNext)
	{
		m_pNext->DisableBeforePseudoModalChild(pMainWindow, bCascadeClosure);
	}
}

// Should be called on the root CControlInfo
void CControlInfo::EnableAfterPseudoModalChild(CWnd *pMainWindow)
{
	if (m_privFlagOne & PF1_REENABLE)
	{
		CWnd *pCWnd = pMainWindow->GetDlgItem(m_ident);

		if (NULL != pCWnd)
		{
			pCWnd->EnableWindow(TRUE);
		}
	}

	if (NULL != m_pNext)
	{
		m_pNext->EnableAfterPseudoModalChild(pMainWindow);
	}
}

// Should be called on the root CControlInfo
// Hide all controls which are in frames in hideMask but not in frames in showMask.
// Show all controls which are in frames in showMask.
// A hidemask of 0xFFFFFFFF, the default, means everything is hidden but those in the showMask which is shown.
// The default "0" frame is not applicable here:
// A zero showMask shows nothing;
// A zero hideMask hides nothing.
void CControlInfo::ChangeFrameVisibility(CWnd *pMainWindow, DWORD showMask, DWORD hideMask/* = 0xFFFFFFFF*/)
{
	if (NULL != pMainWindow)
	{
		if (showMask & m_frameMembership)
		{
			if (! (PF1_INVISIBLE & m_privFlagOne) )
			{
				CWnd *pCWnd = pMainWindow->GetDlgItem(m_ident);

				if (NULL != pCWnd)
				{
					pCWnd->ShowWindow(SW_SHOW);
				}
			}
		}
		else if (hideMask & m_frameMembership)
		{
			CWnd *pCWnd = pMainWindow->GetDlgItem(m_ident);

			if (NULL != pCWnd)
			{
				pCWnd->ShowWindow(SW_HIDE);
			}
		}

		if (NULL != m_pNext)
		{
			m_pNext->ChangeFrameVisibility(pMainWindow, showMask, hideMask);
		}
	}
}

// Disable all controls which are in frames in disableMask but not in frames in enableMask.
// Enables all controls where are in frames in enableMask.
// A disableMask of 0xFFFFFFFF, the default, means everything is disabled but those in enableMask which are enabled.
// The default "0" frame is not applicable here:
// A zero enableMask enables nothing.
// A zero disableMask disables nothing.
void CControlInfo::ChangeFrameEnable(CWnd *pMainWindow, DWORD enableMask, DWORD disableMask/* = 0xFFFFFFFF*/)
{
	if (NULL != pMainWindow)
	{
		if (enableMask & m_frameMembership)
		{
			CWnd *pCWnd = pMainWindow->GetDlgItem(m_ident);

			if (NULL != pCWnd)
			{
				pCWnd->EnableWindow(TRUE);
			}
		}
		else if (disableMask & m_frameMembership)
		{
			CWnd *pCWnd = pMainWindow->GetDlgItem(m_ident);

			if (NULL != pCWnd)
			{
				pCWnd->EnableWindow(FALSE);
			}
		}

		if (NULL != m_pNext)
		{
			m_pNext->ChangeFrameEnable(pMainWindow, enableMask, disableMask);
		}
	}
}

// Calculate the border which is around all controls in the window.
// Returns top/bottom/left/right border values in *pOutBorderRect.
// Invisible controls ignored.
// Returns success.
bool CControlInfo::CalculateWindowBorder(CWnd *pMainWindow, RECT *pMainWindowRect, RECT *pOutBorderRect)
{
	bool result = false;

	if (NULL != pMainWindow)
	{
		// Ignore invisible controls.
		if (!(PF1_INVISIBLE & m_privFlagOne))
		{
			// Calculate the border for this contol.
			CWnd *pOurCWnd = pMainWindow->GetDlgItem(m_ident);

			if (NULL != pOurCWnd)
			{
				RECT ourRect;

				pOurCWnd->GetWindowRect(&ourRect);
				pMainWindow->ScreenToClient(&ourRect);

				pOutBorderRect->top = ourRect.top - pMainWindowRect->top;
				pOutBorderRect->bottom = pMainWindowRect->bottom - ourRect.bottom;
				pOutBorderRect->left = ourRect.left - pMainWindowRect->left;
				pOutBorderRect->right = pMainWindowRect->right - ourRect.right;
			}

			result = true;
		}

		if (NULL != m_pNext)
		{
			RECT childBorderRect;

			if (m_pNext->CalculateWindowBorder(pMainWindow, pMainWindowRect, &childBorderRect))
			{
				// If we got border values, compare and take the minimum.
				// If we didn't, just use those from down the chain.
				if ((!result) || (pOutBorderRect->top > childBorderRect.top))
				{
					pOutBorderRect->top = childBorderRect.top;
				}

				if ((!result) || (pOutBorderRect->bottom > childBorderRect.bottom))
				{
					pOutBorderRect->bottom = childBorderRect.bottom;
				}

				if ((!result) || (pOutBorderRect->left > childBorderRect.left))
				{
					pOutBorderRect->left = childBorderRect.left;
				}

				if ((!result) || (pOutBorderRect->right > childBorderRect.right))
				{
					pOutBorderRect->right = childBorderRect.right;
				}

				result = true;
			}
		}
	}

	return(result);
}

//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================

CSuckLessDialogThread::CSuckLessDialogThread(void)
{
	m_hThread = 0;
	m_pParentSLDlg = NULL;
}

CSuckLessDialogThread::~CSuckLessDialogThread(void)
{
	WaitForThreadTermination();
}

// Must not be called while the thread is running.
// If a parent is set it will be sent WM_SLDLG_THREADTERMINATING just before the
// thread terminates -- use WaitForThreadTermination() after receiving this.
void CSuckLessDialogThread::SetParentSLDlg(CWnd *pParentSLDlg)
{
	if (0 == m_hThread)
	{
		m_pParentSLDlg = pParentSLDlg;
	}
}

// Only the parent thread should call StartThread(), WaitForThreadTermination(), and IsThreadRunning()
// Returns true if the thread started.
bool CSuckLessDialogThread::StartThread(void)
{
	bool result = false;

	// Make sure an existing thread is not already running.
	WaitForThreadTermination();

	unsigned int spambo; // something we ignore.

	// thread handle set to zero if this fails.
	m_hThread = (HANDLE)_beginthreadex(NULL, 0, PrivateMainThread, (void *)(this), 0, &spambo);

	if (0 != m_hThread)
	{
		result = true;
	}

	return(result);
}

// Only the parent thread should call StartThread(), WaitForThreadTermination(), and IsThreadRunning()
void CSuckLessDialogThread::WaitForThreadTermination(void)
{
	if (0 != m_hThread)
	{
		WaitForSingleObject(m_hThread, INFINITE);
		CloseHandle(m_hThread);
		m_hThread = 0;
	}
}

// Only the parent thread should call StartThread(), WaitForThreadTermination(), and IsThreadRunning()
bool CSuckLessDialogThread::IsThreadRunning(void)
{
	bool result = false;

	if (0 != m_hThread)
	{
		DWORD dwWaitRes = WaitForSingleObject(m_hThread, 0); // Poll, don't wait.

		if (WAIT_OBJECT_0 == dwWaitRes)
		{
			result = false; // The thread WAS running but just stopped.
			CloseHandle(m_hThread);
			m_hThread = 0;
		}
		else if (WAIT_TIMEOUT == dwWaitRes)
		{
			result = true; // The thread IS running (or at least was when we polled).
		}
		else
		{
			result = false; // Error waiting. Assume the thread isn't running so things don't deadlock.
		}
	}

	return(result);
}


// ------------------------------------------------------------------------------
// Threads -- Not for external use. Public only for the thread calling mechanism.
// ------------------------------------------------------------------------------
void CSuckLessDialogThread::PrivateMainThread(void)
{
	mainThreadSubClass();

	if (NULL != m_pParentSLDlg)
	{
		// Using SendMessage will cause deadlock.
		m_pParentSLDlg->PostMessage(WM_SLDLG_THREADTERMINATING);
	}
}

//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================

CSuckLessDialog::CSuckLessDialog(UINT nIDTemplate, CWnd* pParentWnd)
	: CDialog(nIDTemplate, pParentWnd)
{
	m_windowIdent = (int)nIDTemplate;

	m_lparamToReturnToParent = 0;

	m_bStayHiddenAfterPseudoModalChild = false;
	m_bCascadeClosure = false;
	m_bCascadeClosurePermanentlyDisabled = false;

	m_nClosureCount = 0;

	m_bHiddenByParentWhileMaximized = false;

	m_controlAndFrameInfoSetup = false;
	m_pControlInfo = NULL;
	m_pFrameInfo = NULL;
	m_activeFrameIdent = 0;

	m_borderRect.top = 0;
	m_borderRect.bottom = 0;
	m_borderRect.left = 0;
	m_borderRect.right = 0;
	
	m_minX = -1;
	m_minY = -1;
	m_maxX = -1;
	m_maxY = -1;

	m_bStatusBar = true;

	m_bIsModal = false;

	m_pChildPseudoModalChildArgs = NULL;
	m_pParentPseudoModalChildArgs = NULL;

	m_pSLDThread = NULL;

	m_hIcon = NULL;
	m_hIconSmall = NULL;
}

CSuckLessDialog::~CSuckLessDialog(void)
{
	m_controlAndFrameInfoSetup = false;

	delete m_pControlInfo;
	m_pControlInfo = NULL;

	delete m_pFrameInfo;
	m_pFrameInfo = NULL;

	delete m_pChildPseudoModalChildArgs;
	m_pChildPseudoModalChildArgs = NULL;

	// m_pParentPseudoModalChildArgs is not ours to delete.

	delete m_pSLDThread;
	m_pSLDThread = NULL;
}

BEGIN_MESSAGE_MAP(CSuckLessDialog, CDialog)
	//{{AFX_MSG_MAP(CSuckLessDialog)
	ON_REGISTERED_MESSAGE(WM_SLDLG_PMCHILDCLOSE, OnSLDlgPMChildCloseEvent)
	ON_REGISTERED_MESSAGE(WM_SLDLG_THREADTERMINATING, OnSLDlgThreadTerminating)
	ON_WM_SIZE()
	ON_WM_GETMINMAXINFO()
	ON_WM_CLOSE()
	ON_WM_SYSCOMMAND()
	ON_WM_SHOWWINDOW()
	ON_WM_ERASEBKGND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// By default, OnCancel() will do nothing while pseudo-modal children are open.
// If you set cascade closure, OnCancel() will close the window, causing the children to close as well.
// Remember not to disable your cancel gadget as leaves the close gadget enabled but it will do nothing.
// Will do nothing if called while a pseudo-modal child is open.
// (Can be called before and after controlAndFrameInfoIsSetup())
void CSuckLessDialog::setCascadeClosure(bool bVal/* = true*/)
{
	if ((NULL == m_pChildPseudoModalChildArgs) && (!m_bCascadeClosurePermanentlyDisabled))
	{
		m_bCascadeClosure = bVal;
	}
}

// To be used in situations where enabling cascade closure would screw up the application,
// for example when a pseudo-modal child or thread is launched on window closure.
// Will do nothing if called while a pseudo-modal child is open.
// (Can be called before and after controlAndFrameInfoIsSetup())
void CSuckLessDialog::setCascadeClosurePermanentlyDisabled(void)
{
	if (NULL == m_pChildPseudoModalChildArgs)
	{
		m_bCascadeClosurePermanentlyDisabled = true;
		m_bCascadeClosure = false;
	}
}


// Will do nothing if called while a pseudo-modal child is open.
// (Can be called before and after controlAndFrameInfoIsSetup())
void CSuckLessDialog::setStayHiddenAfterPseudoModalChild(bool bStayHiddenAfterPseudoModalChild)
{
	if (NULL == m_pChildPseudoModalChildArgs)
	{
		m_bStayHiddenAfterPseudoModalChild = bStayHiddenAfterPseudoModalChild;
	}
}

void CSuckLessDialog::addControlInfo(int ident, DWORD flagOne/* = 0*/, DWORD frameMembership/* = 0*/)
{
	if (!m_controlAndFrameInfoSetup)
	{
		// Adds itself to the list. Handles the case where the list is empty.
		new CControlInfo(&m_pControlInfo, this, m_windowIdent, ident, flagOne, frameMembership);
	}
}

void CSuckLessDialog::setControlRelationship(int identSubject, int identRelative, ...)
{
	if (NULL != m_pControlInfo)
	{
		va_list args;
		va_start(args, identRelative);
		m_pControlInfo->SetControlRelationship(identSubject, identRelative, args);
		va_end(args);
	}
}

void CSuckLessDialog::addFrameInfo(DWORD frameIdent, int frameControlIdent, bool bHorizPercentage, bool bVertPercentage)
{
	if (!m_controlAndFrameInfoSetup)
	{
		// Adds itself to the list. Handles the case where the list is empty.
		new CFrameInfo(&m_pFrameInfo, frameIdent, this, frameControlIdent, bHorizPercentage, bVertPercentage);
	}
}

void CSuckLessDialog::noStatusBar(void)
{
	if (!m_controlAndFrameInfoSetup)
	{
		m_bStatusBar = false;
	}
}

// Will do nothing if addStatusBar() was not called before controlAndFrameInfoIsSetup() was called.
void CSuckLessDialog::setStatusBarText(const TCHAR *statusMsg)
{
	if (m_bStatusBar)
	{
		m_statusBar.SetWindowText(NULL == statusMsg ? _T("") : statusMsg);
	}
}

// Calls CenterWindow()
void CSuckLessDialog::setRememberWindowPosition(const TCHAR *szWinPosRegPath, const TCHAR *szWinPosValueNamePrefix)
{
	CenterWindow();

	if ((NULL != szWinPosRegPath) && (NULL != szWinPosValueNamePrefix))
	{
		m_strWinPosRegPath = szWinPosRegPath;
		m_strWinPosRegValueNamePrefix = szWinPosValueNamePrefix;

		std::basic_string<TCHAR> strW = m_strWinPosRegValueNamePrefix + ".w";
		std::basic_string<TCHAR> strH = m_strWinPosRegValueNamePrefix + ".h";

		DWORD dwW;
		DWORD dwH;

		if (LeetRegQueryDWORDValue(HKEY_CURRENT_USER, m_strWinPosRegPath.c_str(), strW.c_str(), &dwW)
		&&	LeetRegQueryDWORDValue(HKEY_CURRENT_USER, m_strWinPosRegPath.c_str(), strH.c_str(), &dwH))
		{
			RECT rectToCenterIn;
			GetWindowRect(&rectToCenterIn);

			LONG longW = static_cast<LONG>(dwW);
			LONG longH = static_cast<LONG>(dwH);

			LONG longCenterX = ((rectToCenterIn.right - rectToCenterIn.left) / 2) + rectToCenterIn.left;
			LONG longCenterY = ((rectToCenterIn.bottom - rectToCenterIn.top) / 2) + rectToCenterIn.top;

			RECT rectCorrected;
			rectCorrected.left = longCenterX - (longW / 2);
			rectCorrected.right = longCenterX + (longW / 2);
			rectCorrected.top = longCenterY - (longH / 2);
			rectCorrected.bottom = longCenterY + (longH / 2);

			// Keep it on screen.

			RECT rectDesktop;
			GetDesktopWindow()->GetWindowRect(&rectDesktop);

			if (rectCorrected.bottom > rectDesktop.bottom)
			{
				rectCorrected.top    -= (rectCorrected.bottom - rectDesktop.bottom);
				rectCorrected.bottom -= (rectCorrected.bottom - rectDesktop.bottom);
			}
			
			if (rectCorrected.right > rectDesktop.right)
			{
				rectCorrected.left  -= (rectCorrected.right - rectDesktop.right);
				rectCorrected.right -= (rectCorrected.right - rectDesktop.right);
			}

			if (rectCorrected.left < rectDesktop.left)
			{
				rectCorrected.right += (rectDesktop.left - rectCorrected.left);
				rectCorrected.left  += (rectDesktop.left - rectCorrected.left);
			}
			
			if (rectCorrected.top < rectDesktop.top)
			{
				rectCorrected.bottom += (rectDesktop.top - rectCorrected.top);
				rectCorrected.top    += (rectDesktop.top - rectCorrected.top);
			}

			MoveWindow(&rectCorrected, TRUE);
		}
	}
}

void CSuckLessDialog::controlAndFrameInfoIsSetup(void)
{
	if (!m_controlAndFrameInfoSetup) // Protect from accidentally calling multiple times.
	{
		if (m_bStatusBar)
		{
			DWORD windowStyle = this->GetStyle();

			if ( (((WS_MINIMIZE|WS_MAXIMIZE) & windowStyle)) || (!m_statusBar.Create(this)) )
			{
				m_bStatusBar = false;
			}
			else
			{                           
				UINT auIDStatusBar = ID_SEPARATOR;

				m_statusBar.SetIndicators(&auIDStatusBar, 1);
				m_statusBar.SetWindowText("");
				m_statusBar.SetPaneStyle(0, SBPS_STRETCH | SBPS_NOBORDERS);
				this->RepositionBars(AFX_IDW_CONTROLBAR_FIRST, AFX_IDW_CONTROLBAR_LAST, 0);

				CRect statusRect;
				CRect windowRect;
				m_statusBar.GetWindowRect(&statusRect);
				this->GetWindowRect(&windowRect);

				int desiredX = windowRect.Width();
				int desiredY = windowRect.Height() + statusRect.Height();

				this->SetWindowPos(NULL, 0, 0, desiredX, desiredY,
									SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOZORDER);
			}
		}

		if ((NULL != m_hIcon) && (NULL != m_hIconSmall))
		{
			// Set the icon for this dialog.  The framework does this automatically
			//  when the application's main window is not a dialog
			SetIcon(m_hIcon,		TRUE);		// Set big icon
			SetIcon(m_hIcon,		FALSE);		// Set small icon
			SetIcon(m_hIconSmall,	FALSE);		// Set small icon
		}

		if (NULL != m_pControlInfo) // m_pFrameInfo can be NULL at this stage.
		{
			// Define the default window-sized frame (0x00000000) with all controls visible.
			// Adds itself to the list. Handles the case where the list is empty.
			new CFrameInfo(&m_pFrameInfo, 0, this, m_windowIdent, false, false);

			// Calculate the border which is around all controls in the window, including the frames if any.
			RECT mainWindowRect;
			this->GetClientRect(&mainWindowRect);

			if (m_pControlInfo->CalculateWindowBorder(this, &mainWindowRect, &m_borderRect))
			{
				// Remove this border from the default frame so it's like all the others.
				m_pFrameInfo->RemoveBorderFromDefaultFrame(&m_borderRect);

				RECT mainWindowRectExcludingBorders;
				m_pFrameInfo->GetOriginalRect(0, mainWindowRectExcludingBorders, NULL, NULL);

				m_pControlInfo->CalculateInitialControlRelationshipValues(this, &mainWindowRectExcludingBorders);

				m_controlAndFrameInfoSetup = true;
			}
		}
	}
}

// Returns success.
// If setMin is true then MinX / MinY values will be set to the frame's initial size.
// If setMax is true then MaxX / MaxY values will be set to the frame's initial size.
bool CSuckLessDialog::switchToFrame(DWORD frameIdent, bool bSetMinX/* = true*/, bool bSetMinY/* = true*/,
													bool bSetMaxX/* = false*/, bool bSetMaxY/* = false*/)
{
	bool result = true;

	if (!m_controlAndFrameInfoSetup)
	{
		result = false;
	}
	else
	{
		POINT desiredSize;
		if (! m_pFrameInfo->GetNewWindowSize(frameIdent, m_activeFrameIdent, this, &m_borderRect, desiredSize) )
		{
			result = false;
		}
		else
		{
			m_activeFrameIdent = frameIdent;

			// Add on extra size for borders around the frame.
			desiredSize.x += (m_borderRect.right + m_borderRect.left);
			desiredSize.y += (m_borderRect.bottom + m_borderRect.top);

			// Add on extra size for the non-client area of the window.
			RECT mainWindowRect;
			this->GetClientRect(&mainWindowRect);

			RECT mainWindowScreenRect;
			this->GetWindowRect(&mainWindowScreenRect);

			LONG clientDeltaX = (mainWindowScreenRect.right  - mainWindowScreenRect.left) - (mainWindowRect.right  - mainWindowRect.left);
			LONG clientDeltaY = (mainWindowScreenRect.bottom - mainWindowScreenRect.top ) - (mainWindowRect.bottom - mainWindowRect.top );

			desiredSize.x += clientDeltaX;
			desiredSize.y += clientDeltaY;

			// Change the min/max sizes if requested.
			if (bSetMinX || bSetMinY || bSetMaxX || bSetMaxY)
			{
				RECT origRect;

				if (!m_pFrameInfo->GetOriginalRect(frameIdent, origRect, NULL, NULL))
				{
					result = false;
				}
				else
				{
					LONG origWidth  = (origRect.right - origRect.left);
					LONG origHeight = (origRect.bottom - origRect.top);;

					origWidth  += clientDeltaX;
					origHeight += clientDeltaY;

					origWidth  += (m_borderRect.right + m_borderRect.left);
					origHeight += (m_borderRect.bottom + m_borderRect.top);

					if (bSetMinX)
					{
						setMinX(origWidth);
						if (desiredSize.x < origWidth)
						{
							desiredSize.x = origWidth;
						}
					}

					if (bSetMinY)
					{
						setMinY(origHeight);
						if (desiredSize.y < origHeight)
						{
							desiredSize.y = origHeight;
						}
					}

					if (bSetMaxX)
					{
						setMaxX(origWidth);
						if (desiredSize.x > origWidth)
						{
							desiredSize.x = origWidth;
						}
					}

					if (bSetMaxY)
					{
						setMaxY(origHeight);
						if (desiredSize.y > origHeight)
						{
							desiredSize.y = origHeight;
						}
					}
				}
			}

			DWORD windowStyle = this->GetStyle();

			if (WS_MAXIMIZE & windowStyle)
			{
				// The window is maximised and so should not be resized.
				// Sending WM_SIZE with SIZE_MAXIMIZED will will trigger control movement and resizing within the
				// existing window.
				// WM_SIZE uses client size.
				RECT clientRect;
				this->GetClientRect(&clientRect);
				this->SendMessage(WM_SIZE, SIZE_MAXIMIZED, MAKELPARAM((WORD)(clientRect.right - clientRect.left),
																		(WORD)(clientRect.bottom - clientRect.top)));
			}
			else if (!(WS_MINIMIZE & windowStyle))
			{
				this->SetWindowPos(NULL, 0, 0, desiredSize.x, desiredSize.y,
									SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOZORDER);
			}

			// Hide all controls which are not in the desired frame, showing those that are.
			m_pControlInfo->ChangeFrameVisibility(this, frameIdent);

//			this->InvalidateRect(NULL);
		}
	}

	return(result);
}

// If a frame is only used by enableFrameControls() it need not be added with
// addFrameInfo() or associated with a control.
// Does nothing while a pseudo-modal child is open.
void CSuckLessDialog::enableFrameControls(DWORD frameIdentMask, bool bEnable)
{
	if ((m_controlAndFrameInfoSetup) && (NULL == m_pChildPseudoModalChildArgs))
	{
		m_pControlInfo->ChangeFrameEnable(this, bEnable ? frameIdentMask : 0, bEnable ? 0 : frameIdentMask);
	}
}

void CSuckLessDialog::setMinXToCurrent(long numerator/*=1*/, long denominator/*=1*/)
{
	if (0 == denominator)
	{
		denominator = 1;
	}

	RECT mainWindowRect;
	this->GetWindowRect(&mainWindowRect);
	m_minX = ((mainWindowRect.right - mainWindowRect.left) * numerator) / denominator;
}

void CSuckLessDialog::setMinYToCurrent(long numerator/*=1*/, long denominator/*=1*/)
{
	if (0 == denominator)
	{
		denominator = 1;
	}

	RECT mainWindowRect;
	this->GetWindowRect(&mainWindowRect);
	m_minY = ((mainWindowRect.bottom - mainWindowRect.top) * numerator) / denominator;
}

void CSuckLessDialog::setMaxXToCurrent(long numerator/*=1*/, long denominator/*=1*/)
{
	if (0 == denominator)
	{
		denominator = 1;
	}

	RECT mainWindowRect;
	this->GetWindowRect(&mainWindowRect);
	m_maxX = ((mainWindowRect.right - mainWindowRect.left) * numerator) / denominator;
}

void CSuckLessDialog::setMaxYToCurrent(long numerator/*=1*/, long denominator/*=1*/)
{
	if (0 == denominator)
	{
		denominator = 1;
	}

	RECT mainWindowRect;
	this->GetWindowRect(&mainWindowRect);
	m_maxY = ((mainWindowRect.bottom - mainWindowRect.top) * numerator) / denominator;
}

void CSuckLessDialog::setMinX(long minX)
{
	m_minX = minX;
}

void CSuckLessDialog::setMinY(long minY)
{
	m_minY = minY;
}

void CSuckLessDialog::setMaxX(long maxX)
{
	m_maxX = maxX;
}

void CSuckLessDialog::setMaxY(long maxY)
{
	m_maxY = maxY;
}

void CSuckLessDialog::OnSize(UINT nType, int cx, int cy) 
{
	bool didResize = false;

	// This will normally be called once before InitResizeInfoArray()
	if ( m_controlAndFrameInfoSetup && ((SIZE_MAXIMIZED == nType) || (SIZE_RESTORED == nType)) )
	{
		// Get this once and cache it.
		RECT	mainWindowRect;

		// Work out what the unclipped window size would be, excluding borders.
		// Assuming the frames are well defined, they should properly draw in their space.
		POINT fakeSize;
		if (! m_pFrameInfo->GetNewWindowSize(0, m_activeFrameIdent, this, &m_borderRect, fakeSize) )
		{
			// It failed, this'll have to do.
			this->GetClientRect(&mainWindowRect);
			mainWindowRect.top += m_borderRect.top;
			mainWindowRect.left += m_borderRect.left;
			mainWindowRect.right -= m_borderRect.right;
			mainWindowRect.bottom -= m_borderRect.bottom;
		}
		else
		{
			// Offset by the top and left borders to paint in the right place.
			mainWindowRect.left		= m_borderRect.left;
			mainWindowRect.right	= m_borderRect.left + fakeSize.x;
			mainWindowRect.top		= m_borderRect.top;
			mainWindowRect.bottom	= m_borderRect.top  + fakeSize.y;
		}

		m_pControlInfo->DoResize(this, &mainWindowRect, m_pFrameInfo->GetFrameCtrlIdent(m_activeFrameIdent));
		didResize = true;
	}

	CDialog::OnSize(nType, cx, cy);

	if (m_bStatusBar)
	{
		RepositionBars(AFX_IDW_CONTROLBAR_FIRST, AFX_IDW_CONTROLBAR_LAST, 0);
	}

	if (didResize)
	{
		this->InvalidateRect(NULL);
	}

	OnSizeHookEnd();
}

void CSuckLessDialog::OnSysCommand(UINT nID, LPARAM lParam)
{
	if (((nID & 0xFFF0) == SC_MINIMIZE) && ((NULL != m_pParentPseudoModalChildArgs) && (!m_pParentPseudoModalChildArgs->m_bHide)))
	{
		m_pParentPseudoModalChildArgs->m_pParentSuckLessDlg->SendMessage(WM_SYSCOMMAND, SC_MINIMIZE, 0);
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}


void CSuckLessDialog::OnShowWindow(BOOL bShow, UINT nStatus) 
{
	CDialog::OnShowWindow(bShow, nStatus);

	if (bShow && (SW_PARENTOPENING & nStatus) && m_bHiddenByParentWhileMaximized)
	{
		m_bHiddenByParentWhileMaximized = false;

		this->SendMessage(WM_SYSCOMMAND, SC_MAXIMIZE, 0);
	}
	else if ((!bShow) && (SW_PARENTCLOSING & nStatus) && (WS_MAXIMIZE & this->GetStyle())		\
				&& (NULL != m_pParentPseudoModalChildArgs) && (!m_pParentPseudoModalChildArgs->m_bHide)	)
	{
		m_bHiddenByParentWhileMaximized = true;
	}
}


void CSuckLessDialog::OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI) 
{
	if ((-1) != m_minX)
	{
		if (lpMMI->ptMinTrackSize.x < m_minX)
		{
			lpMMI->ptMinTrackSize.x = m_minX;
		}
	}

	if ((-1) != m_minY)
	{
		if (lpMMI->ptMinTrackSize.y < m_minY)
		{
			lpMMI->ptMinTrackSize.y = m_minY;
		}
	}

	if ((-1) != m_maxX)
	{
		if (lpMMI->ptMaxTrackSize.x > m_maxX)
		{
			lpMMI->ptMaxTrackSize.x = m_maxX;
		}
	}

	if ((-1) != m_maxY)
	{
		if (lpMMI->ptMaxTrackSize.y > m_maxY)
		{
			lpMMI->ptMaxTrackSize.y = m_maxY;
		}
	}
  
	CDialog::OnGetMinMaxInfo( lpMMI );
}

// Creates the given dialog.
// Should not be called while another pseudo-modal child is already open.
// Will disable all controls (except those excluded with F1_DISALLOW_DISABLE) while the child is open.
// If bHide is true, will hide the dialog while the child is open.
// The dialog will be deleted once the child closes if bDelDlg is true. If bDelDlg is true and
// dialog creation fails, this function will delete the dialog for you.
//
// If you want the new dialog to appear on the taskbar, as is often the case if you hide the original,
// make sure it has no owner (pSLDlg->SetOwner(NULL);) and make sure it has the WS_EX_APPWINDOW extended
// style:
//
// IDD_DIALOG1 DIALOG DISCARDABLE  0, 0, 114, 41
// STYLE WS_MINIMIZEBOX | WS_MAXIMIZEBOX | WS_POPUP | WS_VISIBLE | WS_CAPTION | 
//    WS_SYSMENU | WS_THICKFRAME
// EXSTYLE WS_EX_APPWINDOW
// CAPTION "Dialog"
// FONT 8, "MS Sans Serif"
// BEGIN
//    DEFPUSHBUTTON   "OK",IDOK,57,7,50,14
//    PUSHBUTTON      "Cancel",IDCANCEL,57,23,50,14
//    LTEXT           "I'm a little pea.",IDC_STATIC,7,7,46,8
// END
bool CSuckLessDialog::openPseudoModalChild(CSuckLessDialog *pSLDlg, int sldIDD,
										   /*CSuckLessDialogArgs *pSLDArgs = NULL,*/
										   bool bHide/* = false*/, bool bDelDlg/* = true*/)
{
	bool result = false;

	// Protect some error creating a pseudo-modal child when one already exists.

	if ((NULL != pSLDlg) && (NULL == m_pChildPseudoModalChildArgs))
	{
		// Disable controls.
		if (m_controlAndFrameInfoSetup)
		{
			m_pControlInfo->DisableBeforePseudoModalChild(this, m_bCascadeClosure);
		}

		if (!m_bCascadeClosure)
		{
			disableClosure();
		}

		OnPseudoModalChildOpen(sldIDD); // Hook function to be overriden by child class.

		m_pChildPseudoModalChildArgs = new CPMCArgs(this, bHide, bDelDlg/*, pSLDArgs*/);
		pSLDlg->setParentPMCArgs(m_pChildPseudoModalChildArgs);

		bool bCreateSuccess = (NULL != pSLDlg->Create(sldIDD, this));

		// Hide us if required.
		if (bHide)
		{
			this->ShowWindow(SW_HIDE);
		}

		if (! bCreateSuccess )
		{
			if (bDelDlg)
			{
				// The dialog was never created and so will not delete itself. We have to.
				delete pSLDlg;
			}

			cleanupAfterPMC();
		}
		else
		{
			result = true;
		}
	}
	else if ((NULL != pSLDlg) && bDelDlg)
	{
		// A pseudo-modal child was already open. We should never have been called, but we might as well
		// delete the dialog as we were asked to avoid memory leaks.
		delete pSLDlg;
	}

	return(result);
}

// Zero is returned by default.
void CSuckLessDialog::setLParamToReturnToParent(LPARAM lparam)
{
	m_lparamToReturnToParent = lparam;
}

void CSuckLessDialog::cleanupAfterPMC(void)
{
	if (NULL != m_pChildPseudoModalChildArgs)
	{
		if (m_controlAndFrameInfoSetup)
		{
			m_pControlInfo->EnableAfterPseudoModalChild(this);
		}

		// Note that setCascadeClosure() does nothing while a pseudo-modal child is open.
		if (!m_bCascadeClosure)
		{
			enableClosure();
		}

		// ShowWindow may be semi-disabled while m_pChildPseudoModalChildArgs is non-NULL.
		if (m_pChildPseudoModalChildArgs->m_bHide)
		{
			if (!m_bStayHiddenAfterPseudoModalChild)
			{
				this->ShowWindow(SW_SHOW);
			}
		}

		// The dialog should delete itself when destroyed if it created. (See overridden PostNCDestroy)

		// Delete the args structure.
		delete m_pChildPseudoModalChildArgs;
		m_pChildPseudoModalChildArgs = NULL; // We no longer have an open pseudo-modal child.
	}
}

// Will cleanup and call OnPseudoModalChildClose() whose default implementation does nothing.
// wparam should be the closed dialog's IDD.
// Do not use this event or OnPseudoModalChildClose() for extra cleanup since it may be possible
// for us to close before the child does, meaning the event is never received.
// If you have things which need to be deleted, make the CSuckLessDialogArgs subclass do it on destruction.
LRESULT CSuckLessDialog::OnSLDlgPMChildCloseEvent(WPARAM wparam, LPARAM lparam)
{
	LRESULT result = 0; // 0 for not handled, 1 for handled.

	if (NULL != m_pChildPseudoModalChildArgs)
	{
		result = 1;

		cleanupAfterPMC();
	}

	// We must call this after cleanupAfterPMC() so that
	// the handler code can open a new pseudo-modal child if it wants to.
	OnPseudoModalChildClose(wparam, lparam);

	return(result);
}

// You should call this function for all CSuckLessDialogs which are the pseudo-modal children of another CSLDlg.
// Called for you by openPseudoModalChild()
// Untested: If dialog isn't deleted, you should be able to reuse it and set a different parent, provided
// you don't do this while it is still the child of another window, which could be difficult in some situations.
// If you call this the dialog will do the following in PostNcDestroy as it closes:
// * Send WM_SLDLG_PMCHILDCLOSE to pPMCArgs->m_pParentSuckLessDlg with pPMCArgs in lparam.
// * Deletes itself if pPMCArgs->m_bDelChildDlg. (Otherwise the CSuckLessDialog class must be deleted manually.)
void CSuckLessDialog::setParentPMCArgs(CPMCArgs *pPMCArgs)
{
	m_pParentPseudoModalChildArgs = pPMCArgs;
}

// Returns boolean success.
// OnSLDlgThreadTerminating() will be called as the thread terminates.
// Unlike pseudo-modal child dialogs, SuckLessDialogThread objects do not auto-delete,
// but they will be deleted by the SuckLessDialog framework:
// * If startThread() fails the thread object will be deleted immediately.
// * If startThread() succeeds the thread object will be deleted in OnSLDlgThreadTerminating()
// - after waiting on the thread handle.
// If you need to have arguments, the parent should create a structure and pass it to the thread.
bool CSuckLessDialog::startThread(CSuckLessDialogThread *pSLDThread)
{
	bool result = false;

	if ((NULL == m_pSLDThread) && (NULL != pSLDThread))
	{
		pSLDThread->SetParentSLDlg(this);

		if (pSLDThread->StartThread())
		{
			m_pSLDThread = pSLDThread;

			disableClosure();

			result = true;
		}
	}

	if ((!result) && (NULL != pSLDThread))
	{
		delete pSLDThread;
		pSLDThread = NULL;
	}

	return(result);
}

LRESULT CSuckLessDialog::OnSLDlgThreadTerminating(WPARAM wparam, LPARAM lparam)
{
	LRESULT result = 0; // 0 for not handled, 1 for handled.

	int threadIdent = -1;

	if (NULL != m_pSLDThread)
	{
		threadIdent = m_pSLDThread->GetIdent();

		// Wait for thread termination to complete before deleting the object.
		// (This is actually redundant as it's done by the thread object destructor...)
		m_pSLDThread->WaitForThreadTermination();

		delete m_pSLDThread;
		m_pSLDThread = NULL;
	}

	enableClosure();

	// We must delete the old thread object before calling this so that
	// the handler code can start a new thread if it wants to.
	OnThreadTermination(threadIdent);

	return(result);
}

void CSuckLessDialog::OnDestroy(void)
{
	if (NULL != m_pSLDThread)
	{
		// If there is a thread running, wait for it to complete before destroying the window.
		// This avoids crashes where the thread posts messages to the window, though we should
		// not make it possible to close the window while the thread is active.
		m_pSLDThread->WaitForThreadTermination();

		// Thread object will be deleted by our destructor.
	}

	WINDOWPLACEMENT winPlac;
	ZeroMemory(&winPlac, sizeof(winPlac));
	winPlac.length = sizeof(winPlac);

	if (GetWindowPlacement(&winPlac) && (!m_strWinPosRegPath.empty()) && (!m_strWinPosRegValueNamePrefix.empty()))
	{
		std::basic_string<TCHAR> strW = m_strWinPosRegValueNamePrefix + ".w";
		std::basic_string<TCHAR> strH = m_strWinPosRegValueNamePrefix + ".h";

		DWORD dwW = static_cast<DWORD>(winPlac.rcNormalPosition.right - winPlac.rcNormalPosition.left);
		DWORD dwH = static_cast<DWORD>(winPlac.rcNormalPosition.bottom - winPlac.rcNormalPosition.top);

		LeetRegSetValue(HKEY_CURRENT_USER, m_strWinPosRegPath.c_str(), strW.c_str(), REG_DWORD, &dwW, sizeof(DWORD));
		LeetRegSetValue(HKEY_CURRENT_USER, m_strWinPosRegPath.c_str(), strH.c_str(), REG_DWORD, &dwH, sizeof(DWORD));
	}

	CDialog::OnDestroy();
}

void CSuckLessDialog::PostNcDestroy(void)
{
	CDialog::PostNcDestroy();

	// Recompensate our parent's closure count
	if (NULL != m_pParentPseudoModalChildArgs)
	{
		while (0 < m_nClosureCount)
		{
			m_nClosureCount--;
			m_pParentPseudoModalChildArgs->m_pParentSuckLessDlg->enableClosure();
		}
	}

	// Set m_bIsModal false so that if someone creates us as modeless now it is correct,
	// while if someone calls DoModal() again it will be set again.
	m_bIsModal = false;

	if (NULL != m_pParentPseudoModalChildArgs)
	{
		// Remember this since m_pParentPseudoModalChildArgs may be deleted after the PostMessage below.
		// (Actually, it probably cannot since our parent shares the same UI thread and we're not using SendMessage().)
		bool bDeleteThis = m_pParentPseudoModalChildArgs->m_bDelChildDlg;

		if (NULL != m_pParentPseudoModalChildArgs->m_pParentSuckLessDlg)
		{
			::PostMessage(m_pParentPseudoModalChildArgs->m_pParentSuckLessDlg->m_hWnd, WM_SLDLG_PMCHILDCLOSE,
							m_windowIdent, m_lparamToReturnToParent);
		}

		if (bDeleteThis)
		{
			// See TN017: Destroying Window Objects.
			delete this;
		}
	}
}

int CSuckLessDialog::DoModal() 
{
	m_bIsModal = true;
	
	return CDialog::DoModal();
}

void CSuckLessDialog::OnOK(void)
{
	if (0 == m_nClosureCount)
	{
		if (!UpdateData(TRUE))
		{
			TRACE0("UpdateData failed during CSuckLessDialog termination.\n");
			// the UpdateData routine will set focus to correct item
			return;
		}

		closeWindow(IDOK);
	}
}

void CSuckLessDialog::OnCancel(void)
{
	if (0 == m_nClosureCount)
	{
		closeWindow(IDCANCEL);
	}
}

// Returns whether closure is now enabled.
// Calls to disableClosure increment a counter with closure only being enabled once
// enableClosure has been called enough times to bring the counter back to zero.
// Closure is enabled by default.
bool CSuckLessDialog::enableClosure(void)
{
	bool result = false;

	if (0 < m_nClosureCount)
	{
		m_nClosureCount--;
	}

	if (0 == m_nClosureCount)
	{
		// Enable the close gadget again if there is one.
		if ((WS_CAPTION|WS_SYSMENU) & GetStyle())
		{
			CMenu *pSysMenu = GetSystemMenu(FALSE);

			if (NULL != pSysMenu)
			{
				pSysMenu->EnableMenuItem(SC_CLOSE, MF_BYCOMMAND | MF_ENABLED);
			}
		}

		result = true;
	}

	// Enable closure of our parent (if nothing else has disabled it).
	if (NULL != m_pParentPseudoModalChildArgs)
	{
		m_pParentPseudoModalChildArgs->m_pParentSuckLessDlg->enableClosure();
	}

	return(result);
}

// Calls to disableClosure increment a counter with closure only being enabled once
// enableClosure has been called enough times to bring the counter back to zero.
// Closure is enabled by default.
void CSuckLessDialog::disableClosure(void)
{
	m_nClosureCount++;

	if (1 == m_nClosureCount)
	{
		// Disable the close gadget if there is one.
		// (It shouldn't work even if pressed due to code in OnCancel())
		if ((WS_CAPTION|WS_SYSMENU) & GetStyle())
		{
			CMenu *pSysMenu = GetSystemMenu(FALSE);

			if (NULL != pSysMenu)
			{
				pSysMenu->EnableMenuItem(SC_CLOSE, MF_BYCOMMAND | MF_GRAYED);
			}
		}
	}

	// Disable closure of our parent.
	if (NULL != m_pParentPseudoModalChildArgs)
	{
		m_pParentPseudoModalChildArgs->m_pParentSuckLessDlg->disableClosure();
	}
}

bool CSuckLessDialog::isClosureEnabled(void)
{
	return(0 == m_nClosureCount);
}


// Call this to close the window.
// Calls DestroyWindow() for modeless CSuckLessDialogs and EndDialog(nModalResult) for modals.
// Pays no attention to whether closure is enabled or not.
// This function is used by CSuckLessDialog::OnOK() and OnCancel() which do respect whether closure is enabled.
void CSuckLessDialog::closeWindow(int nModalResult/* = 0*/)
{
	if (m_bIsModal)//(NULL == m_pParentPseudoModalChildArgs)
	{
		// We should be modal.
		CDialog::EndDialog(nModalResult);
	}
	else
	{
		// We're the pseudo-modal child of another CSuckLessDialog.
		// Call DestroyWindow() so that PostNcDestroy() is called, making us notify our parent and possibly
		// delete ourselves.
		DestroyWindow();
	}
}

void CSuckLessDialog::setWindowIcon(int largeIconIdent)
{
	setWindowIcon(largeIconIdent, largeIconIdent);
}

// We have a separate small icon because SetIcon(m_hIcon, FALSE) is too stupid to read the
// 16x16 image in the large icon and instead scales the 32x32 image down. At least for the main
// application icon, the large icon should still have a 16x16 image as it will be used by Explorer.
void CSuckLessDialog::setWindowIcon(int largeIconIdent, int smallIconIdent)
{
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(largeIconIdent);
	m_hIconSmall = AfxGetApp()->LoadIcon(smallIconIdent);
}

// Code taken from Erwin Tratar's ETSLayout class.
// See: http://www.codeproject.com/dialog/layoutmgr.asp
BOOL CSuckLessDialog::OnEraseBkgnd(CDC* pDC) 
{
	CRect	rcClient;
	this->GetClientRect(rcClient);

	CRgn	rgn;
	rgn.CreateRectRgnIndirect(rcClient);

	CRgn    rgnRect;
	rgnRect.CreateRectRgn(0,0,0,0);

	CRect	rcChild;
	CWnd* pWndChild = this->GetWindow(GW_CHILD);

	TCHAR szClassName[ MAX_PATH ];

	while( pWndChild )
	{
		pWndChild->GetWindowRect(rcChild);
		this->ScreenToClient( rcChild );

		rgnRect.SetRectRgn( rcChild );
	
		::GetClassName( pWndChild->GetSafeHwnd(), szClassName, MAX_PATH );
		DWORD dwStyle = pWndChild->GetStyle();

		// doesn't make sense for hidden children
		if( dwStyle & WS_VISIBLE )
		{
			if( _tcscmp(szClassName,_T("Button"))==0 && (dwStyle & BS_GROUPBOX) )
			{
				// it is a group-box, ignore completly
			}
			else if( _tcscmp(szClassName,_T("SysTabControl32"))==0 )
			{
				// ignore Tab-Control's inside rect
				static_cast<CTabCtrl*>(pWndChild)->AdjustRect(FALSE,rcChild);

				CRgn rgnContent;
				rgnContent.CreateRectRgnIndirect(rcChild);

				rgnRect.CombineRgn( &rgnRect, &rgnContent, RGN_DIFF );
				rgn.CombineRgn( &rgn, &rgnRect, RGN_DIFF );
			}
			else
			{
				rgn.CombineRgn( &rgn, &rgnRect, RGN_DIFF );
			}
		}

		pWndChild = pWndChild->GetNextWindow();
	}


	CBrush backBrush(GetSysColor(COLOR_BTNFACE));
	pDC->FillRgn( &rgn, &backBrush );

	return true;
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CSuckLessDialog::OnPaint() 
{
	if ((NULL != m_hIcon) && IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CSuckLessDialog::OnQueryDragIcon()
{
	HCURSOR result = NULL;

	if (NULL != m_hIcon)
	{
		result = CDialog::OnQueryDragIcon();
	}
	else
	{
		result = (HCURSOR) m_hIcon;
	}

	return(result);
}
