#ifndef __SUCKLESSDIALOG_H_
#define __SUCKLESSDIALOG_H_

// Version 1.5 (15/Apr/2001) Leo Davidson
// -- Added ATOP_CENTER, ABOTTOM_CENTER, ALEFT_CENTER, ARIGHT_CENTER

// Note: Frames which do not start in the top-left corner of the dialog are not supported.
// ***** Support for these shouldn't be a lot of work but I've not needed it yet.
// ***** Hint: Controls will need to be translated by the frame's offset from (0,0).

// Note: If you find that certain control types do not properly erase their backgrounds,
// ***** you probably need to add them as further exceptions CSuckLessDialog::OnEraseBkgnd()

#include "stdafx.h"
#include "resource.h"
#include <afxtempl.h>

// (If you are not using frames you can ignore this.)
// You should define a constant for each of your frames which is an int with a single bit set.
// i.e.
// #define FRAMEFLAG_SIMPLE   1<<0
// #define FRAMEFLAG_ADVANCED 1<<1
// then the frameflag for a control in both frames would be FRAMEFLAG_SIMPLE|FRAMEFLAG_ADVANCED
// Frames can be used for switching between gadget sets (which may overlap), like Simple/Advanced dialogs.
// They may also be used to "group" controls which should be made visible or invisble.
// Controls which are not visible when the dialog is created will NEVER be made visible automatically,
// regardless of frame membership. (So you can make the frame container invisible, etc.)
//
// The frames you define will have a border added to them equal to the border which appears to be around
// the initial dialog. (The gaps between controls and window will be calculated and used to define the border.)

//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================

class CFrameInfo
{
protected:
	CFrameInfo *m_pNext;
	DWORD m_frameIdent; // The identifier of the frame. Usually an int with one bit set.
	// frameIdent of 0 is a special case which is the default frame containing all controls, equal size to the window.

	int m_controlIdent;	// Ident of associated control. Normally a picture of type frame.
	RECT m_originalRect; // The original dimensions of the control.

	bool m_bHorizPercentage;
	bool m_bVertPercentage;

public:
	// Adds itself to the head of the chain pointed to by (*ppFrameInfoNext), which may be NULL.
	// frameIdent is the identifier of the frame. Usually an int with one bit set.
	// frameIdent of 0 is a special case which is the default frame containing all controls, equal size to the window.
	// pMainWindow is the window containing the frame control, not the frame control itself.
	// frameControlIdent is the ident of the associated control, typically a picture frame.
	CFrameInfo(CFrameInfo **ppFrameInfoNext, DWORD frameIdent, CWnd *pMainWindow, int frameControlIdent,
				bool bHorizPercentage, bool bVertPercentage);

	// Deletes the chain.
	~CFrameInfo(void);

	// Should be called on the root.
	// destRect filled in if returns true.
	// Based on the relative sizes of the desired and active frames, works out how large the window
	// should be to switch to the desired frame.
	bool GetNewWindowSize(DWORD desiredFrameIdent, DWORD activeFrameIdent, CWnd *pMainWindow,
									RECT *pBorderRect, POINT &destSize);

	// Should be called on the root.
	// destRect filled in and *pbXXXPercentage set if returns true.
	// Set pbXXXPercentage to NULL if you are not interested in them.
	bool GetOriginalRect(DWORD frameIdent, RECT &destRect, bool *pbHorizPercentage, bool *pbVertPercentage);

	// Should be called on the root.
	// Removes the given border from the default frame (0).
	// Expects top/bottom/left/right border values in *pBorderRect.
	void RemoveBorderFromDefaultFrame(RECT *pBorderRect);

	// Returns the ident for the frame control corresponding to the given frame ident.
	int GetFrameCtrlIdent(DWORD frameIdent);
};

//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================

class CControlInfo
{
public:

	enum // Public Flags One.
	{
		F1_DISALLOW_DISABLE						= 1<<0,	// Don't disable the control when openning a pseudo-modal child.
		F1_DISALLOW_DISABLE_WHEN_CASCADECLOSURE	= 1<<1, // Don't disable the control when cascade closure is on the parent.
		F1_IS_FRAME								= 1<<2,	// Control defines a frame. (Not currently used for anything.)
	};

	// When using AWIDTH_PERCENTAGE  exactly one of ATOP_NONE  and ABOTTOM_NONE should be used.
	// When using AHEIGHT_PERCENTAGE exactly one of ALEFT_NONE and ARIGHT_NONE  should be used.
	enum AlignType
	{
		ATAG_ENDLIST = 0,
		ATOP_NONE,
		ATOP_TOP,
		ATOP_TOP_ACTIVEFRAME,
		ATOP_TOP_NOOFFSET,
		ATOP_BOTTOM,
		ATOP_BOTTOM_ACTIVEFRAME,
		ATOP_BOTTOM_NOOFFSET,
		ATOP_PERCENTAGE,
		ATOP_CENTER,
		ABOTTOM_NONE,
		ABOTTOM_TOP,
		ABOTTOM_TOP_ACTIVEFRAME,
		ABOTTOM_TOP_NOOFFSET,
		ABOTTOM_BOTTOM,
		ABOTTOM_BOTTOM_ACTIVEFRAME,
		ABOTTOM_BOTTOM_NOOFFSET,
		ABOTTOM_PERCENTAGE,
		ABOTTOM_CENTER,
		ATOPBOTTOM_CENTER,
		ALEFT_NONE,
		ALEFT_LEFT,
		ALEFT_LEFT_ACTIVEFRAME,
		ALEFT_LEFT_NOOFFSET,
		ALEFT_RIGHT,
		ALEFT_RIGHT_ACTIVEFRAME,
		ALEFT_RIGHT_NOOFFSET,
		ALEFT_PERCENTAGE,
		ALEFT_CENTER,
		ARIGHT_NONE,
		ARIGHT_LEFT,
		ARIGHT_LEFT_ACTIVEFRAME,
		ARIGHT_LEFT_NOOFFSET,
		ARIGHT_RIGHT,
		ARIGHT_RIGHT_ACTIVEFRAME,
		ARIGHT_RIGHT_NOOFFSET,
		ARIGHT_PERCENTAGE,
		ARIGHT_CENTER,
		ALEFTRIGHT_CENTER,
		// Some aliases for common combinations of the above
		// Given to setControlRelationship, these will act as ATAG_ENDLIST and further AlignTypes will be ignored.
		ACOMMON_TOP_LEFT,
		ACOMMON_TOP_RIGHT,
		ACOMMON_BOTTOM_LEFT,
		ACOMMON_BOTTOM_RIGHT,
		ACOMMON_TOP_RESIZEWIDTH,
		ACOMMON_BOTTOM_RESIZEWIDTH,
		ACOMMON_RESIZEHEIGHT_LEFT,
		ACOMMON_RESIZEHEIGHT_RIGHT,
		ACOMMON_RESIZEHEIGHT_RESIZEWIDTH
	};

protected:

	// When a pseudo-modal child is open the private flag PF1_REENABLE records
	// which controls were disabled and should be re-enabled when the child closes.

	enum // Private Flag One.
	{
		PF1_REENABLE		= 1<<0, // Set iff the control should be enabled when pseudo-modal child closes.
		PF1_INVISIBLE		= 1<<1, // Set iff the control was invisible at creation.
	};

	enum // Constants.
	{
		PERCENTAGE_SCALE_FACTOR = 2000000 // Percentages are multiplied by this to keep them in integers.
	};

protected:

	CControlInfo * m_pNext;

	int	m_ident;
	int m_windowIdent;

	// offset values are height/width values when A*_CENTER is the AlignType.

	// Top
	AlignType	m_topAlign;
	int			m_topRelativeIdent;
	LONG		m_topOffset;
	int			m_topPercentage;
	bool		m_topIsSet;
	LONG		m_topValue;

	// Bottom
	AlignType	m_bottomAlign;
	int			m_bottomRelativeIdent;
	LONG		m_bottomOffset;
	int			m_bottomPercentage;
	bool		m_bottomIsSet;
	LONG		m_bottomValue;

	// Left
	AlignType	m_leftAlign;
	int			m_leftRelativeIdent;
	LONG		m_leftOffset;
	int			m_leftPercentage;
	bool		m_leftIsSet;
	LONG		m_leftValue;

	// Right
	AlignType	m_rightAlign;
	int			m_rightRelativeIdent;
	LONG		m_rightOffset;
	int			m_rightPercentage;
	bool		m_rightIsSet;
	LONG		m_rightValue;

	// Flags
	DWORD m_flagOne;
	DWORD m_frameMembership;
	DWORD m_privFlagOne;

protected:

	void prepareForResize(int activeFrameIdent);
	void calculateAndResize(CControlInfo *pRootControlInfo, CWnd *pMainWindow, RECT *pMainWindowRect);

	// calculate functions
	// -------------------
	// Return -1 if the value cannot be calculated.
	// ident is the ident of the control / window which you want a value for.
	// pRootControlInfo should point to the first CControlInfo in the list. (Used for finding the CControlInfo for ident.)
	// pMainWindow and pMainWindowRect should be the CWnd and RECT of the main window and should not be NULL.

	LONG calculateTop(		int ident, CControlInfo *pRootControlInfo, CWnd *pMainWindow, RECT *pMainWindowRect);
	LONG calculateBottom(	int ident, CControlInfo *pRootControlInfo, CWnd *pMainWindow, RECT *pMainWindowRect);
	LONG calculateLeft(		int ident, CControlInfo *pRootControlInfo, CWnd *pMainWindow, RECT *pMainWindowRect);
	LONG calculateRight(	int ident, CControlInfo *pRootControlInfo, CWnd *pMainWindow, RECT *pMainWindowRect);
	LONG calculateHeight(	int ident, CControlInfo *pRootControlInfo, CWnd *pMainWindow, RECT *pMainWindowRect);
	LONG calculateWidth(	int ident, CControlInfo *pRootControlInfo, CWnd *pMainWindow, RECT *pMainWindowRect);

	bool getCtrlRect(CWnd *pMainWindow, RECT *pMainWindowExcludingBorders, int ctrlIdent, RECT *outRect);

public:

	// All the public functions should be called on the root CControlInfo

	CControlInfo(CControlInfo **ppControlInfoNext, CWnd *pMainWindow, int windowIdent, int controlIdent,
					DWORD flagOne, DWORD frameMembership);
	~CControlInfo(void);

	void SetControlRelationship(int identSubject, int identRelative, va_list args);

	// Calculate the border which is around all controls in the window.
	// Returns top/bottom/left/right border values in *pOutBorderRect.
	// Invisible controls ignored.
	// Returns success.
	bool CalculateWindowBorder(CWnd *pMainWindow, RECT *pMainWindowRect, RECT *pOutBorderRect);

	void CalculateInitialControlRelationshipValues(CWnd *pMainWindow, RECT *pMainWindowExcludingBorders);

	void DoResize(CWnd *pMainWindow, RECT *pMainWindowRect, int activeFrameCtrlIdent);

	void DisableBeforePseudoModalChild(CWnd *pMainWindow, bool bCascadeClosure);
	void EnableAfterPseudoModalChild(CWnd *pMainWindow);

	// Hide all controls which are in frames in hideMask but not in frames in showMask.
	// Show all controls which are in frames in showMask.
	// A hidemask of 0xFFFFFFFF, the default, means everything is hidden but those in the showMask which is shown.
	// The default "0" frame is not applicable here:
	// A zero showMask shows nothing;
	// A zero hideMask hides nothing.
	void ChangeFrameVisibility(CWnd *pMainWindow, DWORD showMask, DWORD hideMask = 0xFFFFFFFF);

	// Disable all controls which are in frames in disableMask but not in frames in enableMask.
	// Enables all controls where are in frames in enableMask.
	// A disableMask of 0xFFFFFFFF, the default, means everything is disabled but those in enableMask which are enabled.
	// The default "0" frame is not applicable here:
	// A zero enableMask enables nothing.
	// A zero disableMask disables nothing.
	void ChangeFrameEnable(CWnd *pMainWindow, DWORD enableMask, DWORD disableMask = 0xFFFFFFFF);
};

//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================

class CSuckLessDialogThread
{
protected:
	HANDLE	m_hThread;
	CWnd *m_pParentSLDlg;

	// Override this to provide your functionality.
	virtual void mainThreadSubClass(void) = 0;

public:
	CSuckLessDialogThread(void);
	virtual ~CSuckLessDialogThread(void);

	// Must not be called while the thread is running.
	// If a parent is set it will be sent WM_SLDLG_THREADTERMINATING just before the
	// thread terminates -- use WaitForThreadTermination() after receiving this.
	void SetParentSLDlg(CWnd *pParentSLDlg);

	// Only the parent thread should call StartThread(), WaitForThreadTermination(), and IsThreadRunning().
	// Returns true if the thread started.
	bool StartThread(void);

	// Only the parent thread should call StartThread(), WaitForThreadTermination(), and IsThreadRunning()
	void WaitForThreadTermination(void);

	// Only the parent thread should call StartThread(), WaitForThreadTermination(), and IsThreadRunning()
	bool IsThreadRunning(void);

	// Returns an ident for the thread.
	// Typically, each implementation of this class within a project will use a unique
	// ident. (-1) should not be used, but any other number is fine.
	// See CSuckLessDialog::OnThreadTermination().
	virtual int GetIdent(void) = 0;

	// Don't call AfxMessageBox in threads because the message box can appear behind the window in some cases.
	// Normally, it's preferable to have a CString which is given to the thread and left empty if the
	// thread runs okay and set to an error message to display when the thread encounters an error or whatever.
	void AfxMessageBox(void){};

	// ------------------------------------------------------------------------------
	// Threads -- Not for external use. Public only for the thread calling mechanism.
	// ------------------------------------------------------------------------------

	void PrivateMainThread(void);

	static unsigned __stdcall PrivateMainThread(void *pMe)
	{
		((CSuckLessDialogThread *)pMe)->PrivateMainThread();

		return(0);
	}
};

//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================

// Child dialogs may use a subclass of this to get init data from their parents
// and to send data back after they've closed.
/*
class CSuckLessDialogArgs
{
	TCHAR *m_sldaTypeName;

public:
	CSuckLessDialogArgs(const TCHAR *sldaTypeName)
	{
		m_sldaTypeName = NULL;

		if (NULL != sldaTypeName)
		{
			m_sldaTypeName = new TCHAR[strlen(sldaTypeName) + 1];
			strcpy(m_sldaTypeName, sldaTypeName);
		}
	}

	virtual ~CSuckLessDialogArgs(void)
	{
		delete [] m_sldaTypeName;
		m_sldaTypeName = NULL;
	}

	bool IsMyTypeName(const TCHAR *sldaTypeName)
	{
		bool result = false;

		if ((NULL != sldaTypeName) && (NULL != m_sldaTypeName) && (0 == strcmp(sldaTypeName, m_sldaTypeName)))
		{
			result = true;
		}

		return(result);
	}
};
*/
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================

// Override OnInitDialog and add items to the control
// array, then set m_minX m_minY and m_controlAndFrameInfoSetup

class CSuckLessDialog : public CDialog
{
public:

	// If hWndParentSuckLessDlg is given it will be sent WM_SLDLG_PMCHILDCLOSE with pPMCArgs in the lparam
	// when the window closes.
	CSuckLessDialog(UINT nIDTemplate, CWnd* pParentWnd);
	~CSuckLessDialog(void);

	//{{AFX_VIRTUAL(CNetpaswdDlg)
	public:
	virtual int DoModal();
	protected:
	virtual void PostNcDestroy();
	//}}AFX_VIRTUAL

private:

	class CPMCArgs
	{
	public:
		CSuckLessDialog *m_pParentSuckLessDlg;
//		CSuckLessDialogArgs *m_pArgsWithParent;

		bool m_bHide;
		bool m_bDelChildDlg;

		CPMCArgs(CSuckLessDialog *pParentSuckLessDlg, bool bHide, bool bDelChildDlg)
		{
			m_pParentSuckLessDlg = pParentSuckLessDlg;
			m_bHide = bHide;
			m_bDelChildDlg = bDelChildDlg;
		}

		~CPMCArgs(void)
		{
		}
	};

	CStatusBar	m_statusBar;

	int m_windowIdent;
	LPARAM m_lparamToReturnToParent;

	bool m_bStayHiddenAfterPseudoModalChild;
	bool m_bCascadeClosure;
	bool m_bCascadeClosurePermanentlyDisabled;

	unsigned int m_nClosureCount;

	bool m_bHiddenByParentWhileMaximized;

	bool m_controlAndFrameInfoSetup;
	// You should always add all your controls, and we don't really support dialogs with zero controls.
	// You need not define frames if you don't need any. The default frame (0x00000000) is always defined
	// when controlAndFrameInfoIsSetup() is called.
	CControlInfo *m_pControlInfo; // Should not be NULL if m_controlAndFrameInfoSetup is set.
	CFrameInfo *m_pFrameInfo; // Should not be NULL if m_controlAndFrameInfoSetup is set.
	int m_activeFrameIdent; // Defaults to 0 which is the default window-sized frame with all controls visible.

	RECT m_borderRect;	// The top/bottom/left/right border values.

	long m_minX;
	long m_minY;
	long m_maxX;
	long m_maxY;

	std::basic_string<TCHAR> m_strWinPosRegPath;
	std::basic_string<TCHAR> m_strWinPosRegValueNamePrefix;

	bool m_bStatusBar;

	bool m_bIsModal;

protected:
	HICON m_hIcon;
private:
	HICON m_hIconSmall;

	// If non-null we have a child open.
	// This is ours to delete when the child closes.
	CPMCArgs *m_pChildPseudoModalChildArgs;

	// If non-null we are the child of a parent.
	// This is *NOT* ours to delete.
	CPMCArgs *m_pParentPseudoModalChildArgs;

	CSuckLessDialogThread *m_pSLDThread;

	void cleanupAfterPMC(void);

protected:

	// By default, OnCancel() will do nothing while pseudo-modal children are open.
	// If you set cascade closure, OnCancel() will close the window, causing the children to close as well.
	// Remember not to disable your cancel gadget as this leaves the close gadget enabled but it will do nothing.
	// Will do nothing if called while a pseudo-modal child is open.
	// (Can be called before and after controlAndFrameInfoIsSetup())
	void setCascadeClosure(bool bVal = true);

	// To be used in situations where enabling cascade closure would screw up the application,
	// for example when a pseudo-modal child or thread is launched on window closure.
	// Will do nothing if called while a pseudo-modal child is open.
	// (Can be called before and after controlAndFrameInfoIsSetup())
	void setCascadeClosurePermanentlyDisabled(void);

	// Will do nothing if called while a pseudo-modal child is open.
	// (Can be called before and after controlAndFrameInfoIsSetup())
	void setStayHiddenAfterPseudoModalChild(bool bStayHiddenAfterPseudoModalChild);

	// Calls to disableClosure increment a counter with closure only being enabled once
	// enableClosure has been called enough times to bring the counter back to zero.

	// Returns whether closure is now enabled.
	bool enableClosure(void);

	void disableClosure(void);

	// Returns whether closure is now enabled.
	bool isClosureEnabled(void);

	// Call this to close the window.
	// Calls DestroyWindow() for modeless CSuckLessDialogs and EndDialog(nModalResult) for modals.
	// Pays no attention to whether closure is enabled or not.
	// This function is used by CSuckLessDialog::OnOK() and OnCancel() which do respect whether closure is enabled.
	void closeWindow(int nModalResult = 0);

	// Call these before controlAndFrameInfoIsSetup()...
	void addControlInfo(int ident, DWORD flagOne = 0, DWORD frameMembership = 0);
	void setControlRelationship(int identSubject, int identRelative, ...);
	void addFrameInfo(DWORD frameIdent, int frameControlIdent, bool bHorizPercentage, bool bVertPercentage);
	void noStatusBar(void);
	void setWindowIcon(int largeIconIdent);
	void setWindowIcon(int largeIconIdent, int smallIconIdent);

	// Call this once the control info is set up and after CSuckLessDialog::OnInitDialog()
	void controlAndFrameInfoIsSetup(void);

	// ******************************************************************** //
	// Everything below should be called after controlAndFrameInfoIsSetup() //
	// ******************************************************************** //

	void setStatusBarText(const TCHAR *statusMsg);

	// Calls CenterWindow()
	void setRememberWindowPosition(const TCHAR *szWinPosRegPath, const TCHAR *szWinPosValueNamePrefix);

	// Returns success.
	// If setMin is true then MinX / MinY values will be set to the frame's initial size.
	// If setMax is true then MaxX / MaxY values will be set to the frame's initial size.
	bool switchToFrame(DWORD frameIdent, bool bSetMinX = true, bool bSetMinY = true,
										bool bSetMaxX = false, bool bSetMaxY = false);

	// If a frame is only used by enableFrameControls() it need not be added with
	// addFrameInfo() or associated with a control.
	// Does nothing while a pseudo-modal child is open.
	void enableFrameControls(DWORD frameIdentMask, bool bEnable);

	void setMinXToCurrent(long numerator = 1, long denominator = 1);
	void setMinYToCurrent(long numerator = 1, long denominator = 1);
	void setMaxXToCurrent(long numerator = 1, long denominator = 1);
	void setMaxYToCurrent(long numerator = 1, long denominator = 1);
	void setMinX(long minX); // Give -1 to use the CDialog default.
	void setMinY(long minY); // Give -1 to use the CDialog default.
	void setMaxX(long maxX); // Give -1 to use the CDialog default.
	void setMaxY(long maxY); // Give -1 to use the CDialog default.

	// Creates the given dialog.
	// Should not be called while another pseudo-modal child is already open.
	// Will disable all controls (except those excluded with F1_DISALLOW_DISABLE) while the child is open.
	// If bHide is true, will hide the dialog while the child is open.
	// The dialog will be deleted once the child closes if bDelDlg is true. If bDelDlg is true and
	// dialog creation fails, this function will delete the dialog for you.
	//
	// If you want the new dialog to appear on the taskbar, as is often the case if you hide the original,
	// make sure it has no owner (pSLDlg->SetOwner(NULL);) and make sure it has the WS_EX_APPWINDOW extended
	// style:
	//
	// IDD_DIALOG1 DIALOG DISCARDABLE  0, 0, 114, 41
	// STYLE WS_MINIMIZEBOX | WS_MAXIMIZEBOX | WS_POPUP | WS_VISIBLE | WS_CAPTION | 
	//    WS_SYSMENU | WS_THICKFRAME
	// EXSTYLE WS_EX_APPWINDOW
	// CAPTION "Dialog"
	// FONT 8, "MS Sans Serif"
	// BEGIN
	//    DEFPUSHBUTTON   "OK",IDOK,57,7,50,14
	//    PUSHBUTTON      "Cancel",IDCANCEL,57,23,50,14
	//    LTEXT           "I'm a little pea.",IDC_STATIC,7,7,46,8
	// END
	bool openPseudoModalChild(CSuckLessDialog *pSLDlg, int sldIDD, bool bHide = false, bool bDelDlg = true);

	// Zero is returned by default.
	void setLParamToReturnToParent(LPARAM lparam);

	// Override this to do stuff before your pseudo-modal child opens.
	void virtual OnPseudoModalChildOpen(int sldIDD)
	{
		// Default action does nothing.
	}

	// Override this to do stuff when your pseudo-modal child closes.
	// Do not use OnPseudoModalChildClose() for cleanup since it may be possible for us
	// to close before the child does, meaning the function doesn't get called.
	void virtual OnPseudoModalChildClose(int sldIDD, LPARAM lparam)
	{
		// Default action does nothing.
	}

	// You should call this function for all CSuckLessDialogs which are the pseudo-modal children of another CSLDlg.
	// Called for you by openPseudoModalChild()
	// Untested: If dialog isn't deleted, you should be able to reuse it and set a different parent, provided
	// you don't do this while it is still the child of another window, which could be difficult in some situations.
	// If you call this the dialog will do the following in PostNcDestroy as it closes:
	// * Send WM_SLDLG_PMCHILDCLOSE to m_pParentPseudoModalChildArgs->m_pParentSuckLessDlg.
	// * Delete itself if m_pParentPseudoModalChildArgs->m_bDelChildDlg. (Otherwise must be deleted manually.)
	void setParentPMCArgs(CPMCArgs *pPMCArgs);

	// Returns boolean success.
	// OnSLDlgThreadTerminating() will be called as the thread terminates.
	// Unlike pseudo-modal child dialogs, SuckLessDialogThread objects do not auto-delete,
	// but they will be deleted by the SuckLessDialog framework:
	// * If startThread() fails the thread object will be deleted immediately.
	// * If startThread() succeeds the thread object will be deleted in OnSLDlgThreadTerminating()
	// - after waiting on the thread handle.
	// If you need to have arguments, the parent should create a structure and pass it to the thread.
	bool startThread(CSuckLessDialogThread *pSLDThread);

	// Override this to do stuff when your thread closes.
	// wparam should be the terminated thread object's ident or (-1) if it could not be found.
	// Do not use OnThreadTermination() for cleanup since it may be possible for us
	// to close before the thread does, meaning the function doesn't get called.
	void virtual OnThreadTermination(int ident)
	{
		// Default action does nothing.
	}

	// Called at the end of resizing.
	void virtual OnSizeHookEnd(void)
	{
		// Default action does nothing.
	}

	// Generated message map functions
	//{{AFX_MSG(CSuckLessDialog)
	afx_msg LRESULT OnSLDlgPMChildCloseEvent(WPARAM wparam, LPARAM lparam);
	afx_msg LRESULT OnSLDlgThreadTerminating(WPARAM wparam, LPARAM lparam);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI);
	virtual void OnOK();
	virtual void OnCancel();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnShowWindow(BOOL bShow, UINT nStatus);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	afx_msg void OnDestroy();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================
//================================================================================

#endif
