// SuperNudelist.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "SuperNudelist.h"
#include <libxml/xmlerror.h>
#include "SuperNudelistDlg.h"
#include "AboutDlg.h"
#include "SNConfig.h"
#include "CCFWrapper.h"
#include "ListingDlg.h"
#include "CCFDLLDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSuperNudelistApp

BEGIN_MESSAGE_MAP(CSuperNudelistApp, CWinApp)
	//{{AFX_MSG_MAP(CSuperNudelistApp)
	//}}AFX_MSG
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSuperNudelistApp construction

CSuperNudelistApp::CSuperNudelistApp()
{
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CSuperNudelistApp object

CSuperNudelistApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CSuperNudelistApp initialization

BOOL CSuperNudelistApp::InitInstance()
{
	// Standard initialization

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	// One-off init.
	InitAlphabetPanelLetterGroupTable();
	CProntoString::InitStatic();

	// Global, one-off LibXml initialisation.
	g_pXMLErrorStringList = new CStringList();

	xmlMemSetup(free, (xmlMallocFunc)malloc, (xmlReallocFunc)realloc, strdup);
	xmlInitParser();
	xmlKeepBlanksDefault(0);
	xmlSetGenericErrorFunc(stderr, NudelXmlErrorFunc);

	InitializeCriticalSection(&g_CSGlobals);
	g_pProgressDlg = NULL;
	g_pStringList = NULL;
	g_bCompiling = false;
	g_pProntoFont = getProntoCFont();

	DWORD dwWelcomeShown = 0;

	bool bReadReg = LeetRegQueryDWORDValue(HKEY_CURRENT_USER, SN_REG_PATH_ROOT, SN_REG_VALUE_WELCOMESHOWN, &dwWelcomeShown);

	if ((!bReadReg) || (0 == dwWelcomeShown))
	{
		CAboutDlg *pAboutDlg = new CAboutDlg(true, NULL);
		pAboutDlg->DoModal();
		delete pAboutDlg;
	}

	bool bStartupSuccess = false;

	CSNConfig *pConfig = new CSNConfig();
	CCCFWrapper *pCCFWrapper = new CCCFWrapper(pConfig);

	set<CSNDisc *> *pSetDiscsWithUnresolvedDevices = NULL;

	// Scope out tError.
	{
		CTypedError tError;
		CStringList slErrors;
		bool bCCFDLLInitError = false;

		CPDTStartUp *pThread = new CPDTStartUp(pConfig, pCCFWrapper, &tError, &slErrors, &bCCFDLLInitError);

		// Delayed popup doesn't work anyway because of DoModal.
		CProgressDlg *pProgDlg = new CProgressDlg(PROGD_FRAME_NOPROG, true, pThread, NULL);

		if (0 == pProgDlg->DoModal())
		{
			if (!slErrors.IsEmpty())
			{
				CListingDlg *pSLDlg = new CListingDlg(SUPERNUDELIST_ERROR_TITLE,
														CTypedError::SafeGetFullErrorTextTempBuffer(&tError),
														&slErrors, false, NULL);
				pSLDlg->DoModal();

				delete pSLDlg;
			}
			else if (bCCFDLLInitError)
			{
				CCCFDLLDlg *pSLDlg = new CCCFDLLDlg(CTypedError::SafeGetFullErrorTextTempBuffer(&tError), NULL);

				pSLDlg->DoModal();

				delete pSLDlg;
			}
			else
			{
				::MessageBox(NULL, CTypedError::SafeGetFullErrorTextTempBuffer(&tError),
									SUPERNUDELIST_ERROR_TITLE, MB_OK | MB_ICONSTOP);
			}
		}
		else
		{
			bStartupSuccess = true;
		}

		delete pProgDlg;
	}

	// Stop showing the welcome screen after one successful startup.
	if ((0 == dwWelcomeShown) && (bStartupSuccess))
	{
		dwWelcomeShown = 1;
		LeetRegSetValue(HKEY_CURRENT_USER, SN_REG_PATH_ROOT, SN_REG_VALUE_WELCOMESHOWN, REG_DWORD, &dwWelcomeShown, sizeof(DWORD));
	}

	CSuperNudelistDlg *pDlg = new CSuperNudelistDlg(pConfig, pCCFWrapper, NULL);
	m_pMainWnd = pDlg; // Note: See CSuperNudelistDlg::OnNcDestroy
	pDlg->DoModal();
	delete pDlg;

	delete pCCFWrapper;
	// Must not delete pConfig before deleting pCCFWrapper.
	delete pConfig;

	freeProntoCFont(g_pProntoFont);
	g_pProntoFont = NULL;

	DeleteCriticalSection(&g_CSGlobals);

	// Global, one-off LibXml cleanup.
	xmlCleanupParser();

	delete g_pXMLErrorStringList;
	g_pXMLErrorStringList = NULL;

	// Since the dialog has been closed, return FALSE so that we exit the
	//  application, rather than start the application's message pump.
	return FALSE;
}

CFont *CSuperNudelistApp::getProntoCFont(void)
{
	CFont *pResult = NULL;

	CDC screenDC;
	const char *faceName = "Pronto";

	if ((screenDC.CreateCompatibleDC(NULL))						\
	&&  (checkForProntoFont(screenDC.m_hAttribDC, faceName))	)
	{
		CFont *pScreenFont = (CFont *)screenDC.SelectStockObject(DEFAULT_GUI_FONT);

		if (NULL != pScreenFont)
		{
			LOGFONT logFontScreen;

			if (pScreenFont->GetLogFont(&logFontScreen))
			{
				LOGFONT logFont;
				ZeroMemory(&logFont, sizeof(LOGFONT));
				logFont.lfHeight = logFontScreen.lfHeight;
				// Must use DEFAULT_CHARSET.
				// SYMBOL_CHARSET breaks the tree control if you
				// populate after setting the font.
				logFont.lfCharSet = DEFAULT_CHARSET;
				strcpy(logFont.lfFaceName, faceName);

				pResult = new CFont();

				if (! pResult->CreateFontIndirect(&logFont) )
				{
					delete pResult;
					pResult = NULL;
				}
			}
		}
	}

	return(pResult);
}

void CSuperNudelistApp::freeProntoCFont(CFont *pFont)
{
	if (NULL != pFont)
	{
		pFont->DeleteObject();
		delete pFont;
	}
}

int CALLBACK enumFontProc(CONST LOGFONT *pLogFont,
						   CONST TEXTMETRIC *pTextMetric,
						   DWORD fontType,
						   LPARAM lparam)
{
	// Pronto font found.
	(*((bool *)lparam)) = true;

	// Stop enumerating fonts.
	return(0);
}

bool CSuperNudelistApp::checkForProntoFont(HDC hdc, const char *faceName)
{
	bool bResult = false;

	LOGFONT logFont;
	ZeroMemory(&logFont, sizeof(logFont));
	logFont.lfCharSet = DEFAULT_CHARSET;
	strcpy(logFont.lfFaceName, faceName);

	EnumFontFamiliesEx(hdc, &logFont, enumFontProc, (LPARAM)&bResult, 0);

	if (!bResult)
	{
		::MessageBox(NULL, "Pronto font (from ProntoEdit) not installed; Pronto symbols will appear incorrectly.",
							SUPERNUDELIST_WARNING_TITLE, MB_OK | MB_ICONEXCLAMATION);
	}

	return(bResult);
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

CPDTStartUp::CPDTStartUp(CSNConfig *pConfig, CCCFWrapper *pCCFWrapper, CTypedError *pTError,
						 CStringList *pStringListErrors, bool *pbCCFDLLInitError)
	: CProgressDlgThread()
{
	m_pConfig = pConfig;
	m_pCCFWrapper = pCCFWrapper;
	m_pTError = pTError;
	m_pStringListErrors = pStringListErrors;
	m_pbCCFDLLInitError = pbCCFDLLInitError;
}

CPDTStartUp::~CPDTStartUp(void)
{
}

// Returns an ident for the thread.
// Typically, each implementation of this class within a project will use a unique
// ident. (-1) should not be used, but any other number is fine.
// See CSuckLessDialog::OnThreadTermination().
int CPDTStartUp::GetIdent(void)
{
	return(SLDTHREADID_STARTUP);
}

// Overriden to provide functionality.
void CPDTStartUp::mainThreadSubClass(void)
{
	bool bResult = true;

	CTypedError::SafeReset(m_pTError);
	m_pStringListErrors->RemoveAll();
	(*m_pbCCFDLLInitError) = false;
	
	// If an error occurs don't report it, the user is trying to
	// close the app and will be told about any problems next time
	// they cause decompilation.

	SetProgressTitle("Super-Nudelist is starting...");

	SetProgressHeading("Loading configuration");
	SetProgressMessage("");

	if (!m_pConfig->Load(m_pProgressDlg, m_pTError, m_pStringListErrors))
	{
		bResult = false;
	}

	SetProgressHeading("Loading CCFDll");
	SetProgressMessage("");

	if (!m_pCCFWrapper->Init(m_pProgressDlg, m_pTError))
	{
		(*m_pbCCFDLLInitError) = true;
		bResult = false;
	}

	// Set return from DoModal.
	this->SetProgressSuccess(bResult);
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
