// SuperNudelistDlg.cpp : implementation file
//

#include "stdafx.h"
#include <io.h>
#include <shlobj.h>
#include "SuperNudelistDlg.h"
#include "SNConfig.h"
#include "CCFWrapper.h"
#include "AboutDlg.h"
#include "ProgressDlg.h"
#include "ListingDlg.h"
#include "DevicesDlg.h"
#include "ListsDlg.h"
#include "DevResDlg.h"
#include "CSFAbstractTree.h"
#include "SNLibrary.h"
#include "CSFSection.h"
#include "SNBuiltInIcons.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSuperNudelistDlg dialog

CSuperNudelistDlg::CSuperNudelistDlg(CSNConfig *pConfig, CCCFWrapper *pCCFWrapper, CWnd* pParent)
	: CSuckLessDialog(CSuperNudelistDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CSuperNudelistDlg)
	m_editInputCCF = _T("");
	m_editOutputCCF = _T("");
	m_checkCompressImages = FALSE;
	//}}AFX_DATA_INIT

	m_pConfig = pConfig;
	m_pCCFWrapper = pCCFWrapper;
	m_pSetDiscsWithUnresolvedDevices = NULL;
	m_bUpdatingPanelCap = false;
}

CSuperNudelistDlg::~CSuperNudelistDlg()
{
	delete m_pSetDiscsWithUnresolvedDevices;
	m_pSetDiscsWithUnresolvedDevices = NULL;
}



void CSuperNudelistDlg::DoDataExchange(CDataExchange* pDX)
{
	CSuckLessDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSuperNudelistDlg)
	DDX_Control(pDX, IDC_CHECK_PANCAP, m_ctrlCheckPanCap);
	DDX_Control(pDX, IDC_SPIN_PANCAP, m_ctrlSpinPanCap);
	DDX_Control(pDX, IDC_EDIT_PANCAP, m_ctrlEditPanCap);
	DDX_Control(pDX, IDC_BUTTON_EXECUTE, m_ctrlButtonExecute);
	DDX_Control(pDX, IDC_CHECK_COMPRESSIMAGES, m_ctrlCheckCompressedImages);
	DDX_Text(pDX, IDC_EDIT_INPUT, m_editInputCCF);
	DDX_Text(pDX, IDC_EDIT_OUTPUT, m_editOutputCCF);
	DDX_Check(pDX, IDC_CHECK_COMPRESSIMAGES, m_checkCompressImages);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CSuperNudelistDlg, CSuckLessDialog)
	//{{AFX_MSG_MAP(CSuperNudelistDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_BUTTON_INPUT, OnButtonInput)
	ON_BN_CLICKED(IDC_BUTTON_OUTPUT, OnButtonOutput)
	ON_BN_CLICKED(IDC_BUTTON_EXECUTE, OnButtonExecute)
	ON_BN_CLICKED(IDC_BUTTON_ABOUT, OnButtonAbout)
	ON_WM_NCDESTROY()
	ON_BN_CLICKED(IDC_BUTTON_DEVICES, OnButtonDevices)
	ON_BN_CLICKED(IDC_BUTTON_LISTS, OnButtonLists)
	ON_BN_CLICKED(IDC_CHECK_PANCAP, OnCheckPancap)
	ON_WM_VSCROLL()
	ON_WM_CLOSE()
	ON_EN_CHANGE(IDC_EDIT_PANCAP, OnChangeEditPancap)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSuperNudelistDlg message handlers

BOOL CSuperNudelistDlg::OnInitDialog()
{
	CSuckLessDialog::OnInitDialog();

	UpdateData(TRUE);

	this->setWindowIcon(IDR_MAINFRAME);

	CTypedError tError;

	// -- Version String --

	const char *verString = m_pConfig->GetExeVersionString(&tError);

	CString winText;
	this->GetWindowText(winText);

	winText += " ";
	
	if (NULL != verString)
	{
		winText += verString;
	}
	else
	{
		winText += CTypedError::SafeGetFullErrorTextTempBuffer(&tError);
	}

	winText += " BETA VERSION";

	winText += " (";
	winText += m_pCCFWrapper->GetCCFDllVersionString();
	winText += ")";

	this->SetWindowText(winText);

	// Reset typed error.
	CTypedError::SafeReset(&tError);

	// -- Initial contents --

	const char *tempString;

	if (NULL != (tempString = m_pConfig->GetInputCCFPath()))
	{
		m_editInputCCF = tempString;
	}

	if (NULL != (tempString = m_pConfig->GetOutputCCFPath()))
	{
		m_editOutputCCF = tempString;
	}

	m_checkCompressImages = m_pConfig->GetCompressImages() ? TRUE : FALSE;


	m_ctrlSpinPanCap.SetRange32(1, CSNConfig::MAX_PANELCAP);

	int nPanCap = m_pConfig->GetPanelCap();

	if (0 >= nPanCap)
	{
		m_ctrlCheckPanCap.SetCheck(0);
		m_ctrlEditPanCap.EnableWindow(FALSE);
		m_ctrlSpinPanCap.EnableWindow(FALSE);
		updatePanelCap(300, true, true);
	}
	else
	{
		m_ctrlCheckPanCap.SetCheck(1);
		m_ctrlEditPanCap.EnableWindow(TRUE);
		m_ctrlSpinPanCap.EnableWindow(TRUE);

		updatePanelCap(nPanCap, true, true);
	}

	// -- Control setup --

	addControlInfo(IDC_STATIC_CCF_FILES);

	addControlInfo(IDC_STATIC_INPUT);
	addControlInfo(IDC_EDIT_INPUT);
	addControlInfo(IDC_BUTTON_INPUT);

	addControlInfo(IDC_STATIC_OUTPUT);
	addControlInfo(IDC_EDIT_OUTPUT);
	addControlInfo(IDC_BUTTON_OUTPUT);
	addControlInfo(IDC_CHECK_PANCAP);
	addControlInfo(IDC_EDIT_PANCAP);
	addControlInfo(IDC_SPIN_PANCAP);
	addControlInfo(IDC_STATIC_PANCAP);
	addControlInfo(IDC_CHECK_COMPRESSIMAGES);

	addControlInfo(IDC_BUTTON_EXECUTE);

	addControlInfo(IDC_BUTTON_HELP);
	addControlInfo(IDC_BUTTON_ABOUT);
	addControlInfo(IDC_BUTTON_DEVICES);
	addControlInfo(IDC_BUTTON_LISTS);

	setControlRelationship(IDC_STATIC_CCF_FILES,	IDD,					CControlInfo::ACOMMON_TOP_RESIZEWIDTH);

	setControlRelationship(IDC_STATIC_INPUT,		IDC_STATIC_CCF_FILES,	CControlInfo::ACOMMON_TOP_LEFT);
	setControlRelationship(IDC_EDIT_INPUT,			IDC_STATIC_CCF_FILES,	CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_BUTTON_INPUT,		IDC_STATIC_CCF_FILES,	CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_STATIC_OUTPUT,		IDC_STATIC_CCF_FILES,	CControlInfo::ACOMMON_TOP_LEFT);
	setControlRelationship(IDC_EDIT_OUTPUT,			IDC_STATIC_CCF_FILES,	CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_BUTTON_OUTPUT,		IDC_STATIC_CCF_FILES,	CControlInfo::ACOMMON_TOP_RIGHT);
	setControlRelationship(IDC_CHECK_PANCAP,		IDC_STATIC_CCF_FILES,	CControlInfo::ACOMMON_TOP_LEFT);
	setControlRelationship(IDC_EDIT_PANCAP,			IDC_STATIC_CCF_FILES,	CControlInfo::ACOMMON_TOP_LEFT);
	setControlRelationship(IDC_SPIN_PANCAP,			IDC_STATIC_CCF_FILES,	CControlInfo::ACOMMON_TOP_LEFT);
	setControlRelationship(IDC_STATIC_PANCAP,		IDC_STATIC_CCF_FILES,	CControlInfo::ACOMMON_TOP_LEFT);
	setControlRelationship(IDC_CHECK_COMPRESSIMAGES,IDC_STATIC_CCF_FILES,	CControlInfo::ACOMMON_TOP_LEFT);

	setControlRelationship(IDC_BUTTON_EXECUTE,		IDC_STATIC_CCF_FILES,	CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_BUTTON_HELP,			IDD,					CControlInfo::ACOMMON_BOTTOM_LEFT);
	setControlRelationship(IDC_BUTTON_ABOUT,		IDD,					CControlInfo::ACOMMON_BOTTOM_LEFT);

	setControlRelationship(IDC_BUTTON_DEVICES,		IDD,					CControlInfo::ACOMMON_BOTTOM_RIGHT);
	setControlRelationship(IDC_BUTTON_LISTS,		IDD,					CControlInfo::ACOMMON_BOTTOM_RIGHT);

	controlAndFrameInfoIsSetup();

	setMinXToCurrent();
	setMinYToCurrent();
	setMaxYToCurrent();

	// Never allow cascade closure because we rely on there being no pseudo-modal child window when we
	// launch the clean-up progress dialog/thread.
	setCascadeClosurePermanentlyDisabled();

	UpdateData(FALSE);

	m_ctrlButtonExecute.SetFocus();

	this->setRememberWindowPosition(SN_REG_PATH_WINDOWPOS, SN_REG_VALUE_WINDOWPOS_MAIN);

	return FALSE;  // return TRUE  unless you set the focus to a control
}

void CSuperNudelistDlg::OnCheckPancap() 
{
	BOOL bEnable = (1 == m_ctrlCheckPanCap.GetCheck() ? TRUE : FALSE);

	m_ctrlEditPanCap.EnableWindow(bEnable);
	m_ctrlSpinPanCap.EnableWindow(bEnable);
}

void CSuperNudelistDlg::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	if (m_ctrlSpinPanCap.GetDlgCtrlID() == pScrollBar->GetDlgCtrlID())
	{
		updatePanelCap(nPos, true, false);
	}

	CSuckLessDialog::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CSuperNudelistDlg::updatePanelCap(int nPanCap, bool bUpdateEdit, bool bUpdateSpin)
{
	if (!m_bUpdatingPanelCap)
	{
		m_bUpdatingPanelCap = true;

		if (bUpdateEdit)
		{
			CString csLabel;

			csLabel.Format("%d", nPanCap);
			m_ctrlEditPanCap.SetWindowText(csLabel);
		}

		if (bUpdateSpin)
		{
			m_ctrlSpinPanCap.SetPos(nPanCap);
		}

//		m_pConfig->SetPanelCap(nPanCap);

		m_bUpdatingPanelCap = false;
	}
}

void CSuperNudelistDlg::OnChangeEditPancap() 
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CSuckLessDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.
	
	CString csLabel;
	m_ctrlEditPanCap.GetWindowText(csLabel);

	bool bUpdateEdit = false;

	int val = 1;

	if (! csLabel.IsEmpty() )
	{
		val = atoi(LPCTSTR(csLabel));

		if (CSNConfig::MAX_PANELCAP < val)
		{
			val = CSNConfig::MAX_PANELCAP;
			bUpdateEdit = true;
		}
		else if (0 >= val)
		{
			val = 1;
			bUpdateEdit = true;
		}
	}

	updatePanelCap(val, bUpdateEdit, true);
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CSuperNudelistDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CSuckLessDialog::OnPaint();
	}
}

HCURSOR CSuperNudelistDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

/*
void CSuperNudelistDlg::OnButtonLoadConfig() 
{
	LPMALLOC pShMalloc;

	if (NOERROR == SHGetMalloc(&pShMalloc))
	{
		char szPath[_MAX_PATH + 1];

		BROWSEINFO bi;
		bi.hwndOwner = this->GetSafeHwnd();
		bi.pidlRoot = NULL;
		bi.pszDisplayName = szPath;
		bi.lpszTitle = "Select existing directory with config:";
		bi.ulFlags = BIF_RETURNONLYFSDIRS;
		bi.lpfn = NULL;
		bi.lParam = 0;
		bi.iImage = 0;

		LPITEMIDLIST pItemIdList = SHBrowseForFolder(&bi);

		if (NULL != pItemIdList)
		{
			// Reuse szPath buffer. We didn't want that shit anyway.
			if (SHGetPathFromIDList(pItemIdList, szPath))
			{
				m_editConfig = szPath;
				UpdateData(FALSE);
			}

			// Free the data returned by the shell.
			pShMalloc->Free(pItemIdList);
		}
	}
}
*/
void CSuperNudelistDlg::OnButtonInput() 
{
	UpdateData(TRUE);

	CFileDialog cfd(TRUE, ".ccf", m_editInputCCF.IsEmpty() ? NULL : LPCTSTR(m_editInputCCF),
					OFN_HIDEREADONLY|OFN_FILEMUSTEXIST|OFN_NOCHANGEDIR|OFN_PATHMUSTEXIST,
					"Pronto Config Files (*.ccf)|*.ccf||", this);

	char *parentPath = NULL;

	cfd.m_ofn.lpstrTitle = "Input CCF File";

	if (!m_editInputCCF.IsEmpty())
	{
		if (NULL != (parentPath = GetParentPath(m_editInputCCF)))
		{
			cfd.m_ofn.lpstrInitialDir = parentPath;
		}
	}

	if (IDOK == cfd.DoModal())
	{
		m_editInputCCF = cfd.GetPathName();
		UpdateData(FALSE);
	}

	delete [] parentPath;
	parentPath = NULL;
}

void CSuperNudelistDlg::OnButtonOutput() 
{
	UpdateData(TRUE);

	CFileDialog cfd(FALSE, ".ccf", m_editOutputCCF.IsEmpty() ? NULL : LPCTSTR(m_editOutputCCF),
					OFN_HIDEREADONLY|OFN_NOCHANGEDIR|OFN_NOREADONLYRETURN|OFN_OVERWRITEPROMPT,
					"Pronto Config Files (*.ccf)|*.ccf||", this);

	char *parentPath = NULL;

	cfd.m_ofn.lpstrTitle = "Output CCF File";

	if (!m_editOutputCCF.IsEmpty())
	{
		if (NULL != (parentPath = GetParentPath(m_editOutputCCF)))
		{
			cfd.m_ofn.lpstrInitialDir = parentPath;
		}
	}

	if (IDOK == cfd.DoModal())
	{
		m_editOutputCCF = cfd.GetPathName();
		UpdateData(FALSE);
	}

	delete [] parentPath;
	parentPath = NULL;
}

void CSuperNudelistDlg::updateConfig(void)
{
	UpdateData(TRUE);

	m_pConfig->SetInputCCFPath(m_editInputCCF);
	m_pConfig->SetOutputCCFPath(m_editOutputCCF);
	m_pConfig->SetCompressImages(m_checkCompressImages ? true : false);

	int nPanCap = 0;

	if (1 == m_ctrlCheckPanCap.GetCheck())
	{
		CString csPanCap;
		m_ctrlEditPanCap.GetWindowText(csPanCap);
		nPanCap = atoi(csPanCap);
	}

	m_pConfig->SetPanelCap(nPanCap);
}

void CSuperNudelistDlg::OnButtonAbout() 
{
	CAboutDlg *pSLDlg = new CAboutDlg(false, this);
	this->openPseudoModalChild(pSLDlg, CAboutDlg::IDD, false, true);
}

void CSuperNudelistDlg::OnButtonDevices() 
{
	CTypedError::SafeReset(&m_threadTError);

	updateConfig();

	if (! m_pConfig->IsReady(&m_threadTError, false) )
	{
		MessageBox(CTypedError::SafeGetFullErrorTextTempBuffer(&m_threadTError), SUPERNUDELIST_ERROR_TITLE, MB_ICONEXCLAMATION);
	}
	else
	{
		m_bThreadResult = true;
		CTypedError::SafeReset(&m_threadTError);
		m_threadErrorList.RemoveAll();

		CPDTPrepForDevs *pThread = new CPDTPrepForDevs(&m_bThreadResult, &m_threadErrorList,
														&m_threadTError,
														m_pConfig, m_pCCFWrapper);

		CProgressDlg *pProgDlg = new CProgressDlg(PROGD_FRAME_NOPROG, false, pThread, this);
		this->openPseudoModalChild(pProgDlg, CProgressDlg::IDD, false, true);
	}
}

void CSuperNudelistDlg::OnButtonLists() 
{
	CTypedError::SafeReset(&m_threadTError);

	updateConfig();

	if (! m_pConfig->IsReady(&m_threadTError, false) )
	{
		MessageBox(CTypedError::SafeGetFullErrorTextTempBuffer(&m_threadTError), SUPERNUDELIST_ERROR_TITLE, MB_ICONEXCLAMATION);
	}
	else
	{
		m_bThreadResult = true;
		CTypedError::SafeReset(&m_threadTError);
		m_threadErrorList.RemoveAll();

		CPDTPrepForLists *pThread = new CPDTPrepForLists(&m_bThreadResult, &m_threadErrorList,
														 &m_threadTError,
														 m_pConfig, m_pCCFWrapper);

		CProgressDlg *pProgDlg = new CProgressDlg(PROGD_FRAME_NOPROG, false, pThread, this);
		this->openPseudoModalChild(pProgDlg, CProgressDlg::IDD, false, true);
	}
}

void CSuperNudelistDlg::OnButtonExecute() 
{
	CTypedError::SafeReset(&m_threadTError);

	if (1 == m_ctrlCheckPanCap.GetCheck())
	{
		CString csPanCap;
		m_ctrlEditPanCap.GetWindowText(csPanCap);
		int nPanCap = atoi(csPanCap);

		if (0 == nPanCap)
		{
			MessageBox("Invalid number in panel cap field", SUPERNUDELIST_ERROR_TITLE, MB_ICONEXCLAMATION);
			return;
		}
	}

	updateConfig();

	if (! m_pConfig->IsReady(&m_threadTError, true) )
	{
		MessageBox(CTypedError::SafeGetFullErrorTextTempBuffer(&m_threadTError), SUPERNUDELIST_ERROR_TITLE, MB_ICONEXCLAMATION);
	}
	else
	{
		if (0 == stricmp(m_pConfig->GetInputCCFPath(), m_pConfig->GetOutputCCFPath()))
		{
			if (IDOK != MessageBox("It is *highly recommended* that you do *not* select the same file for input and output. Doing otherwise can result in loss of your CCF file if there is a problem during compilation.\n\nPress OK to continue or Cancel to abort the operation and pick another output.", SUPERNUDELIST_WARNING_TITLE, MB_ICONEXCLAMATION | MB_OKCANCEL))
			{
				return;
			}
			else if (IDYES != MessageBox("Are you absolutely sure? If you lose your CCF file you can only blame yourself.", SUPERNUDELIST_WARNING_TITLE, MB_ICONQUESTION | MB_YESNO))
			{
				return;
			}
		}
//		else if (0 == _access(m_pConfig->GetOutputCCFPath(), 00))
//		{
//			CString csMsg;
//			csMsg.Format("%s already exists and will be overwritten.\n\nIf there is an error during compilation, the contents of this file (but *not* the input file) may be lost.\n\nPress OK to continue or Cancel to abort the operation and pick another output.", m_pConfig->GetOutputCCFPath());
//
//			if (IDOK != MessageBox(csMsg, SUPERNUDELIST_WARNING_TITLE, MB_ICONEXCLAMATION | MB_OKCANCEL))
//			{
//				return;
//			}
//		}

		m_bThreadResult = true;
		CTypedError::SafeReset(&m_threadTError);
		m_threadErrorList.RemoveAll();

		CPDTPrepForExec *pThread = new CPDTPrepForExec(&m_bThreadResult, &m_threadErrorList,
														&m_threadTError,
														m_pConfig, m_pCCFWrapper);

		CProgressDlg *pProgDlg = new CProgressDlg(PROGD_FRAME_NOPROG, false, pThread, this);
		this->openPseudoModalChild(pProgDlg, CProgressDlg::IDD, false, true);
	}
}

void CSuperNudelistDlg::OnCancel() 
{
	// Don't allow closure if... it's not allowed.
	if (this->isClosureEnabled())
	{
		updateConfig();

		this->disableClosure();
//		this->setStayHiddenAfterPseudoModalChild(true);

		CPDTCleanUp *pThread = new CPDTCleanUp(m_pConfig, m_pCCFWrapper);

		CProgressDlg *pProgDlg = new CProgressDlg(PROGD_FRAME_NOPROG, false, pThread, this);
		this->openPseudoModalChild(pProgDlg, CProgressDlg::IDD, false, true);
	}

//	CSuckLessDialog::OnCancel();
}

void CSuperNudelistDlg::OnPseudoModalChildClose(int sldIDD, LPARAM lparam)
{
	if ((CProgressDlg::IDD == sldIDD) && (SLDTHREADID_CLEANUP == lparam))
	{
		this->closeWindow(0);
	}
	else if (((CProgressDlg::IDD == sldIDD) && (SLDTHREADID_PREPFORDEVS  == lparam))\
	||		 ((CProgressDlg::IDD == sldIDD) && (SLDTHREADID_PREPFORLISTS == lparam))\
	||		 ((CProgressDlg::IDD == sldIDD) && (SLDTHREADID_PREPFOREXECUTE == lparam))\
	||		 ((CProgressDlg::IDD == sldIDD) && (SLDTHREADID_RESOLVEFOREXECUTE == lparam))\
	||		 ((CProgressDlg::IDD == sldIDD) && (SLDTHREADID_FINALEXECUTE == lparam)))
	{
		if (!m_bThreadResult)
		{
			if ((NULL != m_pSetDiscsWithUnresolvedDevices) && (SLDTHREADID_RESOLVEFOREXECUTE == lparam))
			{
				CDevResDlg *pSLDlg = new CDevResDlg(m_pConfig, m_pCCFWrapper, m_pSetDiscsWithUnresolvedDevices, &m_bDevResSuccess, this);
				this->openPseudoModalChild(pSLDlg, CDevResDlg::IDD, false, true);
			}
			else if (m_threadErrorList.IsEmpty())
			{
				MessageBox(CTypedError::SafeGetFullErrorTextTempBuffer(&m_threadTError), SUPERNUDELIST_ERROR_TITLE, MB_ICONEXCLAMATION);
			}
			else
			{
				CListingDlg *pSLDlg = new CListingDlg("CCFDll Operation Log",
														CTypedError::SafeGetFullErrorTextTempBuffer(&m_threadTError),
														&m_threadErrorList, false, this);
				this->openPseudoModalChild(pSLDlg, CListingDlg::IDD, false, true);
			}
		}
		else if (SLDTHREADID_PREPFORDEVS  == lparam)
		{
			CDevicesDlg *pSLDlg = new CDevicesDlg(m_pConfig, m_pCCFWrapper, this);
			this->openPseudoModalChild(pSLDlg, CDevicesDlg::IDD, false, true);
		}
		else if (SLDTHREADID_PREPFORLISTS  == lparam)
		{
			CListsDlg *pSLDlg = new CListsDlg(m_pConfig, m_pCCFWrapper, this);
			this->openPseudoModalChild(pSLDlg, CListsDlg::IDD, false, true);
		}
		else if (SLDTHREADID_PREPFOREXECUTE == lparam)
		{
			if (askToRemoveDevices())
			{
				// in-memory CSF doesn't match what we read from disk anymore.
				m_pCCFWrapper->SetDecompileNeeded();
			}

			m_bThreadResult = true;
			CTypedError::SafeReset(&m_threadTError);
			m_threadErrorList.RemoveAll();

			delete m_pSetDiscsWithUnresolvedDevices;
			m_pSetDiscsWithUnresolvedDevices = NULL;

			CPDTResolveForExec *pThread = new CPDTResolveForExec(&m_bThreadResult,
																 &m_threadTError,
																 m_pConfig, m_pCCFWrapper,
																 &m_pSetDiscsWithUnresolvedDevices);

			CProgressDlg *pProgDlg = new CProgressDlg(PROGD_FRAME_NOPROG, false, pThread, this);
			this->openPseudoModalChild(pProgDlg, CProgressDlg::IDD, false, true);
		}
		else if (SLDTHREADID_RESOLVEFOREXECUTE == lparam)
		{
			delete m_pSetDiscsWithUnresolvedDevices;
			m_pSetDiscsWithUnresolvedDevices = NULL;

			m_bThreadResult = true;
			CTypedError::SafeReset(&m_threadTError);
			m_threadErrorList.RemoveAll();
			m_threadEmptiesList.RemoveAll();

			CPDTFinalExec *pThread = new CPDTFinalExec(&m_bThreadResult, &m_threadErrorList,
														&m_threadTError,
														m_pConfig, m_pCCFWrapper, &m_threadEmptiesList);

			CProgressDlg *pProgDlg = new CProgressDlg(PROGD_FRAME_FULL, false, pThread, this);
			this->openPseudoModalChild(pProgDlg, CProgressDlg::IDD, false, true);
		}
		else if (SLDTHREADID_FINALEXECUTE == lparam)
		{
			if (!m_threadEmptiesList.IsEmpty())
			{
				const char *szMsg;

				if (1 == m_threadEmptiesList.GetCount())
				{
					szMsg = "The following list was not output because it was empty:";
				}
				else
				{
					szMsg = "The following lists were not output because they were empty:";
				}

				CListingDlg *pSLDlg = new CListingDlg(SUPERNUDELIST_WARNING_TITLE, szMsg, &m_threadEmptiesList, true, this);
				this->openPseudoModalChild(pSLDlg, CListingDlg::IDD, false, true);
			}
			else
			{
				// Done. :-)
			}
		}
	}
	else if (CDevResDlg::IDD == sldIDD)
	{
		delete m_pSetDiscsWithUnresolvedDevices;
		m_pSetDiscsWithUnresolvedDevices = NULL;

		if (m_bDevResSuccess)
		{
			m_bThreadResult = true;
			CTypedError::SafeReset(&m_threadTError);
			m_threadErrorList.RemoveAll();
			m_threadEmptiesList.RemoveAll();

			CPDTFinalExec *pThread = new CPDTFinalExec(&m_bThreadResult, &m_threadErrorList,
														&m_threadTError,
														m_pConfig, m_pCCFWrapper, &m_threadEmptiesList);

			CProgressDlg *pProgDlg = new CProgressDlg(PROGD_FRAME_FULL, false, pThread, this);
			this->openPseudoModalChild(pProgDlg, CProgressDlg::IDD, false, true);
		}
	}
}

// Returns true if any were deleted.
bool CSuperNudelistDlg::askToRemoveDevices(void)
{
	bool bRemovedAny = false;

	for (const CSNVolume *pVol = m_pConfig->GetLibrary()->GetFirst();
		 NULL != pVol;
		 pVol = m_pConfig->GetLibrary()->GetNext(pVol))
	{
		int nDeviceNumber = 0;
		int nTotalDevices = 0;
		int nHighestDeviceNumber = 0;

		list<CCSFATDevice *> deviceList;

		while(true)
		{
			list<CCSFATDevice *> deviceListTemp;

			CString csDeviceNamePronto;

			if (0 == nDeviceNumber)
			{
				csDeviceNamePronto = pVol->GetName()->GetPronto(true);
			}
			else
			{
				csDeviceNamePronto.Format("%s.%d", pVol->GetName()->GetPronto(true), nDeviceNumber);
			}

			m_pCCFWrapper->GetCSFAbstractTree()->FindDevicesPronto(deviceListTemp, csDeviceNamePronto);

			if (deviceListTemp.empty())
			{
				break;
			}
			else
			{
				nHighestDeviceNumber = nDeviceNumber;
				nTotalDevices += deviceListTemp.size();
				deviceList.splice(deviceList.end(), deviceListTemp);
				nDeviceNumber++;
			}
		}

		if (0 != nTotalDevices)
		{
			CString csQuestion;

			if (0 != nHighestDeviceNumber)
			{
				csQuestion.Format("Delete existing %d devices named \"%s\" through \"%s.%d\"?",
									nTotalDevices,
									pVol->GetName()->GetPrintable(true),
									pVol->GetName()->GetPrintable(true), nHighestDeviceNumber);
			}
			else if (1 == nTotalDevices)
			{
				csQuestion.Format("Delete existing device named \"%s\"?", pVol->GetName()->GetPrintable(true));
			}
			else
			{
				csQuestion.Format("Delete existing %d devices named \"%s\"?", nTotalDevices, pVol->GetName()->GetPrintable(true));
			}

			if (IDYES == MessageBox(csQuestion, "Replace devices?", MB_YESNO | MB_ICONQUESTION))
			{
				bRemovedAny = true;

				for (list<CCSFATDevice *>::iterator iter = deviceList.begin(); iter != deviceList.end(); iter++)
				{
					m_pCCFWrapper->GetCSFAbstractTree()->DeleteDeviceFromATAndCSF(*iter);
				}
			}
		}
	}

	return(bRemovedAny);
}

void CSuperNudelistDlg::OnNcDestroy() 
{
	// Stop that stupid, lame, useless behaviour where the main app thread is killed
	// because the main window is closed.

	// See "PRB: Windows Flash and Disappear in Dialog-Based Applications" in MSDN.

	// (Could also be done by never setting m_pMainWnd, but maybe something else will depend on that being set.)

	AfxGetApp()->m_pMainWnd = NULL;
	CSuckLessDialog::OnNcDestroy();
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

CPDTCleanUp::CPDTCleanUp(const CSNConfig *pConfig, CCCFWrapper *pCCFWrapper) : CProgressDlgThread()
{
	m_pConfig = pConfig;
	m_pCCFWrapper = pCCFWrapper;
}

CPDTCleanUp::~CPDTCleanUp(void)
{
}

// Returns an ident for the thread.
// Typically, each implementation of this class within a project will use a unique
// ident. (-1) should not be used, but any other number is fine.
// See CSuckLessDialog::OnThreadTermination().
int CPDTCleanUp::GetIdent(void)
{
	return(SLDTHREADID_CLEANUP);
}

// Overriden to provide functionality.
void CPDTCleanUp::mainThreadSubClass(void)
{
	// If an error occurs don't report it, the user is trying to
	// close the app and will be told about any problems next time
	// they cause decompilation.

	SetProgressTitle("SuperNudelist is exiting...");
	
	SetProgressHeading("Saving CCF paths");
	SetProgressMessage("");
	m_pConfig->SaveCCFPathsAndOptions(NULL);

	m_pCCFWrapper->CleanTempDir(m_pProgressDlg, NULL);
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

CPDTPrep::CPDTPrep(bool *pbThreadResult, CStringList *pStrLstLog, CTypedError *pThreadTError,
									CSNConfig *pConfig, CCCFWrapper *pCCFWrapper) : CProgressDlgThread()
{
	m_pbThreadResult = pbThreadResult;
	m_pStrLstLog = pStrLstLog;
	m_pThreadTError = pThreadTError;

	m_pConfig = pConfig;
	m_pCCFWrapper = pCCFWrapper;
}

CPDTPrep::~CPDTPrep(void)
{
}

// Overriden to provide functionality.
void CPDTPrep::mainThreadSubClass(void)
{
	(*m_pbThreadResult) = false;

	SetProgressHeading("Saving CCF paths");
	SetProgressMessage("");
	m_pConfig->SaveCCFPathsAndOptions(NULL);

	if (m_pCCFWrapper->DecompileAndRead(m_pProgressDlg, m_pStrLstLog, m_pThreadTError))
	{
		(*m_pbThreadResult) = true;
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

CPDTPrepForDevs::CPDTPrepForDevs(bool *pbThreadResult, CStringList *pStrLstLog,
								 CTypedError *pThreadTError,
								 CSNConfig *pConfig, CCCFWrapper *pCCFWrapper)
				: CPDTPrep(pbThreadResult, pStrLstLog, pThreadTError, pConfig, pCCFWrapper)
{
}

CPDTPrepForDevs::~CPDTPrepForDevs()
{
}

int CPDTPrepForDevs::GetIdent(void)
{
	return(SLDTHREADID_PREPFORDEVS);
}

// Overriden to provide functionality.
void CPDTPrepForDevs::mainThreadSubClass(void)
{
	SetProgressTitle("Preparing for Players...");

	CPDTPrep::mainThreadSubClass();

	if (*m_pbThreadResult)
	{
		// ...
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

CPDTPrepForLists::CPDTPrepForLists(bool *pbThreadResult, CStringList *pStrLstLog,
								 CTypedError *pThreadTError,
								 CSNConfig *pConfig, CCCFWrapper *pCCFWrapper)
				: CPDTPrep(pbThreadResult, pStrLstLog, pThreadTError, pConfig, pCCFWrapper)
{
}

CPDTPrepForLists::~CPDTPrepForLists()
{
}

int CPDTPrepForLists::GetIdent(void)
{
	return(SLDTHREADID_PREPFORLISTS);
}

// Overriden to provide functionality.
void CPDTPrepForLists::mainThreadSubClass(void)
{
	SetProgressTitle("Preparing for Lists...");

	CPDTPrep::mainThreadSubClass();

	if (*m_pbThreadResult)
	{
		// ...
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

CPDTPrepForExec::CPDTPrepForExec(bool *pbThreadResult, CStringList *pStrLstLog,
								 CTypedError *pThreadTError,
								 CSNConfig *pConfig, CCCFWrapper *pCCFWrapper)
				: CPDTPrep(pbThreadResult, pStrLstLog, pThreadTError, pConfig, pCCFWrapper)
{
}

CPDTPrepForExec::~CPDTPrepForExec()
{
}

int CPDTPrepForExec::GetIdent(void)
{
	return(SLDTHREADID_PREPFOREXECUTE);
}

// Overriden to provide functionality.
void CPDTPrepForExec::mainThreadSubClass(void)
{
	SetProgressTitle("Preparing for execution...");

	CPDTPrep::mainThreadSubClass();

	if (*m_pbThreadResult)
	{
		// ...
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

CPDTResolveForExec::CPDTResolveForExec(bool *pbThreadResult, CTypedError *pThreadTError,
										 CSNConfig *pConfig, CCCFWrapper *pCCFWrapper,
										 set<CSNDisc *> **ppSetDiscsWithUnresolvedDevices)
{
	m_pbThreadResult = pbThreadResult;
	m_pThreadTError = pThreadTError;

	m_pConfig = pConfig;
	m_pCCFWrapper = pCCFWrapper;


	m_ppSetDiscsWithUnresolvedDevices = ppSetDiscsWithUnresolvedDevices;

	delete (*m_ppSetDiscsWithUnresolvedDevices);
	(*m_ppSetDiscsWithUnresolvedDevices) = NULL;
}

CPDTResolveForExec::~CPDTResolveForExec()
{
}

int CPDTResolveForExec::GetIdent(void)
{
	return(SLDTHREADID_RESOLVEFOREXECUTE);
}

// Overriden to provide functionality.
void CPDTResolveForExec::mainThreadSubClass(void)
{
	(*m_pbThreadResult) = false;

	SetProgressTitle("Preparing for execution...");

	SetProgressHeading("Resolving players");
	SetProgressMessage("");

	(*m_ppSetDiscsWithUnresolvedDevices) = m_pConfig->ResolveDevices();

	if (NULL == (*m_ppSetDiscsWithUnresolvedDevices))
	{
		(*m_pbThreadResult) = true;
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

CPDTFinalExec::CPDTFinalExec(bool *pbThreadResult, CStringList *pStrLstLog, CTypedError *pThreadTError,
										 CSNConfig *pConfig, CCCFWrapper *pCCFWrapper, CStringList *pStrLstEmpties)
{
	m_pbThreadResult = pbThreadResult;
	m_pStrLstLog = pStrLstLog;
	m_pThreadTError = pThreadTError;

	m_pStrLstEmpties = pStrLstEmpties;

	m_pConfig = pConfig;
	m_pCCFWrapper = pCCFWrapper;
}

CPDTFinalExec::~CPDTFinalExec()
{
}

int CPDTFinalExec::GetIdent(void)
{
	return(SLDTHREADID_FINALEXECUTE);
}

// Overriden to provide functionality.
void CPDTFinalExec::mainThreadSubClass(void)
{
	(*m_pbThreadResult) = false;

	if (NULL != m_pStrLstEmpties)
	{
		m_pStrLstEmpties->RemoveAll();
	}

	SetProgressHeading("Resolving CSF paths");
	SetProgressMessage("");

	if (m_pConfig->ResolveCSFPaths(m_pThreadTError, m_pCCFWrapper->GetCSFAbstractTree()))
	{
		SetProgressTitle("Executing...");

		SetProgressHeading("Doing layout");
		SetProgressMessage("");

		m_pCCFWrapper->GetRootCSFSection()->CalcHighestButtonNumber();

		m_pConfig->GetLibrary()->AddToAndUpdateRootCSFSection(m_pConfig, m_pCCFWrapper->GetRootCSFSection(), m_pStrLstEmpties);

		// What's in memory is no longer what was read from disk.
		m_pCCFWrapper->SetDecompileNeeded();

		SetProgressMessage("Removing any dangling jumps");

		if (m_pCCFWrapper->RemoveDanglingJumps(m_pThreadTError, true))
		{
			SetProgressHeading("Writing BMP Icons");
			SetProgressMessage("");

			if (NSSNBuiltInIcons::MakeIconsAvailable(m_pThreadTError, m_pConfig))
			{
				SetProgressHeading("Saving and compiling");
				SetProgressMessage("");

				if (m_pCCFWrapper->SaveAndCompile(m_pProgressDlg, m_pStrLstLog, m_pThreadTError))
				{
					(*m_pbThreadResult) = true;
				}
			}
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
