// TrackDlg.cpp : implementation file
//

#include "stdafx.h"
#include "TrackDlg.h"
#include "SNTrack.h"
#include "DiscDlg.h"
#include "SNDevice.h"
#include "ProntoStringDlg.h"
#include "SNDisc.h"
#include "SNConfig.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTrackDlg dialog

CTrackDlg::CTrackDlg(CDiscDlg *pParentDiscDlg,
					 const CSNTrack *pTemplateTrack, const CSNTrack *pUpdateTrack,
					 CSNDisc *pParentDisc)
	: CSuckLessDialog(CTrackDlg::IDD, pParentDiscDlg)
{
	//{{AFX_DATA_INIT(CTrackDlg)
	m_editName = _T("");
	//}}AFX_DATA_INIT

	m_pParentDiscDlg = pParentDiscDlg;

	m_pNewTrack = new CSNTrack();

	if (NULL != pTemplateTrack)
	{
		m_pNewTrack->Set(pTemplateTrack);
	}

	m_pUpdateTrack = pUpdateTrack;

	m_pParentDisc = pParentDisc;

	m_bUpdatingTrackNumber = false;
}

CTrackDlg::~CTrackDlg()
{
	delete m_pNewTrack;
	m_pNewTrack = NULL;
}

void CTrackDlg::DoDataExchange(CDataExchange* pDX)
{
	CSuckLessDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTrackDlg)
	DDX_Control(pDX, IDOK, m_buttonOK);
	DDX_Control(pDX, IDCANCEL, m_buttonCancel);
	DDX_Control(pDX, IDC_SPIN_NUMBER, m_ctrlSpinNumber);
	DDX_Control(pDX, IDC_EDIT_NUMBER, m_ctrlEditNumber);
	DDX_Control(pDX, IDC_EDIT_NAME, m_ctrlEditName);
	DDX_Text(pDX, IDC_EDIT_NAME, m_editName);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTrackDlg, CSuckLessDialog)
	//{{AFX_MSG_MAP(CTrackDlg)
	ON_BN_CLICKED(IDC_BUTTON_NAME, OnButtonName)
	ON_WM_VSCROLL()
	ON_EN_CHANGE(IDC_EDIT_NUMBER, OnChangeEditNumber)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTrackDlg message handlers

BOOL CTrackDlg::OnInitDialog() 
{
	CSuckLessDialog::OnInitDialog();

	UpdateData(TRUE);

	this->setWindowIcon(IDR_MAINFRAME);

	CString winTitle;
	GetWindowText(winTitle);
	if (NULL != m_pUpdateTrack)
	{
		winTitle += " -- (";
		winTitle += m_pUpdateTrack->GetTitle()->GetPrintable(true);
		winTitle += ")";
	}
	else
	{
		winTitle += " -- <New>";
	}
	SetWindowText(winTitle);

	if (NULL == m_pUpdateTrack)
	{
		m_buttonOK.SetWindowText("Add");
		m_buttonCancel.SetWindowText("Close");
	}

	// -- Initial contents --

	if (NULL != g_pProntoFont)
	{
		m_ctrlEditName.SetFont(g_pProntoFont);
	}

	// --

	m_editName		= m_pNewTrack->GetTitle()->GetPronto(true);

	// --

	m_ctrlSpinNumber.SetRange32(0, CSNDevice::MAX_TRACKNUMBER);
	int trackNumber = m_pNewTrack->GetTrackNumber();
	updateTrackNumber(trackNumber, true, true);

	// -- Control setup --

	addControlInfo(IDC_STATIC_NUMBER);
	addControlInfo(IDC_STATIC_TITLE);

	addControlInfo(IDC_EDIT_NUMBER);
	addControlInfo(IDC_SPIN_NUMBER);

	addControlInfo(IDC_EDIT_NAME);
	addControlInfo(IDC_BUTTON_NAME);

	addControlInfo(IDC_BUTTON_HELP);

	addControlInfo(IDOK);
	addControlInfo(IDCANCEL);

	setControlRelationship(IDC_STATIC_NUMBER,		IDD,					CControlInfo::ACOMMON_TOP_LEFT);
	setControlRelationship(IDC_STATIC_TITLE,		IDD,					CControlInfo::ACOMMON_TOP_RESIZEWIDTH);

	setControlRelationship(IDC_EDIT_NUMBER,			IDC_STATIC_NUMBER,		CControlInfo::ACOMMON_TOP_LEFT);
	setControlRelationship(IDC_SPIN_NUMBER,			IDC_STATIC_NUMBER,		CControlInfo::ACOMMON_TOP_LEFT);

	setControlRelationship(IDC_EDIT_NAME,			IDC_STATIC_TITLE,		CControlInfo::ACOMMON_TOP_RESIZEWIDTH);
	setControlRelationship(IDC_BUTTON_NAME,			IDC_STATIC_TITLE,		CControlInfo::ACOMMON_TOP_RIGHT);

	setControlRelationship(IDC_BUTTON_HELP,			IDD,					CControlInfo::ACOMMON_BOTTOM_LEFT);

	setControlRelationship(IDOK,					IDD,					CControlInfo::ACOMMON_BOTTOM_RIGHT);
	setControlRelationship(IDCANCEL,				IDD,					CControlInfo::ACOMMON_BOTTOM_RIGHT);

	controlAndFrameInfoIsSetup();

	setMinXToCurrent();
	setMinYToCurrent();
	setMaxYToCurrent();

	m_ctrlEditName.SetFocus();

	UpdateData(FALSE);

	this->setRememberWindowPosition(SN_REG_PATH_WINDOWPOS, SN_REG_VALUE_WINDOWPOS_TRACK);

	return FALSE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CTrackDlg::OnButtonName() 
{
	CProntoStringDlg *pSLDlg = new CProntoStringDlg(&m_ctrlEditName, "Track Title", this);
	this->openPseudoModalChild(pSLDlg, CProntoStringDlg::IDD, false, true);
}

void CTrackDlg::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	if (m_ctrlSpinNumber.GetDlgCtrlID() == pScrollBar->GetDlgCtrlID())
	{
		updateTrackNumber(nPos, true, false);
	}

	CSuckLessDialog::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CTrackDlg::updateTrackNumber(int trackNumber, bool bUpdateEdit, bool bUpdateSpin)
{
	if (!m_bUpdatingTrackNumber)
	{
		m_bUpdatingTrackNumber = true;

		if (bUpdateEdit)
		{
			CString csLabel;

			csLabel.Format("%d", trackNumber);
			m_ctrlEditNumber.SetWindowText(csLabel);
		}

		if (bUpdateSpin)
		{
			m_ctrlSpinNumber.SetPos(trackNumber);
		}

		m_pNewTrack->SetTrackNumber(trackNumber);

		m_bUpdatingTrackNumber = false;
	}
}

void CTrackDlg::OnChangeEditNumber() 
{
	// TODO: If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CSuckLessDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.
	
	CString csLabel;
	m_ctrlEditNumber.GetWindowText(csLabel);

	bool bUpdateEdit = false;

	int val = 1;

	if (! csLabel.IsEmpty() )
	{
		val = atoi(LPCTSTR(csLabel));

		if (CSNDevice::MAX_TRACKNUMBER < val)
		{
			val = CSNDevice::MAX_TRACKNUMBER;
			bUpdateEdit = true;
		}
		else if (0 >= val)
		{
			val = 1;
			bUpdateEdit = true;
		}
	}

	updateTrackNumber(val, bUpdateEdit, true);
}

void CTrackDlg::OnOK() 
{
	UpdateData(TRUE);

	// -- Put any unsaved values into the disc. --

	m_pNewTrack->GetTitle()->SetPronto(LPCTSTR(m_editName));

	m_pNewTrack->m_pParentDisc = m_pParentDisc;

	// -- Try to add to/update the list. --

	CTypedError tError;

	const CSNTrack *pAddUpdateResult = NULL;

	if (NULL != m_pUpdateTrack)
	{
		pAddUpdateResult = m_pParentDisc->GetTracks()->Update(&tError, m_pNewTrack, m_pUpdateTrack);
	}
	else
	{
		pAddUpdateResult = m_pParentDisc->GetTracks()->AddBelowAndOwn(&tError, m_pNewTrack, NULL);
	}

	if (NULL == pAddUpdateResult)
	{
		MessageBox(CTypedError::SafeGetFullErrorTextTempBuffer(&tError),
				   NULL != m_pUpdateTrack ? "Cannot update track" : "Cannot add track",
				   MB_OK | MB_ICONEXCLAMATION);
	}
	else
	{
		m_pParentDiscDlg->resetAndPopulateTrackList();

		if (NULL != m_pUpdateTrack)
		{
			CSuckLessDialog::OnOK();
		}
		else
		{
			// We no longer own the track and should not delete it on destruction.
			m_pNewTrack = NULL;

			// Create a new track which we do own (for now) and should delete on destruction.
			m_pNewTrack = new CSNTrack();

			// Doing it this way stops the number going out of range.
			m_pNewTrack->SetTrackNumber(pAddUpdateResult->GetTrackNumber() + 1);

			m_editName = m_pNewTrack->GetTitle()->GetPronto(true);
			updateTrackNumber(m_pNewTrack->GetTrackNumber(), true, true);

			UpdateData(FALSE);

			m_ctrlEditName.SetFocus();
		}
	}
}
