#include "stdafx.h"

void CTypedError::sem(const char *nMsg)
{
	delete [] m_errorMessage;
	m_errorMessage = NULL;

	delete [] m_tempFullErrorText;
	m_tempFullErrorText = NULL;

	if ((NULL != nMsg) && ('\0' != nMsg[0]))
	{
		m_errorMessage = new char [strlen(nMsg) + 1];
		if (NULL != m_errorMessage)
		{
			strcpy(m_errorMessage, nMsg);
		}
	}
}

void CTypedError::spm(const char *nMsg)
{
	delete [] m_prependMessage;
	m_prependMessage = NULL;

	delete [] m_tempFullErrorText;
	m_tempFullErrorText = NULL;

	if ((NULL != nMsg) && ('\0' != nMsg[0]))
	{
		m_prependMessage = new char [strlen(nMsg) + 1];
		if (NULL != m_prependMessage)
		{
			strcpy(m_prependMessage, nMsg);
		}
	}
}

CTypedError::CTypedError(void)
{
	m_errorMessage = NULL;
	m_prependMessage = NULL;
	m_tempFullErrorText = NULL;
	this->reset();
}

CTypedError::~CTypedError(void)
{
	this->reset();
}

void CTypedError::reset(void)
{
	m_error = 0;
	m_errorType = ERROR_NONE;
	sem(NULL);
	spm(NULL);
}

bool CTypedError::isNoError(void) const
{
	return(ERROR_NONE == m_errorType);
}

bool CTypedError::isError(void) const
{
	return(ERROR_NONE != m_errorType);
}

DWORD CTypedError::GetError(void) const
{
	return(m_error);
}

CTypedError::ErrorType CTypedError::GetErrorType(void) const
{
	return(m_errorType);
}

const char *CTypedError::GetErrorMessage(void) const
{
	return(m_errorMessage);
}

const char *CTypedError::GetPrependMessage(void) const
{
	return(m_prependMessage);
}

// Static safe-set functions are provided further down. These check a CTypedError pointer for NULL so that
// you can easily and safely provide support for an optional CTypedError without checking it all the time.

void CTypedError::set(const char *msg, DWORD err, ErrorType errorType, va_list args)
{
	if (isNoError())
	{
		m_error = err;
		m_errorType = errorType;

		if ((NULL == args) || (NULL == msg))
		{
			sem(msg);
		}
		else
		{
			char buf[TYPEDERROR_BUFFERSIZE];

			vStringFormat(buf, sizeof(buf), msg, args);

			sem(buf);
		}
	}
}

void CTypedError::SetPrependMessage(const char *preMsg)
{
	spm(preMsg);
}

// Returns full error text in a temporary buffer which is deleted when the typed error is deleted or changed.
const char *CTypedError::getFullErrorTextTempBuffer(void)
{
	delete [] m_tempFullErrorText;
	m_tempFullErrorText = NULL;

	m_tempFullErrorText = this->getFullErrorText();

	return(m_tempFullErrorText);
}

// Returns a new buffer. delete[] when finished with it.
char *CTypedError::getFullErrorText(void) const
{
	char tmpBuf[TYPEDERROR_BUFFERSIZE];

	bool bAddSeparator = false;
	bool bTmpBufContainsExtraMessage = false;
	int reqBufSize = 0;
	int separatorSize = strlen(TE_ERRORSTRINGPARTSEPARATOR);

	if (NULL != m_prependMessage)
	{
		reqBufSize += bAddSeparator ? separatorSize : 0;
		reqBufSize += strlen(m_prependMessage);
		bAddSeparator = true;
	}

	if (NULL != m_errorMessage)
	{
		reqBufSize += bAddSeparator ? separatorSize : 0;
		reqBufSize += strlen(m_errorMessage);
		bAddSeparator = true;
	}

	if (ERROR_SYSTEM == m_errorType)
	{
		CTypedError::GetSystemErrorText(m_error, tmpBuf, sizeof(tmpBuf));
		reqBufSize += bAddSeparator ? separatorSize : 0;
		reqBufSize += strlen(tmpBuf);
		bAddSeparator = true;
		bTmpBufContainsExtraMessage = true;
	}
#if TYPEDERROR_WINSOCK
	else if (ERROR_WINSOCK == m_errorType)
	{
		CTypedError::GetWinSockErrorText(m_error, tmpBuf, sizeof(tmpBuf));
		reqBufSize += bAddSeparator ? separatorSize : 0;
		reqBufSize += strlen(tmpBuf);
		bAddSeparator = true;
		bTmpBufContainsExtraMessage = true;
	}
#endif // TYPEDERROR_WINSOCK
	else if (ERROR_CRUNTIME == m_errorType)
	{
		stringCopy(tmpBuf, strerror(m_error), sizeof(tmpBuf));
		reqBufSize += bAddSeparator ? separatorSize : 0;
		reqBufSize += strlen(tmpBuf);
		bAddSeparator = true;
		bTmpBufContainsExtraMessage = true;
	}
	else if (NULL == m_errorMessage)
	{
		reqBufSize += bAddSeparator ? separatorSize : 0;
		reqBufSize += strlen(TE_NOERRORRECORDED);
		bAddSeparator = true;
	}

	reqBufSize++; // NULL terminator.

	char *result = new char[reqBufSize];
	result[0] = '\0';

	bAddSeparator = false;

	if (NULL != m_prependMessage)
	{
		if (bAddSeparator) strcat(result, TE_ERRORSTRINGPARTSEPARATOR);

		strcat(result, m_prependMessage);
		bAddSeparator = true;
	}

	if (NULL != m_errorMessage)
	{
		if (bAddSeparator) strcat(result, TE_ERRORSTRINGPARTSEPARATOR);

		strcat(result, m_errorMessage);
		bAddSeparator = true;
	}

	if (bTmpBufContainsExtraMessage)
	{
		if (bAddSeparator) strcat(result, TE_ERRORSTRINGPARTSEPARATOR);

		strcat(result, tmpBuf);
		bAddSeparator = true;
	}
	else if (NULL == m_errorMessage)
	{
		if (bAddSeparator) strcat(result, TE_ERRORSTRINGPARTSEPARATOR);

		strcat(result, TE_NOERRORRECORDED);
		bAddSeparator = true;
	}

	return(result);
}

// -- Statics --

void CTypedError::SafeReset(CTypedError *pTError)
{
	if (NULL != pTError)
	{
		pTError->reset();
	}
}

// Defaults to true if pTError is NULL.
bool CTypedError::SafeIsNoError(CTypedError *pTError)
{
	bool bResult = true;

	if (NULL != pTError)
	{
		bResult = pTError->isNoError();
	}

	return(bResult);
}

// Defaults to false if pTError is NULL.
bool CTypedError::SafeIsError(CTypedError *pTError)
{
	bool bResult = false;

	if (NULL != pTError)
	{
		bResult = pTError->isError();
	}

	return(bResult);
}

// Copies one TError to another, if the destination does not already contain an error.
// See also SafeSetTError which is similar.
void CTypedError::SafeCopyTError(CTypedError *pTErrorDestination, const CTypedError *pTErrorSource,
										bool bCopyPrependMessage)
{
	if (NULL != pTErrorDestination)
	{
		if (NULL == pTErrorSource)
		{
			pTErrorDestination->set("<NULL pTErrorSource>", 0, ERROR_MESSAGEONLY, NULL);
		}
		else
		{
			pTErrorDestination->set( pTErrorSource->GetErrorMessage(),
									 pTErrorSource->GetError(),
									 pTErrorSource->GetErrorType(),
									 NULL);

			if (bCopyPrependMessage)
			{
				pTErrorDestination->SetPrependMessage(pTErrorSource->GetPrependMessage());
			}
		}
	}
}

// Returns full error text in a temporary buffer which is deleted when the typed error is deleted or changed.
const char *CTypedError::SafeGetFullErrorTextTempBuffer(const CTypedError *pTError)
{
	const char *result = NULL;

	if (NULL != pTError)
	{
		// Semantically, we're not really changing it... :-)
		result = ((CTypedError *)pTError)->getFullErrorTextTempBuffer();
	}

	if (NULL == result)
	{
		result = TE_NOERRORRECORDED;
	}

	return(result);
}

// Returns a new buffer. delete[] when finished with it.
char *CTypedError::SafeGetFullErrorTextCopy(const CTypedError *pTError)
{
	char *result = NULL;

	if (NULL != pTError)
	{
		result = pTError->getFullErrorText();
	}
	else
	{
		result = new char[strlen(TE_NOERRORRECORDED) + 1];
		strcpy(result, TE_NOERRORRECORDED);
	}

	return(result);
}

// Prefixes your message to the existing message in the TError.
// This is different to calling SetPrependMessage() which sets a *single* prefix buffer which
// appears in front of any messages generated by the object.
// Calling SafePrefixMsgError() multiple times will add each of your messages to the front
// of whatever is there while calling SetPrependMessage() multiple times will result in
// only the last call having any effect.
// SetPrependMessage() should generally only be called by the owner of the TError while
// SafePrefixMsgError() is mainly intended for functions which are generating errors.
// If you call SafePrefixMsgError() and the TError has no error set it is the same as
// calling SafeSetMsgError().
// Important Note: Make sure you only call SafePrefixMsgError when you know that there was
// *************** no error stored beforehand, i.e. that your function set the error.
// *************** (If a CTypedError already has an error set, setting another one does
// *************** nothing until after a reset has been done.)

void CTypedError::SafePrefixMsgError(CTypedError *pTError, const char *msg, ...)
{
	if (NULL != pTError)
	{
		if ((NULL != msg) && ('\0' != msg[0]))
		{
			char buf[TYPEDERROR_BUFFERSIZE];

			va_list args;
			va_start(args, msg);
			vStringFormat(buf, sizeof(buf), msg, args);
			va_end(args);

			// buf is the message to prefix.

			if (pTError->isNoError())
			{
				pTError->set(buf, 0, ERROR_MESSAGEONLY, NULL);
			}
			else
			{
				char *origErrMsg = SafeGetFullErrorTextCopy(pTError);

				pTError->reset(); // Remove existing error message and allow us to set a new one.

				SafeSetMsgError(pTError, "%s%s%s", buf, strlen(origErrMsg) > 0 ? ": " : "", origErrMsg);

				delete [] origErrMsg;
			}
		}
	}
}

void CTypedError::SafeSetMsgError(CTypedError *pTError, const char *msg, ...)
{
	if (NULL != pTError)
	{
		va_list args;
		va_start(args, msg);
		pTError->set(msg, 0, ERROR_MESSAGEONLY, args);
		va_end(args);
	}
}

void CTypedError::SafeSetSystemError(CTypedError *pTError, const char *msg, ...)
{
	if (NULL != pTError)
	{
		int error = GetLastError();
		va_list args;
		va_start(args, msg);
		pTError->set(msg, error, ERROR_SYSTEM, args);
		va_end(args);
		SetLastError(error);
	}
}

void CTypedError::SafeSetSystemErrorNum(CTypedError *pTError, int error, const char *msg, ...)
{
	if (NULL != pTError)
	{
		va_list args;
		va_start(args, msg);
		pTError->set(msg, error, ERROR_SYSTEM, args);
		va_end(args);
	}
}

void CTypedError::SafeSetCRuntimeError(CTypedError *pTError, const char *msg, ...)
{
	if (NULL != pTError)
	{
		int error = errno;
		va_list args;
		va_start(args, msg);
		pTError->set(msg, error, ERROR_CRUNTIME, args);
		va_end(args);
		errno = error;
	}
}

void CTypedError::SafeSetCRuntimeErrorNum(CTypedError *pTError, int error, const char *msg, ...)
{
	if (NULL != pTError)
	{
		va_list args;
		va_start(args, msg);
		pTError->set(msg, error, ERROR_CRUNTIME, args);
		va_end(args);
	}
}

#if TYPEDERROR_WINSOCK
void CTypedError::SafeSetWSError(CTypedError *pTError, const char *msg, ...)
{
	if (NULL != pTError)
	{
		int error = WSAGetLastError();
		va_list args;
		va_start(args, msg);
		pTError->set(msg, error, ERROR_WINSOCK, args);
		va_end(args);
		WSASetLastError(error);
	}
}
#endif // TYPEDERROR_WINSOCK

#if TYPEDERROR_WINSOCK
void CTypedError::SafeSetWSErrorNum(CTypedError *pTError, int error, const char *msg, ...)
{
	if (NULL != pTError)
	{
		va_list args;
		va_start(args, msg);
		pTError->set(msg, error, ERROR_WINSOCK, args);
		va_end(args);
	}
}
#endif // TYPEDERROR_NOWINSOCK

// There is no SafeSetWNetErrorNum().
// If the error isn't ERROR_EXTENDED_ERROR it's a normal system error.
// If the error is ERROR_EXTENDED_ERROR you can only get the extended error information
// with a call to WNetGetLastError (via SafeSetWNetError()) or the information may be lost.

#if TYPEDERROR_WNET
void CTypedError::SafeSetWNetError(CTypedError *pTError, const char *msg, ...)
{
	if (NULL != pTError)
	{
		int err = GetLastError(); //make sure we don't change the error code

		bool bBuffSet = false;
		char buf[TYPEDERROR_BUFFERSIZE];
		char lasterr[TYPEDERROR_BUFFERSIZE];
		char provider[TYPEDERROR_BUFFERSIZE];
		DWORD errorCode;

		if (ERROR_EXTENDED_ERROR == err)
		{
			if (NO_ERROR != WNetGetLastError(&errorCode, lasterr, sizeof(lasterr), provider, sizeof(provider)))
			{
				errorCode = 0;
				stringCopy(lasterr, "<unable to obtain WNet extended error>", sizeof(lasterr));
				stringCopy(provider, "<unknown>", sizeof(provider));
			}
		}

		if ((NULL != msg) && ('\0' != msg[0]))
		{
			va_list args;
			va_start(args, msg);
			vStringFormat(buf, sizeof(buf), msg, args);
			va_end(args);
			bBuffSet = true;
		}

		if (ERROR_EXTENDED_ERROR == err)
		{
			char compBuf[TYPEDERROR_BUFFERSIZE];
			stringFormat(compBuf, sizeof(compBuf), "%s%s%s failed with %lu: %s",
													bBuffSet ? buf  : "",
													bBuffSet ? ": " : "",
													provider, errorCode, lasterr);

			pTError->set(compBuf, 0, ERROR_MESSAGEONLY, NULL);
		}
		else
		{
			pTError->set(bBuffSet ? buf : NULL, err, ERROR_SYSTEM, NULL);
		}

		SetLastError(err);
	}
}
#endif // TYPEDERROR_WNET

// Appends the message from another TError to your message, similar to how the other functions
// append a system message to yours.
// See also SafeCopyTError which is similar.
void CTypedError::SafeSetTError(CTypedError *pTError, const CTypedError *pTErrorToSet, const char *msg, ...)
{
	if (NULL != pTError)
	{
		bool bBuffSet = false;
		char buf[TYPEDERROR_BUFFERSIZE];

		if (NULL != msg)
		{
			va_list args;
			va_start(args, msg);
			vStringFormat(buf, sizeof(buf), msg, args);
			va_end(args);
			bBuffSet = true;
		}

		char composeBuf[TYPEDERROR_BUFFERSIZE];

		stringFormat(composeBuf, sizeof(composeBuf), "%s%s%s",
													bBuffSet ? buf  : "",
													bBuffSet ? ": " : "",
													CTypedError::SafeGetFullErrorTextTempBuffer(pTErrorToSet));

		pTError->set(composeBuf, 0, ERROR_MESSAGEONLY, NULL);
	}
}

// Errors in the NET (Lan Manager) range will be looked up using netmsg.dll if available.
LPTSTR CTypedError::GetSystemErrorText(DWORD err, LPTSTR lpszBuf, DWORD dwSize)
{
    DWORD dwRet;
    LPTSTR lpszTemp = NULL;

	DWORD fmFlags = FORMAT_MESSAGE_MAX_WIDTH_MASK | FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM |FORMAT_MESSAGE_ARGUMENT_ARRAY;
	HMODULE fmSource = NULL;

    if ((err >= NERR_BASE) && (err <= MAX_NERR))
	{
        fmSource = LoadLibraryEx("netmsg.dll", NULL, LOAD_LIBRARY_AS_DATAFILE);
    }
// WinSock error ARE in some of the Winsock DLL's but not in the right order! FFS!
// Have to continue using that lame loopup built-in table.
//	else if ((err >= WSABASEERR) && (err <= WSANO_DATA)) // WSANO_DATA highest I could find...
//	{
//		fmSource = LoadLibraryEx("Mswsock.dll", NULL, LOAD_LIBRARY_AS_DATAFILE);
//	}

    if (fmSource != NULL)
	{
        fmFlags |= FORMAT_MESSAGE_FROM_HMODULE;
	}

    dwRet = FormatMessage(fmFlags, fmSource, err, LANG_NEUTRAL, (LPTSTR)&lpszTemp, 0, NULL);

    if (0 == dwRet)
	{
		stringFormat(lpszBuf, dwSize, "Error %lu", err);
	}
    else
    {
		// remove any whitespace at the end (newlines and stuff like that)
		for (int i = lstrlen(lpszTemp); i > 0; i--)
		{
			if (isspace(lpszTemp[i-1]))
			{
				lpszTemp[i-1] = TEXT('\0');
			}
			else
			{
				break;
			}
		}
//		lpszTemp[lstrlen(lpszTemp)-2] = TEXT('\0');  //remove cr and newline character
		stringCopy(lpszBuf, lpszTemp, dwSize);
    }

    if ( lpszTemp )
	{
        LocalFree((HLOCAL) lpszTemp );
	}

	if (NULL != fmSource)
	{
		FreeLibrary(fmSource);
	}

    return lpszBuf;
}

#if TYPEDERROR_WINSOCK

typedef struct te_nspair
{
	WORD id;
	const char *str;
} TE_NSPair;

//  TYPEDERROR_WINSOCK

// Throw me a frickin' bone here, having to do this is so lame.
TE_NSPair WSErrorMessages[] =
{
	{ 00000, "no error code returned" },
	{ 10000, "no error code returned" },
	{ 10004, "Interrupted system call" },
	{ 10009, "Bad file number" },
	{ 10013, "Permission denied" },
	{ 10014, "Bad address" },
	{ 10022, "Invalid argument" },
	{ 10024, "Too many open files" },
	{ 10035, "Operation would block" },
	{ 10036, "Operation now in progress" },
	{ 10037, "Operation already in progress" },
	{ 10038, "Socket operation on non-socket" },
	{ 10039, "Destination address required" },
	{ 10040, "Message too long" },
	{ 10041, "Protocol wrong type for socket" },
	{ 10042, "Bad protocol option" },
	{ 10043, "Protocol not supported" },
	{ 10044, "Socket type not supported" },
	{ 10045, "Operation not supported on socket" },
	{ 10046, "Protocol family not supported" },
	{ 10047, "Address family not supported by protocol family" },
	{ 10048, "Address already in use" },
	{ 10049, "Can't assign requested address" },
	{ 10050, "Network is down" },
	{ 10051, "Network is unreachable" },
	{ 10052, "Net dropped connection or reset" },
	{ 10053, "Software caused connection abort" },
	{ 10054, "Connection reset by peer" },
	{ 10055, "No buffer space available" },
	{ 10056, "Socket is already connected" },
	{ 10057, "Socket is not connected" },
	{ 10058, "Can't send after socket shutdown" },
	{ 10059, "Too many references, can't splice" },
	{ 10060, "Connection timed out" },
	{ 10061, "Connection refused" },
	{ 10062, "Too many levels of symbolic links" },
	{ 10063, "File name too long" },
	{ 10064, "Host is down" },
	{ 10065, "No Route to Host" },
	{ 10066, "Directory not empty" },
	{ 10067, "Too many processes" },
	{ 10068, "Too many users" },
	{ 10069, "Disc Quota Exceeded" },
	{ 10070, "Stale NFS file handle" },
	{ 10071, "Too many levels of remote in path" },
	{ 10091, "Network SubSystem is unavailable" },
	{ 10092, "WINSOCK DLL Version out of range" },
	{ 10093, "Successful WSASTARTUP not yet performed" },
	{ 11001, "Host not found" },
	{ 11002, "Non-Authoritative Host not found" },
	{ 11003, "Non-Recoverable errors: FORMERR, REFUSED, NOTIMP" },
	{ 11004, "Valid name, no data record of requested type" },
	{ 0, NULL }
};

//  TYPEDERROR_WINSOCK

// Returns buffer.
char *CTypedError::GetWinSockErrorText(int errNum, char *buffer, int bufferSize)
{
	const char *foundString = NULL;

	for (int i=0; NULL != (WSErrorMessages[i].str); i++)
	{
		if (WSErrorMessages[i].id == errNum)
		{
			foundString = WSErrorMessages[i].str;
			break;
		}
	}

	if (NULL != foundString)
	{
		stringCopy(buffer, foundString, bufferSize);
	}
	else
	{
		stringFormat(buffer, bufferSize, "Winsock error %d", errNum);
	}

	return(buffer);
}
#endif // TYPEDERROR_WINSOCK
