// CCFWrapper.h: interface for the CCCFWrapper class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_CCFWRAPPER_H__6DE1D160_83FC_11D4_BF8F_0020AF872188__INCLUDED_)
#define AFX_CCFWRAPPER_H__6DE1D160_83FC_11D4_BF8F_0020AF872188__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CCSFAbstractTree;

/////////////////////////////////////////////////////////////////////////////
// CCF Wrapper defines
/////////////////////////////////////////////////////////////////////////////

#if defined(__cplusplus)
extern "C" {
#endif // __cplusplus

#define CCFDLL_FILENAME "CCFDll.dll"

// Registry key in HKEY_LOCAL_MACHINE which may point to where the DLL is installed.
#define CCFDLL_REG_PATH_ROOT			"Software\\CCF Tools"
#define CCFDLL_REG_VALUE_CCFTOOLSPATH	""

// version
#define GETCCFVERSION "GetCCFVersion"
typedef char* (*GetCCFVersionFunctionPtr)(void); 

// callback
#define SETDISPLAYSTRINGCALLBACK "SetDisplayStringCallback"
typedef void (WINAPI *DISPLAYSTRINGCALLBACK)(char *);
typedef void (*SetDisplayStringCallbackFunctionPtr) (DISPLAYSTRINGCALLBACK DisplayStringCallBack);

// ccfdecompiler
#define CCFRUNDECOMPILER          "CCFRunDecompiler"
typedef int (*CCFRunDecompilerFunctionPtr0_93)(char *szInputCCF,char *szOutputCSF, int iFlags);


// ccfcompiler
#define CCFRUNCOMPILER            "CCFRunCompiler"
typedef int (*CCFRunCompilerFunctionPtr0_93)(char *szInputCSF,char *szOutputCCF, int iFlags);

#if defined(__cplusplus)
}
#endif // __cplusplus

/////////////////////////////////////////////////////////////////////////////
// CCF Wrapper Class
/////////////////////////////////////////////////////////////////////////////

class CTypedError;
class CSNConfig;
class CProgressDlg;
class CCSFSection;

class CCCFWrapper
{
public:

	CCCFWrapper(const CSNConfig *pConfig);
	virtual ~CCCFWrapper();

	bool Init(CProgressDlg *pProgressDlg, CTypedError *pTError);

	const char *GetCCFDllPath(void);

	const char *GetCCFDllVersionString(void);

	bool CleanTempDir(CProgressDlg *pProgressDlg, CTypedError *pTError);

	// Returns success.
	// Will only decompile and read if needed, i.e. if the input has changed since last time.
	// pStrLstLog must NEVER be NULL.
	// pStrLstLog is cleared at start of function and, if successful, cleared at the end.
	bool DecompileAndRead(CProgressDlg *pProgressDlg, CStringList *pStrLstLog, CTypedError *pTError);

	// Returns success.
	// pStrLstLog must NEVER be NULL.
	// pStrLstLog is cleared at start of function and, if successful, cleared at the end.
	bool SaveAndCompile(CProgressDlg *pProgressDlg, CStringList *pStrLstLog, CTypedError *pTError);

		  CCSFSection *GetRootCSFSection(void);
	const CCSFSection *GetRootCSFSection(void) const;
		  CCSFAbstractTree *GetCSFAbstractTree(void);
	const CCSFAbstractTree *GetCSFAbstractTree(void) const;

	void SetDecompileNeeded(void);

	// Returns success.
	// Should only be called after a successful decompilation.
	// Rebuilds the Abstract Tree from the in-memory CSF data; never causes a new decompilation.
	bool RebuildAbstractTree(CTypedError *pTError);

	// Returns success.
	bool RemoveDanglingJumps(CTypedError *pTError, bool bRebuildAbstractTree);

protected:

	enum { CCFDLL_VERSION_MIN_MAJOR = 0 };
	enum { CCFDLL_VERSION_MIN_MINOR = 93 };

	enum { CCFDLL_VERSION_095_MAJOR = 0 };
	enum { CCFDLL_VERSION_095_MINOR = 95 };

	enum {	CCF_OUTPUT_FORMAT_CSF	= 1,
			CCF_OUTPUT_FORMAT_XML	= 2,
			CCF_OUTPUT_FORMAT_GENERATE_BUTTON_ID = 8 };

	enum { CCF_COMPILER_FLAG_COMPRESS_ICON = 1 };

protected:

	const CSNConfig *m_pConfig;

	HINSTANCE m_hInstCCFDLL;
	string m_sCCFDLLPath;
	string m_sCCFDLLVersion;
	int m_iCCFDLLVersionMajor;
	int m_iCCFDLLVersionMinor;

	// NULL if not yet decompiled.
	char *m_szLastDecompileAndReadInput;
	char *m_szPotentialDecompileAndReadInput;
	// Contents invalid if m_szLastDecompileInput NULL.
	ULARGE_INTEGER m_uliLastDecompileAndReadInputWrite;
	// Contents invalid if m_szPotentialDecompileInput NULL.
	ULARGE_INTEGER m_uliPotentialDecompileAndReadInputWrite;

	CCSFSection *m_pRootCSFSection;
	CCSFAbstractTree *m_pCSFAbstractTree;

	typedef struct nudellpafunctable
	{
		FARPROC *ppFunc;
		const char *funcName;
	} NudelLPAFuncTable;

	GetCCFVersionFunctionPtr			m_pGetCCFVersionFunctionFunction;
	SetDisplayStringCallbackFunctionPtr	m_pSetDisplayStringCallbackFunction;
	CCFRunDecompilerFunctionPtr0_93		m_pCCFRunDecompilerFunction0_93;
	CCFRunCompilerFunctionPtr0_93		m_pCCFRunCompilerFunction0_93;

protected:

	bool loadFunctionsFromTable(CTypedError *pTError, NudelLPAFuncTable *funcTable);

	bool removeDanglingJumps(CTypedError *pTError, CCSFSection *pSection);

	// Returns true unless the input filename has just been decompiled and read and has not been changed since.
	// Call commitDecompileAndReadNote() if we go on to successfully decompile and read the changed input.
	bool needToDecompileAndRead(const char *inputFilename);

	// Update internal variables to say that the file last given to needToDecompileAndRead has been
	// successfully decompiled and read.
	// Variables are used by needToDecompileAndRead() to see if the input has changed.
	void commitDecompileAndReadNote(void);

	bool actualDecompile(const char *inputCCFPath, const char *tempCSFPath,
							CProgressDlg *pProgressDlg, CStringList *pStrLstLog, CTypedError *pTError);

	bool actualCompile(const char *tempCSFPath, const char *outputCCFPath, int nTotalCSFLines,
						CProgressDlg *pProgressDlg, CStringList *pStrLstLog, CTypedError *pTError);
};

/////////////////////////////////////////////////////////////////////////////

#endif // !defined(AFX_CCFWRAPPER_H__6DE1D160_83FC_11D4_BF8F_0020AF872188__INCLUDED_)
